// -*- C++ -*-
/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 * 	 
 *	    Copyright (C) 1998 The LyX Team
 *
 *======================================================*/

#include <config.h>

#ifdef __GNUG__
#pragma implementation
#endif

#include "AsciiFormat.h"

void AsciiFormat::write(fmt::Symbol symb) {
	if (symb == fmt::hfill) {
		// Remember the position
		hfills.push_back(column);
		// and pretend it's a non-breakable space
		TextFormat::write(fmt::nbsp);
	} else {
		TextFormat::write(symb);
	}
}

void AsciiFormat::processLine(LString & line) {
	if (line.empty())
	    	return;
	
	// Expand any hfills
	expandspaces(line, hfills, available());

	LString text = line;
	LString margin = line;

	text.substring(leftmargin, text.length()-1);
	if (leftmargin > 0) {
		margin.substring(0, leftmargin-1);
	} else {
	    	margin.erase();
	}

	switch (alignment) {
	case flushleft: break;
	case flushright: {
		int indent = leftmargin + available();
		// available can be negative
		if (indent < 0) {
			indent = 0;
		}
		line = doindent(indent) + text;
		break;
	}
	case block:
		expandspaces(line, spaces, available());
		break;
	case center: {
		int indent = leftmargin + available()/2;
		// available can be negative
		if (indent < 0) {
			indent = 0;
		}
		line = doindent(indent) + text;
	        break;
	}
	}
}

void AsciiFormat::flushline() {
	TextFormat::flushline();
	hfills.clear();
}

void AsciiFormat::expandspaces(LString & line, vector<int> positions, 
			       int amount) {
	if (amount < 1 || positions.size() == 0)
		return;
	int each = amount / positions.size();
	int rem = amount % positions.size();

	// Create a map of how many extra characters each position gets
	vector<int> * extra = new vector<int>(positions.size(), each);

	// The remaining ones are assigned to the first positions
	for (int n=0; n<rem; ++n) {
		(*extra)[n] += 1;
	}

	for (size_t i=0; i<positions.size(); ++i) {
		int pos = positions[i];
		LString head = line;
		LString tail = line;
		head.substring(0, pos);
		if (pos+1 < tail.length()) {
			tail.substring(pos+1, tail.length()-1);
		} else {
			tail.erase();
		}
		for (int j=0; j<(*extra)[i]; ++j) {
			head += " ";
		}
		line = head + tail;
	}
	delete extra;
}
