/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor 	 
 *	     Copyright (C) 1995 Matthias Ettrich
 *           Copyright (C) 1995-1998 The LyX Team.
 *
 *           This file is Copyright (C) 1997-1998
 *           Asger Alstrup
 *
 *======================================================
 */

#include <config.h>

#include <stdlib.h> // atoi

#ifdef __GNUG__
#pragma implementation
#endif

#include "Chktex.h"
#include "LaTeX.h" // TeXErrors
#include "filetools.h"
#include "lyxlex.h"
#include "FileInfo.h"
#include "debug.h"
#include "syscall.h"
#include "syscontr.h"
#include "path.h"
#include "gettext.h"
#include "lstrings.h"

// 	$Id: Chktex.C,v 1.4 1998/10/22 16:56:04 larsbj Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: Chktex.C,v 1.4 1998/10/22 16:56:04 larsbj Exp $";
#endif /* lint */


/*
 * CLASS Chktex
 */

Chktex::Chktex(string const & chktex, string const & f, string const & p)
		: cmd(chktex), file(f), path(p)
{
}


int Chktex::run(TeXErrors &terr)
{
	// run bibtex
	string log = ChangeExtension(file, ".log", true);
	string tmp = cmd + " -q -v0 -b0 -x " + file + " -o " + log;
        Systemcalls one;
	int result= one.Startscript(Systemcalls::System, tmp);
	if (result == 0) {
		result = scanLogFile(terr);
	} else {
		result = -1;
	}
	return result;
}


int Chktex::scanLogFile(TeXErrors &terr)
{
	string token;
	int retval = 0;

	LyXLex lex(0, 0);

	string tmp = ChangeExtension(file, ".log", true);

	if (!lex.setFile(tmp)) {
		// Unable to open file. Return at once
		return -1;
	}

	while (lex.IsOK()) {
		if (lex.EatLine())
			token = lex.GetString();
		else // blank line in the file being read
			continue;

		string file, line, pos, warno, warning;
		token = split(token, file, ':');
		token = split(token, line, ':');
		token = split(token, pos, ':');
		token = split(token, warno, ':');
		token  =split(token, warning, ':');

		int lineno = atoi(line.c_str());
		warno = _("ChkTeX warning id #") + warno;
		terr.insertError(lineno, warno, warning);
		++retval;
	}
	return retval;
}
