// Communicator.C
#include <config.h>

#include "insetcitation.h"
#include "Communicator.h"
#include "LyXViewBase.h"
#include "BufferView.h"
#include "minibuffer.h"
#include "lyxparagraph.h"
#include "lyxtext.h"
#include "buffer.h"
#include "LString.h"
#include "combox.h"
#include "lyxrc.h"


// 	$Id: Communicator.C,v 1.7 1999/04/09 18:16:01 larsbj Exp $

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: Communicator.C,v 1.7 1999/04/09 18:16:01 larsbj Exp $";
#endif /* lint */


#ifdef __GNUG__
#pragma implementation
#endif


Communicator::Communicator(LyXViewBase *lv)
	: l_view(lv)
{
}


Communicator::~Communicator() 
{
	l_view = 0;
}


string Communicator::Dispatch(int action,
			       char const * arg = 0) const
{
	return l_view->getLyXFunc().Dispatch(action, arg);
}


Popups & Communicator::getPopups() const
{
	return l_view->getPopups();
}


string Communicator::getSystemLyXDir() const
{
	extern string system_lyxdir;
	return system_lyxdir;
}


bool Communicator::usingTempDir() const
{
	extern LyXRC *lyxrc;
	return lyxrc->use_tempdir;
}


string Communicator::getDefaultDocumentPath() const
{
	extern LyXRC *lyxrc;
	return lyxrc->document_path;
}


BufferView * Communicator::getCurrentView() const
{
	return l_view->currentView();
}


bool Communicator::isAvailable() const
{
	return l_view->currentView()->available();
}


void Communicator::redoBuffer() const
{
	l_view->currentView()->redoCurrentBuffer();
}


void Communicator::markDirty() const
{
	l_view->currentBuffer()->markDirty();
}


Buffer * Communicator::getBuffer() const
{
	return l_view->currentBuffer();
}


BufferParams & Communicator::getBufferParams() const
{
	return l_view->currentBuffer()->params;
}


int Communicator::switchLayoutsBetweenClasses(char old_c, 
					      char new_c) const
{
	return l_view->currentView()->text->
		SwitchLayoutsBetweenClasses(old_c, new_c,
					    l_view->currentBuffer()->paragraph);
}


void Communicator::setPaperStuff() const
{
	l_view->currentBuffer()->setPaperStuff();
}


bool Communicator::isReadOnly() const
{
	return l_view->currentBuffer()->isReadonly();
}


bool Communicator::isLinuxDoc() const
{
	return l_view->currentBuffer()->isLinuxDoc();
}


string Communicator::getFileName() const
{
	return l_view->currentBuffer()->getFileName();
}


string Communicator::getTmpDir() const
{
#warning Why is getTmpDir a part of Buffer?
	// Did you look at the definition of getTmpDir in
	// buffer.h? getTmpdir just returns the path to the
	// temporary directory. Where else should this be stored?
	//
	// Well, isn't it more global than just Buffer?
	// Shouldn't it be a part of lyxrc?
	return l_view->currentBuffer()->getTmpDir();
}


void Communicator::setPreamble(char const * new_preamble) const
{
	l_view->currentBuffer()->params.preamble = new_preamble;
	l_view->currentBuffer()->markDirty();
	setMinibuffer(_("LaTeX preamble set"));
}


char const * Communicator::getPreamble() const
{
	return l_view->currentBuffer()->params.preamble.c_str();
}


bool Communicator::openNewInset(UpdatableInset * u_inset) const
{
	return l_view->currentView()->open_new_inset(u_inset);
}


void Communicator::updateInset(Inset *inset, bool mark_dirty) const
{
	l_view->currentView()->updateInset(inset, mark_dirty);
}


// This function should be in LyXView when multiframe works ale970302
// Well I've put it here for the time being instead...  It doesn't
// really belong in LyXView and we don't want it in the gui code
// so this seems about the best option ARRae 990103
void Communicator::bibitemUpdate(Combox* combox) const
{
	combox->clear();
    
	if (!isAvailable())
		return;

	// First we check for ordinary bibliography entries
	LyXParagraph *par = l_view->currentBuffer()->paragraph;
	while (par) {
		if (par->bibkey) {
			combox->addto(par->bibkey->getContents().c_str());
		}
		par = par->next;
	}
	
	// If no ordinary bibliography entries we try bibtex entries.
	if (combox->empty()) {
		par = l_view->currentBuffer()->paragraph;
		while (par) {
			Inset *inset;
			int pos = -1;
			while ((inset = par->ReturnNextInsetPointer(pos))) {
				string tmp;
				inset->Latex(tmp,0);
				if (tmp.find(string("\\cite"))
				    != string::npos) {
					// while not the nicest test it is 
					// about the only one that currently
					// works.  I'd certainly be interested
					// in an alternative.
// I have a cool idea to replace most of
// the code in this function that involves
// the use of a Signal1<vector<string>&>
// but I need to be sure I can update
// the vector (I'm not even sure its legal
// Signal syntax).
// There are a heap of other places such
// signals could be used also.  ARRae.
					combox->addto(((InsetCitation*)inset)
						      ->getContents().c_str());
				}
				++pos;
			}
			par = par->next;
		}
	}
}


bool Communicator::hasLockingInset() const
{
	// Temporarily disabled [ale 981117]
	return false;//l_view->the_locking_inset != 0;
}


UpdatableInset * Communicator::getLockingInset() const
{
	// Temporarily disabled [ale 981117]
	return 0;//l_view->the_locking_inset->GetLockingInset();
}


void Communicator::setMinibuffer(char const * new_mb) const
{
	l_view->getMiniBuffer()->Set(new_mb);
}


void Communicator::setParagraph(bool line_top,      bool line_bottom,
				bool pagebreak_top, bool pagebreak_bottom,
				VSpace space_top,   VSpace space_bottom,
				char align,         string labelwidthstring,
				bool noindent) const
{
	l_view->currentView()->text->SetParagraph(line_top, line_bottom,
						  pagebreak_top,
						  pagebreak_bottom,
						  space_top, space_bottom,
						  align,
						  labelwidthstring,
						  noindent);
	l_view->currentView()->update(BufferView::ResetSelection
				      |BufferView::FitCursor
				      |BufferView::BufferChange);
	setMinibuffer(_("Paragraph layout set"));
}


void Communicator::setParagraphExtraOpt(int type,
					char const *width,
					char const *widthp,
					int alignment, bool hfill,
					bool start_minipage) const
{
	l_view->currentView()->text->SetParagraphExtraOpt(type,
							  width,
							  widthp,
							  alignment,
							  hfill,
							  start_minipage);
	l_view->currentView()->update(BufferView::ResetSelection
				      |BufferView::FitCursor
				      |BufferView::BufferChange);
	setMinibuffer(_("ParagraphExtra layout set"));
}


LyXParagraph * Communicator::getParAtCursor() const
{
	return l_view->currentView()->text->cursor.par;
}


int Communicator::getParLayout() const
{
	return l_view->currentView()->text->cursor.par->GetLayout();
}


int Communicator::getParAlign() const
{
	return l_view->currentView()->text->cursor.par->GetAlign();
}


string Communicator::getParLabelWidthString() const
{
	return l_view->currentView()->text->cursor.par->GetLabelWidthString();
}


LyXParagraphParameters & Communicator::getParaParams() const
{
	return l_view->currentView()->text->cursor.par->FirstPhysicalPar()->pparams;
}
