/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 *	     Copyright (C) 1995 Matthias Ettrich
 *           Copyright (C) 1995-1999 The LyX Team.
 *
 *           This file is Copyright (C) 1996-1999
 *           Lars Gullik Bjnnes
 *
 * ======================================================
 */

#include <config.h>

#include <algorithm>
#include FSTREAM_HEADER

#include "DepTable.h"
#include "lstrings.h"
#include "lyxlib.h"

struct DepTable::update_sum {
	void operator()(DepList::value_type & dli) {
		dli.second.old_sum = dli.second.new_sum;
		dli.second.new_sum = lyxsum(dli.first.c_str());
		lyxerr[Debug::INFO] << "Dep. update(" 
				    << dli.first << ", "
				    << dli.second.new_sum << ", " 
				    << dli.second.old_sum << ")\n";
	}
};

class DepTable::write_dep {
public:
	write_dep(ostream & o)
		: ost(o) {}
	void operator()(DepList::value_type & vt) {
		lyxerr[Debug::INFO] << "Dep. write ("
				    << vt.first << ", "
				    << vt.second.new_sum << ", "
				    << vt.second.old_sum << ")\n";
		ost << vt.first.c_str() << ' ' 
		    << vt.second.new_sum << ' ' 
		    << vt.second.old_sum << '\n';
	}
private:
	ostream & ost;
};

void DepTable::insert(string const & f,
		      unsigned long one,
		      unsigned long two)
{
	DepList::const_iterator dli = deplist.find(f);
	if (dli != deplist.end()) return;
	lyxerr[Debug::INFO] << "Dep. insert ("
			    << f << ", " << one << ", " << two << ")\n"; 
	sumsum tmp(one, two);
	deplist[f] = tmp;
}
		

void DepTable::update()
{
	for_each(deplist.begin(), deplist.end(), update_sum());
}


bool DepTable::sumchange()
{
	DepList::const_iterator dli = deplist.begin();
	for(; dli != deplist.end(); ++dli) {
		if((*dli).second.old_sum != (*dli).second.new_sum)
			return true;
	}
	return false;
}


bool DepTable::haschanged(string const & fil)
{
	if (fil.empty()) return false;

	DepList::const_iterator dli = deplist.find(fil);

	if(dli == deplist.end()) return false;

	return (*dli).second.new_sum != (*dli).second.old_sum &&
		(*dli).second.new_sum != 0;
}


void DepTable::write(string const & f)
{
	ofstream fil(f.c_str());
	for_each(deplist.begin(), deplist.end(), write_dep(fil));
}


void DepTable::read(string const & f)
{
	string na;
	unsigned long one = 0;
	unsigned long two = 0;
	// scan the file line by line
	ifstream fil(f.c_str());
	while (fil >> na >> one >> two) {
		lyxerr[Debug::INFO] << "Read dep(" << na << ", " 
				    << one << ", "
				    << two << ")\n";
		insert(na, one, two);
	}
}
