// -*- C++ -*-
/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 * 	 
 *	    Copyright (C) 1997 Asger Alstrup
 *           and the LyX Team.
 *
 *======================================================*/

#ifndef FONTINFO_H
#define FONTINFO_H 

#ifdef __GNUG__
#pragma interface
#endif

#include "LString.h"
#include <map>

/** This class manages a font.
  The idea is to create a FontInfo object with a font name pattern with a 
  wildcard at the size field. Then this object can host request for font-
  instances of any given size. If no exact match is found, the closest size
  is chosen instead. If the font is scalable, the flag
  lyxrc->use_scalable_fonts determines whether to allow scalable fonts
  to give an exact match. */
class FontInfo {
public:
	///
	FontInfo() { init(); }

	///
	FontInfo(string const & pat)
	: pattern(pat) { init(); }

	/// Destructor
	~FontInfo() { release(); }

	/// Does any font match our pattern?
	bool exist() {
		query();
		return !fontList.empty();
	}

	/// Is this font scalable?
	bool isScalable() {
		query();
		return scalable();
	}

	/// Get existing pattern
	string getPattern() const { return pattern; }

	/// Set new pattern
	void setPattern(string const & pat);

	/** Return full name of font close to this size.
	  If impossible, result is the empty string */
	string getFontname(int size);
private:
	/// Font pattern (with wildcard for size)
	string pattern;

	///
	typedef map<unsigned int, string, less<unsigned int> > FontList;
	
	/// A map of all the fonts sizes and corresponding font-strings
        FontList fontList;

	/// Did we query X about this font?
	bool queried;

	// If the font has a size entry of 0 in fontList it is scalable
	bool scalable() {
		return fontList.find(0) != fontList.end();
	}
	
	/// Initialize empty record
	void init()
	{
		queried = false;
	}

	/// Release allocated stuff
	void release();

	/// Ask X11 about this font pattern
	void query();

	/// Build newly sized font string 
	static string resize(string const &, int size);
};
#endif
