/* This file is part of
* ======================================================
* 
*           LyX, The Document Processor
*        
*           Copyright (C) 1995 Matthias Ettrich
*           Copyright (C) 1995-1998 The LyX Team.
*
*======================================================*/

#include <config.h>

#ifdef __GNUG__
#pragma implementation
#endif

#include "lyx_gui_misc.h" // temporary: connecting updateAllVisibleBufferRelatedPopups
#include "LyXView.h"
#include "lyx_main.h"
#include "lyxlookup.h"
#include "toolbar.h"
#include "minibuffer.h"
#include "lyxfunc.h"
#include "Popups.h"
#include "lyx.xpm"
#include "debug.h"
#include "layout_forms.h"
#include "intl.h"
#include "lyxrc.h"
#include "lyxscreen.h"
#include "filetools.h"        // OnlyFilename()
#include "layout.h"
#include "lyxtext.h"
#include "textclasslist.h"
#include "buffer.h"
#include "menus.h"

// 	$Id: LyXView.C,v 1.16 1998/10/17 14:46:10 larsbj Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: LyXView.C,v 1.16 1998/10/17 14:46:10 larsbj Exp $";
#endif /* lint */

MiniBuffer *minibuffer;/* a pointer to the one in LyXView
			  should be removed as soon as possible */

extern void QuitLyX();


LyXView::LyXView(int width, int height)
	: LyXViewBase(width, height)
{
	createMainWindow(width,height);
	// temporary until all popups moved into Popups.
	connect_to_function(popups.updateBufferDependent,
			    &updateAllVisibleBufferRelatedPopups);
}


LyXView::~LyXView()
{
	delete menus;
	delete toolbar;
	delete frameHandler;
	delete minibuffer;
}


/// Redraw the main form.
void LyXView::redraw() {
	lyxerr[Debug::INFO] << "LyXView::redraw()\n";
	fl_redraw_form(_form);
}


// Callback for autosave timer
void LyXView::AutosaveTimerCB(FL_OBJECT *ob, long)
{
	LyXView *v = (LyXView*)ob->u_vdata;
	lyxerr[Debug::INFO] << "Running AutoSave()\n";
	v->currentView()->autoSave();
}


/// Reset autosave timer
void LyXView::resetAutosaveTimer()
{
	if (lyxrc->autosave)
		fl_set_timer(_form_main->timer_autosave, lyxrc->autosave);
}


// Callback for close main form from window manager
int LyXView::atCloseMainFormCB(FL_FORM *, void *)
{
	QuitLyX();
	return FL_IGNORE;
}


void LyXView::setPosition(int x, int y)
{
	position_set = true;
	fl_set_form_position(_form, x, y);
}


void LyXView::show()
{
	fl_show_form(_form,
		     position_set ? FL_PLACE_POSITION :
		     FL_PLACE_CENTER | FL_FREE_SIZE,
		    FL_FULLBORDER , "LyX");
	InitLyXLookup(fl_display, _form->window);
}


void LyXView::createMainWindow(int width, int height)
	/* to make this work as it should, .lyxrc should have been
	 * read first; OR maybe this one should be made dynamic.
	 * Hmmmm. Lgb. 
	 * We will probably not have lyxrc before the main form is
	 * initialized, because error messages from lyxrc parsing 
	 * are presented (and rightly so) in GUI popups. Asger. 
	 */
{
	FL_OBJECT *obj;
	
	FD_form_main *fdui = (FD_form_main *)
		fl_calloc(1, sizeof(FD_form_main));

	_form_main = fdui;

	// the main form
	_form = fdui->form_main = fl_bgn_form(FL_NO_BOX, width, height);
	fl_set_form_atclose(_form, atCloseMainFormCB, 0);
	// Install the raw callback for keyboard events 
	fl_register_raw_callback(_form,
				 KeyPressMask,
				 KeyPressMask_raw_callback);
	fdui->form_main->u_vdata = this;
	obj = fl_add_box(FL_FLAT_BOX,0,0,width,height,"");
	fl_set_object_color(obj,FL_MCOL,FL_MCOL);

	// Parameters for the appearance of the main form
	const int air = 2;
	const int bw = abs(fl_get_border_width());
	
	//
	// THE MENUBAR
	//

	menus = new Menus(this, air);

	//
	// TOOLBAR
	//

	toolbar = new Toolbar(lyxrc->toolbarBackend, this, air, 30 + air + bw);

	// Setup the toolbar
	toolbar->set();

	//
	// WORKAREA
	//

	const int ywork = 60 + 2*air + bw;
	const int workheight = height - ywork - (25 + 2*air);

	frameHandler = new FrameHandler(this, air, ywork,
					width - 3 * air,
					workheight);

	//
	// MINIBUFFER
	//

	minibuffer = new MiniBuffer(this, air, height-(25+air), 
					  width-(2*air), 25);
	::minibuffer = minibuffer; // to be removed later


	//
	// TIMERS
	//
	
	// timer_autosave
	fdui->timer_autosave = obj = fl_add_timer(FL_HIDDEN_TIMER,
						  0,0,0,0,"Timer");
	fl_set_object_callback(obj,AutosaveTimerCB,0);
	obj->u_vdata = this;

	//
	// Misc
	//

        //  assign an icon to main form
        unsigned int w, h;
        Pixmap lyx_p, lyx_mask;
        lyx_p = fl_create_from_pixmapdata(fl_root,
					  lyx_xpm,
					  &w,
					  &h,
					  &lyx_mask,
					  0,
					  0,
					  0); // this leaks
        fl_set_form_icon(fdui->form_main, lyx_p, lyx_mask);

	// set min size
	fl_set_form_minsize(fdui->form_main, 50, 50);
	
	fl_end_form();
}


void LyXView::init()
{
	// Set the textclass choice
	invalidateLayoutChoice();
	updateLayoutChoice();
	updateDocumentClassChoice();
	
	// Start autosave timer
	if (lyxrc->autosave)
		fl_set_timer(_form_main->timer_autosave, lyxrc->autosave);
	
	
        intl.InitKeyMapper(lyxrc->use_kbmap);
}


void LyXView::invalidateLayoutChoice()
{
	last_textclass = -1;
}




// This is necessary, since FL_FREE-Objects doesn't get all keypress events
// as FL_KEYBOARD events :-(   Matthias 280596
int LyXView::KeyPressMask_raw_callback(FL_FORM *fl, void *xev)
{
	//LyXView *view = (LyXView*) fl->u_vdata;
	LyXView *view = static_cast<LyXView*>(fl->u_vdata);
	
	int retval = 0;  // 0 means XForms should have a look at this event

	// Funny. Even though the raw_callback is registered with KeyPressMask,
	// also KeyRelease-events are passed through :-(
	// [It seems that xforms puts them in pairs... (JMarc)]
	if (((XEvent*)xev)->type == KeyPress
	    && view->currentView()->readyForKeypress()) {
		retval = view->getLyXFunc().processKeyEvent((XEvent*)xev);
	}

	return retval;
}


// Updates the title of the window with the filename of the current document
void LyXView::set_window_title(string const & title) {
		fl_set_form_title(_form, title.c_str());
}
