// -*- C++ -*-
/* This file is part of
* ======================================================
* 
*           LyX, The Document Processor
*        
*           Copyright (C) 1995 Matthias Ettrich
*           Copyright (C) 1995-1998 The LyX Team.
*
* ====================================================== */

#include <config.h>

#ifdef __GNUG__
#pragma implementation
#endif

#include "LyXViewBase.h"
#include "debug.h"
#include "toolbar.h"
#include "buffer.h"
#include "textclasslist.h"
#include "filetools.h"
#include "lyxtext.h"

// 	$Id: LyXViewBase.C,v 1.6 1999/05/12 12:13:40 larsbj Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: LyXViewBase.C,v 1.6 1999/05/12 12:13:40 larsbj Exp $";
#endif /* lint */

LyXViewBase::LyXViewBase(int /*width*/, int /*height*/)
	: lyxfunc(this), comm(this), popups(&comm)
{
}


LyXViewBase::~LyXViewBase()
{
}


void LyXViewBase::updateLayoutChoice()
{
	static int current_layout = 0;
	
	// Update the layout display
	if (!toolbar->combox) return;
	
	// This has a side-effect that the layouts are not showed when no
	// document is loaded.
	if (currentBuffer()==0) {
		toolbar->combox->clear();
		toolbar->combox->Redraw();
		return;	
	}

	// If textclass is different, we need to update the list
	if (toolbar->combox->empty() ||
	    (last_textclass != currentBuffer()->params.textclass)) {
		last_textclass = currentBuffer()->params.textclass;
		toolbar->combox->clear();
		for (int i = 0;
		     textclasslist.nameOfLayout(last_textclass, i) !="@@end@@";
		     ++i) {
			LyXLayout const *layout = textclasslist.style(last_textclass, i);
			if (layout->obsoleted_by.empty())
				toolbar->combox->addline(layout->name.c_str());
			else
				toolbar->combox->addline(("@N"+layout->name).c_str());
		}
		current_layout = 0;
	}

	char layout = currentView()->text->cursor.par->GetLayout();

	if (layout != current_layout){
		toolbar->combox->select(layout + 1);
		current_layout = layout;
	}

	// We need to do this
	toolbar->combox->Redraw();
}


void LyXViewBase::updateDocumentClassChoice()
{
	unsigned int i = textclasslist.size();
	vector<string> strvec;
	for(unsigned int j=0; j < i; ++j)
		strvec.push_back(textclasslist.descOfClass(j));
	popups.SetDocumentClassChoice(strvec);
}


// Updates the title of the window with the filename of the current document
void LyXViewBase::updateWindowTitle() {
	static string last_title="LyX";
	string title = "LyX";

	if (currentView()->available()) {
		string cur_title = currentBuffer()->getFileName();
		if (!cur_title.empty()){
			title += ": " + OnlyFilename(cur_title);
			if (!currentBuffer()->isLyxClean())
				title += _(" (Changed)");
			if (currentBuffer()->isReadonly())
				title += _(" (read only)");
		}
	}
	// Don't update title if it's the same as last time
	if (title != last_title) {
		set_window_title(title);
		last_title = title;
	}
}
