/* This file is part of
* ======================================================
* 
*           LyX, The Document Processor
* 	 
*	    Copyright (C) 1995 Matthias Ettrich
*           Copyright (C) 1995-1998 The LyX Team.
*
*======================================================*/

#include <config.h>
#include <stdlib.h>

#ifdef __GNUG__
#pragma implementation "bufferparams.h"
#endif

#include "bufferparams.h"
#include "tex-strings.h"
#include "textclasslist.h"
#include "vspace.h"
#include "debug.h"
#include "lyxlib.h"
#include "lyxlex.h"
#include "lyx_gui_misc.h"

// 	$Id: bufferparams.C,v 1.8 1998/10/22 16:56:10 larsbj Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: bufferparams.C,v 1.8 1998/10/22 16:56:10 larsbj Exp $";
#endif /* lint */


BufferParams::BufferParams()
{
	paragraph_separation = LYX_PARSEP_INDENT;
	defskip = VSpace::MEDSKIP; 
	quotes_language = InsetQuotes::EnglishQ;
	quotes_times = InsetQuotes::DoubleQ;
	fontsize = "default";
	textclass = 0; 
        /*  PaperLayout */
	papersize = PAPER_DEFAULT;
        papersize2 = VM_PAPER_DEFAULT; /* DEFAULT */
        paperpackage = PACKAGE_NONE;
	orientation = ORIENTATION_PORTRAIT;
        use_geometry = false;
        use_amsmath = false;
	secnumdepth = 3;
	tocdepth = 3;
	language = "default";
	fonts = "default";
	inputenc = "default";
	graphicsDriver = "default";
	sides = 1;
	columns = 1;
	pagestyle = "default";
	for(int iter = 0; iter < 4; ++iter) {
		user_defined_bullets[iter] = temp_bullets[iter] 
			                   = ITEMIZE_DEFAULTS[iter];
	}
	allowAccents = false;
}


BufferParams & BufferParams::operator=(BufferParams const & bp)
{
	paragraph_separation = bp.paragraph_separation;
	defskip = bp.defskip;
	quotes_language = bp.quotes_language;
	quotes_times = bp.quotes_times;
	fontsize = bp.fontsize;
	spacing = bp.spacing;
	papersize = bp.papersize;
        papersize2 = bp.papersize2;
        paperpackage = bp.paperpackage;
	orientation = bp.orientation;
        use_geometry = bp.use_geometry;
        use_amsmath = bp.use_amsmath;
        paperwidth = bp.paperwidth;
        paperheight = bp.paperheight;
        leftmargin = bp.leftmargin;
        topmargin = bp.topmargin;
        rightmargin = bp.rightmargin;
        bottommargin = bp.bottommargin;
        headheight = bp.headheight;
        headsep = bp.headsep;
        footskip = bp.footskip;
	secnumdepth = bp.secnumdepth;
	tocdepth = bp.tocdepth;
	language = bp.language;
	fonts = bp.fonts;
	inputenc = bp.inputenc;
	preamble = bp.preamble;
	options = bp.options;
	graphicsDriver = bp.graphicsDriver;
	textclass = bp.textclass;
	
	pagestyle = bp.pagestyle;
	float_placement = bp.float_placement;
	for (int i = 0; i < 4; ++i) {
		temp_bullets[i] = bp.temp_bullets[i];
		user_defined_bullets[i] = bp.user_defined_bullets[i];
	}
	allowAccents=bp.allowAccents;
	
	return *this;
}


void BufferParams::read(LyXLex &lex) {
	int tmpret;

	string token;
	while (lex.IsOK()) {
		lex.nextToken();
		token = lex.GetString();
		
		if (token == "\\begin_preamble") {
			readPreamble(lex);
		} else if (token == "\\textclass") {
			lex.EatLine();
			textclass = textclasslist.numberOfClass(lex.GetString());
			if (textclass == -1) {
				lex.printError("Unknown textclass `$$Token'");
				textclass = 0;
			}
			if (!textclasslist.load(textclass)) {
				// if the textclass wasn't loaded properly
				// we need to either substitute another
				// or stop loading the file.
				// I can substitute but I don't see how I can
				// stop loading... ideas??  ARRae980418
				WriteAlert(_("Textclass Loading Error!"),
				           string(_("Can't load textclass ")) +
					   textclasslist.nameOfClass(textclass),
					   _("-- substituting default"));
				textclass = 0;
			}
		} else if (token == "\\options") {
			lex.EatLine();
			options = lex.GetString();
		} else if (token == "\\language") {
			readLanguage(lex);	    
		} else if (token == "\\fontencoding") {
			lex.EatLine();
		} else if (token == "\\inputencoding") {
			lex.EatLine();
			inputenc = lex.GetString();
		} else if (token == "\\graphics") {
			readGraphicsDriver(lex);
		} else if (token == "\\fontscheme") {
			lex.EatLine();
			fonts = lex.GetString();
		} else if (token == "\\paragraph_separation") {
			tmpret = lex.FindToken(string_paragraph_separation);
			if (tmpret == -1) ++tmpret;
			if (tmpret != LYX_LAYOUT_DEFAULT) 
				paragraph_separation = tmpret;
		} else if (token == "\\defskip") {
			lex.nextToken();
			defskip = VSpace(lex.GetString());
		} else if (token == "\\quotes_language") {
			tmpret = lex.FindToken(string_quotes_language);
			if (tmpret == -1) ++tmpret;
			if (tmpret != LYX_LAYOUT_DEFAULT) {
				InsetQuotes::quote_language tmpl =
					InsetQuotes::EnglishQ;
				switch(tmpret) {
				case 0:
					tmpl = InsetQuotes::EnglishQ;
					break;
				case 1:
					tmpl = InsetQuotes::SwedishQ;
					break;
				case 2:
					tmpl = InsetQuotes::GermanQ;
					break;
				case 3:
					tmpl = InsetQuotes::PolishQ;
					break;
				case 4:
					tmpl = InsetQuotes::FrenchQ;
					break;
				case 5:
					tmpl = InsetQuotes::DanishQ;
					break;	
				}
				quotes_language = tmpl;
			}
		} else if (token == "\\quotes_times") {
			lex.nextToken();
			switch(lex.GetInteger()) {
			case 1: 
				quotes_times = InsetQuotes::SingleQ; 
				break;
			case 2: 
				quotes_times = InsetQuotes::DoubleQ; 
				break;
			}
                } else if (token == "\\papersize") {
			tmpret = lex.FindToken(string_papersize);
			if (tmpret == -1)
                                ++tmpret;
                        else
                                papersize2 = tmpret;
                } else if (token == "\\paperpackage") {
			tmpret = lex.FindToken(string_paperpackages);
			if (tmpret == -1) {
                                ++tmpret;
                                paperpackage = PACKAGE_NONE;
                        } else
                                paperpackage = tmpret;
		} else if (token == "\\use_geometry") {
			lex.nextToken();
			use_geometry = lex.GetInteger();
		} else if (token == "\\use_amsmath") {
			lex.nextToken();
			use_amsmath = lex.GetInteger();
		} else if (token == "\\paperorientation") {
			tmpret = lex.FindToken(string_orientation);
			if (tmpret == -1) ++tmpret;
			if (tmpret != LYX_LAYOUT_DEFAULT) 
				orientation = tmpret;
		} else if (token == "\\paperwidth") {
			lex.next();
			paperwidth = lex.GetString();
		} else if (token == "\\paperheight") {
			lex.next();
			paperheight = lex.GetString();
		} else if (token == "\\leftmargin") {
			lex.next();
			leftmargin = lex.GetString();
		} else if (token == "\\topmargin") {
			lex.next();
			topmargin = lex.GetString();
		} else if (token == "\\rightmargin") {
			lex.next();
			rightmargin = lex.GetString();
		} else if (token == "\\bottommargin") {
			lex.next();
			bottommargin = lex.GetString();
		} else if (token == "\\headheight") {
			lex.next();
			headheight = lex.GetString();
		} else if (token == "\\headsep") {
			lex.next();
			headsep = lex.GetString();
		} else if (token == "\\footskip") {
			lex.next();
			footskip = lex.GetString();
		} else if (token == "\\paperfontsize") {
			lex.nextToken();
			fontsize = strip(lex.GetString());
		} else if (token == "\\papercolumns") {
			lex.nextToken();
			columns = lex.GetInteger();
		} else if (token == "\\papersides") {
			lex.nextToken();
			sides = lex.GetInteger();
		} else if (token == "\\paperpagestyle") {
		        lex.nextToken();
			pagestyle = strip(lex.GetString());
		} else if (token == "\\bullet") {
			lex.nextToken();
			int index = lex.GetInteger();
			lex.nextToken();
			int temp_int = lex.GetInteger();
			user_defined_bullets[index].setFont(temp_int);
			temp_bullets[index].setFont(temp_int);
			lex.nextToken();
			temp_int = lex.GetInteger();
			user_defined_bullets[index].setCharacter(temp_int);
			temp_bullets[index].setCharacter(temp_int);
			lex.nextToken();
			temp_int = lex.GetInteger();
			user_defined_bullets[index].setSize(temp_int);
			temp_bullets[index].setSize(temp_int);
			lex.nextToken();
			string temp_str = lex.GetString();
			if (temp_str != "\\end_bullet") {
				// this element isn't really necessary for
				// parsing but is easier for humans
				// to understand bullets. Put it back and
				// set a debug message?
				lex.printError("\\end_bullet expected, got" + temp_str);
				//how can I put it back?
			}
		} else if (token == "\\bulletLaTeX") {
			lex.nextToken();
			int index = lex.GetInteger();
			lex.next();
			string temp_str = lex.GetString(), sum_str;
			while (temp_str != "\\end_bullet") {
				// this loop structure is needed when user
				// enters an empty string since the first
				// thing returned will be the \\end_bullet
				// OR
				// if the LaTeX entry has spaces. Each element
				// therefore needs to be read in turn
				sum_str += temp_str;
				lex.next();
				temp_str = lex.GetString();
			}
			user_defined_bullets[index].setText(sum_str);
			temp_bullets[index].setText(sum_str);
		} else if (token == "\\secnumdepth") {
			lex.nextToken();
			secnumdepth = lex.GetInteger();
		} else if (token == "\\tocdepth") {
			lex.nextToken();
			tocdepth = lex.GetInteger();
		} else if (token == "\\spacing") {
			lex.next();
			string tmp = strip(lex.GetString());
			if (tmp == "single") {
				spacing.set(Spacing::Single);
			} else if (tmp == "onehalf") {
				spacing.set(Spacing::Onehalf);
			} else if (tmp == "double") {
				spacing.set(Spacing::Double);
			} else if (tmp == "other") {
				lex.next();
				spacing.set(Spacing::Other,
						   lex.GetFloat());
			} else {
				lex.printError("Unknown spacing token: '$$Token'");
			}
		} else if (token == "\\float_placement") {
			lex.nextToken();
			float_placement = lex.GetString();
		} else if (token == "\\end_parameters") {
			// We step this and our work is done.
			break;
		} else {
			lex.printError("Unknown parameter token: `$$Token'");
		}
	}
	// Let's verify that this was read properly.
	if (token != "\\end_parameters") {
		lex.printError("Missing \\end_parameters");
	}
}


void BufferParams::writeFile(FILE *file)
{
	// The top of the file is written by the buffer.
	// Prints out the buffer info into the .lyx file given by file

 	// the textclass
 	fprintf(file, "\\textclass %s\n",
 		textclasslist.nameOfClass(textclass).c_str());
	
	// then the the preamble
	if (!preamble.empty()) {
		fprintf(file, "\\begin_preamble\n");
		// remove '\n' from the end of preamble
		preamble = strip(preamble, '\n');
		
		// write out the whole preamble  in one go
		fwrite(preamble.c_str(),
		       sizeof(char),
		       preamble.length(),
		       file);
		fprintf(file, "\n\\end_preamble\n");
	}
	
      
	// The options 
	if (!options.empty()) {
		fprintf(file,
			"\\options %s\n",
			options.c_str());
	}
   
	// Then the text parameters
	fprintf(file, "\\language %s\n", language.c_str());
	fprintf(file, "\\inputencoding %s\n", inputenc.c_str());
	fprintf(file, "\\fontscheme %s\n", fonts.c_str());
	fprintf(file, "\\graphics %s\n", graphicsDriver.c_str());

	if (!float_placement.empty()) {
		fprintf(file,
			"\\float_placement %s\n",
			float_placement.c_str());
	}
	fprintf(file, "\\paperfontsize %s\n", fontsize.c_str());

	spacing.writeFile(file);

	fprintf(file, "\\papersize %s\n", string_papersize[papersize2]);
	fprintf(file, "\\paperpackage %s\n",
                string_paperpackages[paperpackage]);
	fprintf(file, "\\use_geometry %d\n",use_geometry);
	fprintf(file, "\\use_amsmath %d\n",use_amsmath);
	fprintf(file, "\\paperorientation %s\n",
		string_orientation[orientation]);
        if (!paperwidth.empty())
            fprintf(file, "\\paperwidth %s\n",
                    VSpace(paperwidth).asLyXCommand().c_str());
        if (!paperheight.empty())
            fprintf(file, "\\paperheight %s\n",
                    VSpace(paperheight).asLyXCommand().c_str());
        if (!leftmargin.empty())
            fprintf(file, "\\leftmargin %s\n",
                    VSpace(leftmargin).asLyXCommand().c_str());
        if (!topmargin.empty())
            fprintf(file, "\\topmargin %s\n",
                    VSpace(topmargin).asLyXCommand().c_str());
        if (!rightmargin.empty())
            fprintf(file, "\\rightmargin %s\n",
                    VSpace(rightmargin).asLyXCommand().c_str());
        if (!bottommargin.empty())
            fprintf(file, "\\bottommargin %s\n",
                    VSpace(bottommargin).asLyXCommand().c_str());
        if (!headheight.empty())
            fprintf(file, "\\headheight %s\n",
                    VSpace(headheight).asLyXCommand().c_str());
        if (!headsep.empty())
            fprintf(file, "\\headsep %s\n",
                    VSpace(headsep).asLyXCommand().c_str());
        if (!footskip.empty())
            fprintf(file, "\\footskip %s\n",
                    VSpace(footskip).asLyXCommand().c_str());
	fprintf(file, "\\secnumdepth %d\n", secnumdepth);
	fprintf(file, "\\tocdepth %d\n", tocdepth);
	fprintf(file, "\\paragraph_separation %s\n",
		string_paragraph_separation[paragraph_separation]);
	fprintf(file, "\\defskip %s\n", defskip.asLyXCommand().c_str());
	fprintf(file, "\\quotes_language %s\n",
		string_quotes_language[quotes_language]);
	switch(quotes_times) {
	case InsetQuotes::SingleQ: 
		fprintf(file, "\\quotes_times 1\n"); break;
	case InsetQuotes::DoubleQ: 
		fprintf(file, "\\quotes_times 2\n"); break;
	}		
	fprintf(file, "\\papercolumns %d\n", columns);
	fprintf(file, "\\papersides %d\n", sides);
	fprintf(file, "\\paperpagestyle %s\n", pagestyle.c_str());
	for (int i = 0; i < 4; ++i) {
		if (user_defined_bullets[i] != ITEMIZE_DEFAULTS[i]) {
			if (user_defined_bullets[i].getFont() != -1) {
				fprintf(file, "\\bullet %d\n\t%d\n\t%d\n\t%d\n\\end_bullet\n",
						i,
						user_defined_bullets[i].getFont(),
						user_defined_bullets[i].getCharacter(),
						user_defined_bullets[i].getSize());
			}
			else {
				fprintf(file, "\\bulletLaTeX %d\n\t%s\n\\end_bullet\n",
						i,
						user_defined_bullets[i].c_str());
			}
		}
	}
}


void BufferParams::useClassDefaults() {
	LyXTextClass const *tclass = textclasslist.textClass(textclass);

	sides = tclass->sides;
	columns = tclass->columns;
	pagestyle = tclass->pagestyle;
	options = tclass->options;
	secnumdepth = tclass->secnumdepth;
	tocdepth = tclass->tocdepth;
}


void BufferParams::readPreamble(LyXLex &lex)
{
	if (lex.GetString() != "\\begin_preamble")
		lyxerr << "Error (BufferParams::readPreamble):"
			"consistency check failed.\n";

	preamble = lex.getLongString("\\end_preamble");
}


void BufferParams::readLanguage(LyXLex &lex)
{
	string tmptok;
	string test;
	int n = 0;
	
	if (!lex.next()) return;
	
	tmptok = lex.GetString();
	// check if tmptok is part of tex_babel in tex-defs.h
	while (true) {
		test = tex_babel[n++];
		
		if (test == tmptok) {
			language = tmptok;
			break;
		}
		else if (test == "") {
			lyxerr << "Warning: language `"
			       << tmptok << "' not recognized!\n";
			lyxerr << "         Setting language to `default'.\n";
			language = "default";
			break;	 
		}      
	}
}


void BufferParams::readGraphicsDriver(LyXLex &lex)
{
	string tmptok;
	string test;
	int n=0;
	
	
	if (!lex.next()) return;
	
	tmptok = lex.GetString();
	// check if tmptok is part of tex_graphics in tex_defs.h
	while (true) {
		test = tex_graphics[n++];
		
		if (test == tmptok) {	 
			graphicsDriver = tmptok;
			break;
		}      
		else if (test == "last_item") {
			lex.printError(
				"Warning: graphics driver `$$Token' not recognized!\n"
				"         Setting graphics driver to `default'.\n");
			graphicsDriver = "default";
			break;
		}      
	}
}
