// -*- C++ -*-
/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 * 	 
 *	    Copyright (C) 1999 Asger Alstrup Nielsen
 *
 *======================================================*/

#include <config.h>

#ifdef __GNUG__
#pragma implementation
#endif

#include <algorithm>
using std::lower_bound;

#include "EncLatin9.h"

static
wchar_t a0_ff_to_unicode[] = { 
0x00A0, 0x00A1, 0x00A2, 0x00A3, 0x00A4, 0x00A5, 0x00A6, 0x00A7,
0x00A8, 0x00A9, 0x00AA, 0x00AB, 0x00AC, 0x00AD, 0x00AE, 0x00AF,
0x00B0, 0x00B1, 0x00B2, 0x00B3, 0x00B4, 0x00B5, 0x00B6, 0x00B7,
0x00B8, 0x00B9, 0x00BA, 0x00BB, 0x00BC, 0x00BD, 0x00BE, 0x00BF,
0x00C0, 0x00C1, 0x00C2, 0x00C3, 0x00C4, 0x00C5, 0x00C6, 0x00C7,
0x00C8, 0x00C9, 0x00CA, 0x00CB, 0x00CC, 0x00CD, 0x00CE, 0x00CF,
0x011E, 0x00D1, 0x00D2, 0x00D3, 0x00D4, 0x00D5, 0x00D6, 0x00D7,
0x00D8, 0x00D9, 0x00DA, 0x00DB, 0x00DC, 0x0130, 0x015E, 0x00DF,
0x00E0, 0x00E1, 0x00E2, 0x00E3, 0x00E4, 0x00E5, 0x00E6, 0x00E7,
0x00E8, 0x00E9, 0x00EA, 0x00EB, 0x00EC, 0x00ED, 0x00EE, 0x00EF,
0x011F, 0x00F1, 0x00F2, 0x00F3, 0x00F4, 0x00F5, 0x00F6, 0x00F7,
0x00F8, 0x00F9, 0x00FA, 0x00FB, 0x00FC, 0x0131, 0x015F, 0x00FF
};

static
wchar_t a0_ff_sorted_unicode[] = {
0x00A0, 0x00A1, 0x00A2, 0x00A3, 0x00A4, 0x00A5, 0x00A6, 0x00A7,
0x00A8, 0x00A9, 0x00AA, 0x00AB, 0x00AC, 0x00AD, 0x00AE, 0x00AF,
0x00B0, 0x00B1, 0x00B2, 0x00B3, 0x00B4, 0x00B5, 0x00B6, 0x00B7,
0x00B8, 0x00B9, 0x00BA, 0x00BB, 0x00BC, 0x00BD, 0x00BE, 0x00BF,
0x00C0, 0x00C1, 0x00C2, 0x00C3, 0x00C4, 0x00C5, 0x00C6, 0x00C7,
0x00C8, 0x00C9, 0x00CA, 0x00CB, 0x00CC, 0x00CD, 0x00CE, 0x00CF,
0x00D1, 0x00D2, 0x00D3, 0x00D4, 0x00D5, 0x00D6, 0x00D7, 0x00D8,
0x00D9, 0x00DA, 0x00DB, 0x00DC, 0x00DF, 0x00E0, 0x00E1, 0x00E2,
0x00E3, 0x00E4, 0x00E5, 0x00E6, 0x00E7, 0x00E8, 0x00E9, 0x00EA,
0x00EB, 0x00EC, 0x00ED, 0x00EE, 0x00EF, 0x00F1, 0x00F2, 0x00F3,
0x00F4, 0x00F5, 0x00F6, 0x00F7, 0x00F8, 0x00F9, 0x00FA, 0x00FB,
0x00FC, 0x00FF, 0x011E, 0x011F, 0x0130, 0x0131, 0x015E, 0x015F,
0x0000 // dummy element
};

static
unsigned char a0_ff_sorted_latin9[] = {
0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5, 0xA6, 0xA7,
0xA8, 0xA9, 0xAA, 0xAB, 0xAC, 0xAD, 0xAE, 0xAF,
0xB0, 0xB1, 0xB2, 0xB3, 0xB4, 0xB5, 0xB6, 0xB7,
0xB8, 0xB9, 0xBA, 0xBB, 0xBC, 0xBD, 0xBE, 0xBF,
0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7,
0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF,
0xD1, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6, 0xD7, 0xD8,
0xD9, 0xDA, 0xDB, 0xDC, 0xDF, 0xE0, 0xE1, 0xE2,
0xE3, 0xE4, 0xE5, 0xE6, 0xE7, 0xE8, 0xE9, 0xEA,
0xEB, 0xEC, 0xED, 0xEE, 0xEF, 0xF1, 0xF2, 0xF3,
0xF4, 0xF5, 0xF6, 0xF7, 0xF8, 0xF9, 0xFA, 0xFB,
0xFC, 0xFF, 0xD0, 0xF0, 0xDD, 0xFD, 0xDE, 0xFE
};

static wchar_t to_unicode(LChar c) {
	if (c < 0x20 || (0x7f <= c && c <= 0x9f) || 0x100 <= c)
		return 0;
	if (c <= 0x7e)
		return c;
	return a0_ff_to_unicode[int(c - 0xa0)];
}

static LChar from_unicode(wchar_t c) {
	if (c < 0x20 || (0x7f <= c && c <= 0x9f) || 0x0160 <= c)
		return 0;
	if (c <= 0x7e)
		return c;
	wchar_t * pos = lower_bound(a0_ff_sorted_unicode, a0_ff_sorted_unicode + 0x60, c);
	if (*pos == c)
		return a0_ff_sorted_latin9[pos - a0_ff_sorted_unicode];
	return 0;
}

/// Convert a string to unicode.  
std::pair<wstring, LString::const_iterator>
EncLatin9::toUnicode(LString const & s) {
	wstring result;
	result.reserve(s.size());
	LString::const_iterator i = s.begin();
	for (; i != s.end(); ++i) {
		wchar_t glyph = to_unicode(*i);
		if (glyph == 0)
			break;
		result += to_unicode(*i);
	}
	return std::make_pair(result, i);
}

/// Convert a string from unicode
std::pair<LString, wstring::const_iterator>
EncLatin9::fromUnicode(wstring const & s) {
	LString result;
	result.reserve(s.size());
	wstring::const_iterator i = s.begin();
	for (; i != s.end(); ++i) {
		LChar glyph = from_unicode(*i);
		if (glyph == 0)
			break;
		result += char(*i);
	}
	return std::make_pair(result, i);
}

