// -*- C++ -*-
/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 * 	 
 *	    Copyright (C) 1998 The LyX Team
 *
 *======================================================*/

#ifndef ENCODING_CONVERTER_H
#define ENCODING_CONVERTER_H

#include <config.h>

#ifdef __GNUG__
#pragma interface
#endif

#include <utility>

#include "LString.h"

/** This is an abstract base class for encoding converters.

    Each encoding is implemented by deriving from this abstract base class.
    Have a look at EncLatin1.[Ch] to see how this can be done.  Remember to
    add the new encoding to the dispatcher functions in Encoding.C.

    Check the development/Design/encodings.lyx file for more information.

    (Asger)
*/
class EncodingConverter {
public:
	///
	EncodingConverter() { }

	/// 
	virtual ~EncodingConverter() { }

	/** Convert a string to unicode.  
	    This method returns the converted string, and an iterator
	    that points after the last character in the passed string
	    that was converted exact.
	*/
	virtual std::pair<wstring, LString::const_iterator>
	toUnicode(LString const &);

	/// Convert a string from unicode
	virtual std::pair<LString, wstring::const_iterator>
	fromUnicode(wstring const &);
};

#endif
