#include <config.h>

#ifdef __GNUG__
#pragma implementation
#endif

#include "error.h"
#include <stdio.h>
#include <stdlib.h>
#include <lstrings.h>

// 	$Id: error.C,v 1.3 1998/10/22 16:56:11 larsbj Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: error.C,v 1.3 1998/10/22 16:56:11 larsbj Exp $";
#endif /* lint */

struct error_item {
	Error::DEBUG_LEVELS level;
	const char * name;
	const char * desc;
};

static error_item errorTags[] = {
	{ Error::INFO,		"info",		"General information"},
	{ Error::INIT,		"init",		"Program initialisation"},
	{ Error::KEY,		"key",		"Keyboard events handling"},
	{ Error::TOOLBAR,	"toolbar",	"Toolbar handling"},
	{ Error::PARSER,	"parser",	"Lyxlex grammer parser"},
	{ Error::LYXRC,		"lyxrc",	"Configuration files reading"},
	{ Error::KBMAP,		"kbmap",	"Custom keyboard definition"},
	{ Error::LATEX,		"latex",	"LaTeX generation/execution"},
	{ Error::MATHED,	"mathed",	"Math editor"},
	{ Error::FONT,		"font",		"Font handling"},
	{ Error::TCLASS,	"tclass",	"Textclass files reading"},
	{ Error::LYXVC,		"lyxvc",	"Version control"},
	{ Error::LYXSERVER,	"lyxserver",	"External control interface"},
        { Error::ANY,		"any",          "All debugging messages"}};

static const int numErrorTags = sizeof(errorTags)/sizeof(error_item);

Error::Error(int level) // should loglevel also be an argument?
{
	debuglevel = level;
	loglevel = 0;
}


void Error::setDebugLevel(char const * lev)
{
	string oneLevel;
	int i;
	string levels = split(lev, oneLevel,',');
	
	while(!oneLevel.empty()) {
		// Search for an explicit name
		for (i = 0 ; i<numErrorTags ; ++i) 
			if (compare_no_case(oneLevel,
					    errorTags[i].name) == 0) {
				debuglevel |= errorTags[i].level;
				break;
			}
		// No name found; is it a number?
		if (i == numErrorTags) {
			int num = atoi(oneLevel.c_str());
			if (num)
				debuglevel |= num;
			else
				print("Bad debug feature `"
				      + oneLevel + '\'');
		}
		levels = split(levels, oneLevel,',');
	}

	// Show what features are traced
	for (i = 0 ; i < numErrorTags ; ++i)
		if (errorTags[i].level != Error::ANY
		    && debuglevel & errorTags[i].level)
			print(string("Debugging ") + errorTags[i].name
			      + " (" + errorTags[i].desc + ')');
}


void Error::showTags() {
	for (int i = 0 ; i<numErrorTags ; ++i)
		fprintf(stdout, "  %5d  %-10s%-35s\n", 
			errorTags[i].level,
			errorTags[i].name, 
			errorTags[i].desc);
}


void Error::debug(string const & msg, int level)
{
	if (debuglevel & level)
		print(msg);
	// should also print to the logfile
}

void Error::print(string const & msg)
{
	if (!msg.empty()) 
		fprintf(stderr, "%s\n", msg.c_str());
}


