// -*- C++ -*-

#ifndef FIGLIST_H
#define FIGLIST_H

#include "config.h"
#include <X11/Xlib.h>
#include "LString.h"

class InsetFig;

///
struct figdata {
	/// bitmap data
	Pixmap bitmap;
	/// reference count for this bitmap
	int ref;
        /// rotation angle
        float angle;
        /// width of raw figure w/o rotation
	int raw_wid;
        /// heigt of raw figure w/o rotation
	int raw_hgh;
	/// width and height on screen
	int wid, hgh;
	/// pointer to file name
	string fname;
	/// type; 0-none, 1-B/W, 2-Grayscale, 3-Color
	enum RenderOpt {
		NONE=1,
		MONO=2,
		GREY=4,
		COLOR=8
	};
	RenderOpt flags;
	/// reading request is pending on this figure
	bool reading;
	/// could not read it, because it's broken
	bool broken;
	/// when gs was killed, this says that image is ok
	bool gsdone;
	/// gs pid for reading
	int gspid;
};

class FigureList {
public:
///
	struct Figref {
		/// figure data (image)
		figdata *data;
		/// inset of this figure
		InsetFig *inset;
	};
	FigureList() {
		bitmap_waiting = false;
		figinsref = 0;
	}
	~FigureList() {};
	Figref **figures;  // all the figures
	figdata **bitmaps; // all the bitmaps
	void RegisterFigure(InsetFig *fi);
	void UnregisterFigure(InsetFig *fi);
	int FindFigIndex(Figref *tmpfig);
	void InitFigures();
	void DoneFigures();
	bool bitmap_waiting; // bitmaps are waiting finished
	void getbitmaps();
	figdata *getfigdata(int wid, int hgh, string const & fname, 
			    int psx, int psy, int pswid, int pshgh, 
			    int raw_wid, int raw_hgh, float angle, 
			    figdata::RenderOpt flags);
	void freefigdata(figdata *tmpdata);
	int figinsref;  // number of figures

};

extern FigureList figL;

#endif
