// -*- C++ -*-
/* skeleton.h
 * Skeleton for LyX popup Interface Classes.
 */

#ifndef SKELETON_H
#define SKELETON_H

/** All popups should have the same basic functionality.
 * It should not be necessary to add any other methods
 * to such classes.
 */
class LaTeXPreamble {
public:
	///
	LaTeXPreamble();
	///
	~LaTeXPreamble();
	
	/** Create the popup if necessary, update it and display it.
	 *  We don't need an explicit create() since we'll either
	 *  look at it all the time (bufferframe/main_popup) or only when
	 *  we need to -- as such we will usually only call show() and hide().
	 *  I've added free() mainly to get local dynamic variables deleted.
	 */
	void show();
	/** Hide the popup.
	 *  This method will later be connected to the
	 *  HideAllBufferRelatedPopups signal.
	 *  In some cases you might also want to free the popup (but keep
	 *  whatever local dynamic data you may have).
	 */
	void hide();
	/** Update the popup.
	 *  Will also be used as a target for the UpdateAllBufferRelatedPopups
	 *  (UABRP) signal.  Probably extend it slightly by passing a flag
	 *  indicating the kind of stuff that needs updating.  This would be
	 *  more effective when the UABRP signal since we could use a flag
	 *  equal to BUFFER_SWAP (indicating we've changed buffers).
	 *  Such a flag isn't necessary but may prove useful in reducing
	 *  unnecessary updates -- especially when called from other gui
	 *  components.  
	 *  NOTE: the flag should be defined in gui_common.h 
	 *        -- see the note there for other ideas.
	 */
	void update();
	/** Explicitly free the popup.
	 *  Probably only called at shutdown in which case it is likely to be
	 *  a target for a CloseGUI signal.  Other potential names include
	 *  evaporate(), delete(), kill() and vaporize().
	 */
private:
	///
	void free();
};

#endif
