// -*- C++ -*-
/* This file is part of
 * ======================================================
* 
*           LyX, The Document Processor
*	    Copyright (C) 1995 Matthias Ettrich
*
*           This file is Copyright (C) 1996-1998
*           Lars Gullik Bjnnes
*
*======================================================*/

#ifndef _TOOLBAR_H_
#define _TOOLBAR_H_

#ifdef __GNUG__
#pragma interface
#endif

#include FORMS_H_LOCATION
#include "toolbarbackend.h"
#include "lyxfunc.h"
#include "combox.h"

class LyXView;

/** The LyX toolbar class
  This class {\em is} the LyX toolbar, and is not likely to be enhanced
  further until we begin the move to Qt. We will probably have to make our
  own QToolBar, at least until Troll Tech makes theirs.
  */
class Toolbar {
public:
	///
	Toolbar(ToolbarBackend const &, LyXView *o, int x, int y);

	///
	Toolbar()
	{
		owner = 0;
		sxpos = 0;
		sypos = 0;
		bubble_timer = 0;
		combox = 0;
		reset();
	}

	///
	~Toolbar() {
		clean();
	}

	///
	Toolbar & operator=(const Toolbar & tb) {
		// These values might not be initialized
		// as they should. Look into it if you have the
		// time. (Lgb)
		owner = 0;
		sxpos = 0;
		sypos = 0;
		bubble_timer = 0;
		combox = 0;
		reset();
		toollist = tb.toollist;
		return *this;
	}
	
	///
	int get_toolbar_func(string const & func);
	
	///
	Combox *combox;

	/// (re)sets the toolbar
	void set();

	/// add a new button to the toolbar.
    	void add(ToolbarBackend::TOOLBARITEMS, int lyxfunc);
	/// invokes the n'th icon in the toolbar
	void push(int);
	/// activates the toolbar
        void activate();
	/// deactivates the toolbar
        void deactivate();

private:
	///
	struct toolbarItem
	{
		///
		ToolbarBackend::TOOLBARITEMS baritem;
		///
		int action;
		///
		FL_OBJECT *icon;
		/// 
		bool IsBitmap;
		///
		toolbarItem(){
			baritem = ToolbarBackend::TOOL_NONE;
			action = LFUN_NOACTION;
			icon = 0;
			IsBitmap = false;
		}
		///
		~toolbarItem(){
			if (icon){
				fl_delete_object(icon);
				fl_free_object(icon);
			}
		}

		toolbarItem & operator=(const toolbarItem & ti) {
			// do we have to check icon and IsBitmap too?
			baritem = ti.baritem;
		        action = ti.action;
			icon = 0; // locally we need to get the icon anew
			IsBitmap = false;

			return *this;
		}
	};

	/// typedef to simplify things
	typedef vector<toolbarItem> ToolbarList;
	/// The list containing all the buttons
	ToolbarList toollist;
	///
	LyXView *owner;
	///
	FL_OBJECT *bubble_timer;
	/// Starting position
	int sxpos, sypos;
	///
	int xpos;
	///
	int ypos;
	///
	int buttonwidth;
	///
	int height;
	///
	int standardspacing;
	///
	int sepspace;
	///
	bool cleaned;

	/// removes all toolbar buttons from the toolbar.
	void clean();
	///
	static void ToolbarCB(FL_OBJECT*, long);
	///
	static void BubbleTimerCB(FL_OBJECT *, long);
	///
	static int BubblePost(FL_OBJECT *ob, int event,
			      FL_Coord mx, FL_Coord my, int key, void *xev);

	/** more...
	 */
	void reset(){
		// G++ vector does not have clear defined
		//toollist.clear();
		toollist.erase(toollist.begin(), toollist.end());
		cleaned = false;
		
		lightReset();
	}

	/** more...
	 */
	void lightReset(){
		standardspacing = 2; // the usual space between items
		sepspace = 6; // extra space
		xpos = sxpos - standardspacing;
		ypos = sypos;
		buttonwidth = 30; // the standard button width
		height = 30; // the height of all items in the toolbar
	}
};
#endif
