#include <config.h>
#include FORMS_H_LOCATION

#include "Popups.h"
#include "latexpreamble.h"
#include "filebrowser.h"
#include "latexlog.h"
#include "form_tables.h"
#include "form_paragraphs.h"
#include "form_document.h"
#include "form_copyright.h"
#include "form_credits.h"
#include "form_include.h"
#include "form_index.h"
#include "form_bibkey.h"
#include "form_citation.h"
#include "form_bibtex.h"
#include "form_info.h"


#ifdef __GNUG__
#pragma implementation
#endif


Popups::Popups(Communicator *c)
	: comm(c)
{
	latexpreamble = new LaTeXPreamble(c, this);
	latexlog = new LaTeXLog(c, this);
	formtable = new FormTable(c, this);
        formparagraph = new FormParagraph(c, this);
        formdocument = new FormDocument(c, this);
	formcopyright = new FormCopyright(c, this);
	formcredits = new FormCredits(c, this);
	forminclude = new FormInclude(c, this);
	formindex = new FormIndex(c, this);
	formbibkey = new FormBibkey(c, this);
	formcitation = new FormCitation(c, this);
	formbibtex = new FormBibtex(c, this);
	forminfo = new FormInfo(c, this);
	
	// reduce the number of connections needed in
	// popups by a simple connection here.
	connect_to_signal(this->hideAll,
			  this->hideBufferDependent);
}


Popups::~Popups() 
{
	delete latexpreamble;
	delete latexlog;
	delete formtable;
        delete formparagraph;
	delete formdocument;
	delete formcopyright;
	delete formcredits;
	delete forminclude;
	delete formindex;
	delete formbibkey;
	delete formcitation;
	delete formbibtex;
	delete forminfo;
	
	comm = 0;
}


void Popups::SetDocumentClassChoice(vector<string> const &choices)
{
	formdocument->SetDocumentClassChoice(choices);
}

/*****************************************************************************

Q.  WHY does Popups::Popups pass `this' to popup constructors?

A.  To avoid a segfault.
    The popup constructors need to connect to their
    respective showSomePopup signal(*) but in order to do
    that they need to get the address of the Popups instance
    from LyXView::getPopups().  However, since the Popups
    instance is still being constructed at that time
    LyXView::getPopups() will *not* return the correct
    address because it hasn't finished being constructed.
    A Catch-22 situation (or is that the chicken and the egg...).
    So to get around the problem we pass the address of
    the newly created Popups instance using `this' which
    should hopefully work with all compilers.  If it
    doesn't I have an alternate implementation listed
    below.

    An alternative method of initialisation is to define

      void Popups::init()
      {
          latexpreamble = new LaTeXPreamble(comm);
      }

    which must be called before using anything else ie.

      Popups *p = new Popups(comm);
      p->init();

    This approach works but I decided I didn't want to do it that way.
    There are a couple other possibilities like wrapping Popups in another
    class whos constructor makes the two calls above (construction and init())
    If you have a better idea be sure to make it known.  Cheers, ARRae.

(*) -- I'm using signals exclusively to guarantee that the gui code
       remains hidden from the rest of the system.  In fact the only 
       header related to dialogs/popups that anything in the non-gui-specific
       code gets to see is Popups.h!
       (even Popups.h doesn't know what a LaTeXPreamble class looks like)
       No other gui popup headers are seen outside of the gui-specific
       directories!  This ensures that the gui is completely separate from
       the rest of LyX.  All this through the use of a few simple signals.
       BUT, the price is that during construction we need to connect the
       implementations show() method to the showSomePopup signal and this
       requires that we have an instance of Popups and the problem mentioned
       above.

       Almost all other popups should be able to operate using the same style
       of signalling used for LaTeXPreamble.  Exceptions should be handled
       by adding a specific show or update signal.  For example, spellchecker
       needs to set the next suspect word and its options/replacements so we
       need a:
                 Signal0 updateSpellChecker;

       Since we would have to have a
                 Signal0 showSpellChecker;

       in order to just see the spellchecker and let the user push the [Start]
       button then the updateSpellChecker signal will make the SpellChecker
       popup get the new word and replacements list from LyX.  If you really,
       really wanted to you could define a signal that would pass the new
       word and replacements:
                 Signal2<string, vector<string>> updateSpellChecker;

       (or something similar) but, why bother when the spellchecker can get
       it anyway with a function call or two.  Besides if someone extends
       what a popup does then they also have to change code in the rest of 
       LyX to pass more parameters or get the extra info via a function 
       call anyway.  Thus reducing the independence of the two code bases.

       We don't need a separate update signal for each popup because most of 
       them will be changed only when the buffer is changed (either by closing
       the current open buffer or switching to another buffer in the current
       LyXView -- different BufferView same LyXView or same BufferView same
       LyXView).

       So we minimise signals but maximise independence and programming 
       simplicity, understandability and maintainability.  It's also
       extremely easy to add support for Qt or gtk-- because they use
       signals already. Guis that use callbacks, like xforms, must have their
       code wrapped up like that in the latexpreamble.[Ch] which is awkward
       but will at least allow multiple instances of the same popup.

       Signals will also be a great help in controlling the splashscreen --
       once signalled to hide it can disconnect from the signal and remove
       itself from memory.

******************************************************************************/
