// -*- C++ -*-

#include "browserbase.h"
#include "gettext.h"
#include "path.h"


#ifdef __GNUG__
#pragma implementation
#endif

BrowserBase::BrowserBase(const string &f, const string &t)
		: fname(f), title(t)
{
	popup = 0;
}


BrowserBase::~BrowserBase() { 
	hide(); 
	free(); 
}


void BrowserBase::hide()
{
	if (popup
	    && popup->form
	    && popup->form->visible) {
		fl_hide_form(popup->form);
	}
	Closing(); // emit signal
}


int BrowserBase::BrowserWMClose(FL_FORM * form, void *)
{
	// Ensure that the signals (u and h) are disconnected even if the
	// window manager is used to close the popup.
	BrowserBase *log = (BrowserBase*)form->u_vdata;
	log->hide();
	return FL_CANCEL;
}


void BrowserBase::BrowserClose(FL_OBJECT *ob, long)
{
	BrowserBase *log = (BrowserBase*)ob->form->u_vdata;
	log->hide();	
}


void BrowserBase::BrowserUpdate(FL_OBJECT *ob, long)
{
	BrowserBase *log = (BrowserBase*)ob->form->u_vdata;
	log->update();
}


void BrowserBase::free()
{
	// we don't need to delete u and h here because
	// hide() does that after disconnecting.
	if (popup) {
		if (popup->form
		    && popup->form->visible) {
			hide();
		}
		fl_free_form(popup->form);
		delete popup;
		popup = 0;
	}
}


void BrowserBase::update(const string &f)
{
	if (!f.empty()) fname = f;
	if (fname.empty()) return;
	
	if (popup) {
		Path p(OnlyPath(fname));
		if (!fl_load_browser(popup->browser, fname.c_str()))
			fl_add_browser_line(popup->browser,
					    _("FILE IS EMPTY!"));
	}
}


void BrowserBase::show(const string &f, const string &t)
{
	if (!f.empty()) fname = f;
	if (!t.empty()) title = t;
	
	if (!popup) {
		build();
		fl_set_form_atclose(popup->form,
				    BrowserBase::BrowserWMClose, 0);
	}

	update(); // make sure it is up-to-date

	if (popup->form->visible) {
		fl_raise_form(popup->form);
	} else {
		fl_show_form(popup->form,
			     FL_PLACE_MOUSE | FL_FREE_SIZE,
			     FL_FULLBORDER, title.c_str());
	}
}


void BrowserBase::build()
{
	FL_OBJECT *obj;
	FD_FileBrowser *fdui = popup = new FD_FileBrowser;;
	
	fdui->form = fl_bgn_form(FL_NO_BOX, 470, 380);
	fdui->form->u_vdata = this;
	
	obj = fl_add_box(FL_UP_BOX,0,0,470,380,"");
	fdui->browser = obj = fl_add_browser(FL_NORMAL_BROWSER,10,10,450,320,"");
	obj = fl_add_button(FL_RETURN_BUTTON,270,340,90,30,_("Close"));
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_callback(obj,BrowserClose,0);
	obj = fl_add_button(FL_NORMAL_BUTTON,370,340,90,30,idex(_("Update|#Uu")));fl_set_button_shortcut(obj,scex(_("Update|#Uu")),1);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_callback(obj,BrowserUpdate,0);
	fl_end_form();
	
	fdui->form->fdui = fdui;
}
