/* form_bibkey.C
 * FormBibkey Interface Class Implementation
 */

#include <config.h>
#include "lyx_gui_misc.h"
#include "gettext.h"
#include FORMS_H_LOCATION

#include "form_bibkey.h"
#include "Popups.h"
#include "Communicator.h"
#include "insetbibkey.h"
#include "BufferView.h"
#include "lyxtext.h"

// 	$Id: form_bibkey.C,v 1.1 1999/01/11 06:21:05 arrae Exp $

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: form_bibkey.C,v 1.1 1999/01/11 06:21:05 arrae Exp $";
#endif /* lint */


#ifdef __GNUG__
#pragma implementation
#endif


FormBibkey::FormBibkey(Communicator *c, Popups * p)
	: popup(0), comm(c), d(0), u(0), h(0), status(POPUP_UNMODIFIED)
{
	// let the popup be shown
	// This is a permanent connection so we won't bother
	// storing a copy because we won't be disconnecting.
	connect_to_method(p->showFormBibkey,
			  this,
			  &FormBibkey::show);
	// NOTE:
	// parameter p is used instead of:
	//      comm->getPopups()
	// because we are being contructed from within the
	// Popups constructor and a segfault would occur
	// otherwise.  The segfault occurs because the Popups
	// constructor hasn't returned at this point and so
	// comm->getPopups() will not return the real popups
	// instance.  If this implementation fails see Popups.C
	// ARRae.
}


FormBibkey::~FormBibkey()
{
	free();
}


void FormBibkey::build()
{
	FL_OBJECT *obj;
	FD_form_bibkey *fdui = popup = new FD_form_bibkey;

	fdui->form_bibkey = fl_bgn_form(FL_NO_BOX, 220, 130);
	fdui->form_bibkey->u_vdata = this;

	obj = fl_add_box(FL_UP_BOX,0,0,220,130,"");
	fdui->key = obj = fl_add_input(FL_NORMAL_INPUT,80,10,130,30,idex(_("Key:|#K")));
	fl_set_button_shortcut(obj,scex(_("Key:|#K")),1);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_callback(obj,FormBibkey::InputCB,0);
	fdui->button_ok = obj = fl_add_button(FL_RETURN_BUTTON,20,90,90,30,_("OK"));
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_callback(obj,FormBibkey::BibkeyCB,3);
	fdui->button_cancel = obj = fl_add_button(FL_NORMAL_BUTTON,120,90,90,30,idex(_("Cancel|^[")));
	fl_set_button_shortcut(obj,scex(_("Cancel|^[")),1);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_callback(obj,FormBibkey::BibkeyCB,2);
	fdui->label = obj = fl_add_input(FL_NORMAL_INPUT,80,50,130,30,idex(_("Label:|#L")));
	fl_set_button_shortcut(obj,scex(_("Label:|#L")),1);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_callback(obj,FormBibkey::InputCB,0);
	fl_end_form();

	fdui->form_bibkey->fdui = fdui;
}


void FormBibkey::show(InsetBibKey * inset)
{
	if (!inset) {
		return;
	}

	if (!popup) {
		build();
		fl_set_input_return(popup->key,
				    FL_RETURN_CHANGED);
		fl_set_input_return(popup->label,
				    FL_RETURN_CHANGED);
		fl_set_form_atclose(popup->form_bibkey,
				    FormBibkey::BibkeyWMHideCB, 0);
	}

	if (comm->isReadOnly()) {
		// deactivating an input doesn't seem to work
		// at least not up to 0.88, so we remember instead.
		status = POPUP_READONLY;
		fl_deactivate_object(popup->key);
		fl_deactivate_object(popup->label);
	} else {
		status = POPUP_UNMODIFIED;
		fl_activate_object(popup->key);
		fl_activate_object(popup->label);
	}
	fl_deactivate_object(popup->button_ok);
	fl_set_object_lcol(popup->button_ok, FL_INACTIVE);

        popup->vdata = inset;
	// if the popup is visible and the inset its attached to is deleted
	// the inset will activate this signal warning us of the situation.
	// Since the popup doesn't make sense without an inset we free() it.
	d = connect_to_method(inset->gone,
			      this,
			      &FormBibkey::free);

        fl_set_input(popup->key, inset->getContents().c_str());
        fl_set_input(popup->label, inset->getOptions().c_str());

	if (popup->form_bibkey->visible) {
		fl_raise_form(popup->form_bibkey);
	} else {
		fl_show_form(popup->form_bibkey,
			     FL_PLACE_MOUSE,
			     FL_FULLBORDER,
			     _("Bibliography item"));
		h = connect_to_method(
				comm->getPopups().hideBufferDependent,
				this,
				&FormBibkey::hide);
		// if we change buffers the popup is no longer valid
		// so we'll just hide it when we get an update signal
		u = connect_to_method(
				comm->getPopups().updateBufferDependent,
				this,
				&FormBibkey::hide);
	}
}


void FormBibkey::hide()
{
	if (popup
	    && popup->form_bibkey
	    && popup->form_bibkey->visible) {
		fl_hide_form(popup->form_bibkey);
		d.disconnect();
		u.disconnect();
		h.disconnect();
		popup->vdata = 0;
	}
	free();
}


void FormBibkey::bibkey(long arg)
{
#ifdef WITH_WARNINGS
//#warning still cleaning up to do here
#endif
	InsetBibKey *inset = (InsetBibKey *)popup->vdata;

	switch (arg) {
	case 3: // OK
        {
		if(!comm->isReadOnly()) {
			inset->setContents(fl_get_input(popup->key));
			inset->setOptions(fl_get_input(popup->label));
			hide();

			// Below is the original hack from insetbib.C
			// Why not just use a method like form_index does
			// where the inputs are tested against the contents
			// and if different the assignment is done and
			// comm->updateInset(inset, true) is called?
			// or am I missing something?? ARRae 990103
			// Oh, and I haven't tried this idea yet...

			// Does look like a hack? It is! (but will change at 0.13)
			comm->getCurrentView()->text->RedoParagraph();
			comm->getCurrentView()->update(BufferView::ResetSelection
						       |BufferView::FitCursor
						       |BufferView::BufferChange);
			break;
		} // fall through to Cancel on RO-mode
        }

	case 2: // Cancel
		hide();
		break;
        }
}


void FormBibkey::free()
{
	// we don't need to delete u and h here because
	// hide() does that after disconnecting.
	if (popup) {
		if (popup->form_bibkey
		    && popup->form_bibkey->visible) {
			hide();
		}
		fl_free_form(popup->form_bibkey);
		delete popup;
		popup = 0;
	}
}


int FormBibkey::BibkeyWMHideCB(FL_FORM * form, void *)
{
	// Ensure that the signals (u and h) are disconnected even if the
	// window manager is used to close the popup.
	FormBibkey *pre = (FormBibkey*)form->u_vdata;
	pre->hide();
	return FL_CANCEL;
}


void FormBibkey::BibkeyCB(FL_OBJECT *ob, long data)
{
	FormBibkey *pre = (FormBibkey*)ob->form->u_vdata;
	pre->bibkey(data);
}


void FormBibkey::InputCB(FL_OBJECT *ob, long)
{
	FormBibkey *pre = (FormBibkey*)ob->form->u_vdata;
	if (pre->status == POPUP_UNMODIFIED) {
		pre->status = POPUP_MODIFIED;
		fl_activate_object(pre->popup->button_ok);
		fl_set_object_lcol(pre->popup->button_ok, FL_BLACK);
	}
}
