// -*- C++ -*-
/* form_bibkey.h
 * FormBibkey Interface Class
 */

#ifndef FORMBIBKEY_H
#define FORMBIBKEY_H

#include "lsignal.h"
#include "gui_common.h"

class Communicator;
class Popups;
class InsetBibKey;


/** This class provides an XForms implementation of the FormBibkey Popup.
    The bibkey popup allows user to insert indices
 */
class FormBibkey : public virtual SignalBase {
public:
	/**@name Constructors and Destructors */
	//@{
	/// #FormBibkey x(Communicator ..., Popups ...);#
	FormBibkey(Communicator *, Popups *);
	///
	~FormBibkey();
	//@}
private:
	FormBibkey() {}
	FormBibkey(FormBibkey&) {}

	/**@name Real per-instance Callback Methods */
	//@{
	static int BibkeyWMHideCB(FL_FORM *, void *);
	static void BibkeyCB(FL_OBJECT *, long);
	static void InputCB(FL_OBJECT *, long);
	//@}
	
	/**@name Slot Methods */
	//@{
	/// Create the popup if necessary, update it and display it.
	void show(InsetBibKey *);
	/// Hide the popup.
	void hide();
	/// Real popup handler
	void bibkey(long);
	//@}

	
	/// Build the popup
	void build();
	/// Explicitly free the popup.
	void free();

	typedef struct {
		FL_FORM *form_bibkey;
		void *vdata;
		char *cdata;
		long  ldata;
		FL_OBJECT *key;
		FL_OBJECT *button_ok;
		FL_OBJECT *button_cancel;
		FL_OBJECT *label;
	} FD_form_bibkey;

	/**@name Private Data */
	//@{
	/// Real GUI implementation.
	FD_form_bibkey *popup;
	/// Which LyXView do we belong to?
	Communicator *comm;
	/// Inset-is-being-deleted connection.
	Connection d;
	/// Update connection
	Connection u;
	/// Hide connection.
	Connection h;
	/// has form contents changed? Used to control OK (eventually)
	EnumPopupStatus status;
	//@}
};

#endif
