/* form_bibtex.C
 * FormBibtex Interface Class Implementation
 */

#include <config.h>
#include "lyx_gui_misc.h"
#include "gettext.h"
#include FORMS_H_LOCATION

#include "form_bibtex.h"
#include "Popups.h"
#include "Communicator.h"
#include "insetbibtex.h"
#include "BufferView.h"
#include "lyxtext.h"

// 	$Id: form_bibtex.C,v 1.1 1999/01/11 06:21:05 arrae Exp $

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: form_bibtex.C,v 1.1 1999/01/11 06:21:05 arrae Exp $";
#endif /* lint */


#ifdef __GNUG__
#pragma implementation
#endif


FormBibtex::FormBibtex(Communicator *c, Popups * p)
	: popup(0), comm(c), d(0), u(0), h(0), status(POPUP_UNMODIFIED)
{
	// let the popup be shown
	// This is a permanent connection so we won't bother
	// storing a copy because we won't be disconnecting.
	connect_to_method(p->showFormBibtex,
			  this,
			  &FormBibtex::show);
	// NOTE:
	// parameter p is used instead of:
	//      comm->getPopups()
	// because we are being contructed from within the
	// Popups constructor and a segfault would occur
	// otherwise.  The segfault occurs because the Popups
	// constructor hasn't returned at this point and so
	// comm->getPopups() will not return the real popups
	// instance.  If this implementation fails see Popups.C
	// ARRae.
}


FormBibtex::~FormBibtex()
{
	free();
}


void FormBibtex::build()
{
	FL_OBJECT *obj;
	FD_form_bibtex *fdui = popup = new FD_form_bibtex;

	fdui->form_bibtex = fl_bgn_form(FL_NO_BOX, 220, 130);
	fdui->form_bibtex->u_vdata = this;
  
	obj = fl_add_box(FL_UP_BOX,0,0,220,130,"");
	fdui->input_database = obj = fl_add_input(FL_NORMAL_INPUT,80,10,130,30,idex(_("Database:|#D")));
	fl_set_button_shortcut(obj,scex(_("Database:|#D")),1);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_callback(obj,FormBibtex::InputCB,0);
	fdui->button_ok = obj = fl_add_button(FL_RETURN_BUTTON,20,90,90,30,_("OK"));
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_callback(obj,FormBibtex::BibtexCB,3);
	fdui->button_cancel = obj = fl_add_button(FL_NORMAL_BUTTON,120,90,90,30,idex(_("Cancel|^[")));
	fl_set_button_shortcut(obj,scex(_("Cancel|^[")),1);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_callback(obj,FormBibtex::BibtexCB,2);
	fdui->input_style = obj = fl_add_input(FL_NORMAL_INPUT,80,50,130,30,idex(_("Style:|#S")));
	fl_set_button_shortcut(obj,scex(_("Style:|#S")),1);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_callback(obj,FormBibtex::InputCB,0);
	fl_end_form();

	fdui->form_bibtex->fdui = fdui;
}


void FormBibtex::show(InsetBibtex * inset)
{
	if (!inset) {
		return;
	}

	if (!popup) {
		build();
		fl_set_input_return(popup->input_database,
				    FL_RETURN_CHANGED);
		fl_set_input_return(popup->input_style,
				    FL_RETURN_CHANGED);
		fl_set_form_atclose(popup->form_bibtex,
				    FormBibtex::BibtexWMHideCB, 0);
	}

	if (comm->isReadOnly()) {
		// deactivating an input doesn't seem to work
		// at least not up to 0.88, so we remember instead.
		status = POPUP_READONLY;
		fl_deactivate_object(popup->input_database);
		fl_deactivate_object(popup->input_style);
	} else {
		status = POPUP_UNMODIFIED;
		fl_activate_object(popup->input_database);
		fl_activate_object(popup->input_style);
	}
	fl_deactivate_object(popup->button_ok);
	fl_set_object_lcol(popup->button_ok, FL_INACTIVE);

        popup->vdata = inset;
	// if the popup is visible and the inset its attached to is deleted
	// the inset will activate this signal warning us of the situation.
	// Since the popup doesn't make sense without an inset we free() it.
	d = connect_to_method(inset->gone,
			      this,
			      &FormBibtex::free);

        fl_set_input(popup->input_database, inset->getContents().c_str());
        fl_set_input(popup->input_style, inset->getOptions().c_str());

	if (popup->form_bibtex->visible) {
		fl_raise_form(popup->form_bibtex);
	} else {
		fl_show_form(popup->form_bibtex,
			     FL_PLACE_MOUSE,
			     FL_FULLBORDER,
			     _("BibTeX"));
		h = connect_to_method(
				comm->getPopups().hideBufferDependent,
				this,
				&FormBibtex::hide);
		// if we change buffers the popup is no longer valid
		// so we'll just hide it when we get an update signal
		u = connect_to_method(
				comm->getPopups().updateBufferDependent,
				this,
				&FormBibtex::hide);
	}
}


void FormBibtex::hide()
{
	if (popup
	    && popup->form_bibtex
	    && popup->form_bibtex->visible) {
		fl_hide_form(popup->form_bibtex);
		d.disconnect();
		u.disconnect();
		h.disconnect();
		popup->vdata = 0;
	}
	free();
}


void FormBibtex::bibtex(long arg)
{
#ifdef WITH_WARNINGS
//#warning still cleaning up to do here
#endif
	InsetBibtex *inset = (InsetBibtex *)popup->vdata;

	switch (arg) {
	case 3: // OK
        {
		if(!comm->isReadOnly()) {
			inset->setContents(fl_get_input(popup->input_database));
			inset->setOptions(fl_get_input(popup->input_style));
			hide();

			// Below is the original hack from insetbib.C
			// Why not just use a method like form_index does
			// where the inputs are tested against the contents
			// and if different the assignment is done and
			// comm->updateInset(inset, true) is called?
			// or am I missing something?? ARRae 990103
			// Oh, and I haven't tried this idea yet...

			// Does look like a hack? It is! (but will change at 0.13)
			comm->getCurrentView()->text->RedoParagraph();
			comm->getCurrentView()->update(BufferView::ResetSelection
						       |BufferView::FitCursor
						       |BufferView::BufferChange);
			break;
		} // fall through to Cancel on RO-mode
        }

	case 2: // Cancel
		hide();
		break;
        }
}


void FormBibtex::free()
{
	// we don't need to delete u and h here because
	// hide() does that after disconnecting.
	if (popup) {
		if (popup->form_bibtex
		    && popup->form_bibtex->visible) {
			hide();
		}
		fl_free_form(popup->form_bibtex);
		delete popup;
		popup = 0;
	}
}


int FormBibtex::BibtexWMHideCB(FL_FORM * form, void *)
{
	// Ensure that the signals (u and h) are disconnected even if the
	// window manager is used to close the popup.
	FormBibtex *pre = (FormBibtex*)form->u_vdata;
	pre->hide();
	return FL_CANCEL;
}


void FormBibtex::BibtexCB(FL_OBJECT *ob, long data)
{
	FormBibtex *pre = (FormBibtex*)ob->form->u_vdata;
	pre->bibtex(data);
}


void FormBibtex::InputCB(FL_OBJECT *ob, long)
{
	FormBibtex *pre = (FormBibtex*)ob->form->u_vdata;
	if (pre->status == POPUP_UNMODIFIED) {
		pre->status = POPUP_MODIFIED;
		fl_activate_object(pre->popup->button_ok);
		fl_set_object_lcol(pre->popup->button_ok, FL_BLACK);
	}
}
