// -*- C++ -*-
/* form_bibtex.h
 * FormBibtex Interface Class
 */

#ifndef FORMBIBTEX_H
#define FORMBIBTEX_H

#include "lsignal.h"
#include "gui_common.h"

class Communicator;
class Popups;
class InsetBibtex;


/** This class provides an XForms implementation of the FormBibtex Popup.
    The bibtex popup allows user to insert indices
 */
class FormBibtex : public virtual SignalBase {
public:
	/**@name Constructors and Destructors */
	//@{
	/// #FormBibtex x(Communicator ..., Popups ...);#
	FormBibtex(Communicator *, Popups *);
	///
	~FormBibtex();
	//@}
private:
	FormBibtex() {}
	FormBibtex(FormBibtex&) {}

	/**@name Real per-instance Callback Methods */
	//@{
	static int BibtexWMHideCB(FL_FORM *, void *);
	static void BibtexCB(FL_OBJECT *, long);
	static void InputCB(FL_OBJECT *, long);
	//@}
	
	/**@name Slot Methods */
	//@{
	/// Create the popup if necessary, update it and display it.
	void show(InsetBibtex *);
	/// Hide the popup.
	void hide();
	/// Real popup handler
	void bibtex(long);
	//@}

	
	/// Build the popup
	void build();
	/// Explicitly free the popup.
	void free();

	typedef struct {
		FL_FORM *form_bibtex;
		void *vdata;
		char *cdata;
		long  ldata;
		FL_OBJECT *input_database;
		FL_OBJECT *button_ok;
		FL_OBJECT *button_cancel;
		FL_OBJECT *input_style;
	} FD_form_bibtex;

	/**@name Private Data */
	//@{
	/// Real GUI implementation.
	FD_form_bibtex *popup;
	/// Which LyXView do we belong to?
	Communicator *comm;
	/// Inset-is-being-deleted connection.
	Connection d;
	/// Update connection
	Connection u;
	/// Hide connection.
	Connection h;
	/// has form contents changed? Used to control OK (eventually)
	EnumPopupStatus status;
	//@}
};

#endif
