/* form_citation.C
 * FormCitation Interface Class Implementation
 */

#include <config.h>
#include "lyx_gui_misc.h"
#include "gettext.h"
#include FORMS_H_LOCATION

#include "form_citation.h"
#include "Popups.h"
#include "Communicator.h"
#include "insetcitation.h"
#include "string.h"
#include "combox.h"

// 	$Id: form_citation.C,v 1.2 1999/04/09 18:16:57 larsbj Exp $

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: form_citation.C,v 1.2 1999/04/09 18:16:57 larsbj Exp $";
#endif /* lint */


#ifdef __GNUG__
#pragma implementation
#endif


FormCitation::FormCitation(Communicator *c, Popups * p)
	: popup(0), comm(c), d(0), u(0), h(0), status(POPUP_UNMODIFIED)
{
	// let the popup be shown
	// This is a permanent connection so we won't bother
	// storing a copy because we won't be disconnecting.
	connect_to_method(p->showFormCitation,
			  this,
			  &FormCitation::show);
	// NOTE:
	// parameter p is used instead of:
	//      comm->getPopups()
	// because we are being contructed from within the
	// Popups constructor and a segfault would occur
	// otherwise.  The segfault occurs because the Popups
	// constructor hasn't returned at this point and so
	// comm->getPopups() will not return the real popups
	// instance.  If this implementation fails see Popups.C
	// ARRae.
}


FormCitation::~FormCitation()
{
	free();
}


void FormCitation::build()
{
	FL_OBJECT *obj;
	FD_form_citation *fdui = popup = new FD_form_citation;

	fdui->form_citation = fl_bgn_form(FL_NO_BOX, 220, 130);
	fdui->form_citation->u_vdata = this;

	obj = fl_add_box(FL_UP_BOX,0,0,220,130,"");
// The input key was a place holder...
	fdui->key = obj = fl_add_input(FL_NORMAL_INPUT,80,10,130,30,idex(_("Key:|#K")));
//	fl_set_button_shortcut(obj,scex(_("Key:|#K")),1);
	fl_set_object_boxtype(obj, FL_NO_BOX);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
//	fl_set_object_callback(obj,FormCitation::InputCB,0);
// ... for the combox...
	fdui->bibcombox = new Combox(Combox::FL_COMBOX_INPUT);
	fdui->bibcombox->add(80,10,130,30, 120);
	fdui->bibcombox->setcallback(FormCitation::ComboxCB,this);
	fdui->bibcombox->setinputcallback(FormCitation::ComboxInputCB,this);
	fdui->bibcombox->shortcut(scex(_("Key:|#k")),1);
// ... this really is ugly isn't it? ARRae. 990103

	fdui->button_ok = obj = fl_add_button(FL_RETURN_BUTTON,20,90,90,30,_("OK"));
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_callback(obj,FormCitation::CitationCB,1);
	fdui->button_cancel = obj = fl_add_button(FL_NORMAL_BUTTON,120,90,90,30,idex(_("Cancel|^[")));
	fl_set_button_shortcut(obj,scex(_("Cancel|^[")),1);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_callback(obj,FormCitation::CitationCB,0);
	fdui->label = obj = fl_add_input(FL_NORMAL_INPUT,80,50,130,30,idex(_("Remark:|#R")));
	fl_set_button_shortcut(obj,scex(_("Remark:|#R")),1);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_callback(obj,FormCitation::InputCB,0);

	fl_end_form();

	fdui->form_citation->fdui = fdui;
}


void FormCitation::show(InsetCitation * inset)
{
	if (!inset) {
		return;
	}

	if (!popup) {
		build();
		// this needs to be this often in order to get
		// the Ok button de/activated correctly.  This of
		// course meant that combox filled with every keystroke
		// entered by the user so combox was "fixed" to work 
		// better and not repeat entries then that was "fixed"
		// further by not adding to combo unless the default
		// input return type was used.  This is currently
		// the only place in the code to use an input combox.
		popup->bibcombox->input_return(FL_RETURN_CHANGED);
		fl_set_input_return(popup->label,
				    FL_RETURN_CHANGED);
		fl_set_form_atclose(popup->form_citation,
				    FormCitation::CitationWMHideCB, 0);
	}

	if (comm->isReadOnly()) {
		// deactivating an input doesn't seem to work
		// at least not up to 0.88, so we remember instead.
		status = POPUP_READONLY;
		popup->bibcombox->deactivate();
		fl_deactivate_object(popup->label);
	} else {
		status = POPUP_UNMODIFIED;
		popup->bibcombox->activate();
		fl_activate_object(popup->label);
	}
	fl_deactivate_object(popup->button_ok);
	fl_set_object_lcol(popup->button_ok, FL_INACTIVE);

        popup->vdata = inset;
	// if the popup is visible and the inset its attached to is deleted
	// the inset will activate this signal warning us of the situation.
	// Since the popup doesn't make sense without an inset we free() it.
	d = connect_to_method(inset->gone,
			      this,
			      &FormCitation::free);

	comm->bibitemUpdate(popup->bibcombox);
	if (!popup->bibcombox->select_text(inset->getContents().c_str()))
		popup->bibcombox->addline(inset->getContents().c_str());

        fl_set_input(popup->label, inset->getOptions().c_str());

	if (popup->form_citation->visible) {
		fl_raise_form(popup->form_citation);
	} else {
		fl_show_form(popup->form_citation,
			     FL_PLACE_MOUSE,
			     FL_FULLBORDER,
			     _("Citation"));
		h = connect_to_method(
				comm->getPopups().hideBufferDependent,
				this,
				&FormCitation::hide);
		// if we change buffers the popup is no longer valid
		// so we'll just hide it when we get an update signal
		u = connect_to_method(
				comm->getPopups().updateBufferDependent,
				this,
				&FormCitation::hide);
	}
}


void FormCitation::hide()
{
	if (popup
	    && popup->form_citation
	    && popup->form_citation->visible) {
		fl_hide_form(popup->form_citation);
		d.disconnect();
		u.disconnect();
		h.disconnect();
		popup->vdata = 0;
	}
	free();
}


void FormCitation::citation(long arg)
{
	InsetCitation *inset = (InsetCitation *)popup->vdata;

	switch (arg) {
	case 1: // OK, citation
		if(!comm->isReadOnly()) {
			inset->setContents(popup->bibcombox->getline());
			inset->setOptions(fl_get_input(popup->label));
			hide();
			// shouldn't mark the buffer dirty unless something
			// was actually altered
			comm->updateInset(inset, true);
			break;
		}
		// fall through to Cancel on RO-mode

	case 0: hide();
                break;
	}
}


void FormCitation::free()
{
	// we don't need to delete u and h here because
	// hide() does that after disconnecting.
	if (popup) {
		if (popup->form_citation
		    && popup->form_citation->visible) {
			hide();
		}
		fl_free_form(popup->form_citation);
		delete popup;
		popup = 0;
	}
}


int FormCitation::CitationWMHideCB(FL_FORM * form, void *)
{
	// Ensure that the signals (u and h) are disconnected even if the
	// window manager is used to close the popup.
	FormCitation *pre = (FormCitation*)form->u_vdata;
	pre->hide();
	return FL_CANCEL;
}


void FormCitation::CitationCB(FL_OBJECT *ob, long data)
{
	FormCitation *pre = (FormCitation*)ob->form->u_vdata;
	pre->citation(data);
}


void FormCitation::InputCB(FL_OBJECT *ob, long)
{
	FormCitation *pre = (FormCitation*)ob->form->u_vdata;
	if (pre->status == POPUP_UNMODIFIED) {
		pre->status = POPUP_MODIFIED;
		fl_activate_object(pre->popup->button_ok);
		fl_set_object_lcol(pre->popup->button_ok, FL_BLACK);
	}
}


void FormCitation::ComboxInputCB(void *form)
{
	FormCitation *pre = (FormCitation*)form;
	if (pre->status == POPUP_UNMODIFIED) {
		pre->status = POPUP_MODIFIED;
		fl_activate_object(pre->popup->button_ok);
		fl_set_object_lcol(pre->popup->button_ok, FL_BLACK);
	}
}


void FormCitation::ComboxCB(int, void *form)
{
	FormCitation *pre = (FormCitation*)form;
	if (pre->status == POPUP_UNMODIFIED) {
		pre->status = POPUP_MODIFIED;
		fl_activate_object(pre->popup->button_ok);
		fl_set_object_lcol(pre->popup->button_ok, FL_BLACK);
	}
}
