// -*- C++ -*-
/* form_citation.h
 * FormCitation Interface Class
 */

#ifndef FORMCITATION_H
#define FORMCITATION_H

#include "lsignal.h"
#include "gui_common.h"

class Communicator;
class Popups;
class InsetCitation;
class Combox;


/** This class provides an XForms implementation of the FormCitation Popup.
    The citation popup allows user to insert indices
 */
class FormCitation : public virtual SignalBase {
public:
	/**@name Constructors and Destructors */
	//@{
	/// #FormCitation x(Communicator ..., Popups ...);#
	FormCitation(Communicator *, Popups *);
	///
	~FormCitation();
	//@}
private:
	FormCitation() {}
	FormCitation(FormCitation&) {}

	/**@name Real per-instance Callback Methods */
	//@{
	static int CitationWMHideCB(FL_FORM *, void *);
	static void CitationCB(FL_OBJECT *, long);
	static void InputCB(FL_OBJECT *, long);
	static void ComboxCB(int, void *);
	static void ComboxInputCB(void *);
	//@}
	
	/**@name Slot Methods */
	//@{
	/// Create the popup if necessary, update it and display it.
	void show(InsetCitation *);
	/// Hide the popup.
	void hide();
	/// Real popup handler
	void citation(long);
	//@}

	
	/// Build the popup
	void build();
	/// Explicitly free the popup.
	void free();

	typedef struct {
		FL_FORM *form_citation;
		void *vdata;
		char *cdata;
		long  ldata;
		FL_OBJECT *key;
		FL_OBJECT *button_ok;
		FL_OBJECT *button_cancel;
		FL_OBJECT *label;
		Combox * bibcombox;
	} FD_form_citation;

	/**@name Private Data */
	//@{
	/// Real GUI implementation.
	FD_form_citation *popup;
	/// Which LyXView do we belong to?
	Communicator *comm;
	/// Inset-is-being-deleted connection.
	Connection d;
	/// Update connection
	Connection u;
	/// Hide connection.
	Connection h;
	/// has form contents changed? Used to control OK (eventually)
	EnumPopupStatus status;
	//@}
};

#endif
