/* form_copyright.C
 * FormCopyright Interface Class Implementation
 */

#include <config.h>
#include "lyx_gui_misc.h"
#include "gettext.h"
#include FORMS_H_LOCATION

#include "form_copyright.h"
#include "Popups.h"
#include "Communicator.h"

// 	$Id: form_copyright.C,v 1.1 1998/10/18 03:22:05 arrae Exp $

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: form_copyright.C,v 1.1 1998/10/18 03:22:05 arrae Exp $";
#endif /* lint */


#ifdef __GNUG__
#pragma implementation
#endif


FormCopyright::FormCopyright(Communicator *c, Popups * p)
	: popup(0), comm(c), h(0)
{
	// let the popup be shown
	// This is a permanent connection so we won't bother
	// storing a copy because we won't be disconnecting.
	connect_to_method(p->showCopyright,
			  this,
			  &FormCopyright::show);
	// NOTE:
	// parameter p is used instead of:
	//      comm->getPopups()
	// because we are being contructed from within the
	// Popups constructor and a segfault would occur
	// otherwise.  The segfault occurs because the Popups
	// constructor hasn't returned at this point and so
	// comm->getPopups() will not return the real popups
	// instance.  If this implementation fails see Popups.C
	// ARRae.
}


FormCopyright::~FormCopyright()
{
	free();
}


void FormCopyright::build()
{
	FL_OBJECT *obj;
	FD_form_copyright *fdui = popup = new FD_form_copyright;

	fdui->form_copyright = fl_bgn_form(FL_NO_BOX, 450, 430);
	fdui->form_copyright->u_vdata = this;
	obj = fl_add_box(FL_UP_BOX,0,0,450,430,"");
	obj = fl_add_text(FL_NORMAL_TEXT,10,10,430,50,"LyX is Copyright (C) 1995 by Matthias Ettrich,\n1995-1998 LyX Team");
	fl_set_object_boxtype(obj,FL_FRAME_BOX);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_lalign(obj,FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
	fl_set_object_gravity(obj, FL_NorthWest, FL_NorthEast);
	obj = fl_add_text(FL_NORMAL_TEXT,10,70,430,110,"This program is free software; you can redistribute it\nand/or modify it under the terms of the GNU General\nPublic License as published by the Free Software\nFoundation; either version 2 of the License, or\n(at your option) any later version.");
	fl_set_object_boxtype(obj,FL_FRAME_BOX);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_lalign(obj,FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
	fdui->button_ok = obj = fl_add_button(FL_RETURN_BUTTON,160,390,140,30,"OK");
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_gravity(obj, FL_South, FL_South);
	fl_set_object_resize(obj, FL_RESIZE_NONE);
	fl_set_object_callback(obj,FormCopyright::CopyrightOKCB,0);
	obj = fl_add_text(FL_NORMAL_TEXT,10,190,430,190,"LyX is distributed in the hope that it will\nbe useful, but WITHOUT ANY WARRANTY;\nwithout even the implied warranty of MERCHANTABILITY\nor FITNESS FOR A PARTICULAR PURPOSE.\nSee the GNU General Public License for more details.\nYou should have received a copy of\nthe GNU General Public License\nalong with this program; if not, write to\nthe Free Software Foundation, Inc.,\n675 Mass Ave, Cambridge, MA 02139, USA.");
	fl_set_object_boxtype(obj,FL_FRAME_BOX);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_lalign(obj,FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
	fl_end_form();

	fdui->form_copyright->fdui = fdui;
}


void FormCopyright::show()
{
	if (!popup) {
		build();
		fl_set_form_atclose(popup->form_copyright,
				    FormCopyright::CopyrightWMHideCB, 0);
	}

	if (popup->form_copyright->visible) {
		fl_raise_form(popup->form_copyright);
	} else {
		fl_show_form(popup->form_copyright,
			     FL_PLACE_MOUSE,
			     FL_FULLBORDER,
			     _("Copyright and Warranty"));
		h = connect_to_method(
				comm->getPopups().hideAll,
				this,
				&FormCopyright::hide);
	}
}


void FormCopyright::hide()
{
	if (popup
	    && popup->form_copyright
	    && popup->form_copyright->visible) {
		fl_hide_form(popup->form_copyright);
		h.disconnect();
	}
	free();
}


void FormCopyright::free()
{
	// we don't need to delete h here because
	// hide() does that after disconnecting.
	if (popup) {
		if (popup->form_copyright
		    && popup->form_copyright->visible) {
			hide();
		}
		fl_free_form(popup->form_copyright);
		delete popup;
		popup = 0;
	}
}


int FormCopyright::CopyrightWMHideCB(FL_FORM * form, void *)
{
	// Ensure that the signal h is disconnected even if the
	// window manager is used to close the popup.
	FormCopyright *pre = (FormCopyright*)form->u_vdata;
	pre->hide();
	return FL_CANCEL;
}


void FormCopyright::CopyrightOKCB(FL_OBJECT *ob, long)
{
	FormCopyright *pre = (FormCopyright*)ob->form->u_vdata;
	pre->hide();
}
