/* form_credits.C
 * FormCredits Interface Class Implementation
 */

#include <config.h>
#include "lyx_gui_misc.h"
#include "gettext.h"
#include FORMS_H_LOCATION

#include "form_credits.h"
#include "Popups.h"
#include "Communicator.h"
#include "filetools.h"  // AddName

// 	$Id: form_credits.C,v 1.1 1998/10/18 03:22:05 arrae Exp $

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: form_credits.C,v 1.1 1998/10/18 03:22:05 arrae Exp $";
#endif /* lint */


#ifdef __GNUG__
#pragma implementation
#endif


FormCredits::FormCredits(Communicator *c, Popups * p)
	: popup(0), comm(c), h(0)
{
	// let the popup be shown
	// This is a permanent connection so we won't bother
	// storing a copy because we won't be disconnecting.
	connect_to_method(p->showCredits,
			  this,
			  &FormCredits::show);
	// NOTE:
	// parameter p is used instead of:
	//      comm->getPopups()
	// because we are being contructed from within the
	// Popups constructor and a segfault would occur
	// otherwise.  The segfault occurs because the Popups
	// constructor hasn't returned at this point and so
	// comm->getPopups() will not return the real popups
	// instance.  If this implementation fails see Popups.C
	// ARRae.
}


FormCredits::~FormCredits()
{
	free();
}


void FormCredits::build()
{
	FL_OBJECT *obj;
	FD_form_credits *fdui = popup = new FD_form_credits;

	fdui->form_credits = fl_bgn_form(FL_NO_BOX, 500, 330);
	fdui->form_credits->u_vdata = this;
	
	obj = fl_add_box(FL_UP_BOX,0,0,500,330,"");
	fdui->button_ok = obj = fl_add_button(FL_RETURN_BUTTON,180,290,140,30,"OK");
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_gravity(obj, FL_South, FL_South);
	fl_set_object_resize(obj, FL_RESIZE_NONE);
	fl_set_object_callback(obj,FormCredits::CreditsOKCB,0);
	obj = fl_add_text(FL_NORMAL_TEXT,10,40,480,30,"Matthias");
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_lalign(obj,FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
	fl_set_object_lstyle(obj,FL_ITALIC_STYLE);
	fl_set_object_gravity(obj, FL_NorthWest, FL_NorthEast);
	obj = fl_add_text(FL_NORMAL_TEXT,10,10,480,30,"All these people have contributed to the LyX project. Thanks,");
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_lalign(obj,FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
	fl_set_object_gravity(obj, FL_NorthWest, FL_NorthEast);
	fdui->browser_credits = obj = fl_add_browser(FL_NORMAL_BROWSER,10,80,480,200,"");
	fl_set_object_gravity(obj, FL_NorthWest, FL_SouthEast);
	fl_end_form();

	fdui->form_credits->fdui = fdui;
}


void FormCredits::show()
{
	if (!popup) {
		build();
		fl_set_form_atclose(popup->form_credits,
				    FormCredits::CreditsWMHideCB, 0);

		/* read the credits into the browser */ 
		/* try file LYX_DIR/CREDITS */ 
		string real_file = AddName (comm->getSystemLyXDir(),
					     "CREDITS");

		if (!fl_load_browser(popup->browser_credits, 
				     real_file.c_str())) {
			fl_add_browser_line(popup->browser_credits,
					    _("ERROR: LyX wasn't able to read"
					    " CREDITS file"));
			fl_add_browser_line(popup->browser_credits, "");
			fl_add_browser_line(popup->browser_credits,
					    _("Please install correctly to estimate"
					    " the great"));
			fl_add_browser_line(popup->browser_credits,
					    _("amount of work other people have done"
					    " for the LyX project."));
		}
	}

	if (popup->form_credits->visible) {
		fl_raise_form(popup->form_credits);
	} else {
		fl_show_form(popup->form_credits,
			     FL_PLACE_MOUSE | FL_FREE_SIZE,
			     FL_FULLBORDER,
			     _("Credits"));
		h = connect_to_method(
				comm->getPopups().hideAll,
				this,
				&FormCredits::hide);
	}
}


void FormCredits::hide()
{
	if (popup
	    && popup->form_credits
	    && popup->form_credits->visible) {
		fl_hide_form(popup->form_credits);
		h.disconnect();
	}
	free();
}


void FormCredits::free()
{
	// we don't need to delete h here because
	// hide() does that after disconnecting.
	if (popup) {
		if (popup->form_credits
		    && popup->form_credits->visible) {
			hide();
		}
		fl_free_form(popup->form_credits);
		delete popup;
		popup = 0;
	}
}


int FormCredits::CreditsWMHideCB(FL_FORM * form, void *)
{
	// Ensure that the signal h is disconnected even if the
	// window manager is used to close the popup.
	FormCredits *pre = (FormCredits*)form->u_vdata;
	pre->hide();
	return FL_CANCEL;
}


void FormCredits::CreditsOKCB(FL_OBJECT *ob, long)
{
	FormCredits *pre = (FormCredits*)ob->form->u_vdata;
	pre->hide();
}
