/* form_document.C
 * FormDocument Interface Class Implementation
 */

#include <config.h>
#include "lyx_gui_misc.h"
#include "gettext.h"
#include FORMS_H_LOCATION
#include XPM_H_LOCATION

#include "form_document.h"
#include "Popups.h"
#include "Communicator.h"
#include "textclasslist.h"
#include "combox.h"
#include "tex-strings.h"
#include "bufferparams.h"
#include "insetquotes.h"
#include "vspace.h"
#include "bmtable.h"
#include "filetools.h"

// 	$Id: form_document.C,v 1.6 1998/10/21 23:47:22 larsbj Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: form_document.C,v 1.6 1998/10/21 23:47:22 larsbj Exp $";
#endif /* lint */


#ifdef __GNUG__
#pragma implementation
#endif


FormDocument::FormDocument(Communicator *c, Popups * p)
	: tpopup(0), ppopup(0), cpopup(0), lpopup(0), opopup(0), bpopup(0),
	  comm(c), u(0), h(0), status(POPUP_UNMODIFIED)
{
    // let the popup be shown
    // This is a permanent connection so we won't bother
    // storing a copy because we won't be disconnecting.
    connect_to_method(p->showFormDocument, this, &FormDocument::show);
    // NOTE:
    // parameter p is used instead of:
    //      comm->getPopups()
    // because we are being contructed from within the
    // Popups constructor and a segfault would occur
    // otherwise.  The segfault occurs because the Popups
    // constructor hasn't returned at this point and so
    // comm->getPopups() will not return the real popups
    // instance.  If this implementation fails see Popups.C
    // ARRae.
    build();
}

FormDocument::~FormDocument()
{
    free();
}

void FormDocument::build()
{
    int n;

    // the document paper form
    build_pform();
    fl_addto_choice(ppopup->choice_papersize2,
		    _(" Default | Custom | USletter | USlegal "
		      "| USexecutive | A3 | A4 | A5 | B3 | B4 | B5 "));
    fl_addto_choice(ppopup->choice_paperpackage,
		    _(" None "
		      "| A4 small Margins (only portrait) "
		      "| A4 very small Margins (only portrait) "
		      "| A4 very wide margins (only portrait) "));
    fl_set_input_return(ppopup->input_custom_width, FL_RETURN_ALWAYS);
    fl_set_input_return(ppopup->input_custom_height, FL_RETURN_ALWAYS);
    fl_set_input_return(ppopup->input_top_margin, FL_RETURN_ALWAYS);
    fl_set_input_return(ppopup->input_bottom_margin, FL_RETURN_ALWAYS);
    fl_set_input_return(ppopup->input_left_margin, FL_RETURN_ALWAYS);
    fl_set_input_return(ppopup->input_right_margin, FL_RETURN_ALWAYS);
    fl_set_input_return(ppopup->input_head_height, FL_RETURN_ALWAYS);
    fl_set_input_return(ppopup->input_head_sep, FL_RETURN_ALWAYS);
    fl_set_input_return(ppopup->input_foot_skip, FL_RETURN_ALWAYS);

    // the document class form
    build_cform();
    fl_addto_choice(cpopup->choice_doc_spacing,
		    _(" Single | OneHalf | Double | Other "));
    fl_addto_choice(cpopup->choice_doc_fontsize, "default|10|11|12");
    for (n=0; tex_fonts[n][0]; ++n) {
	fl_addto_choice(cpopup->choice_doc_fonts,tex_fonts[n]);
    }
    fl_addto_choice(cpopup->choice_doc_pagestyle,
		    "default|empty|plain|headings|fancy");
    fl_addto_choice(cpopup->choice_doc_skip,
		    _(" Smallskip | Medskip | Bigskip | Length "));
    fl_set_input_return(cpopup->input_doc_skip, FL_RETURN_ALWAYS);

    // the document language form
    build_lform();
    fl_addto_choice(lpopup->choice_inputenc, "default|latin1|latin2|latin5");
    for (n=0; tex_babel[n][0]; ++n) {
	lpopup->combo_language->addto(tex_babel[n]);
    }
    fl_addto_choice(lpopup->choice_quotes_language,
		    _(" ``text'' | ''text'' | ,,text`` | ,,text'' |"
		      " text | text "));

    // the document options form
    build_oform();
    fl_set_counter_bounds(opopup->slider_secnumdepth,-1,5);
    fl_set_counter_bounds(opopup->slider_tocdepth,-1,5);
    fl_set_counter_step(opopup->slider_secnumdepth,1,1);
    fl_set_counter_step(opopup->slider_tocdepth,1,1);
    fl_set_counter_precision(opopup->slider_secnumdepth, 0);
    fl_set_counter_precision(opopup->slider_tocdepth, 0);
    for (n=0; tex_graphics[n][0]; ++n) {
	fl_addto_choice(opopup->choice_postscript_driver, tex_graphics[n]);
    }
    
    // the document bullets form
    build_bform();
    fl_addto_choice(bpopup->choice_bullet_size,
		    _(" default | tiny | script | footnote | small |"
		      " normal | large | Large | LARGE | huge | Huge"));
    fl_set_choice(bpopup->choice_bullet_size, 1);
    fl_set_input_return(bpopup->input_bullet_latex, FL_RETURN_CHANGED);
    fl_set_input_maxchars(bpopup->input_bullet_latex, 80);

    // the tabbed folder
    build_tform();
    fl_set_form_atclose(tpopup->form_tabbed_document,
                        FormDocument::FormDocumentWMHideCB, 0);
    fl_addto_tabfolder(tpopup->tabbed_folder,_("Document"),
                       cpopup->form_doc_class);
    fl_addto_tabfolder(tpopup->tabbed_folder,_("Paper"),
                       ppopup->form_doc_paper);
    fl_addto_tabfolder(tpopup->tabbed_folder,_("Language"),
                       lpopup->form_doc_lang);
    fl_addto_tabfolder(tpopup->tabbed_folder,_("Extra"),
                       opopup->form_doc_options);
    fbullet = fl_addto_tabfolder(tpopup->tabbed_folder,_("Bullets"),
				 bpopup->form_doc_bullet);
    if ((XpmVersion < 4) || (XpmVersion == 4 && XpmRevision < 7)) {
	    lyxerr << _("Your version of libXpm is older than 4.7.\n"
			"The `bullet' tab of the document popup "
			"has been disabled") << '\n';
	fl_deactivate_object(fbullet);
	fl_set_object_lcol(fbullet, FL_INACTIVE);
    }
}

void FormDocument::show()
{
    update();  // make sure its up-to-date
    if (tpopup->form_tabbed_document->visible) {
        fl_raise_form(tpopup->form_tabbed_document);
    } else {
        fl_show_form(tpopup->form_tabbed_document,
                     FL_PLACE_MOUSE | FL_FREE_SIZE,
                     FL_FULLBORDER,
                     _("Document Layout"));
        u = connect_to_method(comm->getPopups().updateBufferDependent,
                              this, &FormDocument::update);
        h = connect_to_method(comm->getPopups().hideBufferDependent,
                              this, &FormDocument::hide);
    }
}

void FormDocument::hide()
{
    if (tpopup->form_tabbed_document->visible) {
        fl_hide_form(tpopup->form_tabbed_document);
        u.disconnect();
        h.disconnect();
    }
}

void FormDocument::apply()
{
    if (!comm->isAvailable() || !tpopup)
        return;
    bool
	redo;

    redo = capply() || oapply();
    papply();
    lapply();
    bapply();

    if (redo) {
	    comm->redoBuffer();
    }
    comm->markDirty();
    comm->setMinibuffer(_("Document layout set"));
}


bool FormDocument::capply()
{
	bool redo = false;
	BufferParams &params = comm->getBufferParams();

	// If default skip is a "Length" but there's no text in the
	// input field, reset the kind to "Medskip", which is the default.
	if (fl_get_choice (cpopup->choice_doc_skip) == 4
	    && !*(fl_get_input (cpopup->input_doc_skip))) {
		fl_set_choice (cpopup->choice_doc_skip, 2);
	}
	params.fonts = fl_get_choice_text(cpopup->choice_doc_fonts);
	params.fontsize = fl_get_choice_text(cpopup->choice_doc_fontsize);
	params.pagestyle = fl_get_choice_text(cpopup->choice_doc_pagestyle);
   
	char new_class = fl_get_choice(cpopup->choice_doc_class) - 1;
	if (params.textclass != new_class) {
		// try to load new_class
		if (textclasslist.load(new_class)) {
			// successfully loaded
			redo = true;
			comm->setMinibuffer(_("Converting document to new document class..."));
			int ret = comm->switchLayoutsBetweenClasses(params.textclass,
								    new_class);
			if (ret) {
				string s;
				if (ret==1) {
					s= _("One paragraph couldn't be converted");
				} else {
					s += ret;
					s += _(" paragraphs couldn't be converted");
				}
				WriteAlert(_("Conversion Errors!"),s,
					   _("into chosen document class"));
			}
			
			params.textclass = new_class;
		} else {
			// problem changing class -- warn user and retain old style
			WriteAlert(_("Conversion Errors!"),
				   _("Errors loading new document class."),
				   _("Reverting to original document class."));
			fl_set_choice(cpopup->choice_doc_class,
				      params.textclass + 1);
		}
	}
	char tmpsep = params.paragraph_separation;
	if (fl_get_button(cpopup->radio_doc_indent))
		params.paragraph_separation = LYX_PARSEP_INDENT;
	else
		params.paragraph_separation = LYX_PARSEP_SKIP;
	if (tmpsep != params.paragraph_separation)
		redo = true;
	
	VSpace tmpdefskip = params.getDefSkip();
	switch (fl_get_choice (cpopup->choice_doc_skip)) {
	case 1:
		params.setDefSkip(VSpace::SMALLSKIP);
		break;
	case 2:
		params.setDefSkip(VSpace::MEDSKIP);
		break;
    case 3:
	    params.setDefSkip(VSpace::BIGSKIP);
	    break;
	case 4:
		params.setDefSkip
			(VSpace(LyXGlueLength(fl_get_input(cpopup->input_doc_skip))));
		break;
		// DocumentDefskipCB assures that this never happens
	default:
		params.setDefSkip(VSpace::MEDSKIP);
		break;
	}
	if (!(tmpdefskip == params.getDefSkip()))
		redo = true;
	
	if (fl_get_button(cpopup->radio_doc_columns_two))
		params.columns = 2;
	else
		params.columns = 1;
	if (fl_get_button(cpopup->radio_doc_sides_two))
		params.sides = 2;
	else
		params.sides = 1;
	
	Spacing tmpSpacing = params.spacing;
	switch(fl_get_choice(cpopup->choice_doc_spacing)) {
	case 1:
		lyxerr[Debug::INFO] << "Spacing: SINGLE\n";
		params.spacing.set(Spacing::Single);
		break;
	case 2:
		lyxerr[Debug::INFO] << "Spacing: ONEHALF\n";
		params.spacing.set(Spacing::Onehalf);
		break;
	case 3:
		lyxerr[Debug::INFO] << "Spacing: DOUBLE\n";
		params.spacing.set(Spacing::Double);
		break;
	case 4:
		lyxerr[Debug::INFO] << "Spacing: OTHER\n";
		params.spacing.set(Spacing::Other, 
				   fl_get_input(cpopup->input_doc_spacing));
		break;
	}
	if (tmpSpacing != params.spacing)
		redo = true;
	
	params.options = fl_get_input(cpopup->input_doc_extra);
	
	return redo;
}

void FormDocument::papply()
{
    BufferParams
	&params = comm->getBufferParams();
    
    params.papersize2 = fl_get_choice(ppopup->choice_papersize2)-1;
    params.paperpackage = fl_get_choice(ppopup->choice_paperpackage)-1;
    params.use_geometry = fl_get_button(ppopup->push_use_geometry);
    if (fl_get_button(ppopup->radio_landscape))
	params.orientation = ORIENTATION_LANDSCAPE;
    else
	params.orientation = ORIENTATION_PORTRAIT;
    params.paperwidth = fl_get_input(ppopup->input_custom_width);
    params.paperheight = fl_get_input(ppopup->input_custom_height);
    params.leftmargin = fl_get_input(ppopup->input_left_margin);
    params.topmargin = fl_get_input(ppopup->input_top_margin);
    params.rightmargin = fl_get_input(ppopup->input_right_margin);
    params.bottommargin = fl_get_input(ppopup->input_bottom_margin);
    params.headheight = fl_get_input(ppopup->input_head_height);
    params.headsep = fl_get_input(ppopup->input_head_sep);
    params.footskip = fl_get_input(ppopup->input_foot_skip);
    comm->setPaperStuff();
}

void FormDocument::lapply()
{
    BufferParams
	&params = comm->getBufferParams();
    InsetQuotes::quote_language
	lga = InsetQuotes::EnglishQ;

    switch(fl_get_choice(lpopup->choice_quotes_language) - 1) {
    case 0:
	lga = InsetQuotes::EnglishQ;
	break;
    case 1:
	lga = InsetQuotes::SwedishQ;
	break;
    case 2:
	lga = InsetQuotes::GermanQ;
	break;
    case 3:
	lga = InsetQuotes::PolishQ;
		break;
    case 4:
	lga = InsetQuotes::FrenchQ;
	break;
    case 5:
	lga = InsetQuotes::DanishQ;
	break;
    }
    params.quotes_language = lga;
    if (fl_get_button(lpopup->radio_single))   
	params.quotes_times = InsetQuotes::SingleQ;
    else
	params.quotes_times = InsetQuotes::DoubleQ;
    params.language = lpopup->combo_language->getline();
    params.inputenc = fl_get_choice_text(lpopup->choice_inputenc);
    if (fl_get_button(lpopup->choice_accents))
	params.allowAccents = true;
    else
	params.allowAccents = false;
}

bool FormDocument::oapply()
{
    BufferParams
	&params = comm->getBufferParams();
    bool
	redo = false;

    params.graphicsDriver =
	fl_get_choice_text(opopup->choice_postscript_driver);
    params.use_amsmath = fl_get_button(opopup->check_use_amsmath);

#warning check type
    short tmpchar = short(fl_get_counter_value(opopup->slider_secnumdepth));
    if (params.secnumdepth != tmpchar)
	redo = true;
    params.secnumdepth = tmpchar;
   
    params.tocdepth = (signed char) 
	fl_get_counter_value(opopup->slider_tocdepth);;

    params.float_placement =
	fl_get_input(opopup->input_float_placement);

    return redo;
}

void FormDocument::bapply()
{
    /* update the bullet settings */
    BufferParams & param = comm->getBufferParams();
    
    // a little bit of loop unrolling
    param.user_defined_bullets[0] = param.temp_bullets[0];
    param.user_defined_bullets[1] = param.temp_bullets[1];
    param.user_defined_bullets[2] = param.temp_bullets[2];
    param.user_defined_bullets[3] = param.temp_bullets[3];
}

void FormDocument::cancel()
{
    // this avoids confusion when reopening
    BufferParams & param = comm->getBufferParams();
    param.temp_bullets[0] = param.user_defined_bullets[0];
    param.temp_bullets[1] = param.user_defined_bullets[1];
    param.temp_bullets[2] = param.user_defined_bullets[2];
    param.temp_bullets[3] = param.user_defined_bullets[3];
}

void FormDocument::update()
{
    if (!comm->isAvailable() || !tpopup)
        return;
    fl_hide_object(tpopup->text_warning);
    EnableDocumentLayout();
    if (comm->isReadOnly()) {
	DisableDocumentLayout();
	fl_set_object_label(tpopup->text_warning,
			    _("Document is read-only."
			      " No changes to layout permitted."));
	fl_show_object(tpopup->text_warning);
    }
    cupdate();
    pupdate();
    lupdate();
    oupdate();
    bupdate();
}

void FormDocument::cupdate()
{
    if (!cpopup)
        return;
    BufferParams
	const & params = comm->getBufferParams();
    LyXTextClass
	const *tclass = textclasslist.textClass(params.textclass);
	
    fl_set_choice_text(cpopup->choice_doc_class, 
		       textclasslist.descOfClass(params.textclass).c_str());
    fl_set_choice_text(cpopup->choice_doc_fonts, params.fonts.c_str());
    fl_clear_choice(cpopup->choice_doc_fontsize);
    fl_addto_choice(cpopup->choice_doc_fontsize, "default");
    fl_addto_choice(cpopup->choice_doc_fontsize, tclass->opt_fontsize.c_str());
    fl_set_choice(cpopup->choice_doc_fontsize,
		  tokenPos(tclass->opt_fontsize, '|', params.fontsize) + 2);
    fl_clear_choice(cpopup->choice_doc_pagestyle);
    fl_addto_choice(cpopup->choice_doc_pagestyle, "default");
    fl_addto_choice(cpopup->choice_doc_pagestyle,
		    tclass->opt_pagestyle.c_str());
    fl_set_choice(cpopup->choice_doc_pagestyle,
		  tokenPos(tclass->opt_pagestyle, '|', params.pagestyle) + 2);
    fl_set_button(cpopup->radio_doc_indent, 0);
    fl_set_button(cpopup->radio_doc_skip, 0);
    if (params.paragraph_separation == LYX_PARSEP_INDENT)
	fl_set_button(cpopup->radio_doc_indent, 1);
    else
	fl_set_button(cpopup->radio_doc_skip, 1);
    switch (params.getDefSkip().kind()) {
    case VSpace::SMALLSKIP: 
	fl_set_choice (cpopup->choice_doc_skip, 1);
	break;
    case VSpace::MEDSKIP: 
	fl_set_choice (cpopup->choice_doc_skip, 2);
	break;
    case VSpace::BIGSKIP: 
	fl_set_choice (cpopup->choice_doc_skip, 3);
	break;
    case VSpace::LENGTH: 
	fl_set_choice (cpopup->choice_doc_skip, 4);
	fl_set_input (cpopup->input_doc_skip,
		      params.getDefSkip().asLyXCommand().c_str());
	break;
    default:
	fl_set_choice (cpopup->choice_doc_skip, 2);
	break;
    }
    fl_set_button(cpopup->radio_doc_sides_one, 0);
    fl_set_button(cpopup->radio_doc_sides_two, 0);
    if (params.sides == 2)
	fl_set_button(cpopup->radio_doc_sides_two, 1);
    else
	fl_set_button(cpopup->radio_doc_sides_one, 1);
    fl_set_button(cpopup->radio_doc_columns_one, 0);
    fl_set_button(cpopup->radio_doc_columns_two, 0);
    if (params.columns == 2)
	fl_set_button(cpopup->radio_doc_columns_two, 1);
    else
	fl_set_button(cpopup->radio_doc_columns_one, 1);
    fl_set_input(cpopup->input_doc_spacing, "");
    switch (params.spacing.getSpace()) {
    case Spacing::Single:
	// \singlespacing
	fl_set_choice(cpopup->choice_doc_spacing, 1);
	break;
    case Spacing::Onehalf:
	// \onehalfspacing
	fl_set_choice(cpopup->choice_doc_spacing, 2);
	break;
    case Spacing::Double:
	// \doublespacing
	fl_set_choice(cpopup->choice_doc_spacing, 3);
	break;
    case Spacing::Other:
	{
	    fl_set_choice(cpopup->choice_doc_spacing, 4);
	    char sval[20];
	    sprintf(sval,"%g",params.spacing.getValue()); 
	    fl_set_input(cpopup->input_doc_spacing, sval);
	    break;
	}
    }
    if (!params.options.empty())
	fl_set_input(cpopup->input_doc_extra, params.options.c_str());
    else
	fl_set_input(cpopup->input_doc_extra, "");
}

void FormDocument::lupdate()
{
    if (!lpopup)
        return;

    BufferParams
	&params = comm->getBufferParams();

    lpopup->combo_language->select_text(params.language.c_str());
    fl_set_choice_text(lpopup->choice_inputenc, params.inputenc.c_str());
    fl_set_button(lpopup->choice_accents, (int)params.allowAccents);
    fl_set_choice(lpopup->choice_quotes_language, params.quotes_language + 1);
    fl_set_button(lpopup->radio_single, 0);
    fl_set_button(lpopup->radio_double, 0);
    if (params.quotes_times == InsetQuotes::SingleQ)
	fl_set_button(lpopup->radio_single, 1);
    else
	fl_set_button(lpopup->radio_double, 1);
}

void FormDocument::oupdate()
{
    if (!opopup)
        return;

    BufferParams
	&params = comm->getBufferParams();

    fl_set_choice_text(opopup->choice_postscript_driver,
		       params.graphicsDriver.c_str());
    fl_set_button(opopup->check_use_amsmath, params.use_amsmath);
    fl_set_counter_value(opopup->slider_secnumdepth, params.secnumdepth);
    fl_set_counter_value(opopup->slider_tocdepth, params.tocdepth);
    if (!params.float_placement.empty())
	fl_set_input(opopup->input_float_placement,
		     params.float_placement.c_str());
    else
	fl_set_input(opopup->input_float_placement, "");
}

void FormDocument::pupdate()
{
    if (!ppopup)
        return;
    BufferParams
	&params = comm->getBufferParams();

    fl_set_choice(ppopup->choice_papersize2, params.papersize2 + 1);
    fl_set_choice(ppopup->choice_paperpackage, params.paperpackage + 1);
    fl_set_button(ppopup->push_use_geometry, params.use_geometry);
    fl_set_button(ppopup->radio_portrait, 0);
    fl_set_button(ppopup->radio_landscape, 0);
    if (params.orientation == ORIENTATION_LANDSCAPE)
	fl_set_button(ppopup->radio_landscape, 1);
    else
	fl_set_button(ppopup->radio_portrait, 1);
    fl_set_input(ppopup->input_custom_width, params.paperwidth.c_str());
    fl_set_input(ppopup->input_custom_height, params.paperheight.c_str());
    fl_set_input(ppopup->input_left_margin, params.leftmargin.c_str());
    fl_set_input(ppopup->input_top_margin, params.topmargin.c_str());
    fl_set_input(ppopup->input_right_margin, params.rightmargin.c_str());
    fl_set_input(ppopup->input_bottom_margin, params.bottommargin.c_str());
    fl_set_input(ppopup->input_head_height, params.headheight.c_str());
    fl_set_input(ppopup->input_head_sep, params.headsep.c_str());
    fl_set_input(ppopup->input_foot_skip, params.footskip.c_str());
    fl_set_focus_object(ppopup->form_doc_paper, ppopup->choice_papersize2);
}

void FormDocument::bupdate()
{
    if (!bpopup || ((XpmVersion < 4) || (XpmVersion == 4 && XpmRevision < 7)))
        return;

    if (comm->isLinuxDoc()) {
	fl_deactivate_object(fbullet);
	fl_set_object_lcol(fbullet, FL_INACTIVE);
	return;
    } else {
	fl_activate_object(fbullet);
	fl_set_object_lcol(fbullet, FL_BLACK);
    }
    if (comm->isReadOnly()) {
	fl_deactivate_object (bpopup->bmtable_bullet_panel);
	fl_deactivate_object (bpopup->choice_bullet_size);
	fl_deactivate_object (bpopup->input_bullet_latex);
    } else {
	fl_activate_object (bpopup->bmtable_bullet_panel);
	fl_activate_object (bpopup->choice_bullet_size);
	fl_activate_object (bpopup->input_bullet_latex);
    }
    BufferParams
	&params = comm->getBufferParams();

    fl_set_button(bpopup->radio_bullet_depth_1, 1);
    fl_set_input(bpopup->input_bullet_latex,
		 params.user_defined_bullets[0].c_str());
    fl_set_choice(bpopup->choice_bullet_size,
		  params.user_defined_bullets[0].getSize() + 2);
}

void FormDocument::free()
{
    if (tpopup) {
        hide();
        if (cpopup) {
            fl_free_form(cpopup->form_doc_class);
            delete cpopup;
            cpopup = 0;
        }
        if (ppopup) {
            fl_free_form(ppopup->form_doc_paper);
            delete ppopup;
            ppopup = 0;
        }
        if (lpopup) {
	    delete lpopup->combo_language;
            fl_free_form(lpopup->form_doc_lang);
            delete lpopup;
            lpopup = 0;
        }
        if (opopup) {
            fl_free_form(opopup->form_doc_options);
            delete opopup;
            opopup = 0;
        }
        if (bpopup) {
            fl_free_form(bpopup->form_doc_bullet);
            delete bpopup;
            bpopup = 0;
        }
        fl_free_form(tpopup->form_tabbed_document);
        delete tpopup;
        tpopup = 0;
    }
}

int FormDocument::FormDocumentWMHideCB(FL_FORM * form, void *)
{
    // Ensure that the signals (u and h) are disconnected even if the
    // window manager is used to close the popup.
    FormDocument *pre = (FormDocument*)form->u_vdata;
    pre->hide();
    return FL_CANCEL;
}

void FormDocument::FormDocumentCancelCB(FL_OBJECT *ob, long)
{
    FormDocument *pre = (FormDocument*)ob->form->u_vdata;
    pre->cancel();
    pre->hide();
}

void FormDocument::FormDocumentApplyCB(FL_OBJECT *ob, long)
{
    FormDocument *pre = (FormDocument*)ob->form->u_vdata;
    pre->apply();
}

void FormDocument::FormDocumentOKCB(FL_OBJECT *ob, long)
{
    FormDocument *pre = (FormDocument*)ob->form->u_vdata;
    pre->apply();
    pre->hide();
}

void FormDocument::DocumentInputCB(FL_OBJECT *ob, long)
{
    FormDocument *pre = (FormDocument*)ob->form->u_vdata;
    pre->CheckDocumentInput(ob,0);
}

void FormDocument::checkMarginValues()
{
    int allEmpty;
    
    allEmpty = (!strlen(fl_get_input(ppopup->input_top_margin)) &&
		!strlen(fl_get_input(ppopup->input_bottom_margin)) &&
		!strlen(fl_get_input(ppopup->input_left_margin)) &&
		!strlen(fl_get_input(ppopup->input_right_margin)) &&
		!strlen(fl_get_input(ppopup->input_head_height)) &&
		!strlen(fl_get_input(ppopup->input_head_sep)) &&
		!strlen(fl_get_input(ppopup->input_foot_skip)) &&
		!strlen(fl_get_input(ppopup->input_custom_width)) &&
		!strlen(fl_get_input(ppopup->input_custom_height)));
    if (!allEmpty)
	fl_set_button(ppopup->push_use_geometry, 1);
}

void FormDocument::CheckDocumentInput(FL_OBJECT *ob, long)
{
    string
	str;
    char
	val;
    bool
	ok = true;
    
    ActivateDocumentButtons();
    checkMarginValues();
    val = fl_get_choice(ppopup->choice_papersize2)-1;
    if (val == VM_PAPER_DEFAULT) {
	fl_set_button(ppopup->push_use_geometry, 0);
	checkMarginValues();
    } else {
	if ((val != VM_PAPER_USLETTER) && (val != VM_PAPER_USLEGAL) &&
	    (val != VM_PAPER_USEXECUTIVE) && (val != VM_PAPER_A4) &&
	    (val != VM_PAPER_A5) && (val != VM_PAPER_B5)) {
	    fl_set_button(ppopup->push_use_geometry, 1);
	}
	fl_set_choice(ppopup->choice_paperpackage, PACKAGE_NONE + 1);
    }
    val = fl_get_choice(ppopup->choice_paperpackage)-1;
    if (val != PACKAGE_NONE) {
	fl_set_choice(ppopup->choice_papersize2, VM_PAPER_DEFAULT + 1);
	fl_set_button(ppopup->push_use_geometry, 0);
    }
    str = fl_get_input(ppopup->input_custom_width);
    ok = ok && (str.empty() || isValidLength(str));
    str = fl_get_input(ppopup->input_custom_height);
    ok = ok && (str.empty() || isValidLength(str));
    str = fl_get_input(ppopup->input_left_margin);
    ok = ok && (str.empty() || isValidLength(str));
    str = fl_get_input(ppopup->input_right_margin);
    ok = ok && (str.empty() || isValidLength(str));
    str = fl_get_input(ppopup->input_top_margin);
    ok = ok && (str.empty() || isValidLength(str));
    str = fl_get_input(ppopup->input_bottom_margin);
    ok = ok && (str.empty() || isValidLength(str));
    str = fl_get_input(ppopup->input_head_height);
    ok = ok && (str.empty() || isValidLength(str));
    str = fl_get_input(ppopup->input_head_sep);
    ok = ok && (str.empty() || isValidLength(str));
    str = fl_get_input(ppopup->input_foot_skip);
    ok = ok && (str.empty() || isValidLength(str));
    // "Synchronize" the choice and the input field, so that it
    // is impossible to commit senseless data.
    const char* input = fl_get_input (cpopup->input_doc_skip);
    if (ob == cpopup->input_doc_skip) {
	if (!*input) {
	    fl_set_choice (cpopup->choice_doc_skip, 2);
	} else if (isValidGlueLength (input)) {
	    fl_set_choice (cpopup->choice_doc_skip, 4);
	} else {
	    fl_set_choice(cpopup->choice_doc_skip, 4);
	    ok = false;
	}
    } else {
	if (*input && !isValidGlueLength(input))
	    ok = false;
    }
    if (ob == cpopup->choice_doc_skip)
	if ((fl_get_choice(cpopup->choice_doc_skip) == 4) && !*input)
	    ok = false;
    else if (fl_get_choice(cpopup->choice_doc_skip) != 4)
	fl_set_input (cpopup->input_doc_skip, "");
    input = fl_get_input(cpopup->input_doc_spacing);
    if (ob == cpopup->input_doc_spacing) {
	if (!*input) {
	    fl_set_choice (cpopup->choice_doc_spacing, 1);
	} else {
	    fl_set_choice(cpopup->choice_doc_spacing, 4);
	}
    }
    if (ob == cpopup->choice_doc_spacing) {
	fprintf(stderr,"%d\n",fl_get_choice(cpopup->choice_doc_spacing));
	if ((fl_get_choice(cpopup->choice_doc_spacing) == 4) && !*input)
	    ok = false;
	else  if (fl_get_choice(cpopup->choice_doc_spacing) != 4)
	    fl_set_input (cpopup->input_doc_spacing, "");
    }
    if (!ok)
	DeactivateDocumentButtons();
}

void FormDocument::ChoiceBulletSizeCB(FL_OBJECT *ob, long)
{
    FormDocument *pre = (FormDocument*)ob->form->u_vdata;
    pre->ChoiceBulletSize(ob,0);
}

void FormDocument::ChoiceBulletSize(FL_OBJECT * ob, long /*data*/ )
{
    BufferParams & param = comm->getBufferParams();

    // convert from 1-6 range to -1-4 
    param.temp_bullets[current_bullet_depth].setSize(fl_get_choice(ob) - 2);
    fl_set_input(bpopup->input_bullet_latex,
		 param.temp_bullets[current_bullet_depth].c_str());
}

void FormDocument::InputBulletLaTeXCB(FL_OBJECT *ob, long)
{
    FormDocument *pre = (FormDocument*)ob->form->u_vdata;
    pre->InputBulletLaTeX(ob,0);
}

void FormDocument::InputBulletLaTeX(FL_OBJECT *, long)
{
    BufferParams & param = comm->getBufferParams();

    param.temp_bullets[current_bullet_depth].
	setText(fl_get_input(bpopup->input_bullet_latex));
}

void FormDocument::BulletDepthCB(FL_OBJECT *ob, long)
{
    FormDocument *pre = (FormDocument*)ob->form->u_vdata;
    pre->BulletDepth(ob,0);
}

void FormDocument::BulletDepth(FL_OBJECT *ob, long data)
{
    /* Should I do the following:                                 */
    /*  1. change to the panel that the current bullet belongs in */
    /*  2. show that bullet as selected                           */
    /*  3. change the size setting to the size of the bullet in Q.*/
    /*  4. display the latex equivalent in the latex box          */
    /*                                                            */
    /* I'm inclined to just go with 3 and 4 at the moment and     */
    /* maybe try to support the others later                      */
    BufferParams & param = comm->getBufferParams();

    switch (fl_get_button_numb(ob)) {
    case 3:
	// right mouse button resets to default
	param.temp_bullets[data] = ITEMIZE_DEFAULTS[data];
    default:
	current_bullet_depth = data;
	fl_set_input(bpopup->input_bullet_latex,
		     param.temp_bullets[data].c_str());
	fl_set_choice(bpopup->choice_bullet_size,
		      param.temp_bullets[data].getSize() + 2);
    }
}

void FormDocument::BulletPanelCB(FL_OBJECT *ob, long data)
{
    FormDocument *pre = (FormDocument*)ob->form->u_vdata;
    pre->BulletPanel(ob,data);
}

void FormDocument::BulletPanel(FL_OBJECT * /*ob*/, long data)
{
    /* Here we have to change the background pixmap to that selected */
    /* by the user. (eg. standard.xpm, psnfss1.xpm etc...)           */
    
    if (data != current_bullet_panel) {
	fl_freeze_form(bpopup->form_doc_bullet);
	current_bullet_panel = data;

	/* free the current pixmap */
	fl_free_bmtable_pixmap(bpopup->bmtable_bullet_panel);
	string new_panel;
	switch (data) {
	    /* display the new one */
	case 0 :
	    new_panel = "standard";
	    break;
	case 1 :
	    new_panel = "amssymb";
	    break;
	case 2 :
	    new_panel = "psnfss1";
	    break;
	case 3 :
	    new_panel = "psnfss2";
	    break;
	case 4 :
	    new_panel = "psnfss3";
	    break;
	case 5 :
	    new_panel = "psnfss4";
	    break;
	default :
	    /* something very wrong happened */
	    // play it safe for now but should be an exception
	    current_bullet_panel = 0;  // standard panel
	    new_panel = "standard";
	    break;
	}
	new_panel += ".xpm";
	fl_set_bmtable_pixmap_file(bpopup->bmtable_bullet_panel, 6, 6,
				   LibFileSearch("images", new_panel.c_str()).c_str());
	fl_redraw_object(bpopup->bmtable_bullet_panel);
	fl_unfreeze_form(bpopup->form_doc_bullet);
    }
}

void FormDocument::BulletBMTableCB(FL_OBJECT *ob, long)
{
    FormDocument *pre = (FormDocument*)ob->form->u_vdata;
    pre->BulletBMTable(ob,0);
}

void FormDocument::BulletBMTable(FL_OBJECT *ob, long /*data*/ )
{
    /* handle the user input by setting the current bullet depth's pixmap */
    /* to that extracted from the current chosen position of the BMTable  */
    /* Don't forget to free the button's old pixmap first.                */

    BufferParams & param = comm->getBufferParams();
    int bmtable_button = fl_get_bmtable(ob);

    /* try to keep the button held down till another is pushed */
    /*  fl_set_bmtable(ob, 1, bmtable_button); */
    param.temp_bullets[current_bullet_depth].setFont(current_bullet_panel);
    param.temp_bullets[current_bullet_depth].setCharacter(bmtable_button);
    fl_set_input(bpopup->input_bullet_latex,
		 param.temp_bullets[current_bullet_depth].c_str());
}

void FormDocument::DeactivateDocumentButtons()
{
    fl_deactivate_object(tpopup->button_ok);
    fl_deactivate_object(tpopup->button_apply);
    fl_set_object_lcol(tpopup->button_ok, FL_INACTIVE);
    fl_set_object_lcol(tpopup->button_apply, FL_INACTIVE);
}

void FormDocument::ActivateDocumentButtons()
{
    fl_activate_object(tpopup->button_ok);
    fl_activate_object(tpopup->button_apply);
    fl_set_object_lcol(tpopup->button_ok, FL_BLACK);
    fl_set_object_lcol(tpopup->button_apply, FL_BLACK);
}

void FormDocument::EnableDocumentLayout()
{
        ActivateDocumentButtons ();
	fl_activate_object (cpopup->radio_doc_indent);
	fl_activate_object (cpopup->radio_doc_skip);
	fl_activate_object (cpopup->choice_doc_class);
	fl_activate_object (cpopup->choice_doc_pagestyle);
	fl_activate_object (cpopup->choice_doc_fonts);
	fl_activate_object (cpopup->choice_doc_fontsize);
	fl_activate_object (cpopup->radio_doc_sides_one);
	fl_activate_object (cpopup->radio_doc_sides_two);
	fl_activate_object (cpopup->radio_doc_columns_one);
	fl_activate_object (cpopup->radio_doc_columns_two);
	fl_activate_object (cpopup->input_doc_extra);
	fl_activate_object (cpopup->input_doc_skip);
	fl_activate_object (cpopup->choice_doc_skip);
	fl_activate_object (cpopup->choice_doc_spacing);
	fl_activate_object (cpopup->input_doc_spacing);

	fl_activate_object (lpopup->choice_language);
	lpopup->combo_language->activate();
	fl_activate_object (lpopup->choice_inputenc);

	fl_activate_object (opopup->slider_secnumdepth);
	fl_activate_object (opopup->slider_tocdepth);
	fl_activate_object (opopup->check_use_amsmath);
	fl_activate_object (opopup->input_float_placement);
	fl_activate_object (opopup->choice_postscript_driver);

        fl_activate_object (ppopup->choice_paperpackage);
	fl_activate_object (ppopup->greoup_radio_orientation);
	fl_activate_object (ppopup->radio_portrait);
	fl_activate_object (ppopup->radio_landscape);
	fl_activate_object (ppopup->choice_papersize2);
	fl_activate_object (ppopup->push_use_geometry);
	fl_activate_object (ppopup->input_custom_width);
	fl_activate_object (ppopup->input_custom_height);
	fl_activate_object (ppopup->input_top_margin);
	fl_activate_object (ppopup->input_bottom_margin);
	fl_activate_object (ppopup->input_left_margin);
	fl_activate_object (ppopup->input_right_margin);
	fl_activate_object (ppopup->input_head_height);
	fl_activate_object (ppopup->input_head_sep);
	fl_activate_object (ppopup->input_foot_skip);
}

void FormDocument::DisableDocumentLayout()
{
        DeactivateDocumentButtons ();
	fl_deactivate_object (cpopup->radio_doc_indent);
	fl_deactivate_object (cpopup->radio_doc_skip);
	fl_deactivate_object (cpopup->choice_doc_class);
	fl_deactivate_object (cpopup->choice_doc_pagestyle);
	fl_deactivate_object (cpopup->choice_doc_fonts);
	fl_deactivate_object (cpopup->choice_doc_fontsize);
	fl_deactivate_object (cpopup->radio_doc_sides_one);
	fl_deactivate_object (cpopup->radio_doc_sides_two);
	fl_deactivate_object (cpopup->radio_doc_columns_one);
	fl_deactivate_object (cpopup->radio_doc_columns_two);
	fl_deactivate_object (cpopup->input_doc_extra);
	fl_deactivate_object (cpopup->input_doc_skip);
	fl_deactivate_object (cpopup->choice_doc_skip);
	fl_deactivate_object (cpopup->choice_doc_spacing);
	fl_deactivate_object (cpopup->input_doc_spacing);

	fl_deactivate_object (lpopup->choice_language);
	lpopup->combo_language->deactivate();
	fl_deactivate_object (lpopup->choice_inputenc);

	fl_deactivate_object (opopup->slider_secnumdepth);
	fl_deactivate_object (opopup->slider_tocdepth);
	fl_deactivate_object (opopup->check_use_amsmath);
	fl_deactivate_object (opopup->input_float_placement);
	fl_deactivate_object (opopup->choice_postscript_driver);

        fl_deactivate_object (ppopup->choice_paperpackage);
	fl_deactivate_object (ppopup->greoup_radio_orientation);
	fl_deactivate_object (ppopup->radio_portrait);
	fl_deactivate_object (ppopup->radio_landscape);
	fl_deactivate_object (ppopup->choice_papersize2);
	fl_deactivate_object (ppopup->push_use_geometry);
	fl_deactivate_object (ppopup->input_custom_width);
	fl_deactivate_object (ppopup->input_custom_height);
	fl_deactivate_object (ppopup->input_top_margin);
	fl_deactivate_object (ppopup->input_bottom_margin);
	fl_deactivate_object (ppopup->input_left_margin);
	fl_deactivate_object (ppopup->input_right_margin);
	fl_deactivate_object (ppopup->input_head_height);
	fl_deactivate_object (ppopup->input_head_sep);
	fl_deactivate_object (ppopup->input_foot_skip);
}

void FormDocument::SetDocumentClassChoice(vector<string> const &choices)
{
	vector<string>::const_iterator cit = choices.begin();

	fl_clear_choice(cpopup->choice_doc_class);
	for (; cit != choices.end(); ++cit)
		fl_addto_choice(cpopup->choice_doc_class,(*cit).c_str());
}

void FormDocument::build_tform()
{
    FL_OBJECT *obj;
    tpopup = (FD_form_tabbed_document *) fl_calloc(1, sizeof(*tpopup));

    tpopup->form_tabbed_document = fl_bgn_form(FL_NO_BOX, 465, 445);
    tpopup->form_tabbed_document->u_vdata = this;

    obj = fl_add_box(FL_FLAT_BOX,0,0,465,445,"");
    tpopup->tabbed_folder = obj =
	fl_add_tabfolder(FL_TOP_TABFOLDER,20,15,435,360,_("Tabbed folder"));
    obj = fl_add_button(FL_NORMAL_BUTTON,355,405,100,30,idex(_("Cancel|^[")));
    fl_set_button_shortcut(obj,scex(_("Cancel|^[")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormDocumentCancelCB,0);
    tpopup->button_apply = obj =
	fl_add_button(FL_NORMAL_BUTTON,245,405,100,30,idex(_("Apply|#A")));
    fl_set_button_shortcut(obj,scex(_("Apply|#A")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormDocumentApplyCB,0);
    tpopup->button_ok = obj =
	fl_add_button(FL_RETURN_BUTTON,135,405,100,30,_("OK"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormDocumentOKCB,0);
    tpopup->text_warning = obj = fl_add_text(FL_NORMAL_TEXT,20,375,435,30,"");
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_end_form();
    
    tpopup->form_tabbed_document->fdui = tpopup;
}

void FormDocument::build_cform()
{
    FL_OBJECT *obj;
    cpopup = (FD_form_doc_class *) fl_calloc(1, sizeof(*cpopup));

    cpopup->form_doc_class = fl_bgn_form(FL_NO_BOX, 440, 345);
    cpopup->form_doc_class->u_vdata = this;

    obj = fl_add_box(FL_FLAT_BOX,0,0,440,345,"");
    obj = fl_add_labelframe(FL_ENGRAVED_FRAME,300,200,120,80,_("Separation"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    obj = fl_add_labelframe(FL_ENGRAVED_FRAME,300,110,120,70,_("Columns"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    obj = fl_add_labelframe(FL_ENGRAVED_FRAME,300,20,120,70,_("Sides"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    cpopup->choice_doc_fonts = obj =
	fl_add_choice(FL_NORMAL_CHOICE,120,100,160,30,idex(_("Fonts:|#F")));
    fl_set_button_shortcut(obj,scex(_("Fonts:|#F")),1);
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    cpopup->choice_doc_fontsize = obj =
	fl_add_choice(FL_NORMAL_CHOICE,120,140,160,30,idex(_("Font Size:|#O")));
    fl_set_button_shortcut(obj,scex(_("Font Size:|#O")),1);
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    cpopup->choice_doc_class = obj =
	fl_add_choice(FL_NORMAL_CHOICE,120,20,160,30,idex(_("Class:|#C")));
    fl_set_button_shortcut(obj,scex(_("Class:|#C")),1);
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    cpopup->choice_doc_pagestyle = obj =
	fl_add_choice(FL_NORMAL_CHOICE,120,60,160,30,idex(_("Pagestyle:|#P")));
    fl_set_button_shortcut(obj,scex(_("Pagestyle:|#P")),1);
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);

    cpopup->group_doc_sides = fl_bgn_group();
    cpopup->radio_doc_sides_one = obj =
	fl_add_checkbutton(FL_RADIO_BUTTON,300,30,120,30,idex(_("One|#n")));
    fl_set_button_shortcut(obj,scex(_("One|#n")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    cpopup->radio_doc_sides_two = obj =
	fl_add_checkbutton(FL_RADIO_BUTTON,300,60,120,30,idex(_("Two|#T")));
    fl_set_button_shortcut(obj,scex(_("Two|#T")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_end_group();

    cpopup->group_doc_columns = fl_bgn_group();
    cpopup->radio_doc_columns_one = obj =
	fl_add_checkbutton(FL_RADIO_BUTTON,300,120,110,30,idex(_("One|#e")));
    fl_set_button_shortcut(obj,scex(_("One|#e")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    cpopup->radio_doc_columns_two = obj =
	fl_add_checkbutton(FL_RADIO_BUTTON,300,150,110,30,idex(_("Two|#w")));
    fl_set_button_shortcut(obj,scex(_("Two|#w")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_end_group();
    
    
    cpopup->group_doc_sep = fl_bgn_group();
    cpopup->radio_doc_indent = obj =
	fl_add_checkbutton(FL_RADIO_BUTTON,300,210,110,30,idex(_("Indent|#I")));
    fl_set_button_shortcut(obj,scex(_("Indent|#I")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    cpopup->radio_doc_skip = obj =
	fl_add_checkbutton(FL_RADIO_BUTTON,300,240,110,30,idex(_("Skip|#K")));
    fl_set_button_shortcut(obj,scex(_("Skip|#K")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_button(obj, 1);
    fl_end_group();
    
    cpopup->input_doc_extra = obj =
	fl_add_input(FL_NORMAL_INPUT,120,185,160,30,idex(_("Extra Options:|#X")));
    fl_set_button_shortcut(obj,scex(_("Extra Options:|#X")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    cpopup->input_doc_skip = obj =
	fl_add_input(FL_NORMAL_INPUT,220,225,60,30,"");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,DocumentInputCB,0);
    cpopup->choice_doc_skip = obj =
	fl_add_choice(FL_NORMAL_CHOICE,120,225,90,30,idex(_("Default Skip:|#u")));
    fl_set_button_shortcut(obj,scex(_("Default Skip:|#u")),1);
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,DocumentInputCB,0);
    cpopup->choice_doc_spacing = obj =
	fl_add_choice(FL_NORMAL_CHOICE,120,290,160,30,idex(_("Spacing|#g")));
    fl_set_button_shortcut(obj,scex(_("Spacing|#g")),1);
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,DocumentInputCB,0);
    cpopup->input_doc_spacing = obj =
	fl_add_input(FL_NORMAL_INPUT,300,290,120,30,"");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,DocumentInputCB,0);
    fl_end_form();
    
    cpopup->form_doc_class->fdui = cpopup;
}

void FormDocument::build_pform()
{
    FL_OBJECT *obj;
    ppopup = (FD_form_doc_paper *) fl_calloc(1, sizeof(*ppopup));
    
    ppopup->form_doc_paper = fl_bgn_form(FL_NO_BOX, 440, 345);
    ppopup->form_doc_paper->u_vdata = this;

    obj = fl_add_box(FL_FLAT_BOX,0,0,440,345,"");
    obj = fl_add_frame(FL_ENGRAVED_FRAME,230,85,200,100,"");
    fl_set_object_color(obj,FL_COL1,FL_COL1);
    fl_set_object_lalign(obj,FL_ALIGN_TOP_LEFT);
    ppopup->choice_paperpackage = obj =
	fl_add_choice(FL_NORMAL_CHOICE,290,15,140,30,idex(_("Special:|#S")));
    fl_set_button_shortcut(obj,scex(_("Special:|#S")),1);
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,DocumentInputCB,0);
    obj = fl_add_frame(FL_ENGRAVED_FRAME,230,205,200,130,"");
    fl_set_object_color(obj,FL_COL1,FL_COL1);
    fl_set_object_lalign(obj,FL_ALIGN_TOP_LEFT);
    obj = fl_add_frame(FL_ENGRAVED_FRAME,10,165,190,170,"");
    fl_set_object_color(obj,FL_COL1,FL_COL1);
    fl_set_object_lalign(obj,FL_ALIGN_TOP_LEFT);
    obj = fl_add_text(FL_NORMAL_TEXT,20,155,70,20,_("Margins"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    obj = fl_add_text(FL_NORMAL_TEXT,240,195,160,20,_("Foot/Head Margins"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    obj = fl_add_frame(FL_ENGRAVED_FRAME,440,360,10,10,"");
    fl_set_object_color(obj,FL_COL1,FL_COL1);
    obj = fl_add_frame(FL_ENGRAVED_FRAME,440,350,10,10,"");
    fl_set_object_color(obj,FL_COL1,FL_COL1);
    obj = fl_add_frame(FL_ENGRAVED_FRAME,10,85,190,70,"");
    fl_set_object_color(obj,FL_COL1,FL_COL1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    obj = fl_add_text(FL_NORMAL_TEXT,20,75,95,20,_("Orientation"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);

    ppopup->greoup_radio_orientation = fl_bgn_group();
    ppopup->radio_portrait = obj =
	fl_add_checkbutton(FL_RADIO_BUTTON,20,90,120,30,idex(_("Portrait|#o")));
    fl_set_button_shortcut(obj,scex(_("Portrait|#o")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    ppopup->radio_landscape = obj =
	fl_add_checkbutton(FL_RADIO_BUTTON,20,120,120,30,idex(_("Landscape|#L")));
    fl_set_button_shortcut(obj,scex(_("Landscape|#L")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_end_group();

    ppopup->choice_papersize2 = obj =
	fl_add_choice(FL_NORMAL_CHOICE,70,15,160,30,idex(_("Papersize:|#P")));
    fl_set_button_shortcut(obj,scex(_("Papersize:|#P")),1);
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,DocumentInputCB,0);
    obj = fl_add_text(FL_NORMAL_TEXT,240,75,150,20,_("Custom Papersize"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    ppopup->push_use_geometry = obj =
	fl_add_checkbutton(FL_PUSH_BUTTON,20,45,170,30,idex(_("Use Geometry Package|#U")));
    fl_set_button_shortcut(obj,scex(_("Use Geometry Package|#U")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    ppopup->input_custom_width = obj =
	fl_add_input(FL_NORMAL_INPUT,330,100,90,30,idex(_("Width:|#W")));
    fl_set_button_shortcut(obj,scex(_("Width:|#W")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,DocumentInputCB,0);
    ppopup->input_custom_height = obj =
	fl_add_input(FL_NORMAL_INPUT,330,140,90,30,idex(_("Height:|#H")));
    fl_set_button_shortcut(obj,scex(_("Height:|#H")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,DocumentInputCB,0);
    ppopup->input_top_margin = obj =
	fl_add_input(FL_NORMAL_INPUT,100,175,90,30,idex(_("Top:|#T")));
    fl_set_button_shortcut(obj,scex(_("Top:|#T")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,DocumentInputCB,0);
    ppopup->input_bottom_margin = obj =
	fl_add_input(FL_NORMAL_INPUT,100,215,90,30,idex(_("Bottom:|#B")));
    fl_set_button_shortcut(obj,scex(_("Bottom:|#B")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,DocumentInputCB,0);
    ppopup->input_left_margin = obj =
	fl_add_input(FL_NORMAL_INPUT,100,255,90,30,idex(_("Left:|#e")));
    fl_set_button_shortcut(obj,scex(_("Left:|#e")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,DocumentInputCB,0);
    ppopup->input_right_margin = obj =
	fl_add_input(FL_NORMAL_INPUT,100,295,90,30,idex(_("Right:|#R")));
    fl_set_button_shortcut(obj,scex(_("Right:|#R")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,DocumentInputCB,0);
    ppopup->input_head_height = obj =
	fl_add_input(FL_NORMAL_INPUT,330,215,90,30,idex(_("Headheight:|#i")));
    fl_set_button_shortcut(obj,scex(_("Headheight:|#i")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,DocumentInputCB,0);
    ppopup->input_head_sep = obj =
	fl_add_input(FL_NORMAL_INPUT,330,255,90,30,idex(_("Headsep:|#d")));
    fl_set_button_shortcut(obj,scex(_("Headsep:|#d")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,DocumentInputCB,0);
    ppopup->input_foot_skip = obj =
	fl_add_input(FL_NORMAL_INPUT,330,295,90,30,idex(_("Footskip:|#F")));
    fl_set_button_shortcut(obj,scex(_("Footskip:|#F")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,DocumentInputCB,0);
    ppopup->text_warning = obj = fl_add_text(FL_NORMAL_TEXT,10,360,420,20,"");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_end_form();

    ppopup->form_doc_paper->fdui = ppopup;
}

void FormDocument::build_lform()
{
    FL_OBJECT *obj;
    lpopup = (FD_form_doc_lang *) fl_calloc(1, sizeof(*lpopup));

    lpopup->form_doc_lang = fl_bgn_form(FL_NO_BOX, 440, 345);
    lpopup->form_doc_lang->u_vdata = this;

    obj = fl_add_box(FL_FLAT_BOX,0,0,440,345,"");
    obj = fl_add_labelframe(FL_ENGRAVED_FRAME,50,160,290,110,
			    _("Quote Style    "));
    lpopup->choice_accents = obj =
	fl_add_checkbutton(FL_PUSH_BUTTON,90,120,310,30,
			   idex(_("Allow accents on ALL characters|#w")));
    fl_set_button_shortcut(obj,scex(_("Allow accents on ALL characters|#w")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    lpopup->choice_language = obj =
	fl_add_box(FL_FRAME_BOX,120,30,190,30,_("Language:"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT);
    lpopup->choice_inputenc = obj =
	fl_add_choice(FL_NORMAL_CHOICE,120,70,190,30,idex(_("Encoding:|#D")));
    fl_set_button_shortcut(obj,scex(_("Encoding:|#D")),1);
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    lpopup->choice_quotes_language = obj =
	fl_add_choice(FL_NORMAL_CHOICE,120,180,190,30,idex(_("Type:|#T")));
    fl_set_button_shortcut(obj,scex(_("Type:|#T")),1);
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_bgn_group();
    lpopup->radio_single = obj =
	fl_add_checkbutton(FL_RADIO_BUTTON,120,220,90,30,idex(_("Single|#S")));
    fl_set_button_shortcut(obj,scex(_("Single|#S")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    lpopup->radio_double = obj =
	fl_add_checkbutton(FL_RADIO_BUTTON,220,220,90,30,idex(_("Double|#D")));
    fl_set_button_shortcut(obj,scex(_("Double|#D")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_end_group();

    // this is added by hand !!!
    obj = lpopup->choice_language;
    lpopup->combo_language = new Combox(Combox::FL_COMBOX_DROPLIST);
    lpopup->combo_language->add(obj->x, obj->y, obj->w, obj->h, 200);
    lpopup->combo_language->shortcut("#G",1);
    fl_end_form();

    lpopup->form_doc_lang->fdui = lpopup;
}

void FormDocument::build_oform()
{
    FL_OBJECT *obj;
    opopup = (FD_form_doc_options *) fl_calloc(1, sizeof(*opopup));

    opopup->form_doc_options = fl_bgn_form(FL_NO_BOX, 440, 345);
    opopup->form_doc_options->u_vdata = this;

    obj = fl_add_box(FL_FLAT_BOX,0,0,440,345,"");
    opopup->input_float_placement = obj = fl_add_input(FL_NORMAL_INPUT,155,60,120,30,idex(_("Float Placement:|#L")));fl_set_button_shortcut(obj,scex(_("Float Placement:|#L")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    opopup->slider_secnumdepth = obj = fl_add_counter(FL_SIMPLE_COUNTER,155,110,80,30,_("Section number depth"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT);
    opopup->slider_tocdepth = obj = fl_add_counter(FL_SIMPLE_COUNTER,155,150,80,30,_("Table of contents depth"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT);
    opopup->choice_postscript_driver = obj = fl_add_choice(FL_NORMAL_CHOICE,155,205,190,30,idex(_("PS Driver:|#S")));fl_set_button_shortcut(obj,scex(_("PS Driver:|#S")),1);
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    opopup->check_use_amsmath = obj = fl_add_checkbutton(FL_PUSH_BUTTON,140,250,200,30,idex(_("Use AMS Math|#M")));fl_set_button_shortcut(obj,scex(_("Use AMS Math|#M")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_end_form();

    opopup->form_doc_options->fdui = opopup;
}

void FormDocument::build_bform()
{
    FL_OBJECT *obj;
    bpopup = (FD_form_doc_bullet *) fl_calloc(1, sizeof(*bpopup));

    bpopup->form_doc_bullet = fl_bgn_form(FL_NO_BOX, 440, 345);
    bpopup->form_doc_bullet->u_vdata = this;

    // added by hand !!!
    fl_set_border_width(-1);
    
    obj = fl_add_box(FL_FLAT_BOX,0,0,440,345,"");
    // make the border thick again to improve the bmtable looks
    fl_set_border_width(-3);
    bpopup->bmtable_bullet_panel = obj = fl_add_bmtable(1,90,105,265,180,"");
    fl_set_object_callback(obj, BulletBMTableCB, 0);
    fl_set_object_lcol(obj, FL_BLUE);
    fl_set_object_boxtype(obj, FL_UP_BOX);
    fl_set_bmtable_pixmap_file(obj, 6, 6,
			       LibFileSearch("images","standard.xpm").c_str());
    fl_set_border_width(-1);
    // end changed by hand
    obj = fl_add_frame(FL_ENGRAVED_FRAME,95,20,255,70,"");
    bpopup->choice_bullet_size = obj =
	fl_add_choice(FL_NORMAL_CHOICE,15,45,65,30,idex(_("Size|#z")));
    fl_set_button_shortcut(obj,scex(_("Size|#z")),1);
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_TOP);
    fl_set_object_callback(obj,ChoiceBulletSizeCB,0);
    bpopup->input_bullet_latex = obj =
	fl_add_input(FL_NORMAL_INPUT,80,300,275,30,idex(_("LaTeX|#L")));
    fl_set_button_shortcut(obj,scex(_("LaTeX|#L")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_FIXED_STYLE);
    fl_set_object_callback(obj,InputBulletLaTeXCB,0);
    bpopup->radio_bullet_depth = fl_bgn_group();
    bpopup->radio_bullet_depth_1 = obj =
	fl_add_button(FL_RADIO_BUTTON,105,35,55,40,idex(_("1|#1")));
    fl_set_button_shortcut(obj,scex(_("1|#1")),1);
    fl_set_object_lsize(obj,FL_LARGE_SIZE);
    fl_set_object_callback(obj,BulletDepthCB,0);
    fl_set_button(obj, 1);
    bpopup->radio_bullet_depth_2 = obj =
	fl_add_button(FL_RADIO_BUTTON,165,35,55,40,idex(_("2|#2")));
    fl_set_button_shortcut(obj,scex(_("2|#2")),1);
    fl_set_object_lsize(obj,FL_LARGE_SIZE);
    fl_set_object_callback(obj,BulletDepthCB,1);
    bpopup->radio_bullet_depth_3 = obj =
	fl_add_button(FL_RADIO_BUTTON,225,35,55,40,idex(_("3|#3")));
    fl_set_button_shortcut(obj,scex(_("3|#3")),1);
    fl_set_object_lsize(obj,FL_LARGE_SIZE);
    fl_set_object_callback(obj,BulletDepthCB,2);
    bpopup->radio_bullet_depth_4 = obj =
	fl_add_button(FL_RADIO_BUTTON,285,35,55,40,idex(_("4|#4")));
    fl_set_button_shortcut(obj,scex(_("4|#4")),1);
    fl_set_object_lsize(obj,FL_LARGE_SIZE);
    fl_set_object_callback(obj,BulletDepthCB,3);
    fl_end_group();

    obj = fl_add_text(FL_NORMAL_TEXT,105,10,85,20,_("Bullet Depth"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);

    bpopup->radio_bullet_panel = fl_bgn_group();
    bpopup->radio_bullet_panel_standard = obj =
	fl_add_button(FL_RADIO_BUTTON,15,105,65,30,idex(_("Standard|#S")));
    fl_set_button_shortcut(obj,scex(_("Standard|#S")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_resize(obj, FL_RESIZE_NONE);
    fl_set_object_callback(obj,BulletPanelCB,0);
    fl_set_button(obj, 1);
    bpopup->radio_bullet_panel_maths = obj =
	fl_add_button(FL_RADIO_BUTTON,15,135,65,30,idex(_("Maths|#M")));
    fl_set_button_shortcut(obj,scex(_("Maths|#M")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_resize(obj, FL_RESIZE_NONE);
    fl_set_object_callback(obj,BulletPanelCB,1);
    bpopup->radio_bullet_panel_ding2 = obj =
	fl_add_button(FL_RADIO_BUTTON,15,195,65,30,idex(_("Ding 2|#i")));
    fl_set_button_shortcut(obj,scex(_("Ding 2|#i")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_resize(obj, FL_RESIZE_NONE);
    fl_set_object_callback(obj,BulletPanelCB,3);
    bpopup->radio_bullet_panel_ding3 = obj =
	fl_add_button(FL_RADIO_BUTTON,15,225,65,30,idex(_("Ding 3|#n")));
    fl_set_button_shortcut(obj,scex(_("Ding 3|#n")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_resize(obj, FL_RESIZE_NONE);
    fl_set_object_callback(obj,BulletPanelCB,4);
    bpopup->radio_bullet_panel_ding4 = obj =
	fl_add_button(FL_RADIO_BUTTON,15,255,65,30,idex(_("Ding 4|#g")));
    fl_set_button_shortcut(obj,scex(_("Ding 4|#g")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_resize(obj, FL_RESIZE_NONE);
    fl_set_object_callback(obj,BulletPanelCB,5);
    bpopup->radio_bullet_panel_ding1 = obj =
	fl_add_button(FL_RADIO_BUTTON,15,165,65,30,idex(_("Ding 1|#D")));
    fl_set_button_shortcut(obj,scex(_("Ding 1|#D")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_resize(obj, FL_RESIZE_NONE);
    fl_set_object_callback(obj,BulletPanelCB,2);
    fl_end_group();


    fl_end_form();

    bpopup->form_doc_bullet->fdui = bpopup;
}
