// -*- C++ -*-
/* form_document.h
 * FormDocument Interface Class
 */

#ifndef FORM_DOCUMENT_H
#define FORM_DOCUMENT_H

#include <vector>
#include "gui_common.h"
#include "lsignal.h"

class Communicator;
class Popups;
class Combox;

/** This class provides an XForms implementation of the FormDocument Popup.
    The table-layout-form here changes values for latex-tabulars
 */
class FormDocument : public virtual SignalBase {
public:
    /**@name Constructors and Destructors */
    //@{
    /// #FormDocument x(Communicator ..., Popups ...);#
    FormDocument(Communicator *, Popups *);
    ///
    ~FormDocument();
    void SetDocumentClassChoice(vector<string> const &choices);
    //@}
private:
    FormDocument() {}
    FormDocument(FormDocument&) {}
    void DeactivateDocumentButtons();
    void ActivateDocumentButtons();
    void EnableDocumentLayout();
    void DisableDocumentLayout();
    void CheckDocumentInput(FL_OBJECT *ob, long);
    void ChoiceBulletSize(FL_OBJECT *ob, long);
    void InputBulletLaTeX(FL_OBJECT *ob, long);
    void BulletDepth(FL_OBJECT *ob, long);
    void BulletPanel(FL_OBJECT *ob, long);
    void BulletBMTable(FL_OBJECT *ob, long);
    void checkMarginValues();

    /**@name Real per-instance Callback Methods */
    //@{
    static int FormDocumentWMHideCB(FL_FORM *, void *);
    static void FormDocumentCancelCB(FL_OBJECT *, long);
    static void FormDocumentApplyCB(FL_OBJECT *, long);
    static void FormDocumentOKCB(FL_OBJECT *, long);
    static void DocumentInputCB(FL_OBJECT *ob, long);
    static void ChoiceBulletSizeCB(FL_OBJECT *ob, long);
    static void InputBulletLaTeXCB(FL_OBJECT *ob, long);
    static void BulletDepthCB(FL_OBJECT *ob, long);
    static void BulletPanelCB(FL_OBJECT *ob, long);
    static void BulletBMTableCB(FL_OBJECT *ob, long);
    //@}
	
    /**@name Slot Methods */
    //@{
    /// Create the popup if necessary, update it and display it.
    void show();
    /// Hide the popup.
    void hide();
    /// Update the popup.
    void update();
    void pupdate(); // paper
    void cupdate(); // class
    void lupdate(); // language
    void oupdate(); // options
    void bupdate(); // bullets
    /// Apply from popup
    void apply();
    void papply(); // paper
    bool capply(); // class
    void lapply(); // language
    bool oapply(); // options
    void bapply(); // bullets
    /// Cancel from popup
    void cancel();
    //@}
    /// Build the popup
    void build();
    void build_tform(); // tabbed folder
    void build_pform(); // paper
    void build_cform(); // class
    void build_lform(); // language
    void build_oform(); // options
    void build_bform(); // bullets
    ///
    /// Explicitly free the popup.
    void free();

    /// Typedefinitions from the fdesign produced Header file
    typedef struct {
	FL_FORM *form_tabbed_document;
	void *vdata;
	char *cdata;
	long  ldata;
	FL_OBJECT *tabbed_folder;
	FL_OBJECT *button_apply;
	FL_OBJECT *button_ok;
	FL_OBJECT *text_warning;
    } FD_form_tabbed_document;

    typedef struct {
	FL_FORM *form_doc_paper;
	void *vdata;
	char *cdata;
	long  ldata;
	FL_OBJECT *choice_paperpackage;
	FL_OBJECT *greoup_radio_orientation;
	FL_OBJECT *radio_portrait;
	FL_OBJECT *radio_landscape;
	FL_OBJECT *choice_papersize2;
	FL_OBJECT *push_use_geometry;
	FL_OBJECT *input_custom_width;
	FL_OBJECT *input_custom_height;
	FL_OBJECT *input_top_margin;
	FL_OBJECT *input_bottom_margin;
	FL_OBJECT *input_left_margin;
	FL_OBJECT *input_right_margin;
	FL_OBJECT *input_head_height;
	FL_OBJECT *input_head_sep;
	FL_OBJECT *input_foot_skip;
	FL_OBJECT *text_warning;
    } FD_form_doc_paper;

    typedef struct {
	FL_FORM *form_doc_class;
	void *vdata;
	char *cdata;
	long  ldata;
	FL_OBJECT *choice_doc_fonts;
	FL_OBJECT *choice_doc_fontsize;
	FL_OBJECT *choice_doc_class;
	FL_OBJECT *choice_doc_pagestyle;
	FL_OBJECT *choice_doc_spacing;
	FL_OBJECT *input_doc_extra;
	FL_OBJECT *input_doc_skip;
	FL_OBJECT *choice_doc_skip;
	FL_OBJECT *group_doc_sides;
	FL_OBJECT *radio_doc_sides_one;
	FL_OBJECT *radio_doc_sides_two;
	FL_OBJECT *group_doc_columns;
	FL_OBJECT *radio_doc_columns_one;
	FL_OBJECT *radio_doc_columns_two;
	FL_OBJECT *group_doc_sep;
	FL_OBJECT *radio_doc_indent;
	FL_OBJECT *radio_doc_skip;
	FL_OBJECT *input_doc_spacing;
    } FD_form_doc_class;

    typedef struct {
	FL_FORM *form_doc_lang;
	void *vdata;
	char *cdata;
	long  ldata;
	FL_OBJECT *choice_accents;
	FL_OBJECT *choice_language;
	FL_OBJECT *choice_inputenc;
	FL_OBJECT *choice_quotes_language;
	FL_OBJECT *radio_single;
	FL_OBJECT *radio_double;
	// added by hand !!!
	Combox *combo_language;
    } FD_form_doc_lang;

    typedef struct {
	FL_FORM *form_doc_options;
	void *vdata;
	char *cdata;
	long  ldata;
	FL_OBJECT *input_float_placement;
	FL_OBJECT *slider_secnumdepth;
	FL_OBJECT *slider_tocdepth;
	FL_OBJECT *choice_postscript_driver;
	FL_OBJECT *check_use_amsmath;
    } FD_form_doc_options;

    typedef struct {
	FL_FORM *form_doc_bullet;
	void *vdata;
	char *cdata;
	long  ldata;
	FL_OBJECT *bmtable_bullet_panel;
	FL_OBJECT *choice_bullet_size;
	FL_OBJECT *input_bullet_latex;
	FL_OBJECT *radio_bullet_depth;
	FL_OBJECT *radio_bullet_depth_1;
	FL_OBJECT *radio_bullet_depth_2;
	FL_OBJECT *radio_bullet_depth_3;
	FL_OBJECT *radio_bullet_depth_4;
	FL_OBJECT *radio_bullet_panel;
	FL_OBJECT *radio_bullet_panel_standard;
	FL_OBJECT *radio_bullet_panel_maths;
	FL_OBJECT *radio_bullet_panel_ding2;
	FL_OBJECT *radio_bullet_panel_ding3;
	FL_OBJECT *radio_bullet_panel_ding4;
	FL_OBJECT *radio_bullet_panel_ding1;
    } FD_form_doc_bullet;

    /**@name Private Data */
    //@{
    /// Real GUI implementation.
    FD_form_tabbed_document *tpopup; // tabbed folder
    FD_form_doc_paper       *ppopup; // paper
    FD_form_doc_class       *cpopup; // class
    FD_form_doc_lang        *lpopup; // language
    FD_form_doc_options     *opopup; // options
    FD_form_doc_bullet      *bpopup; // bullets
    /// Which LyXView do we belong to?
    Communicator *comm;
    /// Update connection.
    Connection u;
    /// Hide connection.
    Connection h;
    /// has form contents changed? Used to control OK/Apply
    EnumPopupStatus status;
    //@}
    int
        ActCell, Confirmed,
	current_bullet_panel, current_bullet_depth;
    ///
    FL_OBJECT *fbullet;
};

#endif
