/* form_include.C
 * FormInclude Interface Class Implementation
 */

#include <config.h>
#include "lyx_gui_misc.h"
#include "gettext.h"
#include FORMS_H_LOCATION

#include "form_include.h"
#include "Popups.h"
#include "Communicator.h"
// temporary until filedlg is moved into
// gui independence
#include "../../filedlg.h" 
#include "filetools.h"
#include "insetinclude.h"
#include "commandtags.h"
#include "lstrings.h"

// 	$Id: form_include.C,v 1.2 1998/10/21 23:47:23 larsbj Exp $

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: form_include.C,v 1.2 1998/10/21 23:47:23 larsbj Exp $";
#endif /* lint */


#ifdef __GNUG__
#pragma implementation
#endif


FormInclude::FormInclude(Communicator *c, Popups * p)
	: popup(0), comm(c), d(0), u(0), h(0), status(POPUP_UNMODIFIED)
{
	// let the popup be shown
	// This is a permanent connection so we won't bother
	// storing a copy because we won't be disconnecting.
	connect_to_method(p->showFormInclude,
			  this,
			  &FormInclude::show);
	// NOTE:
	// parameter p is used instead of:
	//      comm->getPopups()
	// because we are being contructed from within the
	// Popups constructor and a segfault would occur
	// otherwise.  The segfault occurs because the Popups
	// constructor hasn't returned at this point and so
	// comm->getPopups() will not return the real popups
	// instance.  If this implementation fails see Popups.C
	// ARRae.
}


FormInclude::~FormInclude()
{
	free();
}


void FormInclude::build()
{
	FL_OBJECT *obj;
	FD_form_include *fdui = popup = new FD_form_include;

	fdui->form_include = fl_bgn_form(FL_NO_BOX, 340, 210);
	fdui->form_include->u_vdata = this;

	obj = fl_add_box(FL_UP_BOX,0,0,340,210,"");
	obj = fl_add_frame(FL_ENGRAVED_FRAME,10,70,160,90,"");
	fdui->browsebt = obj = fl_add_button(FL_NORMAL_BUTTON,230,30,100,30,idex(_("Browse|#B")));
	fl_set_button_shortcut(obj,scex(_("Browse|#B")),1);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_callback(obj,FormInclude::IncludeCB,0);
	fdui->flag1 = obj = fl_add_checkbutton(FL_PUSH_BUTTON,180,70,150,30,idex(_("Don't typeset|#D")));
	fl_set_button_shortcut(obj,scex(_("Don't typeset|#D")),1);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fdui->button_ok = obj = fl_add_button(FL_RETURN_BUTTON,120,170,100,30,_("OK"));
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_callback(obj,FormInclude::IncludeCB,1);
	obj = fl_add_button(FL_NORMAL_BUTTON,230,170,100,30,idex(_("Cancel|^[")));
	fl_set_button_shortcut(obj,scex(_("Cancel|^[")),1);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_callback(obj,FormInclude::IncludeCB,2);
	obj = fl_add_button(FL_NORMAL_BUTTON,230,130,100,30,idex(_("Load|#L")));
	fl_set_button_shortcut(obj,scex(_("Load|#L")),1);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_callback(obj,FormInclude::IncludeCB,5);
	fdui->input = obj = fl_add_input(FL_NORMAL_INPUT,10,30,210,30,idex(_("File name:|#F")));
	fl_set_button_shortcut(obj,scex(_("File name:|#F")),1);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_lalign(obj,FL_ALIGN_TOP_LEFT);
	fl_set_object_callback(obj,FormInclude::InputCB,0);
	fdui->flag41 = obj = fl_add_checkbutton(FL_PUSH_BUTTON,180,100,150,30,idex(_("Visible space|#s")));
	fl_set_button_shortcut(obj,scex(_("Visible space|#s")),1);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);

	fdui->include_grp = fl_bgn_group();
	fdui->flag4 = obj = fl_add_checkbutton(FL_RADIO_BUTTON,10,130,160,30,idex(_("Verbatim|#V")));
	fl_set_button_shortcut(obj,scex(_("Verbatim|#V")),1);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_callback(obj,FormInclude::IncludeCB,10);
	fdui->flag2 = obj = fl_add_checkbutton(FL_RADIO_BUTTON,10,100,160,30,idex(_("Use input|#i")));
	fl_set_button_shortcut(obj,scex(_("Use input|#i")),1);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_callback(obj,FormInclude::IncludeCB,11);
	fdui->flag3 = obj = fl_add_checkbutton(FL_RADIO_BUTTON,10,70,160,30,idex(_("Use include|#U")));
	fl_set_button_shortcut(obj,scex(_("Use include|#U")),1);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_callback(obj,FormInclude::IncludeCB,11);
	fl_end_group();

	fl_end_form();

	fdui->form_include->fdui = fdui;
}


void FormInclude::show(InsetInclude * inset)
{
	if (!inset) {
		return;
	}

	if (!popup) {
		build();
		fl_set_input_return(popup->input,
				    FL_RETURN_CHANGED);
		fl_set_form_atclose(popup->form_include,
				    FormInclude::IncludeWMHideCB, 0);
	}

	if (comm->isReadOnly()) {
		// deactivating an input doesn't seem to work
		// at least not up to 0.88, so we remember instead.
		status = POPUP_READONLY;
		fl_deactivate_object(popup->input);
	} else {
		status = POPUP_UNMODIFIED;
		fl_activate_object(popup->input);
	}
	fl_deactivate_object(popup->button_ok);
	fl_set_object_lcol(popup->button_ok, FL_INACTIVE);

        popup->vdata = inset;
	// if the popup is visible and the inset its attached to is deleted
	// the inset will activate this signal warning us of the situation.
	// Since the popup doesn't make sense without an inset we free() it.
	d = connect_to_method(inset->gone,
			      this,
			      &FormInclude::free);

        fl_set_input(popup->input, inset->getContents().c_str());
	fl_set_button(popup->flag1, int(inset->isNoLoad()));
	fl_set_button(popup->flag2, int(inset->isInput()));
	fl_set_button(popup->flag3, int(inset->isInclude()));
	fl_set_button(popup->flag4, int(inset->isVerb()));
        if (inset->isVerb()) {
            fl_set_button(popup->flag41, int(inset->isVerbVisibleSpace()));
        } else {
	    fl_set_button(popup->flag41, 0);
            fl_deactivate_object(popup->flag41);
	    fl_set_object_lcol(popup->flag41, FL_INACTIVE);
	}

	if (popup->form_include->visible) {
		fl_raise_form(popup->form_include);
	} else {
		fl_show_form(popup->form_include,
			     FL_PLACE_MOUSE,
			     FL_FULLBORDER,
			     _("Include"));
		h = connect_to_method(
				comm->getPopups().hideBufferDependent,
				this,
				&FormInclude::hide);
		// if we change buffers the popup is no longer valid
		// so we'll just hide it when we get an update signal
		u = connect_to_method(
				comm->getPopups().updateBufferDependent,
				this,
				&FormInclude::hide);
	}
}


void FormInclude::hide()
{
	if (popup
	    && popup->form_include
	    && popup->form_include->visible) {
		fl_hide_form(popup->form_include);
		d.disconnect();
		u.disconnect();
		h.disconnect();
		popup->vdata = 0;
	}
	free();
}


void FormInclude::include(long arg)
{
#ifdef WITH_WARNINGS
#warning still cleaning up to do here
	// Can a .lyx file be input? I think not yet we allow it
	// Can a .tex file be included?  Isn't this only for .lyx?
	// Ensure correct  use of the status variable for readonly.  ARRae
#endif
	InsetInclude *inset = (InsetInclude *) popup->vdata;
	
	switch (arg) {
	case 0:
	{
		// Should browsing too be disabled in RO-mode?
		if (status != POPUP_READONLY) {
			LyXFileDlg fileDlg;
			string mpath = OnlyPath(inset->getMasterFilename());
			string ext;
    
			if (fl_get_button(popup->flag2)) {
				// Use Input Button
				ext = "*.tex";
			} else if (fl_get_button(popup->flag4)) {
				// Verbatim all files
				ext = "*";
			} else {
				ext = "*.lyx";
			}
			
			// launches dialog
			fileDlg.SetButton(0,
					  _("Documents"),
					  comm->getDefaultDocumentPath().c_str());
    
			// Use by default the master's path
			string filename = 
				fileDlg.Select(_("Select Child Document"),
					       mpath, ext, 
					       inset->getContents());
 
			// check selected filename
			if (!filename.empty()) {
				string filename2 = MakeRelPath(filename,
								mpath);
				if (prefixIs(filename2, "..")) {
					fl_set_input(popup->input,
						     filename.c_str());
				} else {
					fl_set_input(popup->input,
						     filename2.c_str());
				}
				status = POPUP_MODIFIED;
			}
		}
		break;
	}

	case 1:
		if(status != POPUP_READONLY) {
			inset->setContents(fl_get_input(popup->input));
			// don't typeset
			inset->setNoLoad(fl_get_button(popup->flag1));
			if (fl_get_button(popup->flag2)) {
				inset->setInput();
			} else if (fl_get_button(popup->flag3)) {
				inset->setInclude();
			} else if (fl_get_button(popup->flag4)) {
				inset->setVerb();
				inset->setVisibleSpace(
					fl_get_button(popup->flag41));
			}

			hide();
			comm->updateInset(inset, true);
			break;
		} // fall through

	case 2:
		hide();
		break;

	case 5:
		if(status != POPUP_READONLY) {
			inset->setContents(fl_get_input(popup->input));
			inset->setNoLoad(fl_get_button(popup->flag1));
			if (fl_get_button(popup->flag2)) {
				inset->setInput();
			} else if (fl_get_button(popup->flag3)) {
				inset->setInclude();
			} else if (fl_get_button(popup->flag4)) {
				inset->setVerb();
				inset->setVisibleSpace(
					fl_get_button(popup->flag41));
			}

			hide();
			comm->updateInset(inset, true);
			comm->Dispatch(LFUN_CHILDOPEN,
				       inset->getContents().c_str());
                }
                break;
		
        case 10:
		if (status != POPUP_READONLY) {
			fl_activate_object(popup->flag41);
			fl_set_object_lcol(popup->flag41, FL_BLACK);
		}
                break;

        case 11:
		if (status != POPUP_READONLY) {
			fl_deactivate_object(popup->flag41);
			fl_set_object_lcol(popup->flag41, FL_INACTIVE);
			fl_set_button(popup->flag41, 0);
		}
                break;
	}
}


void FormInclude::free()
{
	// we don't need to delete u and h here because
	// hide() does that after disconnecting.
	if (popup) {
		if (popup->form_include
		    && popup->form_include->visible) {
			hide();
		}
		fl_free_form(popup->form_include);
		delete popup;
		popup = 0;
	}
}


int FormInclude::IncludeWMHideCB(FL_FORM * form, void *)
{
	// Ensure that the signals (u and h) are disconnected even if the
	// window manager is used to close the popup.
	FormInclude *pre = (FormInclude*)form->u_vdata;
	pre->hide();
	return FL_CANCEL;
}


void FormInclude::IncludeCB(FL_OBJECT *ob, long data)
{
	FormInclude *pre = (FormInclude*)ob->form->u_vdata;
	pre->include(data);
}


void FormInclude::InputCB(FL_OBJECT *ob, long)
{
	FormInclude *pre = (FormInclude*)ob->form->u_vdata;
	if (pre->status == POPUP_UNMODIFIED) {
		pre->status = POPUP_MODIFIED;
		fl_activate_object(pre->popup->button_ok);
		fl_set_object_lcol(pre->popup->button_ok, FL_BLACK);
	}
}
