// -*- C++ -*-
/* forminclude.h
 * Forminclude Interface Class
 */

#ifndef FORMINCLUDE_H
#define FORMINCLUDE_H

#include "lsignal.h"
#include "gui_common.h"

class Communicator;
class Popups;
class InsetInclude;


/** This class provides an XForms implementation of the Forminclude Popup.
    The LaTeX include popup allows user access to a buffers include.
 */
class FormInclude : public virtual SignalBase {
public:
	/**@name Constructors and Destructors */
	//@{
	/// #Forminclude x(Communicator ..., Popups ...);#
	FormInclude(Communicator *, Popups *);
	///
	~FormInclude();
	//@}
private:
	FormInclude() {}
	FormInclude(FormInclude&) {}

	/**@name Real per-instance Callback Methods */
	//@{
	static int IncludeWMHideCB(FL_FORM *, void *);
	static void IncludeCB(FL_OBJECT *, long);
	static void InputCB(FL_OBJECT *, long);
	//@}
	
	/**@name Slot Methods */
	//@{
	/// Create the popup if necessary, update it and display it.
	void show(InsetInclude *);
	/// Hide the popup.
	void hide();
	/// Real popup handler
	void include(long);
	//@}

	
	/// Build the popup
	void build();
	/// Explicitly free the popup.
	void free();

	typedef struct {
		FL_FORM *form_include;
		void *vdata;
		char *cdata;
		long  ldata;
		FL_OBJECT *browsebt;
		FL_OBJECT *flag1;
		FL_OBJECT *button_ok;
		FL_OBJECT *input;
		FL_OBJECT *flag41;
		FL_OBJECT *include_grp;
		FL_OBJECT *flag4;
		FL_OBJECT *flag2;
		FL_OBJECT *flag3;
	} FD_form_include;

	/**@name Private Data */
	//@{
	/// Real GUI implementation.
	FD_form_include *popup;
	/// Which LyXView do we belong to?
	Communicator *comm;
	/// Inset-is-being-deleted connection.
	Connection d;
	/// Update connection
	Connection u;
	/// Hide connection.
	Connection h;
	/// has form contents changed? Used to control OK (eventually)
	EnumPopupStatus status;
	//@}
};

#endif
