/* form_index.C
 * FormIndex Interface Class Implementation
 */

#include <config.h>
#include "lyx_gui_misc.h"
#include "gettext.h"
#include FORMS_H_LOCATION

#include "form_index.h"
#include "Popups.h"
#include "Communicator.h"
#include "insetindex.h"

// 	$Id: form_index.C,v 1.2 1999/04/09 18:17:00 larsbj Exp $

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: form_index.C,v 1.2 1999/04/09 18:17:00 larsbj Exp $";
#endif /* lint */


#ifdef __GNUG__
#pragma implementation
#endif


FormIndex::FormIndex(Communicator *c, Popups * p)
	: popup(0), comm(c), d(0), u(0), h(0), status(POPUP_UNMODIFIED)
{
	// let the popup be shown
	// This is a permanent connection so we won't bother
	// storing a copy because we won't be disconnecting.
	connect_to_method(p->showFormIndex,
			  this,
			  &FormIndex::show);
	// NOTE:
	// parameter p is used instead of:
	//      comm->getPopups()
	// because we are being contructed from within the
	// Popups constructor and a segfault would occur
	// otherwise.  The segfault occurs because the Popups
	// constructor hasn't returned at this point and so
	// comm->getPopups() will not return the real popups
	// instance.  If this implementation fails see Popups.C
	// ARRae.
}


FormIndex::~FormIndex()
{
	free();
}


void FormIndex::build()
{
#ifdef WITH_WARNINGS
#warning NOTE:  There is no corresponding .fd file
#endif
//        Perhaps setup a TextInputBase class for text inputs (label, index, etc.)
//        then again we could just use askForText from lyx_gui_misc.  ARRae
	FL_OBJECT *obj;
	FD_form_index *fdui = popup = new FD_form_index;

	fdui->form_index = fl_bgn_form(FL_NO_BOX, 258, 130);
	fdui->form_index->u_vdata = this;
	obj = fl_add_box(FL_UP_BOX,0,0,258,196,"");
	fdui->key = obj = fl_add_input(FL_NORMAL_INPUT,95,26,130,30,
				       idex(_("Keyword:|#K")));
	  fl_set_object_shortcut(obj,scex(_("Keyword:|#K")),1);
	  fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	  fl_set_object_callback(obj,FormIndex::InputCB,0);
	fdui->button_ok = obj = fl_add_button(FL_RETURN_BUTTON,45,90,80,30,_("OK"));
	  fl_set_object_callback(obj,FormIndex::IndexCB,1);
	fdui->button_cancel = obj = fl_add_button(FL_NORMAL_BUTTON,145,90,80,30,
			    idex(_("Cancel|^[")));
	  fl_set_object_shortcut(obj,scex(_("Cancel|^[")),1);
	  fl_set_object_callback(obj,FormIndex::IndexCB,0);
	fl_end_form();

	fdui->form_index->fdui = fdui;
}


void FormIndex::show(InsetIndex * inset)
{
	if (!inset) {
		return;
	}

	if (!popup) {
		build();
		fl_set_input_return(popup->key,
				    FL_RETURN_CHANGED);
		fl_set_form_atclose(popup->form_index,
				    FormIndex::IndexWMHideCB, 0);
	}

	if (comm->isReadOnly()) {
		// deactivating an input doesn't seem to work
		// at least not up to 0.88, so we remember instead.
		status = POPUP_READONLY;
		fl_deactivate_object(popup->key);
	} else {
		status = POPUP_UNMODIFIED;
		fl_activate_object(popup->key);
	}
	fl_deactivate_object(popup->button_ok);
	fl_set_object_lcol(popup->button_ok, FL_INACTIVE);

        popup->vdata = inset;
	// if the popup is visible and the inset its attached to is deleted
	// the inset will activate this signal warning us of the situation.
	// Since the popup doesn't make sense without an inset we free() it.
	d = connect_to_method(inset->gone,
			      this,
			      &FormIndex::free);

        fl_set_input(popup->key, inset->getContents().c_str());

	if (popup->form_index->visible) {
		fl_raise_form(popup->form_index);
	} else {
		fl_show_form(popup->form_index,
			     FL_PLACE_MOUSE,
			     FL_FULLBORDER,
			     _("Index"));
		h = connect_to_method(
				comm->getPopups().hideBufferDependent,
				this,
				&FormIndex::hide);
		// if we change buffers the popup is no longer valid
		// so we'll just hide it when we get an update signal
		u = connect_to_method(
				comm->getPopups().updateBufferDependent,
				this,
				&FormIndex::hide);
	}
}


void FormIndex::hide()
{
	if (popup
	    && popup->form_index
	    && popup->form_index->visible) {
		fl_hide_form(popup->form_index);
		d.disconnect();
		u.disconnect();
		h.disconnect();
		popup->vdata = 0;
	}
	free();
}


void FormIndex::index(long arg)
{
	InsetIndex *inset = (InsetIndex*)popup->vdata;

	switch (arg) {
	case 1: // OK
		if(!comm->isReadOnly()) {
			string tmp = fl_get_input(popup->key);
			if(tmp != inset->getContents())	{
				inset->setContents(tmp);
				hide();
				comm->updateInset(inset, true);
				break;
			}
		} // fall through to Cancel on RO
	case 0: // Cancel
		hide();
		break;
	}
}


void FormIndex::free()
{
	// we don't need to delete u and h here because
	// hide() does that after disconnecting.
	if (popup) {
		if (popup->form_index
		    && popup->form_index->visible) {
			hide();
		}
		fl_free_form(popup->form_index);
		delete popup;
		popup = 0;
	}
}


int FormIndex::IndexWMHideCB(FL_FORM * form, void *)
{
	// Ensure that the signals (u and h) are disconnected even if the
	// window manager is used to close the popup.
	FormIndex *pre = (FormIndex*)form->u_vdata;
	pre->hide();
	return FL_CANCEL;
}


void FormIndex::IndexCB(FL_OBJECT *ob, long data)
{
	FormIndex *pre = (FormIndex*)ob->form->u_vdata;
	pre->index(data);
}


void FormIndex::InputCB(FL_OBJECT *ob, long)
{
	FormIndex *pre = (FormIndex*)ob->form->u_vdata;
	if (pre->status == POPUP_UNMODIFIED) {
		pre->status = POPUP_MODIFIED;
		fl_activate_object(pre->popup->button_ok);
		fl_set_object_lcol(pre->popup->button_ok, FL_BLACK);
	}
}
