// -*- C++ -*-
/* formindex.h
 * Formindex Interface Class
 */

#ifndef FORMINDEX_H
#define FORMINDEX_H

#include "lsignal.h"
#include "gui_common.h"

class Communicator;
class Popups;
class InsetIndex;


/** This class provides an XForms implementation of the FormIndex Popup.
    The index popup allows user to insert indices
 */
class FormIndex : public virtual SignalBase {
public:
	/**@name Constructors and Destructors */
	//@{
	/// #FormIndex x(Communicator ..., Popups ...);#
	FormIndex(Communicator *, Popups *);
	///
	~FormIndex();
	//@}
private:
	FormIndex() {}
	FormIndex(FormIndex&) {}

	/**@name Real per-instance Callback Methods */
	//@{
	static int IndexWMHideCB(FL_FORM *, void *);
	static void IndexCB(FL_OBJECT *, long);
	static void InputCB(FL_OBJECT *, long);
	//@}
	
	/**@name Slot Methods */
	//@{
	/// Create the popup if necessary, update it and display it.
	void show(InsetIndex *);
	/// Hide the popup.
	void hide();
	/// Real popup handler
	void index(long);
	//@}

	
	/// Build the popup
	void build();
	/// Explicitly free the popup.
	void free();

	typedef struct {
		FL_FORM *form_index;
		void *vdata;
		char *cdata;
		long  ldata;
		FL_OBJECT *button_ok;
		FL_OBJECT *button_cancel;
		FL_OBJECT *key;
	} FD_form_index;

	/**@name Private Data */
	//@{
	/// Real GUI implementation.
	FD_form_index *popup;
	/// Which LyXView do we belong to?
	Communicator *comm;
	/// Inset-is-being-deleted connection.
	Connection d;
	/// Update connection
	Connection u;
	/// Hide connection.
	Connection h;
	/// has form contents changed? Used to control OK (eventually)
	EnumPopupStatus status;
	//@}
};

#endif
