/* form_info.C
 * FormInfo Interface Class Implementation
 */

#include <config.h>
#include "lyx_gui_misc.h"
#include "gettext.h"
#include FORMS_H_LOCATION

#include "form_info.h"
#include "Popups.h"
#include "Communicator.h"
#include "insetinfo.h"

// 	$Id: form_info.C,v 1.2 1999/04/09 18:17:01 larsbj Exp $

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: form_info.C,v 1.2 1999/04/09 18:17:01 larsbj Exp $";
#endif /* lint */


#ifdef __GNUG__
#pragma implementation
#endif


FormInfo::FormInfo(Communicator *c, Popups * p)
	: popup(0), comm(c), d(0), u(0), h(0), status(POPUP_UNMODIFIED)
{
	// let the popup be shown
	// This is a permanent connection so we won't bother
	// storing a copy because we won't be disconnecting.
	connect_to_method(p->showFormInfo,
			  this,
			  &FormInfo::show);
	// NOTE:
	// parameter p is used instead of:
	//      comm->getPopups()
	// because we are being contructed from within the
	// Popups constructor and a segfault would occur
	// otherwise.  The segfault occurs because the Popups
	// constructor hasn't returned at this point and so
	// comm->getPopups() will not return the real popups
	// instance.  If this implementation fails see Popups.C
	// ARRae.
}


FormInfo::~FormInfo()
{
	free();
}


void FormInfo::build()
{
#ifdef WITH_WARNINGS
#warning NOTE:  There is no corresponding .fd file
#endif
//        Perhaps setup a TextInputBase class for text inputs (label, info, etc.)
// Note could use the same base class as LaTeXPreamble (if one existed) since just
// the size of the form (and hence multiline input) is different.  ARRae.
	FL_OBJECT *obj;
	FD_form_info *fdui = popup = new FD_form_info;

	fdui->form_info = fl_bgn_form(FL_UP_BOX,400,180);
	fdui->form_info->u_vdata = this;

	fdui->input_info = obj = fl_add_input(FL_MULTILINE_INPUT,10,10,380,120,"");
	fl_set_object_color(obj,FL_MCOL,FL_MCOL);
	fl_set_object_resize(obj, FL_RESIZE_ALL);
	fl_set_object_gravity(obj, NorthWestGravity, SouthEastGravity);
	fl_set_object_callback(obj,FormInfo::InputCB,0);
	
	fdui->button_ok = obj = fl_add_button(FL_RETURN_BUTTON,110,140,80,30,_("OK"));
	fl_set_object_resize(obj, FL_RESIZE_NONE);
	fl_set_object_callback(obj,FormInfo::InfoCB,1);
	fdui->button_cancel = obj = fl_add_button(FL_NORMAL_BUTTON,210,140,80,30,idex(_("Cancel|^[")));
	fl_set_object_shortcut(obj,scex(_("Cancel|^[")),1);
	fl_set_object_resize(obj, FL_RESIZE_NONE);
	fl_set_object_callback(obj,FormInfo::InfoCB,0);
	fl_end_form();

	fdui->form_info->fdui = fdui;
}


void FormInfo::show(InsetInfo * inset)
{
	if (!inset) {
		return;
	}

	if (!popup) {
		build();
		fl_set_input_return(popup->input_info,
				    FL_RETURN_CHANGED);
		fl_set_form_atclose(popup->form_info,
				    FormInfo::InfoWMHideCB, 0);
	}

	if (comm->isReadOnly()) {
		// deactivating an input doesn't seem to work
		// at least not up to 0.88, so we remember instead.
		status = POPUP_READONLY;
		fl_deactivate_object(popup->input_info);
	} else {
		status = POPUP_UNMODIFIED;
		fl_activate_object(popup->input_info);
	}
	fl_deactivate_object(popup->button_ok);
	fl_set_object_lcol(popup->button_ok, FL_INACTIVE);

        popup->vdata = inset;
	// if the popup is visible and the inset its attached to is deleted
	// the inset will activate this signal warning us of the situation.
	// Since the popup doesn't make sense without an inset we free() it.
	d = connect_to_method(inset->gone,
			      this,
			      &FormInfo::free);

        fl_set_input(popup->input_info, inset->getContents().c_str());

	if (popup->form_info->visible) {
		fl_raise_form(popup->form_info);
	} else {
		fl_show_form(popup->form_info,
			     FL_PLACE_MOUSE | FL_FREE_SIZE,
			     FL_FULLBORDER,
			     _("Note"));
		h = connect_to_method(
				comm->getPopups().hideBufferDependent,
				this,
				&FormInfo::hide);
		// if we change buffers the popup is no longer valid
		// so we'll just hide it when we get an update signal
		u = connect_to_method(
				comm->getPopups().updateBufferDependent,
				this,
				&FormInfo::hide);
	}
}


void FormInfo::hide()
{
	if (popup
	    && popup->form_info
	    && popup->form_info->visible) {
		fl_hide_form(popup->form_info);
		d.disconnect();
		u.disconnect();
		h.disconnect();
		popup->vdata = 0;
	}
	free();
}


void FormInfo::info(long arg)
{
	InsetInfo *inset = (InsetInfo*)popup->vdata;

	switch (arg) {
	case 1: // OK
		if(!comm->isReadOnly()) {
			string tmp = fl_get_input(popup->input_info);
			if(tmp != inset->getContents())	{
				inset->setContents(tmp);
				hide();
				comm->markDirty();
				break;
			}
		} // fall through to Cancel on RO
	case 0: // Cancel
		hide();
		break;
	}
}


void FormInfo::free()
{
	// we don't need to delete u and h here because
	// hide() does that after disconnecting.
	if (popup) {
		if (popup->form_info
		    && popup->form_info->visible) {
			hide();
		}
		fl_free_form(popup->form_info);
		delete popup;
		popup = 0;
	}
}


int FormInfo::InfoWMHideCB(FL_FORM * form, void *)
{
	// Ensure that the signals (u and h) are disconnected even if the
	// window manager is used to close the popup.
	FormInfo *pre = (FormInfo*)form->u_vdata;
	pre->hide();
	return FL_CANCEL;
}


void FormInfo::InfoCB(FL_OBJECT *ob, long data)
{
	FormInfo *pre = (FormInfo*)ob->form->u_vdata;
	pre->info(data);
}


void FormInfo::InputCB(FL_OBJECT *ob, long)
{
	FormInfo *pre = (FormInfo*)ob->form->u_vdata;
	if (pre->status == POPUP_UNMODIFIED) {
		pre->status = POPUP_MODIFIED;
		fl_activate_object(pre->popup->button_ok);
		fl_set_object_lcol(pre->popup->button_ok, FL_BLACK);
	}
}
