// -*- C++ -*-
/* form_info.h
 * FormInfo Interface Class
 */

#ifndef FORMINFO_H
#define FORMINFO_H

#include "lsignal.h"
#include "gui_common.h"

class Communicator;
class Popups;
class InsetInfo;


/** This class provides an XForms implementation of the FormInfo Popup.
    The info popup allows user to insert indices
 */
class FormInfo : public virtual SignalBase {
public:
	/**@name Constructors and Destructors */
	//@{
	/// #FormInfo x(Communicator ..., Popups ...);#
	FormInfo(Communicator *, Popups *);
	///
	~FormInfo();
	//@}
private:
	FormInfo() {}
	FormInfo(FormInfo&) {}

	/**@name Real per-instance Callback Methods */
	//@{
	static int InfoWMHideCB(FL_FORM *, void *);
	static void InfoCB(FL_OBJECT *, long);
	static void InputCB(FL_OBJECT *, long);
	//@}
	
	/**@name Slot Methods */
	//@{
	/// Create the popup if necessary, update it and display it.
	void show(InsetInfo *);
	/// Hide the popup.
	void hide();
	/// Real popup handler
	void info(long);
	//@}

	
	/// Build the popup
	void build();
	/// Explicitly free the popup.
	void free();

	typedef struct {
		FL_FORM *form_info;
		void *vdata;
		char *cdata;
		long  ldata;
		FL_OBJECT *button_ok;
		FL_OBJECT *button_cancel;
		FL_OBJECT *input_info;
	} FD_form_info;

	/**@name Private Data */
	//@{
	/// Real GUI implementation.
	FD_form_info *popup;
	/// Which LyXView do we belong to?
	Communicator *comm;
	/// Inset-is-being-deleted connection.
	Connection d;
	/// Update connection
	Connection u;
	/// Hide connection.
	Connection h;
	/// has form contents changed? Used to control OK (eventually)
	EnumPopupStatus status;
	//@}
};

#endif
