/* form_paragraph.C
 * FormParagraph Interface Class Implementation
 */

#include <config.h>
#include "lyx_gui_misc.h"
#include "gettext.h"
#include FORMS_H_LOCATION

#include "form_paragraphs.h"
#include "Popups.h"
#include "Communicator.h"
#include "textclasslist.h"
#include "bufferparams.h"
// lyxparagraph.h is only needed until the pextra stuff is shifted to the
// paragraph params then only paraparams.h will be needed
#include "lyxparagraph.h"

// 	$Id: form_paragraphs.C,v 1.8 1998/10/21 10:12:34 lasgoutt Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: form_paragraphs.C,v 1.8 1998/10/21 10:12:34 lasgoutt Exp $";
#endif /* lint */


#ifdef __GNUG__
#pragma implementation
#endif


FormParagraph::FormParagraph(Communicator *c, Popups * p)
	: ppopup(0), epopup(0), tpopup(0), comm(c), u(0), h(0),
          status(POPUP_UNMODIFIED)
{
    // let the popup be shown
    // This is a permanent connection so we won't bother
    // storing a copy because we won't be disconnecting.
    connect_to_method(p->showFormParagraph, this, &FormParagraph::show);
    // NOTE:
    // parameter p is used instead of:
    //      comm->getPopups()
    // because we are being contructed from within the
    // Popups constructor and a segfault would occur
    // otherwise.  The segfault occurs because the Popups
    // constructor hasn't returned at this point and so
    // comm->getPopups() will not return the real popups
    // instance.  If this implementation fails see Popups.C
    // ARRae.
    build();
}


FormParagraph::~FormParagraph()
{
    free();
}

void FormParagraph::build()
{
    // the paragraph form
    build_pform();
    fl_addto_choice(ppopup->choice_space_above,
                    _(" None | Defskip | Smallskip "
                      "| Medskip | Bigskip | VFill | Length "));
    fl_addto_choice(ppopup->choice_space_below,
                    _(" None | Defskip | Smallskip "
                      "| Medskip | Bigskip | VFill | Length ")); 
    fl_set_input_return(ppopup->input_space_above, FL_RETURN_ALWAYS);
    fl_set_input_return(ppopup->input_space_below, FL_RETURN_ALWAYS);
    // the paragraph extra form
    build_eform();
    fl_set_input_return(epopup->input_pextra_width, FL_RETURN_ALWAYS);
    fl_set_input_return(epopup->input_pextra_widthp, FL_RETURN_ALWAYS);
    // the tabbed folder
    build_tform();
    fl_set_form_atclose(tpopup->form_tabbed_paragraph,
                        FormParagraph::FormParagraphWMHideCB, 0);
    fl_addto_tabfolder(tpopup->tabbed_folder,_("Paragraph"),
                       ppopup->form_paragraph);
    fl_addto_tabfolder(tpopup->tabbed_folder,_("Extra"),
                       epopup->form_paragraph_extra);
}

void FormParagraph::build_pform()
{
    FL_OBJECT *obj;
    ppopup = (FD_form_paragraph *) fl_calloc(1, sizeof(*ppopup));
    
    ppopup->form_paragraph = fl_bgn_form(FL_NO_BOX, 490, 260);
    ppopup->form_paragraph->u_vdata = this;

    obj = fl_add_box(FL_FLAT_BOX,0,0,490,260,"");
    obj = fl_add_frame(FL_ENGRAVED_FRAME,360,110,120,50,"");
    fl_set_object_color(obj,FL_COL1,FL_COL1);
    obj = fl_add_frame(FL_ENGRAVED_FRAME,10,110,340,90,"");
    fl_set_object_color(obj,FL_COL1,FL_COL1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    obj = fl_add_frame(FL_ENGRAVED_FRAME,360,20,120,70,"");
    fl_set_object_color(obj,FL_COL1,FL_COL1);
    obj = fl_add_frame(FL_ENGRAVED_FRAME,210,20,140,70,"");
    fl_set_object_color(obj,FL_COL1,FL_COL1);
    obj = fl_add_frame(FL_ENGRAVED_FRAME,10,20,190,70,"");
    fl_set_object_color(obj,FL_COL1,FL_COL1);
    ppopup->input_labelwidth = obj = fl_add_input(FL_NORMAL_INPUT,100,210,360,30,idex(_("Label Width:|#d")));fl_set_button_shortcut(obj,scex(_("Label Width:|#d")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_gravity(obj, FL_West, FL_East);
    fl_set_object_resize(obj, FL_RESIZE_X);
    obj = fl_add_text(FL_NORMAL_TEXT,370,100,60,20,_("Indent"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    ppopup->check_lines_top = obj = fl_add_checkbutton(FL_PUSH_BUTTON,230,30,120,30,idex(_("Above|#b")));fl_set_button_shortcut(obj,scex(_("Above|#b")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    ppopup->check_lines_bottom = obj = fl_add_checkbutton(FL_PUSH_BUTTON,230,60,120,30,idex(_("Below|#E")));fl_set_button_shortcut(obj,scex(_("Below|#E")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    ppopup->check_pagebreaks_top = obj = fl_add_checkbutton(FL_PUSH_BUTTON,360,30,120,30,idex(_("Above|#o")));fl_set_button_shortcut(obj,scex(_("Above|#o")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    ppopup->check_pagebreaks_bottom = obj = fl_add_checkbutton(FL_PUSH_BUTTON,360,60,120,30,idex(_("Below|#l")));fl_set_button_shortcut(obj,scex(_("Below|#l")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    ppopup->check_noindent = obj = fl_add_checkbutton(FL_PUSH_BUTTON,360,120,120,30,idex(_("No Indent|#I")));fl_set_button_shortcut(obj,scex(_("No Indent|#I")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);

    ppopup->group_radio_alignment = fl_bgn_group();
    ppopup->radio_align_right = obj = fl_add_checkbutton(FL_RADIO_BUTTON,10,30,80,30,idex(_("Right|#R")));fl_set_button_shortcut(obj,scex(_("Right|#R")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    ppopup->radio_align_left = obj = fl_add_checkbutton(FL_RADIO_BUTTON,10,60,80,30,idex(_("Left|#f")));fl_set_button_shortcut(obj,scex(_("Left|#f")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    ppopup->radio_align_block = obj = fl_add_checkbutton(FL_RADIO_BUTTON,100,30,80,30,idex(_("Block|#c")));fl_set_button_shortcut(obj,scex(_("Block|#c")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    ppopup->radio_align_center = obj = fl_add_checkbutton(FL_RADIO_BUTTON,100,60,80,30,idex(_("Center|#n")));fl_set_button_shortcut(obj,scex(_("Center|#n")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_end_group();

    ppopup->input_space_above = obj = fl_add_input(FL_NORMAL_INPUT,180,120,90,30,"");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,ParagraphVSpaceCB,0);
    ppopup->input_space_below = obj = fl_add_input(FL_NORMAL_INPUT,180,160,90,30,"");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,ParagraphVSpaceCB,0);
    ppopup->choice_space_above = obj = fl_add_choice(FL_NORMAL_CHOICE,70,120,100,30,idex(_("Above:|#v")));fl_set_button_shortcut(obj,scex(_("Above:|#v")),1);
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,ParagraphVSpaceCB,0);
    ppopup->choice_space_below = obj = fl_add_choice(FL_NORMAL_CHOICE,70,160,100,30,idex(_("Below:|#w")));fl_set_button_shortcut(obj,scex(_("Below:|#w")),1);
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,ParagraphVSpaceCB,0);
    obj = fl_add_text(FL_NORMAL_TEXT,370,10,100,20,_("Pagebreaks"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    obj = fl_add_text(FL_NORMAL_TEXT,230,10,50,20,_("Lines"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    obj = fl_add_text(FL_NORMAL_TEXT,20,10,80,20,_("Alignment"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    obj = fl_add_text(FL_NORMAL_TEXT,20,100,130,20,_("Vertical Spaces"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    ppopup->check_space_above = obj = fl_add_checkbutton(FL_PUSH_BUTTON,270,120,40,30,idex(_("Keep|#K")));fl_set_button_shortcut(obj,scex(_("Keep|#K")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    ppopup->check_space_below = obj = fl_add_checkbutton(FL_PUSH_BUTTON,270,160,40,30,idex(_("Keep|#p")));fl_set_button_shortcut(obj,scex(_("Keep|#p")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_end_form();

    ppopup->form_paragraph->fdui = ppopup;
}

void FormParagraph::build_eform()
{
    FL_OBJECT *obj;
    epopup = (FD_form_paragraph_extra *) fl_calloc(1, sizeof(*epopup));

    epopup->form_paragraph_extra = fl_bgn_form(FL_NO_BOX, 490, 260);
    epopup->form_paragraph_extra->u_vdata = this;

    obj = fl_add_box(FL_FLAT_BOX,0,0,490,260,"");
    obj = fl_add_frame(FL_ENGRAVED_FRAME,10,20,170,100,"");
    fl_set_object_color(obj,FL_COL1,FL_COL1);
    obj = fl_add_frame(FL_ENGRAVED_FRAME,350,20,100,100,"");
    fl_set_object_color(obj,FL_COL1,FL_COL1);
    obj = fl_add_frame(FL_ENGRAVED_FRAME,190,20,150,100,"");
    fl_set_object_color(obj,FL_COL1,FL_COL1);
    obj = fl_add_text(FL_NORMAL_TEXT,20,10,120,20,_("Extra Options"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    epopup->input_pextra_width = obj = fl_add_input(FL_NORMAL_INPUT,240,30,90,30,idex(_("Length|#L")));fl_set_button_shortcut(obj,scex(_("Length|#L")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormParagraphInputCB,0);
    epopup->input_pextra_widthp = obj = fl_add_input(FL_INT_INPUT,240,70,90,30,idex(_("or %|#o")));fl_set_button_shortcut(obj,scex(_("or %|#o")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormParagraphInputCB,0);
    obj = fl_add_text(FL_NORMAL_TEXT,200,10,60,20,_("Width"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    obj = fl_add_text(FL_NORMAL_TEXT,350,10,90,20,_("Alignment"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);

    epopup->group_alignment = fl_bgn_group();
    epopup->radio_pextra_top = obj = fl_add_checkbutton(FL_RADIO_BUTTON,360,30,80,30,idex(_("Top|#T")));fl_set_button_shortcut(obj,scex(_("Top|#T")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormParagraphInputCB,0);
    epopup->radio_pextra_middle = obj = fl_add_checkbutton(FL_RADIO_BUTTON,360,60,80,30,idex(_("Middle|#d")));fl_set_button_shortcut(obj,scex(_("Middle|#d")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormParagraphInputCB,0);
    epopup->radio_pextra_bottom = obj = fl_add_checkbutton(FL_RADIO_BUTTON,360,90,80,30,idex(_("Bottom|#B")));fl_set_button_shortcut(obj,scex(_("Bottom|#B")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormParagraphInputCB,0);
    fl_end_group();

    epopup->text_warning = obj = fl_add_text(FL_NORMAL_TEXT,10,190,470,30,"");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    epopup->radio_pextra_hfill = obj = fl_add_checkbutton(FL_PUSH_BUTTON,10,130,300,30,idex(_("HFill between Minipage paragraphs|#H")));fl_set_button_shortcut(obj,scex(_("HFill between Minipage paragraphs|#H")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    epopup->radio_pextra_startmp = obj = fl_add_checkbutton(FL_PUSH_BUTTON,10,160,260,30,idex(_("Start new Minipage|#S")));fl_set_button_shortcut(obj,scex(_("Start new Minipage|#S")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);

    epopup->group_extraopt = fl_bgn_group();
    epopup->radio_pextra_indent = obj = fl_add_checkbutton(FL_PUSH_BUTTON,10,30,160,30,idex(_("Indented Paragraph|#I")));fl_set_button_shortcut(obj,scex(_("Indented Paragraph|#I")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormParagraphInputCB,0);
    epopup->radio_pextra_minipage = obj = fl_add_checkbutton(FL_PUSH_BUTTON,10,60,160,30,idex(_("Minipage|#M")));fl_set_button_shortcut(obj,scex(_("Minipage|#M")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormParagraphInputCB,0);
    epopup->radio_pextra_floatflt = obj = fl_add_checkbutton(FL_PUSH_BUTTON,10,90,160,30,idex(_("Floatflt|#F")));fl_set_button_shortcut(obj,scex(_("Floatflt|#F")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormParagraphInputCB,0);
    fl_end_group();

    fl_end_form();

    epopup->form_paragraph_extra->fdui = epopup;
}

void FormParagraph::build_tform()
{
    FL_OBJECT *obj;
    tpopup = (FD_form_tabbed_paragraph *) fl_calloc(1, sizeof(*tpopup));

    tpopup->form_tabbed_paragraph = fl_bgn_form(FL_NO_BOX, 510, 340);
    tpopup->form_tabbed_paragraph->u_vdata = this;

    obj = fl_add_box(FL_FLAT_BOX,0,0,510,340,"");
    tpopup->tabbed_folder = obj = fl_add_tabfolder(FL_TOP_TABFOLDER,10,10,490,280,_("Tabbed folder"));
    fl_set_object_resize(obj, FL_RESIZE_ALL);
    obj = fl_add_button(FL_NORMAL_BUTTON,400,300,100,30,idex(_("Cancel|^[")));fl_set_button_shortcut(obj,scex(_("Cancel|^[")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormParagraphCancelCB,0);
    tpopup->button_apply = obj = fl_add_button(FL_NORMAL_BUTTON,290,300,100,30,idex(_("Apply|#A")));fl_set_button_shortcut(obj,scex(_("Apply|#A")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormParagraphApplyCB,0);
    tpopup->button_ok = obj = fl_add_button(FL_RETURN_BUTTON,180,300,100,30,_("OK"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormParagraphOKCB,0);
    fl_end_form();
    
    tpopup->form_tabbed_paragraph->fdui = tpopup;
}

void FormParagraph::show()
{
    update();  // make sure its up-to-date
    if (tpopup->form_tabbed_paragraph->visible) {
        fl_raise_form(tpopup->form_tabbed_paragraph);
    } else {
        fl_show_form(tpopup->form_tabbed_paragraph,
                     FL_PLACE_MOUSE,// | FL_FREE_SIZE,
                     FL_FULLBORDER,
                     _("Paragraph Layout"));
        u = connect_to_method(comm->getPopups().updateBufferDependent,
                              this, &FormParagraph::update);
        h = connect_to_method(comm->getPopups().hideBufferDependent,
                              this, &FormParagraph::hide);
    }
}

void FormParagraph::hide()
{
    if (tpopup->form_tabbed_paragraph->visible) {
        fl_hide_form(tpopup->form_tabbed_paragraph);
        u.disconnect();
        h.disconnect();
    }
}

void FormParagraph::apply()
{
    if (!comm->isAvailable() || !tpopup)
        return;

    if (fl_get_active_folder(tpopup->tabbed_folder) == ppopup->form_paragraph)
        papply();
    else
        eapply();
}

void FormParagraph::papply()
{
    bool
        line_top, line_bottom, pagebreak_top, pagebreak_bottom, noindent;
    VSpace
        space_top, space_bottom;
    char
        align;
    string
        labelwidthstring;

    // If a vspace kind is "Length" but there's no text in
    // the input field, reset the kind to "None". 
    if (fl_get_choice (ppopup->choice_space_above) == 7
        && !*(fl_get_input (ppopup->input_space_above))) {
        fl_set_choice (ppopup->choice_space_above, 1);
    }
    if (fl_get_choice (ppopup->choice_space_below) == 7
        && !*(fl_get_input (ppopup->input_space_below))) {
        fl_set_choice (ppopup->choice_space_below, 1);
    }
    line_top = fl_get_button(ppopup->check_lines_top);
    line_bottom = fl_get_button(ppopup->check_lines_bottom);
    pagebreak_top = fl_get_button(ppopup->check_pagebreaks_top);
    pagebreak_bottom = fl_get_button(ppopup->check_pagebreaks_bottom);
    switch (fl_get_choice (ppopup->choice_space_above)) {
      case 1:
          space_top = VSpace::NONE;
          break;
      case 2:
          space_top = VSpace::DEFSKIP;
          break;
      case 3:
          space_top = VSpace::SMALLSKIP;
          break;
      case 4:
          space_top = VSpace::MEDSKIP;
          break;
      case 5:
          space_top = VSpace::BIGSKIP;
          break;
      case 6:
          space_top = VSpace::VFILL;
          break;
      case 7:
          space_top = LyXGlueLength (fl_get_input (ppopup->input_space_above));
          break;
    }
    if (fl_get_button (ppopup->check_space_above))
        space_top.setKeep (true);
    switch (fl_get_choice (ppopup->choice_space_below)) {
      case 1:
          space_bottom = VSpace::NONE;
          break;
      case 2:
          space_bottom = VSpace::DEFSKIP;
          break;
      case 3:
          space_bottom = VSpace::SMALLSKIP;
          break;
      case 4:
          space_bottom = VSpace::MEDSKIP;
          break;
      case 5:
          space_bottom = VSpace::BIGSKIP;
          break;
      case 6:
          space_bottom = VSpace::VFILL;
          break;
      case 7:
          space_bottom=LyXGlueLength(fl_get_input(ppopup->input_space_below));
          break;
    }
    if (fl_get_button (ppopup->check_space_below))
        space_bottom.setKeep (true);

    if (fl_get_button(ppopup->radio_align_left))
        align = LYX_ALIGN_LEFT;
    else if (fl_get_button(ppopup->radio_align_right))
        align = LYX_ALIGN_RIGHT;
    else if (fl_get_button(ppopup->radio_align_center))
        align = LYX_ALIGN_CENTER;
    else 
        align = LYX_ALIGN_BLOCK;
   
    labelwidthstring = fl_get_input(ppopup->input_labelwidth);
    noindent = fl_get_button(ppopup->check_noindent);
   
    comm->setParagraph(line_top, line_bottom, pagebreak_top,
		       pagebreak_bottom, space_top, space_bottom,
		       align, labelwidthstring, noindent);
}

void FormParagraph::eapply()
{
    const char
        *width = fl_get_input(epopup->input_pextra_width),
        *widthp = fl_get_input(epopup->input_pextra_widthp);
    int
        type = PEXTRA_NONE,
        alignment = 0;
    bool
        hfill = false,
        start_minipage = false;

    if (fl_get_button(epopup->radio_pextra_indent)) {
        type = PEXTRA_INDENT;
    } else if (fl_get_button(epopup->radio_pextra_minipage)) {
        type = PEXTRA_MINIPAGE;
        hfill = fl_get_button(epopup->radio_pextra_hfill);
        start_minipage = fl_get_button(epopup->radio_pextra_startmp);
        if (fl_get_button(epopup->radio_pextra_top))
            alignment = MINIPAGE_ALIGN_TOP;
        else if (fl_get_button(epopup->radio_pextra_middle))
            alignment = MINIPAGE_ALIGN_MIDDLE;
        else if (fl_get_button(epopup->radio_pextra_bottom))
            alignment = MINIPAGE_ALIGN_BOTTOM;
    } else if (fl_get_button(epopup->radio_pextra_floatflt)) {
        type = PEXTRA_FLOATFLT;
    }
    comm->setParagraphExtraOpt(type,width,widthp,alignment,hfill,
                               start_minipage);
}

void FormParagraph::update()
{
    if (!comm->isAvailable() || !tpopup)
        return;
    pupdate();
    eupdate();
}

void FormParagraph::pupdate()
{
    if (!ppopup)
        return;
    int
        align = comm->getParAlign();

    fl_set_input(ppopup->input_labelwidth,
                 comm->getParLabelWidthString().c_str());
    fl_set_button(ppopup->radio_align_right, 0);
    fl_set_button(ppopup->radio_align_left, 0);
    fl_set_button(ppopup->radio_align_center, 0);
    fl_set_button(ppopup->radio_align_block, 0);
    if (align == LYX_ALIGN_LAYOUT)
        align =	textclasslist.style(comm->getBufferParams().textclass,
				    comm->getParLayout())->align;
    switch (align) {
      case LYX_ALIGN_RIGHT:
          fl_set_button(ppopup->radio_align_right, 1);
          break;
      case LYX_ALIGN_LEFT:
          fl_set_button(ppopup->radio_align_left, 1);
          break;
      case LYX_ALIGN_CENTER:
          fl_set_button(ppopup->radio_align_center, 1);
          break;
      default:
          fl_set_button(ppopup->radio_align_block, 1);
          break;
    }
    fl_set_button(ppopup->check_lines_top,
                  comm->getParaParams().line_top);
    fl_set_button(ppopup->check_lines_bottom,
                  comm->getParaParams().line_bottom);
    fl_set_button(ppopup->check_pagebreaks_top,
                  comm->getParaParams().pagebreak_top);
    fl_set_button(ppopup->check_pagebreaks_bottom,
                  comm->getParaParams().pagebreak_bottom);
    fl_set_button(ppopup->check_noindent,
                  comm->getParaParams().noindent);
    fl_set_input (ppopup->input_space_above, "");
    switch (comm->getParaParams().added_space_top.kind())
    {
      case VSpace::NONE:
          fl_set_choice (ppopup->choice_space_above, 1);
          break;
      case VSpace::DEFSKIP:
          fl_set_choice (ppopup->choice_space_above, 2);
          break;
      case VSpace::SMALLSKIP:
          fl_set_choice (ppopup->choice_space_above, 3);
          break;
      case VSpace::MEDSKIP:
          fl_set_choice (ppopup->choice_space_above, 4);
          break;
      case VSpace::BIGSKIP:
          fl_set_choice (ppopup->choice_space_above, 5);
          break;
      case VSpace::VFILL:
          fl_set_choice (ppopup->choice_space_above, 6);
          break;
      case VSpace::LENGTH:
          fl_set_choice (ppopup->choice_space_above, 7); 
          fl_set_input  (ppopup->input_space_above, 
                         comm->getParaParams().added_space_top.length().asString().c_str());
          break;
    }
    fl_set_button (ppopup->check_space_above,
                   comm->getParaParams().added_space_top.keep());
    fl_set_input (ppopup->input_space_below, "");
    switch (comm->getParaParams().added_space_bottom.kind()) {
      case VSpace::NONE:
          fl_set_choice (ppopup->choice_space_below, 1);
          break;
      case VSpace::DEFSKIP:
          fl_set_choice (ppopup->choice_space_below, 2);
          break;
      case VSpace::SMALLSKIP:
          fl_set_choice (ppopup->choice_space_below, 3);
          break;
      case VSpace::MEDSKIP:
          fl_set_choice (ppopup->choice_space_below, 4);
          break;
      case VSpace::BIGSKIP:
          fl_set_choice (ppopup->choice_space_below, 5);
          break;
      case VSpace::VFILL:
          fl_set_choice (ppopup->choice_space_below, 6);
          break;
      case VSpace::LENGTH:
          fl_set_choice (ppopup->choice_space_below, 7); 
          fl_set_input  (ppopup->input_space_below, 
                         comm->getParaParams().added_space_bottom.length().asString().c_str());
          break;
    }
    fl_set_button (ppopup->check_space_below,
                   comm->getParaParams().added_space_bottom.keep());
    fl_set_button(ppopup->check_noindent,
                  comm->getParaParams().noindent);
}

void FormParagraph::eupdate()
{
    if (!epopup)
        return;
    LyXParagraph
        *par = comm->getParAtCursor();

    EnableParagraphExtra();
    fl_set_input(epopup->input_pextra_width, par->pextra_width.c_str());
    fl_set_input(epopup->input_pextra_widthp, par->pextra_widthp.c_str());
    switch(par->pextra_alignment) {
      case MINIPAGE_ALIGN_TOP:
          fl_set_button(epopup->radio_pextra_top,1);
          break;
      case MINIPAGE_ALIGN_MIDDLE:
          fl_set_button(epopup->radio_pextra_middle,1);
          break;
      case MINIPAGE_ALIGN_BOTTOM:
          fl_set_button(epopup->radio_pextra_bottom,1);
          break;
    }
    fl_set_button(epopup->radio_pextra_hfill, par->pextra_hfill);
    fl_set_button(epopup->radio_pextra_startmp, par->pextra_start_minipage);
    CheckInputWidth();
    if (par->pextra_type == PEXTRA_INDENT) {
        fl_set_button(epopup->radio_pextra_indent,1);
        fl_set_button(epopup->radio_pextra_minipage,0);
        fl_set_button(epopup->radio_pextra_floatflt,0);
        fl_deactivate_object(epopup->radio_pextra_top);
        fl_deactivate_object(epopup->radio_pextra_middle);
        fl_deactivate_object(epopup->radio_pextra_bottom);
    } else if (par->pextra_type == PEXTRA_MINIPAGE) {
        fl_set_button(epopup->radio_pextra_indent,0);
        fl_set_button(epopup->radio_pextra_minipage,1);
        fl_set_button(epopup->radio_pextra_floatflt,0);
        fl_activate_object(epopup->radio_pextra_top);
        fl_activate_object(epopup->radio_pextra_middle);
        fl_activate_object(epopup->radio_pextra_bottom);
    } else if (par->pextra_type == PEXTRA_FLOATFLT) {
        fl_set_button(epopup->radio_pextra_indent,0);
        fl_set_button(epopup->radio_pextra_minipage,0);
        fl_set_button(epopup->radio_pextra_floatflt,1);
        fl_deactivate_object(epopup->radio_pextra_top);
        fl_deactivate_object(epopup->radio_pextra_middle);
        fl_deactivate_object(epopup->radio_pextra_bottom);
    } else {
        fl_set_button(epopup->radio_pextra_indent,0);
        fl_set_button(epopup->radio_pextra_minipage,0);
        fl_deactivate_object(epopup->input_pextra_width);
        fl_deactivate_object(epopup->input_pextra_widthp);
        fl_deactivate_object(epopup->radio_pextra_top);
        fl_deactivate_object(epopup->radio_pextra_middle);
        fl_deactivate_object(epopup->radio_pextra_bottom);
    }
    if (par->pextra_type == PEXTRA_NONE)
        ActivateParagraphButtons();
    fl_hide_object(epopup->text_warning);
    if (comm->isReadOnly()) {
        DisableParagraphExtra();
        fl_set_object_label(epopup->text_warning,
                            _("Document is read-only. No changes to layout permitted."));
        fl_show_object(epopup->text_warning);
    }
}

void FormParagraph::free()
{
    if (tpopup) {
        hide();
        if (epopup) {
            fl_free_form(epopup->form_paragraph_extra);
            delete epopup;
            epopup = 0;
        }
        if (ppopup) {
            fl_free_form(ppopup->form_paragraph);
            delete ppopup;
            ppopup = 0;
        }
        fl_free_form(tpopup->form_tabbed_paragraph);
        delete tpopup;
        tpopup = 0;
    }
}

int FormParagraph::FormParagraphWMHideCB(FL_FORM * form, void *)
{
    // Ensure that the signals (u and h) are disconnected even if the
    // window manager is used to close the popup.
    FormParagraph *pre = (FormParagraph*)form->u_vdata;
    pre->hide();
    return FL_CANCEL;
}

void FormParagraph::FormParagraphCancelCB(FL_OBJECT *ob, long)
{
    FormParagraph *pre = (FormParagraph*)ob->form->u_vdata;
    pre->hide();
}

void FormParagraph::FormParagraphApplyCB(FL_OBJECT *ob, long)
{
    FormParagraph *pre = (FormParagraph*)ob->form->u_vdata;
    pre->apply();
}

void FormParagraph::FormParagraphOKCB(FL_OBJECT *ob, long)
{
    FormParagraph *pre = (FormParagraph*)ob->form->u_vdata;
    pre->apply();
    pre->hide();
}

void FormParagraph::ParagraphVSpaceCB(FL_OBJECT *ob, long)
{
    FormParagraph *pre = (FormParagraph*)ob->form->u_vdata;
    pre->CheckParagraphVSpace(ob,0);
}

void FormParagraph::FormParagraphInputCB(FL_OBJECT *ob, long)
{
    FormParagraph *pre = (FormParagraph*)ob->form->u_vdata;
    pre->CheckParagraphInput(ob,0);
}

void FormParagraph::UpdateParagraphCB(FL_OBJECT *ob, long)
{
    FormParagraph *pre = (FormParagraph*)ob->form->u_vdata;
    pre->pupdate();
}

void FormParagraph::UpdateParagraphExtraCB(FL_OBJECT *ob, long)
{
    FormParagraph *pre = (FormParagraph*)ob->form->u_vdata;
    pre->eupdate();
}

void FormParagraph::CheckParagraphInput(FL_OBJECT *ob, long)
{
    int
        n;
    string
        s1, s2;
    
    ActivateParagraphButtons();
    CheckInputWidth();
    if (ob == epopup->radio_pextra_indent) {
        n = fl_get_button(epopup->radio_pextra_indent);
        if (n) {
            fl_set_button(epopup->radio_pextra_minipage,0);
            fl_set_button(epopup->radio_pextra_floatflt,0);
            fl_activate_object(epopup->input_pextra_width);
            fl_activate_object(epopup->input_pextra_widthp);
            fl_deactivate_object(epopup->radio_pextra_top);
            fl_deactivate_object(epopup->radio_pextra_middle);
            fl_deactivate_object(epopup->radio_pextra_bottom);
        } else {
            fl_deactivate_object(epopup->input_pextra_width);
            fl_deactivate_object(epopup->input_pextra_widthp);
            ActivateParagraphButtons();
        }
    } else if (ob == epopup->radio_pextra_minipage) {
        n = fl_get_button(epopup->radio_pextra_minipage);
        if (n) {
            fl_set_button(epopup->radio_pextra_indent,0);
            fl_set_button(epopup->radio_pextra_floatflt,0);
            fl_activate_object(epopup->input_pextra_width);
            fl_activate_object(epopup->input_pextra_widthp);
            fl_activate_object(epopup->radio_pextra_top);
            fl_activate_object(epopup->radio_pextra_middle);
            fl_activate_object(epopup->radio_pextra_bottom);
        } else {
            fl_deactivate_object(epopup->input_pextra_width);
            fl_deactivate_object(epopup->input_pextra_widthp);
            fl_deactivate_object(epopup->radio_pextra_top);
            fl_deactivate_object(epopup->radio_pextra_middle);
            fl_deactivate_object(epopup->radio_pextra_bottom);
            ActivateParagraphButtons();
        }
    } else if (ob == epopup->radio_pextra_floatflt) {
        n = fl_get_button(epopup->radio_pextra_floatflt);
        if (n) {
            fl_set_button(epopup->radio_pextra_indent,0);
            fl_set_button(epopup->radio_pextra_minipage,0);
            fl_activate_object(epopup->input_pextra_width);
            fl_activate_object(epopup->input_pextra_widthp);
            fl_activate_object(epopup->radio_pextra_top);
            fl_activate_object(epopup->radio_pextra_middle);
            fl_activate_object(epopup->radio_pextra_bottom);
        } else {
            fl_deactivate_object(epopup->input_pextra_width);
            fl_deactivate_object(epopup->input_pextra_widthp);
            fl_deactivate_object(epopup->radio_pextra_top);
            fl_deactivate_object(epopup->radio_pextra_middle);
            fl_deactivate_object(epopup->radio_pextra_bottom);
            ActivateParagraphButtons();
        }
    }
}

void FormParagraph::CheckParagraphVSpace(FL_OBJECT *obj, long)
{
    // "Synchronize" the choices and input fields, making it
    // impossible to commit senseless data.

    if (obj == ppopup->choice_space_above) {
        if (fl_get_choice (ppopup->choice_space_above) != 7) {
            fl_set_input (ppopup->input_space_above, "");
            ActivateParagraphButtons();
        }
    } else if (obj == ppopup->choice_space_below) {
        if (fl_get_choice (ppopup->choice_space_below) != 7) {
            fl_set_input (ppopup->input_space_below, "");
            ActivateParagraphButtons();
        }
    } else if (obj == ppopup->input_space_above) {
        string input = fl_get_input (ppopup->input_space_above);
        
        if (input.empty()) {
            fl_set_choice (ppopup->choice_space_above, 1);
            ActivateParagraphButtons();
        }
        else if (isValidGlueLength (input)) {
            fl_set_choice (ppopup->choice_space_above, 7);
            ActivateParagraphButtons();
        }
        else {
            fl_set_choice (ppopup->choice_space_above, 7);
            DeactivateParagraphButtons();
        }
    } else if (obj == ppopup->input_space_below) {
        string input = fl_get_input (ppopup->input_space_below);
        
        if (input.empty()) {
            fl_set_choice (ppopup->choice_space_below, 1);
            ActivateParagraphButtons();
        }
        else if (isValidGlueLength (input)) {
            fl_set_choice (ppopup->choice_space_below, 7);
            ActivateParagraphButtons();
        }
        else {
            fl_set_choice (ppopup->choice_space_below, 7);
            DeactivateParagraphButtons();
        }
    }
}

void FormParagraph::DeactivateParagraphButtons()
{
    fl_deactivate_object(tpopup->button_ok);
    fl_deactivate_object(tpopup->button_apply);
    fl_set_object_lcol(tpopup->button_ok, FL_INACTIVE);
    fl_set_object_lcol(tpopup->button_apply, FL_INACTIVE);
}

void FormParagraph::ActivateParagraphButtons()
{
    fl_activate_object(tpopup->button_ok);
    fl_activate_object(tpopup->button_apply);
    fl_set_object_lcol(tpopup->button_ok, FL_BLACK);
    fl_set_object_lcol(tpopup->button_apply, FL_BLACK);
}

void FormParagraph::DisableParagraphExtra ()
{
    DeactivateParagraphButtons();
    fl_deactivate_object (epopup->input_pextra_width);
    fl_deactivate_object (epopup->input_pextra_widthp);
    fl_deactivate_object (epopup->group_alignment);
    fl_deactivate_object (epopup->radio_pextra_top);
    fl_deactivate_object (epopup->radio_pextra_middle);
    fl_deactivate_object (epopup->radio_pextra_bottom);
    fl_deactivate_object (epopup->text_warning);
    fl_deactivate_object (epopup->group_extraopt);
    fl_deactivate_object (epopup->radio_pextra_indent);
    fl_deactivate_object (epopup->radio_pextra_minipage);
    fl_deactivate_object (epopup->radio_pextra_floatflt);
    fl_deactivate_object (epopup->radio_pextra_hfill);
    fl_deactivate_object (epopup->radio_pextra_startmp);
}	

void FormParagraph::EnableParagraphExtra ()
{
    ActivateParagraphButtons();
    fl_activate_object (epopup->input_pextra_width);
    fl_activate_object (epopup->input_pextra_widthp);
    fl_activate_object (epopup->group_alignment);
    fl_activate_object (epopup->radio_pextra_top);
    fl_activate_object (epopup->radio_pextra_middle);
    fl_activate_object (epopup->radio_pextra_bottom);
    fl_activate_object (epopup->text_warning);
    fl_activate_object (epopup->group_extraopt);
    fl_activate_object (epopup->radio_pextra_indent);
    fl_activate_object (epopup->radio_pextra_minipage);
    fl_activate_object (epopup->radio_pextra_floatflt);
    fl_activate_object (epopup->radio_pextra_hfill);
    fl_activate_object (epopup->radio_pextra_startmp);
}	

bool FormParagraph::CheckInputWidth()
{
    string
        s1,s2;

    s1 = fl_get_input(epopup->input_pextra_width);
    s2 = fl_get_input(epopup->input_pextra_widthp);
    if (s1.empty() && s2.empty()) {
        fl_activate_object(epopup->input_pextra_width);
        fl_activate_object(epopup->input_pextra_widthp);
        fl_hide_object(epopup->text_warning);
        DeactivateParagraphButtons();
        return false;
    }
    if (!s1.empty()) { // LyXLength parameter
        fl_activate_object(epopup->input_pextra_width);
        fl_deactivate_object(epopup->input_pextra_widthp);
        if (!isValidLength(s1)) {
            DeactivateParagraphButtons();
            fl_set_object_label(epopup->text_warning,
                                _("Warning: Invalid Length (valid example: 10mm)"));
            fl_show_object(epopup->text_warning);
            return false;
        }
    } else { // !s2.empty() % parameter
        fl_deactivate_object(epopup->input_pextra_width);
        fl_activate_object(epopup->input_pextra_widthp);
        if ((atoi(s2.c_str()) < 0 ) || (atoi(s2.c_str()) > 100)) {
            DeactivateParagraphButtons();
            fl_set_object_label(epopup->text_warning,
                                _("Warning: Invalid percent value (0-100)"));
            fl_show_object(epopup->text_warning);
            return false;
        }
    }
    fl_hide_object(epopup->text_warning);
    return true;
}
