// -*- C++ -*-
/* form_paragraph.h
 * FormParagraph Interface Class
 */

#ifndef FORM_PARAGRAPH_H
#define FORM_PARAGRAPH_H

#include "lsignal.h"
#include "gui_common.h"

class Communicator;
class Popups;

/** This class provides an XForms implementation of the FormParagraph Popup.
    The table-layout-form here changes values for latex-tabulars
 */
class FormParagraph : public virtual SignalBase {
public:
    /**@name Constructors and Destructors */
    //@{
    /// #FormParagraph x(Communicator ..., Popups ...);#
    FormParagraph(Communicator *, Popups *);
    ///
    ~FormParagraph();
    //@}
private:
    FormParagraph() {}
    FormParagraph(FormParagraph&) {}
    void DeactivateParagraphButtons();
    void ActivateParagraphButtons();
    void DisableParagraphExtra();
    void EnableParagraphExtra();
    bool CheckInputWidth();
    void CheckParagraphInput(FL_OBJECT *ob, long);
    void CheckParagraphVSpace(FL_OBJECT *ob, long);

    /**@name Real per-instance Callback Methods */
    //@{
    static int FormParagraphWMHideCB(FL_FORM *, void *);
    static void FormParagraphCancelCB(FL_OBJECT *, long);
    static void FormParagraphApplyCB(FL_OBJECT *, long);
    static void FormParagraphOKCB(FL_OBJECT *, long);
    static void FormParagraphInputCB(FL_OBJECT *, long);
    static void ParagraphVSpaceCB(FL_OBJECT *ob, long);
    static void UpdateParagraphCB(FL_OBJECT *ob, long);
    static void UpdateParagraphExtraCB(FL_OBJECT *ob, long);
    //@}
	
    /**@name Slot Methods */
    //@{
    /// Create the popup if necessary, update it and display it.
    void show();
    /// Hide the popup.
    void hide();
    /// Update the popup.
    void update();
    void pupdate();
    void eupdate();
    /// Apply from popup
    void apply();
    void papply();
    void eapply();
    //@}

	
    /// Build the popup
    void build();
    void build_pform();
    void build_eform();
    void build_tform();
    ///
    /// Explicitly free the popup.
    void free();
    /// Typedefinitions from the fdesign produced Header file
    typedef struct {
	FL_FORM *form_paragraph;
	void *vdata;
	char *cdata;
	long  ldata;
	FL_OBJECT *input_labelwidth;
	FL_OBJECT *check_lines_top;
	FL_OBJECT *check_lines_bottom;
	FL_OBJECT *check_pagebreaks_top;
	FL_OBJECT *check_pagebreaks_bottom;
	FL_OBJECT *check_noindent;
	FL_OBJECT *group_radio_alignment;
	FL_OBJECT *radio_align_right;
	FL_OBJECT *radio_align_left;
	FL_OBJECT *radio_align_block;
	FL_OBJECT *radio_align_center;
	FL_OBJECT *input_space_above;
	FL_OBJECT *input_space_below;
	FL_OBJECT *choice_space_above;
	FL_OBJECT *choice_space_below;
	FL_OBJECT *check_space_above;
	FL_OBJECT *check_space_below;
    } FD_form_paragraph;

    typedef struct {
	FL_FORM *form_paragraph_extra;
	void *vdata;
	char *cdata;
	long  ldata;
	FL_OBJECT *input_pextra_width;
	FL_OBJECT *input_pextra_widthp;
	FL_OBJECT *group_alignment;
	FL_OBJECT *radio_pextra_top;
	FL_OBJECT *radio_pextra_middle;
	FL_OBJECT *radio_pextra_bottom;
	FL_OBJECT *text_warning;
	FL_OBJECT *radio_pextra_hfill;
	FL_OBJECT *radio_pextra_startmp;
	FL_OBJECT *group_extraopt;
	FL_OBJECT *radio_pextra_indent;
	FL_OBJECT *radio_pextra_minipage;
	FL_OBJECT *radio_pextra_floatflt;
    } FD_form_paragraph_extra;

    typedef struct {
	FL_FORM *form_tabbed_paragraph;
	void *vdata;
	char *cdata;
	long  ldata;
        FL_OBJECT *tabbed_folder;
	FL_OBJECT *button_apply;
	FL_OBJECT *button_ok;
    } FD_form_tabbed_paragraph;

    /**@name Private Data */
    //@{
    /// Real GUI implementation.
    FD_form_paragraph *ppopup;
    FD_form_paragraph_extra *epopup;
    FD_form_tabbed_paragraph *tpopup;
    /// Which Communicator do we talk to?
    Communicator *comm;
    /// Update connection.
    Connection u;
    /// Hide connection.
    Connection h;
    /// has form contents changed? Used to control OK/Apply
    EnumPopupStatus status;
    //@}
    int ActCell, Confirmed;
};

#endif
