/* form_table.C
 * FormTable Interface Class Implementation
 */

#include <config.h>
#include "lyx_gui_misc.h"
#include "gettext.h"
#include FORMS_H_LOCATION

#include "form_tables.h"
#include "Popups.h"
#include "Communicator.h"
#include "insettabular.h"

// 	$Id: form_tables.C,v 1.10 1998/10/21 19:05:38 jug Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: form_tables.C,v 1.10 1998/10/21 19:05:38 jug Exp $";
#endif /* lint */


#ifdef __GNUG__
#pragma implementation
#endif


FormTable::FormTable(Communicator *c, Popups * p)
	: tpopup(0), opopup(0), cpopup(0), lpopup(0), epopup(0),
          comm(c), u(0), h(0)
{
    // let the popup be shown
    // This is a permanent connection so we won't bother
    // storing a copy because we won't be disconnecting.
    connect_to_method(p->showFormTable, this, &FormTable::show);
    connect_to_method(p->updateFormTable, this, &FormTable::update);
    connect_to_method(p->showFormTableNew, this, &FormTable::show_new);
    // NOTE:
    // parameter p is used instead of:
    //      comm->getPopups()
    // because we are being contructed from within the
    // Popups constructor and a segfault would occur
    // otherwise.  The segfault occurs because the Popups
    // constructor hasn't returned at this point and so
    // comm->getPopups() will not return the real popups
    // instance.  If this implementation fails see Popups.C
    // ARRae.
    build();
}


FormTable::~FormTable()
{
    free();
}

void FormTable::build()
{
    build_tform();
    build_oform();
    build_eform();
    build_cform();
    build_lform();
    build_nform();
    fl_set_form_atclose(tpopup->form_tabbed_table,
                        FormTable::FormTableWMHideCB, 0);
    fl_set_input_return(cpopup->input_column_width, FL_RETURN_CHANGED);
    fl_set_input_return(epopup->input_special_alignment, FL_RETURN_CHANGED);
    fl_set_input_return(epopup->input_special_multialign, FL_RETURN_CHANGED);
    fl_set_form_atclose(npopup->form_table, 
                        FormTable::FormTableNewWMHideCB, 0);
    fl_addto_tabfolder(tpopup->tabbed_folder,_("Cell"),
                       cpopup->form_cell_options);
    fl_addto_tabfolder(tpopup->tabbed_folder,_("Table"),
                       opopup->form_table_options);
    fl_addto_tabfolder(tpopup->tabbed_folder,_("Longtable"),
                       lpopup->form_longtable_options);
    fl_addto_tabfolder(tpopup->tabbed_folder,_("Extra"),
                       epopup->form_table_extra);
}

void FormTable::show()
{
    local_update(true);  // make sure its up-to-date
    if (tpopup->form_tabbed_table->visible) {
        fl_raise_form(tpopup->form_tabbed_table);
    } else {
        fl_show_form(tpopup->form_tabbed_table,
                     FL_PLACE_MOUSE | FL_FREE_SIZE,
                     FL_FULLBORDER,
                     _("Table Layout"));
        u = connect_to_method(comm->getPopups().updateBufferDependent,
                              this, &FormTable::update);
        h = connect_to_method(comm->getPopups().hideBufferDependent,
                              this, &FormTable::hide);
    }
}

void FormTable::show_new()
{
    if (npopup->form_table->visible) {
        fl_raise_form(npopup->form_table);
    } else {
        fl_show_form(npopup->form_table, FL_PLACE_MOUSE | FL_FREE_SIZE,
                     FL_FULLBORDER, _("Insert Table"));
    }
}

void FormTable::hide()
{
    if (tpopup->form_tabbed_table->visible) {
        fl_hide_form(tpopup->form_tabbed_table);
        if (epopup->form_table_extra->visible)
            fl_hide_form(epopup->form_table_extra);
        u.disconnect();
        h.disconnect();
    }
}

void FormTable::nhide()
{
    if (npopup->form_table->visible)
        fl_hide_form(npopup->form_table);
}

void FormTable::napply()
{
    int
        xsize,ysize;

    if (!comm->isAvailable()) return;

    comm->setMinibuffer(_("Inserting tabular inset..."));
    ysize = int(fl_get_slider_value(npopup->slider_columns) + 0.5);
    xsize = int(fl_get_slider_value(npopup->slider_rows) + 0.5);
    if (comm->openNewInset(new InsetTabular(comm->getBuffer(),xsize,ysize))) {
	    comm->setMinibuffer(_("Tabular mode"));
    }
}

void FormTable::update()
{
    if (tpopup && tpopup->form_tabbed_table->visible)
        local_update(true);
}

bool FormTable::local_update(bool flag)
{
    if (!comm->isAvailable() || !comm->hasLockingInset())
	return false;

    InsetTabular
        *inset = (InsetTabular *)comm->getLockingInset();

    if (inset->LyxCode() != Inset::TABULAR_CODE)
        return false;

    LyXTable
        *table = inset->tabular;
    int
        align,
        cell,
        column,row;
    char
        buf[12];
    string
        pwidth, special;

    cell = inset->GetActCell();
    ActCell = cell;
    column = table->column_of_cell(cell)+1;
    fl_set_object_label(tpopup->text_warning,"");
    Confirmed = false;
    fl_activate_object(epopup->input_special_alignment);
    fl_activate_object(epopup->input_special_multialign);
    fl_activate_object(cpopup->input_column_width);
    sprintf(buf,"%d",column);
    fl_set_input(tpopup->input_table_column, buf);
    fl_deactivate_object(tpopup->input_table_column);
    row = table->row_of_cell(cell)+1;
    sprintf(buf,"%d",row);
    fl_set_input(tpopup->input_table_row, buf);
    fl_deactivate_object(tpopup->input_table_row);
    if (table->IsMultiColumn(cell))
        fl_set_button(cpopup->radio_multicolumn, 1);
    else
        fl_set_button(cpopup->radio_multicolumn, 0);
    if (table->RotateCell(cell))
        fl_set_button(cpopup->radio_rotate_cell, 1);
    else
        fl_set_button(cpopup->radio_rotate_cell, 0);
    if (table->TopLine(cell))
        fl_set_button(cpopup->radio_border_top, 1);
    else
        fl_set_button(cpopup->radio_border_top, 0);
    if (table->BottomLine(cell))
        fl_set_button(cpopup->radio_border_bottom, 1);
    else
        fl_set_button(cpopup->radio_border_bottom, 0);
    if (table->LeftLine(cell))
        fl_set_button(cpopup->radio_border_left, 1);
    else
        fl_set_button(cpopup->radio_border_left, 0);
    if (table->RightLine(cell))
        fl_set_button(cpopup->radio_border_right, 1);
    else
        fl_set_button(cpopup->radio_border_right, 0);
    align = table->GetAlignment(cell);
    fl_set_button(cpopup->radio_align_left, 0);
    fl_set_button(cpopup->radio_align_right, 0);
    fl_set_button(cpopup->radio_align_center, 0);
    special = table->GetAlignSpecial(cell,LyXTable::SET_SPECIAL_COLUMN);
    if (flag)
        fl_set_input(epopup->input_special_alignment, special.c_str());
    if (comm->isReadOnly()) 
        fl_deactivate_object(epopup->input_special_alignment);
    special = table->GetAlignSpecial(cell,LyXTable::SET_SPECIAL_MULTI);
    if (flag)
        fl_set_input(epopup->input_special_multialign, special.c_str());
    if (comm->isReadOnly()) 
        fl_deactivate_object(epopup->input_special_multialign);
    pwidth = table->GetPWidth(cell);
    if (flag)
        fl_set_input(cpopup->input_column_width,pwidth.c_str());
    if (comm->isReadOnly()) {
        fl_deactivate_object(cpopup->input_column_width);
    } else {
        fl_activate_object(cpopup->input_column_width);
    }
    if (!pwidth.empty()) {
        fl_activate_object(cpopup->radio_linebreak_cell);
        fl_set_object_lcol(cpopup->radio_linebreak_cell, FL_BLACK);
        fl_set_button(cpopup->radio_linebreak_cell,
                      table->Linebreaks(table->FirstVirtualCell(cell)));
    } else {
        fl_deactivate_object(cpopup->radio_linebreak_cell);
        fl_set_object_lcol(cpopup->radio_linebreak_cell, FL_INACTIVE);
        fl_set_button(cpopup->radio_linebreak_cell,0);
    }
    if ((!pwidth.empty() && !table->IsMultiColumn(cell)) ||
        (align == LYX_ALIGN_LEFT))
        fl_set_button(cpopup->radio_align_left, 1);
    else if (align == LYX_ALIGN_RIGHT)
        fl_set_button(cpopup->radio_align_right, 1);
    else
        fl_set_button(cpopup->radio_align_center, 1);
    if (!pwidth.empty() && !table->IsMultiColumn(cell)) {
        fl_deactivate_object(cpopup->radio_align_left);
        fl_deactivate_object(cpopup->radio_align_right);
        fl_deactivate_object(cpopup->radio_align_center);
        fl_set_object_lcol(cpopup->radio_align_left, FL_INACTIVE);
        fl_set_object_lcol(cpopup->radio_align_right, FL_INACTIVE);
        fl_set_object_lcol(cpopup->radio_align_center, FL_INACTIVE);
    } else {
        fl_activate_object(cpopup->radio_align_left);
        fl_activate_object(cpopup->radio_align_right);
        fl_activate_object(cpopup->radio_align_center);
        fl_set_object_lcol(cpopup->radio_align_left, FL_BLACK);
        fl_set_object_lcol(cpopup->radio_align_right, FL_BLACK);
        fl_set_object_lcol(cpopup->radio_align_center, FL_BLACK);
    }
    fl_set_button(opopup->radio_longtable,table->IsLongTable());
    if (table->IsLongTable()) {
        fl_activate_object(lpopup->radio_lt_firsthead);
        fl_activate_object(lpopup->radio_lt_head);
        fl_activate_object(lpopup->radio_lt_foot);
        fl_activate_object(lpopup->radio_lt_lastfoot);
        fl_activate_object(lpopup->radio_lt_newpage);
        fl_set_object_lcol(lpopup->radio_lt_firsthead, FL_BLACK);
        fl_set_object_lcol(lpopup->radio_lt_head, FL_BLACK);
        fl_set_object_lcol(lpopup->radio_lt_foot, FL_BLACK);
        fl_set_object_lcol(lpopup->radio_lt_lastfoot, FL_BLACK);
        fl_set_object_lcol(lpopup->radio_lt_newpage, FL_BLACK);
        fl_set_button(lpopup->radio_lt_firsthead,
                      table->RowOfLTFirstHead(cell));
        fl_set_button(lpopup->radio_lt_head, table->RowOfLTHead(cell));
        fl_set_button(lpopup->radio_lt_foot, table->RowOfLTFoot(cell));
        fl_set_button(lpopup->radio_lt_lastfoot,
                      table->RowOfLTLastFoot(cell));
        fl_set_button(lpopup->radio_lt_newpage, table->LTNewPage(cell));
    } else {
        fl_deactivate_object(lpopup->radio_lt_firsthead);
        fl_deactivate_object(lpopup->radio_lt_head);
        fl_deactivate_object(lpopup->radio_lt_foot);
        fl_deactivate_object(lpopup->radio_lt_lastfoot);
        fl_deactivate_object(lpopup->radio_lt_newpage);
        fl_set_button(lpopup->radio_lt_firsthead,0);
        fl_set_button(lpopup->radio_lt_head,0);
        fl_set_button(lpopup->radio_lt_foot,0);
        fl_set_button(lpopup->radio_lt_lastfoot,0);
        fl_set_button(lpopup->radio_lt_newpage,0);
        fl_set_object_lcol(lpopup->radio_lt_firsthead, FL_INACTIVE);
        fl_set_object_lcol(lpopup->radio_lt_head, FL_INACTIVE);
        fl_set_object_lcol(lpopup->radio_lt_foot, FL_INACTIVE);
        fl_set_object_lcol(lpopup->radio_lt_lastfoot, FL_INACTIVE);
        fl_set_object_lcol(lpopup->radio_lt_newpage, FL_INACTIVE);
    }
    fl_set_button(opopup->radio_rotate_table,table->RotateTable());
    return true;
}

void FormTable::free()
{
    if (tpopup) {
        hide();
        fl_free_form(tpopup->form_tabbed_table);
        delete tpopup;
        tpopup = 0;
    }
    if (opopup) {
        fl_free_form(opopup->form_table_options);
        delete opopup;
        opopup = 0;
    }
    if (epopup) {
        fl_free_form(epopup->form_table_extra);
        delete epopup;
        epopup = 0;
    }
    if (cpopup) {
        fl_free_form(cpopup->form_cell_options);
        delete cpopup;
        cpopup = 0;
    }
    if (lpopup) {
        fl_free_form(lpopup->form_longtable_options);
        delete lpopup;
        lpopup = 0;
    }
    if (npopup) {
        nhide();
        fl_free_form(npopup->form_table);
        delete npopup;
        npopup = 0;
    }
}

int FormTable::FormTableWMHideCB(FL_FORM * form, void *)
{
    // Ensure that the signals (u and h) are disconnected even if the
    // window manager is used to close the popup.
    FormTable *pre = (FormTable*)form->u_vdata;
    pre->hide();
    return FL_CANCEL;
}

int FormTable::FormTableNewWMHideCB(FL_FORM * form, void *)
{
    FormTable *pre = (FormTable*)form->u_vdata;
    pre->nhide();
    return FL_CANCEL;
}

void FormTable::FormTableCloseCB(FL_OBJECT *ob, long)
{
    FormTable *pre = (FormTable*)ob->form->u_vdata;
    pre->hide();
}

void FormTable::FormTableNewCancelCB(FL_OBJECT *ob, long)
{
    FormTable *pre = (FormTable*)ob->form->u_vdata;
    pre->nhide();
}

void FormTable::FormTableNewOKCB(FL_OBJECT *ob, long)
{
    FormTable *pre = (FormTable*)ob->form->u_vdata;
    pre->napply();
    pre->nhide();
}

void FormTable::FormTableNewApplyCB(FL_OBJECT *ob, long)
{
    FormTable *pre = (FormTable*)ob->form->u_vdata;
    pre->napply();
}

void FormTable::FormTableInputCB(FL_OBJECT *ob, long)
{
    FormTable *pre = (FormTable*)ob->form->u_vdata;
    pre->SetTableOptions(ob,0);
}

void FormTable::SetTableOptions(FL_OBJECT *ob, long)
{
    if (!comm->isAvailable() || !comm->hasLockingInset())
	return;

    InsetTabular
        *inset = (InsetTabular *)comm->getLockingInset();

    if (inset->LyxCode() != Inset::TABULAR_CODE)
        return;

    LyXTable
        *table = inset->tabular;
    int
        cell,
        s,
        num = 0;
    string
        special,
        str;

    cell = inset->GetActCell();
    if (ActCell != cell) {
        local_update(false);
        fl_set_object_label(tpopup->text_warning,
                     _("Warning: Wrong Cursor position, updated window"));
        fl_show_object(tpopup->text_warning);
        return;
    }
    // No point in processing directives that you can't do anything with
    // anyhow, so exit now if the buffer is read-only.
    if (comm->isReadOnly()) {
      local_update(false);
      return;
    }
    if (ob != opopup->button_table_delete) {
        fl_set_object_label(tpopup->text_warning,"");
        Confirmed = false;
    }
    if (ob == cpopup->input_column_width) {
        string
            str;
        str = fl_get_input(ob);
        if (!str.empty() && !isValidLength(str)) {
            fl_set_object_label(tpopup->text_warning,
                 _("Warning: Invalid Length (valid example: 10mm)"));
            fl_show_object(tpopup->text_warning);
            return;
        }
        inset->TableFeatures(LyXTable::SET_PWIDTH,str);
        local_update(false); // update for alignment
        return;
    }
    str = fl_get_input(cpopup->input_column_width);
    if (!str.empty() && !isValidLength(str)) {
        fl_set_object_label(tpopup->text_warning,
                            _("Warning: Invalid Length (valid example: 10mm)"));
        fl_show_object(tpopup->text_warning);
        return;
    }
    if (((ob==opopup->button_delete_row) && (table->rows<=1)) ||
        ((ob==opopup->button_delete_column) && (table->columns<=1)))
        ob = opopup->button_table_delete;
    if (ob == opopup->button_append_row)
        num = LyXTable::APPEND_ROW;
    else if (ob == opopup->button_append_column)
        num = LyXTable::APPEND_COLUMN;
    else if (ob == opopup->button_delete_row)
        num = LyXTable::DELETE_ROW;
    else if (ob == opopup->button_delete_column)
        num = LyXTable::DELETE_COLUMN;
    else if (ob == opopup->button_set_borders)
        num = LyXTable::SET_ALL_LINES;
    else if (ob == opopup->button_unset_borders)
        num = LyXTable::UNSET_ALL_LINES;
    else if (ob == cpopup->radio_border_top)
        num = LyXTable::TOGGLE_LINE_TOP;
    else if (ob == cpopup->radio_border_bottom)
        num = LyXTable::TOGGLE_LINE_BOTTOM;
    else if (ob == cpopup->radio_border_left)
        num = LyXTable::TOGGLE_LINE_LEFT;
    else if (ob == cpopup->radio_border_right)
        num = LyXTable::TOGGLE_LINE_RIGHT;
    else if (ob == cpopup->radio_align_left)
        num = LyXTable::ALIGN_LEFT;
    else if (ob == cpopup->radio_align_right)
        num = LyXTable::ALIGN_RIGHT;
    else if (ob == cpopup->radio_align_center)
        num = LyXTable::ALIGN_CENTER;
    else if ((ob==opopup->button_table_delete) && !Confirmed) {
        fl_set_object_label(tpopup->text_warning,
                            _("Confirm: press Delete-Button again"));
        Confirmed = true;
        return;
    } else if ((ob == opopup->button_table_delete) 
	       && Confirmed) {
        num = LyXTable::DELETE_TABLE;
        Confirmed = false;
    } else if (ob == cpopup->radio_multicolumn)
        num = LyXTable::MULTICOLUMN;
    else if (ob == opopup->radio_longtable) {
        s=fl_get_button(opopup->radio_longtable);
        if (s) {
            num = LyXTable::SET_LONGTABLE;
            fl_activate_object(lpopup->radio_lt_firsthead);
            fl_activate_object(lpopup->radio_lt_head);
            fl_activate_object(lpopup->radio_lt_foot);
            fl_activate_object(lpopup->radio_lt_lastfoot);
            fl_activate_object(lpopup->radio_lt_newpage);
            fl_set_button(lpopup->radio_lt_firsthead,
                          table->RowOfLTFirstHead(cell));
            fl_set_button(lpopup->radio_lt_head, table->RowOfLTHead(cell));
            fl_set_button(lpopup->radio_lt_foot, table->RowOfLTFoot(cell));
            fl_set_button(lpopup->radio_lt_lastfoot,
                          table->RowOfLTLastFoot(cell));
            fl_set_button(lpopup->radio_lt_firsthead, table->LTNewPage(cell));
        } else {
	    num = LyXTable::UNSET_LONGTABLE;
            fl_deactivate_object(lpopup->radio_lt_firsthead);
            fl_deactivate_object(lpopup->radio_lt_head);
            fl_deactivate_object(lpopup->radio_lt_foot);
            fl_deactivate_object(lpopup->radio_lt_lastfoot);
            fl_deactivate_object(lpopup->radio_lt_newpage);
            fl_set_button(lpopup->radio_lt_firsthead,0);
            fl_set_button(lpopup->radio_lt_head,0);
            fl_set_button(lpopup->radio_lt_foot,0);
            fl_set_button(lpopup->radio_lt_lastfoot,0);
            fl_set_button(lpopup->radio_lt_newpage,0);
	    fl_set_object_lcol(lpopup->radio_lt_firsthead, FL_INACTIVE);
	    fl_set_object_lcol(lpopup->radio_lt_head, FL_INACTIVE);
	    fl_set_object_lcol(lpopup->radio_lt_foot, FL_INACTIVE);
	    fl_set_object_lcol(lpopup->radio_lt_lastfoot, FL_INACTIVE);
	    fl_set_object_lcol(lpopup->radio_lt_newpage, FL_INACTIVE);
        }
    } else if (ob == opopup->radio_rotate_table) {
        s=fl_get_button(opopup->radio_rotate_table);
	if (s)
            num = LyXTable::SET_ROTATE_TABLE;
	else
	    num = LyXTable::UNSET_ROTATE_TABLE;
    } else if (ob == cpopup->radio_rotate_cell) {
        s=fl_get_button(cpopup->radio_rotate_cell);
	if (s)
            num = LyXTable::SET_ROTATE_CELL;
	else
	    num = LyXTable::UNSET_ROTATE_CELL;
    } else if (ob == cpopup->radio_linebreak_cell) {
        num = LyXTable::SET_LINEBREAKS;
    } else if (ob == lpopup->radio_lt_firsthead) {
        num = LyXTable::SET_LTFIRSTHEAD;
    } else if (ob == lpopup->radio_lt_head) {
        num = LyXTable::SET_LTHEAD;
    } else if (ob == lpopup->radio_lt_foot) {
        num = LyXTable::SET_LTFOOT;
    } else if (ob == lpopup->radio_lt_lastfoot) {
        num = LyXTable::SET_LTLASTFOOT;
    } else if (ob == lpopup->radio_lt_newpage) {
        num = LyXTable::SET_LTNEWPAGE;
    } else if (ob == epopup->input_special_alignment) {
        special = fl_get_input(epopup->input_special_alignment);
        num = LyXTable::SET_SPECIAL_COLUMN;
    } else if (ob == epopup->input_special_multialign) {
        special = fl_get_input(epopup->input_special_multialign);
        num = LyXTable::SET_SPECIAL_MULTI;
    } else
        return;
    
    inset->TableFeatures(num,special);
    local_update(false);
}

/*
 * ----------------------------------------------------------------------------
 *               CODE INSERTED AND MODIFIED FROM forms/form_tables.fd
 * ----------------------------------------------------------------------------
 */
void FormTable::build_tform()
{
    FL_OBJECT *obj;
    tpopup = (FD_form_tabbed_table *) fl_calloc(1, sizeof(*tpopup));

    tpopup->form_tabbed_table = fl_bgn_form(FL_NO_BOX, 510, 340);
    tpopup->form_tabbed_table->u_vdata = this;

    obj = fl_add_box(FL_FLAT_BOX,0,0,510,340,"");
    tpopup->tabbed_folder = obj = fl_add_tabfolder(FL_TOP_TABFOLDER,10,10,485,240,_("Tabbed folder"));
    fl_set_object_resize(obj, FL_RESIZE_ALL);
    tpopup->button_close = obj = fl_add_button(FL_RETURN_BUTTON,395,300,100,30,_("Close"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormTableCloseCB,0);
    tpopup->input_table_column = obj = fl_add_input(FL_NORMAL_INPUT,15,300,60,30,_("Column"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_TOP_LEFT);
    tpopup->input_table_row = obj = fl_add_input(FL_NORMAL_INPUT,85,300,60,30,_("Row"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_TOP_LEFT);
    tpopup->text_warning = obj = fl_add_text(FL_NORMAL_TEXT,10,250,485,30,"");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_end_form();

    tpopup->form_tabbed_table->fdui = tpopup;
}

void FormTable::build_oform()
{
    FL_OBJECT *obj;
    opopup = (FD_form_table_options *) fl_calloc(1, sizeof(*opopup));
    
    opopup->form_table_options = fl_bgn_form(FL_NO_BOX, 485, 220);
    opopup->form_table_options->u_vdata = this;

    obj = fl_add_box(FL_FLAT_BOX,0,0,485,220,"");
    obj = fl_add_frame(FL_ENGRAVED_FRAME,280,20,120,75,"");
    fl_set_object_color(obj,FL_COL1,FL_COL1);
    opopup->button_append_column = obj = fl_add_button(FL_NORMAL_BUTTON,10,40,120,30,idex(_("Append Column|#A")));fl_set_button_shortcut(obj,scex(_("Append Column|#A")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormTableInputCB,0);
    opopup->button_delete_column = obj = fl_add_button(FL_NORMAL_BUTTON,140,40,120,30,idex(_("Delete Column|#O")));fl_set_button_shortcut(obj,scex(_("Delete Column|#O")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormTableInputCB,0);
    opopup->button_append_row = obj = fl_add_button(FL_NORMAL_BUTTON,10,70,120,30,idex(_("Append Row|#p")));fl_set_button_shortcut(obj,scex(_("Append Row|#p")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormTableInputCB,0);
    opopup->button_delete_row = obj = fl_add_button(FL_NORMAL_BUTTON,140,70,120,30,idex(_("Delete Row|#w")));fl_set_button_shortcut(obj,scex(_("Delete Row|#w")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormTableInputCB,0);
    opopup->button_table_delete = obj = fl_add_button(FL_NORMAL_BUTTON,10,110,250,30,idex(_("Delete Table|#D")));fl_set_button_shortcut(obj,scex(_("Delete Table|#D")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormTableInputCB,0);
    opopup->button_set_borders = obj = fl_add_button(FL_NORMAL_BUTTON,10,10,120,30,idex(_("Set Borders|#S")));fl_set_button_shortcut(obj,scex(_("Set Borders|#S")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormTableInputCB,0);
    opopup->button_unset_borders = obj = fl_add_button(FL_NORMAL_BUTTON,140,10,120,30,idex(_("Unset Borders|#U")));fl_set_button_shortcut(obj,scex(_("Unset Borders|#U")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormTableInputCB,0);
    opopup->radio_longtable = obj = fl_add_checkbutton(FL_PUSH_BUTTON,280,30,90,25,_("Longtable"));
    fl_set_object_callback(obj,FormTableInputCB,0);
    opopup->radio_rotate_table = obj = fl_add_checkbutton(FL_PUSH_BUTTON,280,55,90,25,idex(_("Rotate 90|#9")));fl_set_button_shortcut(obj,scex(_("Rotate 90|#9")),1);
    fl_set_object_callback(obj,FormTableInputCB,0);
    obj = fl_add_text(FL_NORMAL_TEXT,285,10,100,20,_("Spec. Table"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_end_form();

    opopup->form_table_options->fdui = opopup;
}

void FormTable::build_eform()
{
    FL_OBJECT *obj;
    epopup = (FD_form_table_extra *) fl_calloc(1, sizeof(*epopup));

    epopup->form_table_extra = fl_bgn_form(FL_NO_BOX, 485, 220);
    epopup->form_table_extra->u_vdata = this;
    
    obj = fl_add_box(FL_FLAT_BOX,0,0,485,220,"");
    obj = fl_add_frame(FL_ENGRAVED_FRAME,15,95,305,60,"");
    fl_set_object_color(obj,FL_COL1,FL_COL1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    obj = fl_add_text(FL_NORMAL_TEXT,20,85,205,20,_("Special Multicolumn Alignment"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    obj = fl_add_frame(FL_ENGRAVED_FRAME,15,20,305,60,"");
    fl_set_object_color(obj,FL_COL1,FL_COL1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    epopup->input_special_alignment = obj = fl_add_input(FL_NORMAL_INPUT,25,35,280,30,"");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormTableInputCB,0);
    obj = fl_add_text(FL_NORMAL_TEXT,20,10,170,20,_("Special Column Alignment"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    epopup->input_special_multialign = obj = fl_add_input(FL_NORMAL_INPUT,25,110,280,30,"");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormTableInputCB,0);
    fl_end_form();

    epopup->form_table_extra->fdui = epopup;
}

void FormTable::build_cform()
{
    FL_OBJECT *obj;
    cpopup = (FD_form_cell_options *) fl_calloc(1, sizeof(*cpopup));

    cpopup->form_cell_options = fl_bgn_form(FL_NO_BOX, 485, 220);
    cpopup->form_cell_options->u_vdata = this;
    
    obj = fl_add_box(FL_FLAT_BOX,0,0,485,220,"");
    obj = fl_add_frame(FL_ENGRAVED_FRAME,20,20,145,105,"");
    fl_set_object_color(obj,FL_COL1,FL_COL1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    obj = fl_add_text(FL_NORMAL_TEXT,25,10,70,20,_("Borders"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    cpopup->radio_border_top = obj = fl_add_checkbutton(FL_PUSH_BUTTON,75,45,25,25,idex(_("Top|#T")));fl_set_button_shortcut(obj,scex(_("Top|#T")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_TOP_LEFT);
    fl_set_object_callback(obj,FormTableInputCB,0);
    cpopup->radio_border_bottom = obj = fl_add_checkbutton(FL_PUSH_BUTTON,75,75,25,25,idex(_("Bottom|#B")));fl_set_button_shortcut(obj,scex(_("Bottom|#B")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_BOTTOM_LEFT);
    fl_set_object_callback(obj,FormTableInputCB,0);
    cpopup->radio_border_left = obj = fl_add_checkbutton(FL_PUSH_BUTTON,55,60,25,25,idex(_("Left|#L")));fl_set_button_shortcut(obj,scex(_("Left|#L")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT);
    fl_set_object_callback(obj,FormTableInputCB,0);
    cpopup->radio_border_right = obj = fl_add_checkbutton(FL_PUSH_BUTTON,95,60,25,25,idex(_("Right|#R")));fl_set_button_shortcut(obj,scex(_("Right|#R")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_RIGHT);
    fl_set_object_callback(obj,FormTableInputCB,0);
    obj = fl_add_frame(FL_ENGRAVED_FRAME,200,20,105,105,"");
    fl_set_object_color(obj,FL_COL1,FL_COL1);
    obj = fl_add_text(FL_NORMAL_TEXT,205,10,90,20,_("Alignment"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    cpopup->radio_align_left = obj = fl_add_checkbutton(FL_RADIO_BUTTON,200,30,80,25,idex(_("Left|#e")));fl_set_button_shortcut(obj,scex(_("Left|#e")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormTableInputCB,0);
    cpopup->radio_align_right = obj = fl_add_checkbutton(FL_RADIO_BUTTON,200,55,80,25,idex(_("Right|#i")));fl_set_button_shortcut(obj,scex(_("Right|#i")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormTableInputCB,0);
    cpopup->radio_align_center = obj = fl_add_checkbutton(FL_RADIO_BUTTON,200,80,80,25,idex(_("Center|#C")));fl_set_button_shortcut(obj,scex(_("Center|#C")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormTableInputCB,0);
    obj = fl_add_frame(FL_ENGRAVED_FRAME,340,20,110,105,"");
    obj = fl_add_text(FL_NORMAL_TEXT,345,10,100,20,_("Special Cell"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    cpopup->radio_multicolumn = obj = fl_add_checkbutton(FL_PUSH_BUTTON,340,30,95,25,idex(_("Multicolumn|#M")));fl_set_button_shortcut(obj,scex(_("Multicolumn|#M")),1);
    fl_set_object_callback(obj,FormTableInputCB,0);
    cpopup->radio_linebreak_cell = obj = fl_add_checkbutton(FL_PUSH_BUTTON,340,55,95,25,idex(_("Linebreaks|#N")));fl_set_button_shortcut(obj,scex(_("Linebreaks|#N")),1);
    fl_set_object_callback(obj,FormTableInputCB,0);
    cpopup->radio_rotate_cell = obj = fl_add_checkbutton(FL_PUSH_BUTTON,340,80,95,25,_("Rotate 90"));
    fl_set_object_callback(obj,FormTableInputCB,0);
    cpopup->input_column_width = obj = fl_add_input(FL_NORMAL_INPUT,355,145,95,30,_("Width of cell:"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormTableInputCB,0);
    fl_end_form();

    cpopup->form_cell_options->fdui = cpopup;
}

void FormTable::build_lform()
{
    FL_OBJECT *obj;
    lpopup = (FD_form_longtable_options *) fl_calloc(1, sizeof(*lpopup));

    lpopup->form_longtable_options = fl_bgn_form(FL_NO_BOX, 485, 220);
    lpopup->form_longtable_options->u_vdata = this;
    
    obj = fl_add_box(FL_FLAT_BOX,0,0,485,220,"");
    obj = fl_add_frame(FL_ENGRAVED_FRAME,10,20,100,75,"");
    obj = fl_add_text(FL_NORMAL_TEXT,15,10,85,20,_("Header"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    lpopup->radio_lt_firsthead = obj = fl_add_checkbutton(FL_PUSH_BUTTON,15,30,90,25,_("First Head"));
    fl_set_object_callback(obj,FormTableInputCB,0);
    lpopup->radio_lt_head = obj = fl_add_checkbutton(FL_PUSH_BUTTON,15,60,90,25,_("Head"));
    fl_set_object_callback(obj,FormTableInputCB,0);
    obj = fl_add_frame(FL_ENGRAVED_FRAME,130,20,100,75,"");
    obj = fl_add_text(FL_NORMAL_TEXT,135,10,85,20,_("Footer"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    lpopup->radio_lt_foot = obj = fl_add_checkbutton(FL_PUSH_BUTTON,135,30,90,25,_("Foot"));
    fl_set_object_callback(obj,FormTableInputCB,0);
    lpopup->radio_lt_lastfoot = obj = fl_add_checkbutton(FL_PUSH_BUTTON,135,60,90,25,_("Last Foot"));
    fl_set_object_callback(obj,FormTableInputCB,0);
    obj = fl_add_frame(FL_ENGRAVED_FRAME,245,20,95,75,"");
    obj = fl_add_text(FL_NORMAL_TEXT,250,10,85,20,_("Special"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    lpopup->radio_lt_newpage = obj = fl_add_checkbutton(FL_PUSH_BUTTON,250,30,90,25,_("New Page"));
    fl_set_object_callback(obj,FormTableInputCB,0);
    fl_end_form();

    lpopup->form_longtable_options->fdui = lpopup;
}

void FormTable::build_nform()
{
    FL_OBJECT *obj;
    npopup = (FD_form_table *) fl_calloc(1, sizeof(*npopup));
    
    npopup->form_table = fl_bgn_form(FL_NO_BOX, 310, 130);
    npopup->form_table->u_vdata = this;

    obj = fl_add_box(FL_FLAT_BOX,0,0,310,130,"");
    npopup->button_ok = obj = fl_add_button(FL_RETURN_BUTTON,10,90,90,30,_("OK"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormTableNewOKCB,0);
    npopup->button_apply = obj = fl_add_button(FL_NORMAL_BUTTON,110,90,90,30,idex(_("Apply|#A")));fl_set_button_shortcut(obj,scex(_("Apply|#A")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormTableNewApplyCB,0);
    npopup->button_cancel = obj = fl_add_button(FL_NORMAL_BUTTON,210,90,90,30,idex(_("Cancel|^[")));fl_set_button_shortcut(obj,scex(_("Cancel|^[")),1);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,FormTableNewCancelCB,0);
    npopup->slider_columns = obj = fl_add_valslider(FL_HOR_SLIDER,80,50,220,30,_("Columns"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT);
    fl_set_object_resize(obj, FL_RESIZE_X);
    npopup->slider_rows = obj = fl_add_valslider(FL_HOR_SLIDER,80,10,220,30,_("Rows"));
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT);
    fl_set_object_resize(obj, FL_RESIZE_X);
    fl_end_form();
    
    npopup->form_table->fdui = npopup;
    fl_set_slider_bounds(npopup->slider_rows,1,50);
    fl_set_slider_bounds(npopup->slider_columns,1,50);
    fl_set_slider_value(npopup->slider_rows, 5);
    fl_set_slider_value(npopup->slider_columns, 5);
    fl_set_slider_precision(npopup->slider_rows, 0);
    fl_set_slider_precision(npopup->slider_columns, 0);
}
