// -*- C++ -*-
/* form_table.h
 * FormTable Interface Class
 */

#ifndef FORM_TABLE_H
#define FORM_TABLE_H

#include "lsignal.h"

class Communicator;
class Popups;

/** This class provides an XForms implementation of the FormTable Popup.
    The table-layout-form here changes values for latex-tabulars
 */
class FormTable : public virtual SignalBase {
public:
    /**@name Constructors and Destructors */
    //@{
    /// #FormTable x(Communicator ..., Popups ...);#
    FormTable(Communicator *, Popups *);
    ///
    ~FormTable();
    //@}
private:
    FormTable() {}
    FormTable(FormTable&) {}

    /**@name Real per-instance Callback Methods */
    //@{
    static int FormTableWMHideCB(FL_FORM *, void *);
    static int FormTableSpeWMHideCB(FL_FORM *, void *);
    static int FormTableNewWMHideCB(FL_FORM *, void *);
    static void FormTableCloseCB(FL_OBJECT *, long);
    static void FormTableSpeCloseCB(FL_OBJECT *, long);
    static void FormTableNewCancelCB(FL_OBJECT *, long);
    static void FormTableNewOKCB(FL_OBJECT *, long);
    static void FormTableNewApplyCB(FL_OBJECT *, long);
    static void FormTableInputCB(FL_OBJECT *, long);
    //@}
	
    /**@name Slot Methods */
    //@{
    /// Create the popup if necessary, update it and display it.
    void show();
    void show_new();
    /// Hide the popup.
    void hide();
    void nhide();
    /// Update the popup.
    bool local_update(bool flag);
    void update();
    /// Apply from popup
    void napply();
    ///
    void SetTableOptions(FL_OBJECT *ob, long);
    //@}

    /// Build the popup
    void build();
    //
    void build_oform(); // table options
    void build_eform(); // extra options
    void build_cform(); // cell options
    void build_lform(); // longtable options
    void build_tform(); // tabbed folder
    void build_nform(); // new table form
    /// Explicitly free the popup.
    void free();

    typedef struct {
	FL_FORM *form_tabbed_table;
	void *vdata;
	char *cdata;
	long  ldata;
	FL_OBJECT *tabbed_folder;
	FL_OBJECT *button_close;
	FL_OBJECT *input_table_column;
	FL_OBJECT *input_table_row;
	FL_OBJECT *text_warning;
    } FD_form_tabbed_table;

    typedef struct {
	FL_FORM *form_table_options;
	void *vdata;
	char *cdata;
	long  ldata;
	FL_OBJECT *button_append_column;
	FL_OBJECT *button_delete_column;
	FL_OBJECT *button_append_row;
	FL_OBJECT *button_delete_row;
	FL_OBJECT *button_table_delete;
	FL_OBJECT *button_set_borders;
	FL_OBJECT *button_unset_borders;
	FL_OBJECT *radio_longtable;
	FL_OBJECT *radio_rotate_table;
    } FD_form_table_options;

    typedef struct {
	FL_FORM *form_cell_options;
	void *vdata;
	char *cdata;
	long  ldata;
	FL_OBJECT *radio_border_top;
	FL_OBJECT *radio_border_bottom;
	FL_OBJECT *radio_border_left;
	FL_OBJECT *radio_border_right;
	FL_OBJECT *radio_align_left;
	FL_OBJECT *radio_align_right;
	FL_OBJECT *radio_align_center;
	FL_OBJECT *radio_multicolumn;
	FL_OBJECT *radio_linebreak_cell;
	FL_OBJECT *radio_rotate_cell;
	FL_OBJECT *input_column_width;
    } FD_form_cell_options;

    typedef struct {
	FL_FORM *form_longtable_options;
	void *vdata;
	char *cdata;
	long  ldata;
	FL_OBJECT *radio_lt_firsthead;
	FL_OBJECT *radio_lt_head;
	FL_OBJECT *radio_lt_foot;
	FL_OBJECT *radio_lt_lastfoot;
	FL_OBJECT *radio_lt_newpage;
    } FD_form_longtable_options;

     typedef struct {
	FL_FORM *form_table_extra;
	void *vdata;
	char *cdata;
	long  ldata;
	FL_OBJECT *input_special_alignment;
	FL_OBJECT *input_special_multialign;
    } FD_form_table_extra;

    typedef struct {
	FL_FORM *form_table;
	void *vdata;
	char *cdata;
	long  ldata;
	FL_OBJECT *button_ok;
	FL_OBJECT *button_apply;
	FL_OBJECT *button_cancel;
	FL_OBJECT *slider_columns;
	FL_OBJECT *slider_rows;
    } FD_form_table;

    /**@name Private Data */
    //@{
    /// Real GUI implementation.
    FD_form_tabbed_table *tpopup;
    FD_form_table_options *opopup;
    FD_form_cell_options *cpopup;
    FD_form_longtable_options *lpopup;
    FD_form_table_extra *epopup;
    FD_form_table *npopup;
    /// Which Communicator do we talk to?
    Communicator *comm;
    /// Update connection.
    Connection u;
    /// Hide connection.
    Connection h;
    //@}
    int ActCell, Confirmed;
};

#endif
