/* latexpreamble.C
 * LaTeXPreamble Interface Class Implementation
 */

#include <config.h>
#include "lyx_gui_misc.h"
#include "gettext.h"
#include FORMS_H_LOCATION

#include "latexpreamble.h"
#include "Communicator.h"
#include "Popups.h"

// 	$Id: latexpreamble.C,v 1.13 1998/09/22 16:27:55 asger Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: latexpreamble.C,v 1.13 1998/09/22 16:27:55 asger Exp $";
#endif /* lint */


#ifdef __GNUG__
#pragma implementation
#endif


LaTeXPreamble::LaTeXPreamble(Communicator *c, Popups * p)
	: popup(0), comm(c), u(0), h(0), status(POPUP_UNMODIFIED)
{
	// let the popup be shown
	// This is a permanent connection so we won't bother
	// storing a copy because we won't be disconnecting.
	connect_to_method(p->showLaTeXPreamble,
			  this,
			  &LaTeXPreamble::show);
	// NOTE:
	// parameter p is used instead of:
	//      comm->getPopups()
	// because we are being contructed from within the
	// Popups constructor and a segfault would occur
	// otherwise.  The segfault occurs because the Popups
	// constructor hasn't returned at this point and so
	// comm->getPopups() will not return the real popups
	// instance.  If this implementation fails see Popups.C
	// ARRae.
}


LaTeXPreamble::~LaTeXPreamble()
{
	free();
}


void LaTeXPreamble::build()
{
	FL_OBJECT *obj;
	FD_form_preamble *fdui = popup = new FD_form_preamble;

	fdui->form_preamble = fl_bgn_form(FL_NO_BOX, 380, 420);
	fdui->form_preamble->u_vdata = this;

	obj = fl_add_box(FL_UP_BOX,0,0,380,420,"");
	fdui->button_ok = obj = fl_add_button(FL_NORMAL_BUTTON,50,380,100,30,idex(_("OK|#O")));
	fl_set_button_shortcut(obj,scex(_("OK|#O")),1);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_gravity(obj, FL_SouthEast, FL_SouthEast);
	fl_set_object_callback(obj,LaTeXPreamble::PreambleOKCB,0);
	fdui->button_apply = obj = fl_add_button(FL_NORMAL_BUTTON,160,380,100,30,idex(_("Apply|#A")));
	fl_set_button_shortcut(obj,scex(_("Apply|#A")),1);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_gravity(obj, FL_SouthEast, FL_SouthEast);
	fl_set_object_callback(obj,LaTeXPreamble::PreambleApplyCB,0);
	fdui->button_cancel = obj = fl_add_button(FL_NORMAL_BUTTON,270,380,100,30,idex(_("Cancel|^[")));
	fl_set_button_shortcut(obj,scex(_("Cancel|^[")),1);
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_gravity(obj, FL_SouthEast, FL_SouthEast);
	fl_set_object_callback(obj,LaTeXPreamble::PreambleCancelCB,0);
	fdui->input_preamble = obj = fl_add_input(FL_MULTILINE_INPUT,10,10,360,360,"");
	fl_set_object_lsize(obj,FL_NORMAL_SIZE);
	fl_set_object_gravity(obj, FL_NorthWest, FL_SouthEast);
	fl_set_object_callback(obj,LaTeXPreamble::PreambleInputCB,0);
	fl_end_form();

	fdui->form_preamble->fdui = fdui;
}


void LaTeXPreamble::show()
{
	if (!popup) {
		build();
		fl_set_input_return(popup->input_preamble,
				    FL_RETURN_CHANGED);
		fl_set_form_atclose(popup->form_preamble,
				    LaTeXPreamble::PreambleWMHideCB, 0);
	}
	
	update();  // make sure its up-to-date
	
	if (popup->form_preamble->visible) {
		fl_raise_form(popup->form_preamble);
	} else {
		fl_show_form(popup->form_preamble,
			     FL_PLACE_MOUSE | FL_FREE_SIZE,
			     FL_FULLBORDER,
			     _("LaTeX Preamble"));
		u = connect_to_method(
				comm->getPopups().updateBufferDependent,
				this,
				&LaTeXPreamble::update);
		h = connect_to_method(
				comm->getPopups().hideBufferDependent,
				this,
				&LaTeXPreamble::hide);
	}
}


void LaTeXPreamble::hide()
{
	if (popup
	    && popup->form_preamble
	    && popup->form_preamble->visible) {
		fl_hide_form(popup->form_preamble);
		u.disconnect();
		h.disconnect();
	}
	free();
}


void LaTeXPreamble::apply()
{
	if (!comm->isAvailable()) {
		return;
	}
	
	comm->setPreamble(fl_get_input(popup->input_preamble));
	fl_deactivate_object(popup->button_ok);
	fl_deactivate_object(popup->button_apply);
	fl_set_object_lcol(popup->button_ok, FL_INACTIVE);
	fl_set_object_lcol(popup->button_apply, FL_INACTIVE);
	status = POPUP_UNMODIFIED;
}


void LaTeXPreamble::update()
{
	if (popup
	    && comm->isAvailable()) {
		fl_set_input(popup->input_preamble,
			     comm->getPreamble());
		
		if (comm->isReadOnly()) {
			// deactivating an input doesn't seem to work
			// at least not up to 0.88, so we remember instead.
			status = POPUP_READONLY;
			fl_deactivate_object(popup->input_preamble);
		} else {
			status = POPUP_UNMODIFIED;
			fl_activate_object(popup->input_preamble);
		}
		fl_deactivate_object(popup->button_ok);
		fl_deactivate_object(popup->button_apply);
		fl_set_object_lcol(popup->button_ok, FL_INACTIVE);
		fl_set_object_lcol(popup->button_apply, FL_INACTIVE);
	}
}


void LaTeXPreamble::free()
{
	// we don't need to delete u and h here because
	// hide() does that after disconnecting.
	if (popup) {
		if (popup->form_preamble
		    && popup->form_preamble->visible) {
			hide();
		}
		fl_free_form(popup->form_preamble);
		delete popup;
		popup = 0;
	}
}


int LaTeXPreamble::PreambleWMHideCB(FL_FORM * form, void *)
{
	// Ensure that the signals (u and h) are disconnected even if the
	// window manager is used to close the popup.
	LaTeXPreamble *pre = (LaTeXPreamble*)form->u_vdata;
	pre->hide();
	return FL_CANCEL;
}


void LaTeXPreamble::PreambleOKCB(FL_OBJECT *ob, long)
{
	LaTeXPreamble *pre = (LaTeXPreamble*)ob->form->u_vdata;
	pre->apply();
	pre->hide();
}


void LaTeXPreamble::PreambleApplyCB(FL_OBJECT *ob, long)
{
	LaTeXPreamble *pre = (LaTeXPreamble*)ob->form->u_vdata;
	pre->apply();
}


void LaTeXPreamble::PreambleCancelCB(FL_OBJECT *ob, long)
{
	LaTeXPreamble *pre = (LaTeXPreamble*)ob->form->u_vdata;
	pre->hide();
}


void LaTeXPreamble::PreambleInputCB(FL_OBJECT *ob, long)
{
	LaTeXPreamble *pre = (LaTeXPreamble*)ob->form->u_vdata;
	if (pre->status == POPUP_UNMODIFIED) {
		pre->status = POPUP_MODIFIED;
		fl_activate_object(pre->popup->button_ok);
		fl_activate_object(pre->popup->button_apply);
		fl_set_object_lcol(pre->popup->button_ok, FL_BLACK);
		fl_set_object_lcol(pre->popup->button_apply, FL_BLACK);
	}
}
