// -*- C++ -*-
/* latexpreamble.h
 * LaTeXPreamble Interface Class
 */

#ifndef LATEXPREAMBLE_H
#define LATEXPREAMBLE_H

#include "lsignal.h"
#include "gui_common.h"

class Communicator;
class Popups;

/** This class provides an XForms implementation of the LaTeXPreamble Popup.
    The LaTeX preamble popup allows user access to a buffers preamble.
 */
class LaTeXPreamble : public virtual SignalBase {
public:
	/**@name Constructors and Destructors */
	//@{
	/// #LaTeXPreamble x(Communicator ..., Popups ...);#
	LaTeXPreamble(Communicator *, Popups *);
	///
	~LaTeXPreamble();
	//@}
private:
	LaTeXPreamble() {}
	LaTeXPreamble(LaTeXPreamble&) {}

	/**@name Real per-instance Callback Methods */
	//@{
	static int PreambleWMHideCB(FL_FORM *, void *);
	static void PreambleOKCB(FL_OBJECT *, long);
	static void PreambleApplyCB(FL_OBJECT *, long);
	static void PreambleCancelCB(FL_OBJECT *, long);
	static void PreambleInputCB(FL_OBJECT *, long);
	//@}
	
	/**@name Slot Methods */
	//@{
	/// Create the popup if necessary, update it and display it.
	void show();
	/// Hide the popup.
	void hide();
	/// Update the popup.
	void update();
	/// Apply from popup
	void apply();
	//@}

	
	/// Build the popup
	void build();
	/// Explicitly free the popup.
	void free();

	typedef struct {
		FL_FORM *form_preamble;
		void *vdata;
		char *cdata;
		long  ldata;
		FL_OBJECT *button_ok;
		FL_OBJECT *button_apply;
		FL_OBJECT *button_cancel;
		FL_OBJECT *input_preamble;
	} FD_form_preamble;

	/**@name Private Data */
	//@{
	/// Real GUI implementation.
	FD_form_preamble *popup;
	/// Which LyXView do we belong to?
	Communicator *comm;
	/// Update connection.
	Connection u;
	/// Hide connection.
	Connection h;
	/// has form contents changed? Used to control OK/Apply
	EnumPopupStatus status;
	//@}
};

#endif
