/* This file is part of
* ======================================================
* 
*           LyX, The Document Processor
*
*	    Copyright (C) 1995 Matthias Ettrich
*           Copyright (C) 1995-1998 The LyX Team.
*
*           This file is Copyright 1996-1998
*           Lars Gullik Bjnnes
*
*======================================================*/

//  Added pseudo-action handling, asierra 180296

#include <config.h>

#ifdef __GNUG__
#pragma implementation
#endif

#include "lyx_main.h"
#include "lyx_gui_misc.h"
#include "lyx.h"
#include "toolbar.h"
#include "LyXAction.h"
#include "lyxfunc.h"
#include "debug.h"
#include "filetools.h"
#include "combox.h"
#include "LyXView.h"
#include "lstrings.h"

// 	$Id: toolbar.C,v 1.16 1998/10/22 16:56:23 larsbj Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: toolbar.C,v 1.16 1998/10/22 16:56:23 larsbj Exp $";
#endif /* lint */

extern void LayoutsCB(int, void*);
extern char** get_pixmap_from_symbol(char const *arg, int, int);
extern LyXAction lyxaction;


Toolbar::Toolbar(ToolbarBackend const &rct, LyXView *o, int x, int y)
	: owner(o), sxpos(x), sypos(y)
{
	combox = 0;
	bubble_timer = 0;
	reset();

	ToolbarBackend::FuncList::const_iterator p = rct.toollist.begin();
	for (; p != rct.toollist.end(); ++p) {
		switch(p->baritem) {
		case ToolbarBackend::TOOL_NONE:
			// nothing to do
			break;
		case ToolbarBackend::TOOL_SEPARATOR:			
		case ToolbarBackend::TOOL_LAYOUTS:
		case ToolbarBackend::TOOL_NEWLINE:
		case ToolbarBackend::TOOL_FUNC:
			lyxerr[Debug::TOOLBAR] << "tool action: " 
					       << p->action << '\n';
			add(p->baritem, p->action);
			break;
		}
	}
}


void Toolbar::BubbleTimerCB(FL_OBJECT *, long data)
{
	FL_OBJECT* ob = (FL_OBJECT*) data;
	// The trick we use to get the help text is to read the
	// argument of the callback that has been registered for
	// ToolBarCB.  (JMarc)
	string help = lyxaction.helpText(ob->argument);
	fl_show_oneliner(help.c_str(), ob->form->x + ob->x,
			 ob->form->y + ob->y + ob->h);
}


// post_handler for bubble-help (Matthias)
int Toolbar::BubblePost(FL_OBJECT *ob, int event,
	     FL_Coord /*mx*/, FL_Coord /*my*/, int /*key*/, void */*xev*/)
{
	Toolbar *t = (Toolbar*)ob->u_ldata;
	
	// We do not test for empty help here, since this can never happen
	if (event == FL_ENTER) {
		fl_set_object_callback(t->bubble_timer,
				       BubbleTimerCB, (long) ob);
		fl_set_timer(t->bubble_timer, 1);
	} else if (event != FL_MOTION) {
		fl_set_timer(t->bubble_timer, 0);
		fl_hide_oneliner();
	}
	return 0;
}


void Toolbar::activate()
{
	ToolbarList::const_iterator p = toollist.begin();
	for (; p != toollist.end(); ++p) {
		if (p->icon) {
			fl_activate_object(p->icon);
		}
	}
}


void Toolbar::deactivate()
{
	ToolbarList::const_iterator p = toollist.begin();
	for (; p != toollist.end(); ++p) {
		if (p->icon) {
			fl_deactivate_object(p->icon);
		}
	}
}


void Toolbar::ToolbarCB(FL_OBJECT *ob, long ac)
{
	Toolbar *t = (Toolbar*)ob->u_ldata;
	
	string res = t->owner->getLyXFunc().Dispatch(int(ac));
	if(!res.empty())
		lyxerr[Debug::TOOLBAR] << res << '\n';
}


#warning This one is buggy, fix it.
int Toolbar::get_toolbar_func(string const & func)
{
	int action = lyxaction.LookupFunc(func.c_str());
	if (action == -1) {
		if (func == "separator"){
			action = ToolbarBackend::TOOL_SEPARATOR;
		} else if (func == "layouts"){
                        action = ToolbarBackend::TOOL_LAYOUTS;
                } else action = 0;
	}
	return action;
}


static
void setPixmap(FL_OBJECT *obj, int action, int buttonwidth, int height) {
	string name, arg, xpm_name;
	kb_action act;

	if (lyxaction.isPseudoAction(action)) {
		lyxerr[Debug::TOOLBAR] << "Pseudo action " << action << '\n';

		act = lyxaction.retrieveActionArg(action, arg);
		name = lyxaction.getActionName(act);
		xpm_name = subst(name + ' ' + arg, ' ','_');
	} else {
		act = (kb_action)action;
		name = lyxaction.getActionName(action);
		xpm_name = name;
	}

	lyxerr[Debug::TOOLBAR] << "Icon name for action " << action
			       << " is `" << xpm_name << "'\n";
	
	string fullname = LibFileSearch("images", xpm_name, "xpm");

	if (!fullname.empty()) {
		lyxerr[Debug::TOOLBAR] << "Full icon name is `" 
				       << fullname << "'\n";
		fl_set_pixmapbutton_file(obj, fullname.c_str());
		return;
	}

	if (act == LFUN_INSERT_MATH && !arg.empty()) {
		lyxerr[Debug::TOOLBAR] << "Using mathed-provided icon\n";
		char ** pixmap = get_pixmap_from_symbol(arg.c_str(),
							buttonwidth,
							height);
		fl_set_pixmapbutton_data(obj, pixmap);
		return;
	}
	
	lyxerr << "Unable to find icon `" << xpm_name << "'\n";
	fullname = LibFileSearch("images", "unknown", "xpm");
	if (!fullname.empty()) {
		lyxerr[Debug::TOOLBAR] << "Using default `unknown' icon\n";
		fl_set_pixmapbutton_file(obj, fullname.c_str());
	}
}


void Toolbar::set()
{
	FL_OBJECT *obj;
	
	fl_freeze_form(owner->getForm());
	//fl_addto_form(owner->getForm());

	// add the time if it don't exist
	if (bubble_timer == 0)
		bubble_timer = fl_add_timer(FL_HIDDEN_TIMER,
					    xpos,ypos,0,0,"Timer");
		
	ToolbarList::iterator item = toollist.begin();
	for (; item != toollist.end(); ++item) {
		switch(item->baritem){
		case ToolbarBackend::TOOL_NONE: break;
		case ToolbarBackend::TOOL_NEWLINE: break;
		case ToolbarBackend::TOOL_SEPARATOR:
			xpos += sepspace;
			break;
		case ToolbarBackend::TOOL_LAYOUTS:
			xpos += standardspacing;
			if (!combox)
				combox = new Combox(Combox::FL_COMBOX_DROPLIST);
			combox->add(xpos, ypos, 135, height, 300);
			combox->setcallback(LayoutsCB);
			combox->resize(FL_RESIZE_ALL);
			combox->gravity(NorthWestGravity, NorthWestGravity);
			xpos += 135;
			// Here the layoutdropdown should also connect
			// to the layoutslist signal
			break;
		case ToolbarBackend::TOOL_FUNC:
			xpos += standardspacing;
			item->icon = obj =
				fl_add_pixmapbutton(FL_NORMAL_BUTTON,
						    xpos,ypos,
						    buttonwidth,
						    height,"");
			fl_set_object_boxtype(obj,FL_UP_BOX);
			fl_set_object_color(obj,FL_MCOL,FL_BLUE);
			fl_set_object_resize(obj, FL_RESIZE_ALL);
			fl_set_object_gravity(obj,
					      NorthWestGravity,
					      NorthWestGravity);
			fl_set_object_callback(obj,ToolbarCB,
					       long(item->action));
			// Remove the blue feedback rectangle
			fl_set_pixmapbutton_focus_outline(obj,0);
			
			// we need to know what toolbar this item
			// belongs to. (Lgb)
			obj->u_ldata = long(this);
			
			// Setup the callback for bubble help
			fl_set_object_posthandler(obj, BubblePost);
			
			//set the icon
			setPixmap(obj, item->action, buttonwidth, height);
			
			// we must remember to update the positions
			xpos += buttonwidth;
			// ypos is constant
			/* Here will come a check to see if the new
			 * pos is within the bounds of the main frame,
			 * and perhaps wrap the toolbar if not.
			 */
			break;
		}
	}
	//fl_end_form();
	fl_unfreeze_form(owner->getForm());
	// Should be safe to do this here. SHOULD not be done here.
	//owner->updateLayoutChoice();
}


void Toolbar::add(ToolbarBackend::TOOLBARITEMS baritem, int action)
{
	toolbarItem newItem;
	newItem.baritem = baritem;
	newItem.action = action;
	toollist.push_back(newItem);
}


void Toolbar::clean()
{
	reset();

	//now delete all the objects..
	if (owner) fl_freeze_form(owner->getForm());

	// G++ vector does not have clear defined
	//toollist.clear();
	toollist.erase(toollist.begin(), toollist.end());
	
	if (owner) fl_unfreeze_form(owner->getForm());
	
	lyxerr[Debug::TOOLBAR] << "toolbar cleaned\n";
	cleaned = true;
}


void Toolbar::push(int nth)
{
	lyxerr[Debug::TOOLBAR] << "Toolbar::push: trying to trigger no `"
			       << nth << "'\n";
	
	if (0 <= nth || nth >= int(toollist.size())) {
		// item nth not found...
		LyXBell();
		return;
	}

	fl_trigger_object(toollist[nth - 1].icon);
}
