
#include <config.h>
#include <unistd.h>

#include "gsqueue.h"
#include "error.h"
#include "lyx_main.h"
#include "filetools.h"
#include "lyxrc.h"
#include "syscall.h"

extern long int background_pixels;
extern FL_OBJECT *figinset_canvas;
extern char **environ; // is this only redundtant on linux systems? Lgb.

bool gs_color;			// do we allocate colors for gs?
static bool gs_xcolor = false;		// allocated extended colors
bool gs_gray;			// is grayscale?

// the queue of images to be rendered.
GSQueue gsQ;

const unsigned short MAXGS = 3;			/* maximum 3 gs's at a time */

// yes I know, a terrible function name.
static void wait_for_deleted_property(Display *disp, Window win) {
	while (true) {
		// grab server to prevent other child interfering
		// with setting GHOSTVIEW property
		lyxerr.debug("Grabbing the server");
		XGrabServer(tempdisp);
		int nprop = 0;
		Atom *prop = XListProperties(tempdisp,
					     fl_get_canvas_id(figinset_canvas),
					     &nprop);
		if (!prop) break;
		
		bool found = false;
		for (i = 0; i < nprop; ++i) {
			char *p = XGetAtomName(tempdisp, prop[i]);
			if (strcmp(p, "GHOSTVIEW") == 0) {
				found = true;
				break; // out of for
			}
			XFree(p);
		}
		XFree((char *)prop);    /* jc: */
		if (!found) break; // out of while
		
		// release the server
		XUngrabServer(tempdisp);
		// ok, property found, we must wait until ghostscript
		// deletes it
		lyxerr.debug("Releasing the server");
		lyxerr.debug(LString("[") +
			     int(getpid()) +
			     "] GHOSTVIEW property"
			     " found. Waiting.");
	}
}


void GSQueue::runqueue()
{
	// run queued requests for ghostscript, if any
	if (!gsworking() && gs_color && !gs_xcolor) {
		// here alloc all colors, so that gs will use only
		// those we allocated for it
		// *****
		gs_xcolor = true;
	}
	
	while (gsrunning < MAXGS) {
		Queue *p;
		int pid;
		LString tbuf;
		LString tbuf2;
		int i;

		if (!queue) {
			if (!gsworking() && gs_xcolor) {
				// de-allocate rest of colors
				// *****
				gs_xcolor = false;
			}
			return;
		}
		p = queue;

		if (!p->data) {
			delete p;
			continue;
		}

		switch (pid=fork()) {
		case -1: // no fork
			if (lyxerr.debugging()) {
				printf("GS start error!!! Cannot fork.\n");
			}
			p->data->broken = true;
			p->data->reading = false;
			return;
		case 0: // child
		{
			LString rbuf;
			LString gbuf;
			

			// create translation file
			tbuf = system_tempdir + "/~lyxgs"
				+ int(getpid()) + ".ps";
			
			FilePtr	f(tbuf, FilePtr::write);
			fprintf(f, "gsave clippath pathbbox grestore\n"
				"4 dict begin\n"
				"/ury exch def /urx exch def /lly exch def " 
				"/llx exch def\n"
				"%g %g translate\n"
				"%g rotate\n"
				"%g %g translate\n"
				"%g %g scale\n"
				"%d %d translate\nend\n",
				p->data->wid / 2.0, p->data->hgh / 2.0,
				p->data->angle,
				- (p->data->raw_wid / 2.0), -(p->data->raw_hgh / 2.0),
				p->rx / 72.0, p->ry / 72.0,
				-p->ofsx, -p->ofsy
				);

			// DON'T EVER remove this!!
			f.close(); // was this all? (Lgb)
			
			// gs process - set ghostview environment first
			tbuf2 = LString("GHOSTVIEW=")
				+ long(fl_get_canvas_id(figinset_canvas))
				       + LString(" ") + long(p->data->bitmap);
			// now set up ghostview property on a window
			tbuf = LString("0 0 0 0 ")
				+ int(p->data->wid) + LString(" ")
				+ int(p->data->hgh) + "72 72 0 0 0 0";
			
			lyxerr.debug("Will set GHOSTVIEW property to ["
				     + tbuf + "]");
			// wait until property is deleted if executing multiple
			// ghostscripts
			Display* tempdisp = XOpenDisplay(XDisplayName(NULL));
			
			wait_for_deleted_property(tempdisp, fl_get_canvas_id(figinset_canvas));
						  
			XChangeProperty(tempdisp, 
					fl_get_canvas_id(figinset_canvas),
					XInternAtom(tempdisp, "GHOSTVIEW", false),
					XInternAtom(tempdisp, "STRING", false),
					8, PropModeAppend, 
					(const unsigned char *)tbuf.c_str(),
					tbuf.length());
			
			switch (p->data->flags & 3) {
			case 0: // Hidden
				tbuf = "H"; break;
			case 1: // Mono
				tbuf = "M"; break;
			case 2: // Gray
				tbuf = "G"; break;
			case 3:
				if (gs_color && !gs_gray) 
					// Color
					tbuf = "C";
				else 
					// Gray
					tbuf = "G";
				break;
			}

			tbuf += LString(" ")
				+ long(BlackPixelOfScreen(DefaultScreenOfDisplay(fl_display)))
				+ LString(" ")
				+ long(background_pixels);
			
			XChangeProperty(tempdisp, 
					fl_get_canvas_id(figinset_canvas),
					XInternAtom(tempdisp, "GHOSTVIEW_COLORS", false),
					XInternAtom(tempdisp, "STRING", false),
					8, PropModeReplace, 
					(const unsigned char *) tbuf.c_str(),
					tbuf.length());
			XUngrabServer(tempdisp);
			lyxerr.debug("Releasing the server.");
			XCloseDisplay(tempdisp);

			// set up environment
			int ne = 0;
			while (environ[ne]) ++ne;
			char **env = (char **) malloc(sizeof(char*)*(ne+2));
#warning Oops.
			env[0] = strdup(tbuf2.c_str());
			memcpy(&env[1], environ, sizeof(char*)*(ne+1));
			environ = env;

			// now make gs command
			// close(0);
			// close(1); do NOT close. If GS writes out
			// errors it would hang. (Matthias 290596) 
			//sprintf(rbuf, "-r%gx%g", p->rx, p->ry);
			//sprintf(gbuf, "-g%dx%d", p->data->wid, p->data->hgh);
			// now chdir into dir with .eps file, to be on the safe
			// side
			Path(OnlyPath(p->data->fname));
			// make temp file name
			//sprintf(tbuf, "%s/~lyxgs%d.ps", system_tempdir.c_str(),
				//int(getpid()));
			//if (lyxerr.debugging()) {
			//	printf("starting gs %s %s, pid: %d\n", tbuf,
			//	       p->data->fname.c_str(), int(getpid()));
			//}

			LString argv = lyxrc->ps_command + " -sDEVICE=x11 -dNOPAUSE -dSAFER";
#warning Fix this!
			argv += LString(" -r") + int(p->rx) + LString("x") + int(p->ry); 
			argv += LString(" -g") + int(p->data->wid) + LString("x") + int(p->data->hgh);
			argv += " " + system_tempdir + "/~lyxgs" + int(getpid()) + ".ps";
			argv += " showpage.ps";
			lyxerr.debug("starting:" + argv);

			Systemcalls(Systemcalls::DontWait, argv);
			// if we are still there, an error occurred.
			lyxerr.print(LString("Error executing ghostscript. "));
			lyxerr.print("Cmd: "
				     +argv);
			_exit(0);	// no gs?
		}
		default:
			// normal process (parent)
			if (lyxerr.debugging()) {
				printf("GS [%d] started\n", pid);
			}
			queue = queue->next;
			gsrunning++;
			p->data->gspid = pid;
			delete p;
			break;
		}
	}
}
