// -*- C++ -*-
/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 * 	 
 *	    Copyright (C) 1998 The LyX Team
 *
 *======================================================*/

#ifndef ASCIIFORMAT_H
#define ASCIIFORMAT_H

#include <config.h>

#ifdef __GNUG__
#pragma interface
#endif

#include "TextFormat.h"

/** This is a class to control output of Ascii file.

    In addition to the margin, word-wrapping and indentation control
    of the TextFormatter, it implements alignments and hfills.
    (Asger)
*/
class AsciiFormat : public TextFormat {
public:
	/// What kind of text alignment do we want?
	enum Alignment {
		flushleft,
		flushright,
		block,
		center
	};

	/** What encoding do we get, and what encoding should we write?  
	    What end-of-line string should we use?
	    How wide is a line at the most in the output file? 
	*/
	AsciiFormat(Encoding & inenc_, Encoding & outenc_, 
		    string eol_ = string("\n"), int width_ = 75)
		: TextFormat(inenc_, outenc_, eol_, width_)
	{
		alignment = flushleft;
	}

	/// We override this to get hfill support
	void write(fmt::Symbol);

	/// What kind of file format are we?
	Format kind() const {
		return FileFormat::ASCII;
	}

	/// Set the alignment
	void setAlignment(Alignment a) {
		alignment = a;
	}

protected:
	/// We override this to implement hfill and alignment support
	void processLine(string & line);

	/// We override this to support hfill
	void flushline();

private:
	/// Expand spaces with amount at given positions
	void expandspaces(string & line, vector<int> positions, int amount);

	/// What kind of alignment are we using?
	Alignment alignment;

	/// Where do we have hfills in the current line?
	vector<int> hfills;
};

#endif ASCIIFORMAT_H
