// -*- C++ -*-
/* This file is part of
* ======================================================
* 
*           LyX, The Document Processor
*        
*           Copyright (C) 1995 Matthias Ettrich
*           Copyright (C) 1995-1998 The LyX Team.
*
*======================================================*/

#ifndef BUFFER_VIEW_H
#define BUFFER_VIEW_H

#ifdef __GNUG__
#pragma interface
#endif

#include "painter.h"
#include "timer.h"
#include "undo.h"
#include "BackStack.h"

class LyXView;
class Buffer;
class LyXText;
class WorkArea;
class LyXScreen;
class Inset;
class UpdatableInset;

/**
 * A buffer view holds a buffer and displays it along with a scrollbar.
 * The main access from the user interface to a buffer goes through a
 * BufferView.
 */
class BufferView : public virtual SignalBase {
public:
	///
	BufferView(LyXView *owner, int ,int ,int, int);
        ///
        ~BufferView();

	///
	Buffer *currentBuffer() { return buffer_; }

	BufferView & operator=(BufferView const &)
	{
		// This will need to be expanded/completed on a later date
		//#warning note!
		return *this;
	}
	
	/// Are we ready to accept a keypress?
	bool readyForKeypress() const;

	/// Set focus to this buffer view
	void setFocus() const;

	/// If we set buffer to zero, we close the buffer
	void setBuffer(Buffer *b);

	/// Get the painter
	Painter & getPainter();

	///
	void resize(int xpos, int ypos, int width, int height);
	///
	void resize();
	///
	void redraw();
	///
	void fitCursor();
	///
	void update();
	///
	void smallUpdate(signed char f);

	/* Moved here from buffer.[Ch] */

        /// Open and lock an updatable inset
        bool open_new_inset(UpdatableInset*);

	///
	enum UpdateEnum {
		RebreakOnly = 0,
		ResetSelection = 1,
		FitCursor = 2,
		BufferChange = 4
	};
	
	void update(unsigned short);

	/** Insert an inset into the buffer
	    Insert inset into buffer, placing it in a layout of lout,
	    if no_table make sure that it doesn't end up in a table. */
	void insertInset(Inset *, string const &lout=string(), 
			 bool no_table = false);

	
	///
	void updateScrollbar();
	///
	void redoCurrentBuffer();
	///
	int resizeCurrentBuffer();
	///
	bool gotoLabel(const string &label);
	///
	void gotoError();
	///
	void gotoNote();
	///
	void cursorPrevious();
	///
	void cursorNext();
	/// 
	bool available() const;
	///
	LyXView *getOwner() { return owner_; }
	
        ///
        void savePosition();
        ///
        void restorePosition();
	///
	LyXText *text;
	///
	void insertCorrectQuote();
	///
	void changeDepth(FL_OBJECT *ob, long decInc);
	///
	void melt();
	///
	void paste();
	///
	void cut();
	///
	void copy();
	///
	void copyEnvironment();
	///
	void pasteEnvironment();
	///
	void margin();
	///
	void hyphenationPoint();
	///
	void protectedSpace();
	///
	void ldots();
	///
	void endOfSentenceDot();
	///
	void newline();
	///
	void hfill();
	///
	void openStuff();
	///
	void toggleFloat();

	///
	enum FloatAct {
		///
		OpenFootMargin,
		///
		CloseFootMargin,
		///
		OpenFigTab,
		///
		CloseFigTab
	};

	///
	void allFloats(FloatAct flact);
	///
	void foot();
	///
	void beforeChange();
	///
	void note();
	///
	void undo();
	///
	void redo();
	///
	void fontSize(string const & size);
	///
	void emph();
	///
	void noun();
	///
	void bold();
	///
	void underline();
	///
	void code();
	///
	void sans();
	///
	void roman();
	///
	void tex();
	///
	void styleReset();
	///
	void userFree();
	///
	void autoSave();
	///
	void toggleReadOnly();
	///
	void center();
	///
	void appendix();
	///
	void insertTOC();
	///
	void insertLOF();
	///
	void insertLOA();
	///
	void insertLOT();
	///
	void removeErrors();
	///
	void footMelt();
	///
	void marginMelt();
	///
	void upcaseWord();
	///
	void lowercaseWord();
	///
	void capitalizeWord();
	///
	void right(bool sel);
	///
	void left(bool sel);
	///
	void up(bool sel);
	///
	void down(bool sel);
	///
	void upParagraph(bool sel);
	///
	void downParagraph(bool sel);
	///
	void prior(bool sel);
	///
	void next(bool sel);
	///
	void home(bool sel);
	///
	void end(bool sel);
	///
	void tab();
	///
	void wordRight(bool sel);
	///
	void wordLeft(bool sel);
	///
	void beginBuff(bool sel);
	///
	void endBuff(bool sel);
	///
	void insertSpecialChar(LYX_META_KIND kind);
	/// return true if mark was set, false if it was removed
	bool setMark();
	///
	void regionDelete();
	///
	void regionDeleteSkip();
	///
	void deleteWordForward();
	///
	void deleteWordBackward();
	///
	void deleteLineForward();
	///
	void markOff();
	///
	void markOn();
	///
	void backspace();
	///
	void backspaceSkip();
	///
	void paragraphBreak(bool keep_layout);
	///
	void paragraphBreakSkip();
	///
	void insertChar(char);
	///
	void insertAsciiFile(string const & f, bool asParagraph);
	///
	void insertLabel();
	///
	void pasteSelection(char at);
	///
	void MenuMakeLaTeX();
	///
	int runLaTeX();
	///
	int runChktex();
	///
	int runLinuxDoc(int flag, string const & filename);
	///
	void layoutSave();
	///
	char getCurrentChar();
	///
	void selfInsert(string const &);
	///
	void moveCursorUpdate(bool selecting);
	///
	void freeUpdateTimer();
	///
	void setUpdateTimer(float time = 0.3);
	///
	int getMaxWidth(UpdatableInset*) const;
	///
	bool InsertGreek(char const);
	///
	void InsertMathSymbol(char const*);

private:
	///
	struct InsetUpdateStruct {
		///
		Inset* inset;
		///
		InsetUpdateStruct* next;
	};
	
	/* 
	   This is the inset locking stuff needed for mathed ------------

	   An inset can simple call LockInset in it's edit call and *ONLY*
	   in it's edit call.
	   
	   Inset::Edit() can only be called by the main lyx module.

	   Then the inset may modify the menu's and/or iconbars. 

	   Unlocking is either done by LyX or the inset itself with
	   a UnlockInset-call.

	   During the lock, all button and keyboard events will be modified
	   and send to the inset through the following inset-features.
	   Note that Inset::InsetUnlock will be called from inside
	   UnlockInset. It is meant to contain the code for restoring
	   the menus and things like this.
   
	   virtual void InsetButtonPress(int x, int y, int button);
	   virtual void InsetButtonRelease(int x, int y, int button);
	   virtual void InsetKeyPress(XKeyEvent *ev);
	   virtual void InsetMotionNotify(int x, int y, int state);
	   virtual void InsetUnlock();

	   If a inset wishes any redraw and/or update it just has to call
	   UpdateInset(this).
	   It's is completly irrelevant, where the inset is. UpdateInset will
	   find it in any paragraph in any buffer. 
	   Of course the_locking_inset and the insets in the current
	   paragraph/buffer are checked first, so no performance problem
	   should occur.
   
	   Hope that's ok for the beginning, Alejandro,
	   sorry that I needed so much time,
	   
	           Matthias
	*/

	///
        inline void InsetSleep();
	
	///
        inline void InsetWakeup();

	///
	UpdatableInset* the_locking_inset;

	///
        bool inset_slept;
	
	///
        int  slx, sly;

public:
	///
	void updateInset(Inset *inset, bool mark_dirty);
	///
	bool lockInset(UpdatableInset* inset);

	///
	void showLockedInsetCursor(long x, long y, int asc, int desc);
	///
	void hideLockedInsetCursor(long x, long y, int asc, int desc);
	///
	void fitLockedInsetCursor(long x, long y, int asc, int desc);
	///
	int unlockInset(UpdatableInset* inset);
	///
	void lockedInsetStoreUndo(Undo::undo_kind kind);
	
	/* This is for asyncron updating. UpdateInsetUpdateList will
	   be called automatically from LyX. Just insert the Inset
	   into the Updatelist */
	///
	void putInsetIntoInsetUpdateList(Inset* inset);
	///
	void updateInsetUpdateList();
	///
	void toggleSelection(bool b = true);
	///
	int paperWidth() const;
	///
	int width() const;
	///
	int height() const;
	///
	int xpos() const;
	///
	int ypos() const;
	///
	void hideCursor();
	///
	void showCursor();
private:
	///
	InsetUpdateStruct *InsetUpdateList;
	bool updatetimer;
	///
	void toggleAndShow(LyXFont const & font);
	/// Update pixmap of screen
	void updateScreen();
	///
	void workAreaExpose();
	///
	void create_view(int, int, int, int);
	///
	bool checkInsetHit(int &x, int &y);
	/// 
	int ScrollUp(long time);
	///
	int ScrollDown(long time);
	///
	void ScrollUpOnePage();
	///
	void ScrollDownOnePage();

	/// A Slot
	void UpdateTimerCB();

	/// A slot for the up arrow in the scrollbar.
	void UpCB(long time, int button);

	/// A slot for the slider in the scrollbar.
	void ScrollCB(double value);

	/// A slot for the down arrow in the scrollbar.
	void DownCB(long time, int button);

	///
	void WorkAreaMotionNotify(int x, int y, unsigned int);

	///
	void WorkAreaSelectionNotify(Window, XEvent*);

	///
	void WorkAreaButtonPress(int x, int y, unsigned int);

	///
	void WorkAreaButtonRelease(int x, int y, unsigned int);

	///
	void CursorToggleCB();
	
	///
	LyXView *owner_;
	///
	Buffer *buffer_;
	///
	LyXScreen *screen_;
	///
	long current_scrollbar_value;
	///
	bool lyx_focus;
	///
	bool work_area_focus;
	///
	WorkArea *workarea;
	///
	Timer timer_cursor;
	///
	Timer timer_update;
        ///
        BackStack backstack;
	///
	friend class LyXFunc;
};



inline  
void BufferView::InsetSleep()
{
	if (the_locking_inset && !inset_slept) {
		the_locking_inset->GetCursorPos(slx, sly);
		the_locking_inset->InsetUnlock();
		inset_slept = true;
	}
}


inline  
void BufferView::InsetWakeup()
{
	if (the_locking_inset && inset_slept) {
		the_locking_inset->Edit(slx, sly, 0, this);
		inset_slept = false;
	}
}


#endif
