// -*- C++ -*-
/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 * 	 
 *	    Copyright (C) 1999 Asger Alstrup Nielsen
 *
 *======================================================*/

#ifndef ENCODING_H
#define ENCODING_H

#include <config.h>

#ifdef __GNUG__
#pragma interface
#endif

#include <utility>

#include "LString.h"

// It would be ideal to put all of this stuff in a namespace

/// The different kinds of encodings we support
enum Encoding {
	/// The Latin family
	ISO8859_1, ISO8859_2, ISO8859_3, ISO8859_4, ISO8859_5, ISO8859_6, 
	ISO8859_7, ISO8859_8, ISO8859_9, ISO8859_14, ISO8859_15
	/// 7-bit LaTeX
	// , LaTeX,
	/// 8-bit LaTeX
	// T1,
	/// SGML entities
	// SGML
	///
#ifdef WIDESTRINGS
	// Here, 16-bit encodings should be listed.
	,
	///
	Unicode
#endif
};

/** Convert a string to a different encoding
    This function returns the converted string, and an iterator
    that points after the last character in the passed string
    that was converted exact. 
*/
std::pair<string, string::const_iterator>
convertEncoding(string const &, Encoding from, Encoding to);

/// Convert a string to Unicode
std::pair<wstring, LString::const_iterator>
convertToUnicode(LString const &, Encoding from);

/// Convert a string from Unicode
std::pair<LString, wstring::const_iterator>
convertFromUnicode(wstring const &, Encoding to);

/// Return the name of an encoding as a single word in ASCII
LString getEncodingName(Encoding);

/// Get the encoding from an ASCII name
Encoding getEncodingFromName(string const &);

/// Return the GUI name of an encoding
LString getEncodingGuiName(Encoding);

/// Get the encoding from a GUI name
Encoding getEncodingFromGuiName(string const &);

/// Can all chars of an encoding be contained in a single LChar?
bool isEncodingFixedWidth(Encoding);

/// Can this encoding be converted to Unicode? 
bool canConvertToUnicode(Encoding);

#endif
