// -*- C++ -*-
#ifndef _LYXACTION_H_
#define _LYXACTION_H_

#ifdef __GNUG__
#pragma interface
#endif

#include <map>
#include "commandtags.h"
#include "LString.h"

/** This class encapsulates LyX action and user command operations.
 */
class LyXAction {
private:
	///
	struct func_info {
		string name;
		unsigned int attrib;
		string helpText;
	};

	///
	struct pseudo_func {
		kb_action action;
		string arg;
	};
public:
	///
	typedef map<string, kb_action, less<string> > func_map;
	///
	typedef map<kb_action, func_info, less<kb_action> > info_map;
	///
	typedef map<unsigned int, pseudo_func, less<unsigned int> > pseudo_map;
	///
	typedef map<string, unsigned int, less<string> > arg_item;
	///
	typedef map<kb_action, arg_item, less<kb_action> > arg_map;

	///
	enum func_attrib {
		/// nothing special about this func
		NOOP = 0,
		/// can not be used in RO mode (perhaps this should change)
		ReadOnly = 1 // ,
		//Interactive = 2, // Is interactive (requires a GUI)
		//Argument=4,      // Requires argument
		//MathOnly = 8,    // Only math mode
		//EtcEtc = ...     // Or other attributes...
	};
	
	///
	LyXAction();
	///
	~LyXAction();
    
	/** Returns an action tag from a string. Returns kb_action.
	  Include arguments in func_name ONLY if you
	  want to create new pseudo actions. */
	int LookupFunc(string const &func_name); 

        /** Returns an action tag which name is the most similar to a string.
	    Don't include arguments, they would be ignored. */
        int getApproxFunc(string const &func);

        /** Returns an action name the most similar to a string.
	    Don't include arguments, they would be ignored. */
        string getApproxFuncName(string const &func);

	/// Returns a pseudo-action given an action and its argument.
	int getPseudoAction(kb_action action, string const &arg);

	/// Retrieves the real action and its argument.
	kb_action retrieveActionArg(int i, string &arg) const;
    
	/// Search for an existent pseudoaction, return -1 if it doesn't exist.
	int searchActionArg(kb_action action, string const &arg);

	/// Check if a value is a pseudo-action. 
	bool isPseudoAction(int) const;
    
	/// Not sure if this function should be here 
	int bindKey(string const &seq, int action);
	
	/// Return the name associated with command
	string getActionName(int action) const;

	/// Return one line help text associated with (pseudo)action
	string helpText(int action) const;

	/// True if the command is Read Only (not allowed for RO buffers)
	bool isFuncRO(kb_action action) const;

private:
	///
	void init();
	///
	void newFunc(kb_action, string const &name,
		     string const &helpText, unsigned int attrib);
	
	/** This is a list of all the LyXFunc names with the
	  coresponding action number. It is usually only used by the
	  minibuffer or when assigning commands to keys during init. */
	func_map lyx_func_map;
	
	/** This is a mapping from action number to an object holding
	  info about this action. f.ex. helptext, command name (string),
	  command attributes (ro) */
	info_map lyx_info_map;
	
	/** A mapping from the automatically created pseudo action number
	  to the real action and its argument. */
	pseudo_map lyx_pseudo_map;
	
	/** A (multi) mapping from the lyx action to all the generated
	  pseudofuncs and the arguments the action should use. */
	arg_map lyx_arg_map;
};
     

/* --------------------   Inlines  ------------------ */
    
     
inline
bool LyXAction::isPseudoAction(int a) const
{ 
	return a > int(LFUN_LASTACTION); 
}
     

#endif
