// -*- C++ -*-
/* This file is part of
* ======================================================
* 
*           LyX, The Document Processor
*        
*           Copyright (C) 1995 Matthias Ettrich
*           Copyright (C) 1995-1998 The LyX Team.
*
*======================================================*/

#ifndef LYXVIEWBASE_H
#define LYXVIEWBASE_H

#ifdef __GNUG__
#pragma interface
#endif

#include "lyxfunc.h"
#include "intl.h"
#include "Popups.h"
#include "Communicator.h"
//#include "BufferView.h"
#include "frame.h"

class Toolbar;
class MiniBuffer;
class Menus;

///
class LyXViewBase {
public:
	///
	LyXViewBase(int, int);
	///
	virtual ~LyXViewBase();
	/// Where to place the form.
	virtual void setPosition(int, int) = 0;
	///
	virtual void show() = 0;
	///
	void updateLayoutChoice();
	///
	void updateWindowTitle();
	///
	FrameHandler *getFrameHandler()
	{
		return frameHandler;
	}
	
	/// Set focus to the current buffer in this window
	//void setFocus() { bufferview->setFocus(); }
	void setFocus() { frameHandler->setFocus(); }
	/// returns the buffer currently shown in the main form.
	//Buffer *currentBuffer() { return bufferview->currentBuffer();}
	Buffer *currentBuffer() { return frameHandler->currentBuffer();}
	///
	//BufferView *currentView() { return bufferview; }
	BufferView *currentView() { return frameHandler->currentView(); }
	/// return a pointer to the toolbar
	Toolbar* getToolbar() { return toolbar; }
	///
	LyXFunc & getLyXFunc() { return lyxfunc; }
	///
	Menus *getMenus() { return menus; }
	///
	Intl & getIntl() { return intl; }
	///
	Popups & getPopups() { return popups; }
	/// return a pointer to the minibuffer
	MiniBuffer *getMiniBuffer() { return minibuffer; }
protected:

	///
	virtual void createMainWindow(int width, int height) = 0;
	///
	virtual void set_window_title(string const &) = 0;
	///
	void updateDocumentClassChoice();
	///
	LyXFunc lyxfunc;
	///
	Toolbar *toolbar;
	///
	MiniBuffer *minibuffer;
	///
	Menus *menus;
	///
	Intl intl;
	///
	Communicator comm;
	///
	Popups popups;
	/** This is supposed to be a pointer or a list of pointers to the
	    BufferViews currently being shown in the LyXView. So far	
	    this is not used, but that should change pretty soon. (Lgb) */
	//BufferView *bufferview;
	FrameHandler *frameHandler;
	
	/** The last textclass layout list in the layout choice selector
	    This should probably be moved to the toolbar, but for now it's
	  here. (Asger) */
	int last_textclass;
private:
};

#endif
