// -*- C++ -*-
/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 * 	 
 *	    Copyright (C) 1998 The LyX Team
 *
 *======================================================*/

#ifndef OUTPUTCONTROLLER_H
#define OUTPUTCONTROLLER_H

#include <config.h>

#ifdef __GNUG__
#pragma interface
#endif

#include <vector>

#include "LString.h"
#include "filetools.h"
#include "FileFormat.h"

/** This class provides a bunch of functionality that is useful when
    generating output of the buffer.  At first, this class will help
    with generation of file formats.  At the moment textual file formats
    are supported the best, but raw binary file formats are also possible.

    However, one goal is to expand this to the abstract painter, such 
    that the painting on the screen can be handled by this class.
*/
class OutputController {
public:
	/** Instantiate an OutputController that produces output to
	    a file with the given filename (including full path),
	    in the format as given by the FileFormat object.
	*/
	OutputController(string const & filename_, 
			 FileFormat * fileformat_);

	// Later, we could add a constructor something this that produce
	// graphical output(!)
	// OutputController(Painter const &);

	///
	~OutputController();

	// Get the format we are doing
	FileFormat::Format format() const;

	// Get controller for the specific file format we are using
	FileFormat & formatter();

	/** Write a string to target.  This string is NOT allowed
	    to contain line breaks.  Use "<< fmt::br" for that. 
	*/
	OutputController & operator<<(string const & string);

	/// Write a symbol to target
	OutputController & operator<<(fmt::Symbol);

	/** Add a message concerning the output generation.
	    Typically, this will concern missing features that
	    can't be output properly. */
	void message(string const &);

	/// The type of the collected messages list
	typedef vector<string> Messages;

	/// Get the messages collected during output
	Messages getMessages() const;
private:
	///
	string filename;

	/// Which format do we output in?
	FileFormat * fileformat;

	/// A collection of messages worth keeping
	Messages messages;
};

#endif
