// -*- C++ -*-
/* Popups.h
 * Container of all popups and signals a LyXView needs/uses to access them.
 */

#ifndef POPUPS_H
#define POPUPS_H

#include <vector>
#include "lsignal.h"
#include "LString.h"

class Communicator;

class LaTeXPreamble;
class LaTeXLog;
class FormTable;
class FormParagraph;
class FormDocument;
class FormCopyright;
class FormCredits;
class FormInclude;
class FormIndex;
class FormBibkey;
class FormCitation;
class FormBibtex;
class FormInfo;

class InsetInclude;
class InsetIndex;
class InsetBibKey;
class InsetCitation;
class InsetBibtex;
class InsetInfo;


///
class Popups {
public:
	///
	Popups(Communicator *);
	///
	~Popups();
	///
	void SetDocumentClassChoice(vector<string> const &);
	
	/// hide all visible popups
	Signal0 hideAll;
	
	/// hide any popups that require a buffer for them to operate
	Signal0 hideBufferDependent;
	
	/// Visible popups updated
	Signal0 updateBufferDependent;
	
	/**@name Popup Access Signals */
	//@{
	/// Opens the LaTeXPreamble Popup.
	Signal0 showLaTeXPreamble;
	///
	Signal0 showLaTeXLog;
        ///
        Signal0 showFormTable;
	///
	Signal0 updateFormTable; // needed for InsetTabular-Update
	///
        Signal0 showFormTableNew;
	///
	Signal0 showFormParagraph;
	///
	Signal0 showFormDocument;
	///
	Signal0 showCopyright;
	///
	Signal0 showCredits;
	///
	Signal1<InsetInclude*> showFormInclude;
	///
	Signal1<InsetIndex*> showFormIndex;
	///
	Signal1<InsetBibKey*> showFormBibkey;
	///
	Signal1<InsetCitation*> showFormCitation;
	///
	Signal1<InsetBibtex*> showFormBibtex;
	///
	Signal1<InsetInfo*> showFormInfo;
	//@}
private:
	///
	Popups() {}
	///
	Popups(Popups&) {}
	///
	Communicator *comm;
	///
	LaTeXPreamble *latexpreamble;
	///
	LaTeXLog *latexlog;
	///
	FormTable *formtable;
	///
	FormParagraph *formparagraph;
	///
	FormDocument *formdocument;
	///
	FormCopyright * formcopyright;
	///
	FormCredits * formcredits;
	///
	FormInclude * forminclude;
	///
	FormIndex * formindex;
	///
	FormBibkey * formbibkey;
	///
	FormCitation * formcitation;
	///
	FormBibtex * formbibtex;
	///
	FormInfo * forminfo;
};

#endif
