// -*- C++ -*-
/* This file is part of
* ======================================================
* 
*           LyX, The Document Processor 	 
*	    Copyright (C) 1995 Matthias Ettrich
*
*           This file is Copyleft (C) 1996
*           Lars Gullik Bjnnes
*
*======================================================*/

#ifndef BUFFER_LIST_H
#define BUFFER_LIST_H

#ifdef __GNUG__
#pragma interface
#endif

#include <vector>

#include "buffer.h"
#include "debug.h"


/** The class governing all the open buffers
  This class governs all the currently open buffers. Currently all the buffer
  are located in a static array, soon this will change and we will have a
  linked list instead.
 */
class BufferList {
public:
	///
	//typedef vector<Buffer>::iterator iterator;
	//typedef vector<Buffer>::const_iterator const_iterator;
	
	///
 	BufferList();

	///
	~BufferList();

	/// state info
	enum list_state {
		///
		OK,
		///
		CLOSING
	};

	/// returns the state of the bufferlist
	list_state getState() { return _state; }
	
	/** loads a LyX file or...
	  If the optional argument tolastfiles is false (default is
            true), the file name will not be added to the last opened
	    files list
	    */  
	Buffer * loadLyXFile(string const & filename, 
			    bool tolastfiles = true);
	
	///
	bool empty() const;

	///
	void menuWrite(Buffer * buf);

	///
	void menuWriteAs(Buffer * buf);
	
	/// Saves buffer. Returns false if unsuccesful.
	bool write(Buffer *, bool makeBackup = true);

	///
        bool QwriteAll();

	/// Close all open buffers.
	void closeAll();

	///
	void resize();

	/// Read a file into a buffer readonly or not.
	Buffer* readFile(string const &, bool ro);

	/// Make a new file (buffer) using a template
	Buffer* newFile(string const &, string);

	/** This one must be moved to some other place.
	 */
	void makePup(int);

	///
	void updateInset(Inset *, bool = true);

	///
	int unlockInset(UpdatableInset *);

	///
	void updateIncludedTeXfiles(string const &);

	///
	void emergencyWriteAll();

	/** closes buffer
	  Returns false if operation was canceled
	  */
	bool close(Buffer *);

	///
	Buffer * first();
	
	/// returns true if the buffer exists already
	bool exists(string const &) const;

	/// returns a pointer to the buffer with the given name.
	Buffer * getBuffer(string const &);
	/// returns a pointer to the buffer with the given number.
	Buffer * getBuffer(unsigned int);

private:
	/// users are not allowed to use this.
	BufferList(const BufferList &) { };
	/// nor this
	void operator=(const BufferList &){ };
	///
	typedef vector<Buffer> BStore;
	///
	BStore bstore;
	///
	list_state _state;
};


#endif
