// -*- C++ -*-
/*
 *  File:        formula.h
 *  Purpose:     Declaration of formula inset
 *  Author:      Alejandro Aguilar Sierra <asierra@servidor.unam.mx> 
 *  Created:     January 1996
 *  Description: Allows the edition of math paragraphs inside Lyx. 
 *
 *  Copyright: (c) 1996, Alejandro Aguilar Sierra
 *
 *  Version: 0.4, Lyx project.
 *
 *   You are free to use and modify this code under the terms of
 *   the GNU General Public Licence version 2 or later.
 */

#ifndef _INSET_FORMULA_MACRO_H 
#define _INSET_FORMULA_MACRO_H

#ifdef __GNUG__
#pragma interface
#endif

#include "formula.h"




class MathMacroTemplate;


///
class InsetFormulaMacro: public InsetFormula {
public:
	///
	InsetFormulaMacro();
	///
	InsetFormulaMacro(string name, int na=0, bool env=false);
	///
	~InsetFormulaMacro();

	///
	int ascent(Painter &, LyXFont const &) const;
	///
	int descent(Painter &, LyXFont const &) const;
	///
	int width(Painter &, LyXFont const &) const;
	///
	void draw(Painter &, LyXFont const &, int baseline, float &x);

	///
	void Read(LyXLex &lex);
        ///
	void Write(FILE *file);
	///
	int Latex(FILE *file, signed char fragile);
	///
	int Latex(string &file, signed char fragile);
	///
	Inset* Clone();

	/// what appears in the minibuffer when opening
	string EditMessage() const { return "Math macro editor mode"; }
	///
	void Edit(int x, int y, unsigned int, BufferView *);
	///
	void InsetUnlock();
	///
	UpdatableInset::RESULT LocalDispatch(int, char const*);
   
private:
	///
        bool opened;
	///
        string name;
	///
        class MathMacroTemplate* tmacro;
};

#endif
