// -*- C++ -*-
/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 *
 *           Copyright (C) 1998 The LyX Team.
 *
 *======================================================
 */


#ifndef INSETCOLLAPSABLE_H
#define INSETCOLLAPSABLE_H

#ifdef __GNUG__
#pragma interface
#endif

#include "insettext.h"
#include "lyxfont.h"


class Painter;

/** A colapsable text inset
  
*/
class InsetCollapsable : public InsetText {
public:
    ///
    static int const TEXT_TO_TOP_OFFSET = 2;
    ///
    static int const TEXT_TO_BOTTOM_OFFSET = 2;
    ///
    InsetCollapsable(Buffer *);
    ///
    ~InsetCollapsable() {};
    ///
    Inset* InsetCollapsable::Clone();
    ///
    int ascent(Painter &, LyXFont const &) const;
    ///
    int descent(Painter &, LyXFont const &) const;
    ///
    int width(Painter &, LyXFont const &f) const;
    ///
    void draw(Painter &pain, const LyXFont &, int , float &);
    ///
    void Edit(int x, int y, unsigned int, BufferView *);
    ///
    void InsetUnlock();
    ///
    void UpdateLocal(bool flag=true);
    ///
    bool display() const { return (!collapsed); }
    ///
    void InsetButtonRelease(int, int, int);
    ///
    void InsetButtonPress(int, int, int);
    ///
    void InsetMotionNotify(int, int, int);
    ///
    void setLabel(LString const &l) { label = l; }
    ///
    void setFont(LyXFont &f) { labelfont = f; }
    ///
    void setAutoCollapse(bool f) { autocolapse = f; }

protected:
    ///
    int ascent_collapsed(Painter &, LyXFont const &) const;
    ///
    int descent_collapsed(Painter &, LyXFont const &) const;
    ///
    int width_collapsed(Painter &, LyXFont const &f) const;
    ///
    void draw_collapsed(Painter &pain, const LyXFont &, int , float &);
    
private:
    ///
    bool collapsed;
    ///
    LString label;
    ///
    LyXFont labelfont;
    ///
    bool autocolapse;
    ///
    int
        localInsetWidth,
        top_baseline, top_x,
	button_x;
};

#endif
