// -*- C++ -*-
/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 *
 *	     Copyright (C) 1995 Matthias Ettrich
 *           Copyright (C) 1995-1998 The LyX Team.
 *
 *======================================================
 */
// This is the rewrite of the tabular (table) support.

// It will probably be a lot of work.

// One first goal could be to make the inset read the old table format
// and just output it again... no viewing at all.

// When making the internal structure of tabular support I really think
// that STL containers should be used. This will separate the container from
// the rest of the code, which is a good thing.

// Ideally the tabular support should do as the mathed and use
// LaTeX in the .lyx file too.

// Things to think of when desingning the new tabular support:
// - color support (colortbl, color)
// - decimal alignment (dcloumn)
// - custom lines (hhline)
// - rotation
// - multicolumn
// - multirow
// - column styles

// This is what I have written about tabular support in the LyX3-Tasks file:
//
//  o rewrite of table code. Should probably be written as some
//          kind of an inset. At least get the code out of the kernel.
//                - colortbl  -multirow
//                - hhline    -multicolumn
//                - dcolumn
// o enhance longtable support

// Lgb

#ifndef INSETTABULAR_H
#define INSETTABULAR_H

#ifdef __GNUG__
#pragma interface
#endif

#include "lyxinset.h"
#include "lyxparagraph.h"
#include "table.h"
#include "LString.h"
#include "buffer.h"

class LyXLex;
class Painter;

class InsetTabular : public UpdatableInset {
public:
    ///
    InsetTabular(Buffer *, int rows=1, int columns=1);
    ///
    InsetTabular(InsetTabular const &, Buffer *);
    ///
    ~InsetTabular();
    ///
    InsetTabular* InsetTabular::Clone();
    ///
    void Read(LyXLex &);
    ///
    void Write(FILE *);
    ///
    int ascent(Painter &, LyXFont const &) const;
    ///
    int descent(Painter &, LyXFont const &) const;
    ///
    int width(Painter &, LyXFont const &f) const;
    ///
    void draw(Painter &pain, const LyXFont &, int , float &);
    ///
    string EditMessage() const;
    ///
    void Edit(int x, int y, unsigned int, BufferView *);
    ///
    void InsetUnlock();
    ///
    bool UnlockInsetInInset(Inset *, bool lr=false);
    ///
    void UpdateLocal(bool flag = true);
    ///
    bool UpdateInsetInInset(BufferView *, Inset *);
    ///
    bool display() const { return tabular->IsLongTable(); }
    ///
    void InsetButtonRelease(int, int, int);
    ///
    void InsetButtonPress(int, int, int);
    ///
    void InsetMotionNotify(int, int, int);
    ///
    void InsetKeyPress(XKeyEvent *);
    ///
    UpdatableInset::RESULT LocalDispatch(int, char const *);
    ///
    int Latex(FILE *, signed char);
    ///
    int Latex(string &, signed char);
    ///
    void Validate(LaTeXFeatures &features) const;
    ///
    Inset::Code LyxCode() const { return Inset::TABULAR_CODE; }
    ///
    void GetCursorPos(int &x, int &y);
    ///
    void ToggleInsetCursor();
    ///
    void TableFeatures(int feature, string val="");
    ///
    int GetActCell() { return actcell; }
    ///
    bool InsertInset(Inset *);
    ///
    void SetFont(LyXFont const &);
    ///
#warning Jurgen. Look here! I did, look yourself now
    ///
    // Can you ever imagine a table inset without a tabular?
    // why not make tabular _be_ a LyXTable and not a pointer to one.
    // (if you feel the need to "delete" the tabular, just introduce a
    // erase() method to it.

    // I imagine this question is from Lgb :), then imagine how you should
    // know here how large a LyXTable should be? I aggree that par could
    // not be a pointer, but does this really matter, we have to rewrite
    // this whole stuff so that LyXTable holds ALL of the information and
    // does all the moving around stuff so it really does not matter now!
    //   -- Jug --
    ///
    /// Public structures and variables
    ///
    LyXTable *tabular;
    ///
    LyXParagraph *par;

private:
    bool calculate_width_of_cells();
    ///
    int SingleWidth(LyXParagraph *par, int pos);
    ///
    void SingleHeight(LyXParagraph *par, int pos, int &, int &);
    ///
    LyXFont GetFont(LyXParagraph *par, int pos);
    ///
    int BeginningOfMainBody(LyXParagraph *par);
    ///
    void DrawCellLines(Painter &, int x, int baseline, int row, int cell);
    ///
    void ShowInsetCursor();
    ///
    void HideInsetCursor();
    ///
    void setPos(int x, int y, bool activate_inset=true);
    ///
    void setWidthOfCell(int pos, int cell, int row);
    ///
    bool moveRight(bool activate_inset=true);
    bool moveLeft(bool activate_inset=true);
    bool moveUp(bool activate_inset=true);
    bool moveDown(bool activate_inset=true);
    bool moveNextCell();
    bool movePrevCell();
    bool Delete();
    ///
    void resetPos();
    ///
    void RemoveTableRow();
    ///
    bool SimpleTeXOneTablePar(string &file, TexRow &texrow);
    bool TeXContTableRows(string &file, int i, int current_cell_number,
                          int &column, TexRow &texrow);
    bool hasCharSelection() {return (sel_pos_start != sel_pos_end);}
    bool hasCellSelection() {return hasCharSelection() &&
				 (sel_cell_start != sel_cell_end);}
    ///
    void SetCharFont(int pos, LyXFont font);
    ///
    /// Private structures and variables
    ///
    UpdatableInset
        *the_locking_inset;
    Buffer
        *buffer;
    LyXFont
        current_font,
        real_current_font;
    int
        inset_pos,
        inset_x, inset_y,
        sel_pos_start,
	sel_pos_end,
	sel_cell_start,
	sel_cell_end,
        old_x,
        cx, cy,
        actpos,
        actcell,
        actcol,
        actrow;
    bool
        no_selection;
};
#endif
