// -*- C++ -*-
/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 *
 *           Copyright (C) 1998 The LyX Team.
 *
 *======================================================
 */
// The pristine updatable inset: Text


#ifndef INSETTEXT_H
#define INSETTEXT_H

#ifdef __GNUG__
#pragma interface
#endif

#include "lyxinset.h"
#include "lyxparagraph.h"
#include "LString.h"
#include "buffer.h"

class Painter;

/** A text inset is like a TeX box
  
  To write full text (including styles and other insets) in a given
  space. 
*/
class InsetText : public UpdatableInset {
public:
	///
	enum { TEXT_TO_INSET_OFFSET = 1 };
    ///
    InsetText(Buffer *);
    ///
     InsetText(InsetText const &, Buffer *);
    ///
    ~InsetText();
    ///
    Inset* InsetText::Clone();
    ///
    void Read(LyXLex &);
    ///
    void Write(FILE *);
    ///
    int ascent(Painter &, LyXFont const &) const;
    ///
    int descent(Painter &, LyXFont const &) const;
    ///
    int width(Painter &, LyXFont const &f) const;
    ///
    int getMaxWidth(UpdatableInset *) const;
    ///
    void draw(Painter &pain, const LyXFont &, int , float &);
    ///
    string EditMessage() const;
    ///
    void Edit(int x, int y, unsigned int, BufferView *);
    ///
    void InsetUnlock();
    ///
    bool UnlockInsetInInset(Inset *, bool lr=false);
    ///
    void UpdateLocal(bool flag=true);
    ///
    bool UpdateInsetInInset(BufferView *, Inset *);
    ///
    void InsetButtonRelease(int, int, int);
    ///
    void InsetButtonPress(int, int, int);
    ///
    void InsetMotionNotify(int, int, int);
    ///
    void InsetKeyPress(XKeyEvent *);
    ///
    UpdatableInset::RESULT LocalDispatch(int, char const *);
    ///
    int Latex(FILE *, signed char);
    ///
    int Latex(string &, signed char);
    ///
    void Validate(LaTeXFeatures &features) const;
    ///
    Inset::Code LyxCode() const { return Inset::TEXT_CODE; }
    ///
    void GetCursorPos(int &x, int &y);
    ///
    int InsetInInsetY();
    ///
    void ToggleInsetCursor();
    ///
    bool InsertInset(Inset *);
    ///
    UpdatableInset * GetLockingInset();
    ///
    void SetFont(LyXFont const &);
    ///
    LyXParagraph *par;

protected:
    ///
    void WriteParagraphData(FILE *);
    ///
    void resetPos();
    ///
    void drawSelection(Painter &pain, int pos, int baseline, float x);
    ///
    void SingleHeight(LyXParagraph *par,int pos, int &asc, int &desc);
    ///
    int SingleWidth(LyXParagraph *par, int pos);
    ///
    LyXFont GetFont(LyXParagraph *par, int pos);
    ///
    Buffer *buffer;
    ///
    LyXFont current_font;
	///
        LyXFont real_current_font;
	///
	int maxWidth;
	///
        int maxAscent;
	///
	int maxDescent;
	///
	int insetWidth;

private:
    ///
    void drawRowSelection(Painter &, int startpos, int endpos, int row,
			  int baseline, float x);
    ///
    void drawRowText(Painter &, int startpos, int endpos, int baseline,
		     float x);
    ///
    void computeTextRows();
    ///
    void computeBaselines(int);
    ///
    int BeginningOfMainBody(LyXParagraph *par);
    ///
    void ShowInsetCursor();
    ///
    void HideInsetCursor();
    ///
    void setPos(int x, int y, bool activate_inset=true);
    ///
    bool moveRight(bool activate_inset = true);
    bool moveLeft(bool activate_inset = true);
    bool moveUp(bool activate_inset = true);
    bool moveDown(bool activate_inset = true);
    bool Delete();
    ///
    bool hasSelection() {return (selection_start != selection_end);}
    ///
    void SetCharFont(int pos, LyXFont font);
    ///
    string getText(int);
	
	/* Private structures and variables */
	///
	int inset_pos;
	///
        int inset_x;
	///
	int inset_y;
	///
        int interline_space;
	///
        int selection_start;
	///
	int selection_end;
	///
        int old_x;
	///
        int cx;
	///
	int cy;
	///
        int actpos;
	///
        int actrow;
	///
	bool no_selection;
	///
	UpdatableInset *the_locking_inset;
	///
	struct row_struct {
		///
		int asc;
		///
		int desc;
		///
		int pos;
		///
		int baseline;
	};
	///
	typedef vector<row_struct> RowList;
	///
	RowList rows;
	InsetText & operator = (InsetText const &it) {
		par = it.par;
		buffer = it.buffer; // suspect
		current_font = it.current_font;
		real_current_font = it.real_current_font;
		maxWidth = it.maxWidth;
		maxAscent = it.maxAscent;
		maxDescent = it.maxDescent;
		insetWidth = it.insetWidth;
		inset_pos = it.inset_pos;
		inset_x = it.inset_x;
		inset_y = it.inset_y;
		interline_space = it.interline_space;
		selection_start = it.selection_start;
		selection_end = it.selection_end;
		old_x = it.old_x;
		cx = it.cx;
		cy = it.cy;
		actpos = it.actpos;
		actrow = it.actrow;
		no_selection = it.no_selection;
		the_locking_inset = it.the_locking_inset; // suspect
		rows = it.rows;
		return *this;
	}
};
#endif
