// -*- C++ -*-
/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 * 	 
 *	    Copyright (C) 1995 Matthias Ettrich
 *          Copyright (C) 1995-1998 The LyX Team.
 *
 *======================================================*/

#ifndef LAYOUT_H
#define LAYOUT_H

#ifdef __GNUG__
#pragma interface
#endif

#include "definitions.h"
#include "lyxfont.h"
#include "Spacing.h"

class LyXLex;

/// The different margin types
enum LYX_MARGIN_TYPE {
	///
	MARGIN_MANUAL,
	///
	MARGIN_FIRST_DYNAMIC,
	///
	MARGIN_DYNAMIC,
	///
	MARGIN_STATIC,
	///
	MARGIN_RIGHT_ADDRESS_BOX
};

///
enum LYX_ALIGNMENT {
	///
	LYX_ALIGN_BLOCK = 1,
	///
	LYX_ALIGN_LEFT = 2,
	///
	LYX_ALIGN_RIGHT = 4,
	///
	LYX_ALIGN_CENTER = 8,
	///
	LYX_ALIGN_LAYOUT = 16,
	///
        LYX_ALIGN_WIDTH = 32
};

/// The different LaTeX-Types
enum LYX_LATEX_TYPES {
	///
	LATEX_PARAGRAPH,
	///
	LATEX_COMMAND,
	///
	LATEX_ENVIRONMENT,
	///
	LATEX_ITEM_ENVIRONMENT,
	///
	LATEX_LIST_ENVIRONMENT
};

/// The different label types
enum LYX_LABEL_TYPES {
	///
	LABEL_NO_LABEL,
	///
	LABEL_MANUAL,
	///
        LABEL_BIBLIO,
	///
	LABEL_TOP_ENVIRONMENT,
	///
	LABEL_CENTERED_TOP_ENVIRONMENT,
	
	// the flushright labels following now must start with LABEL_STATIC
	///
	LABEL_STATIC,
	///
	LABEL_SENSITIVE,
	///
	LABEL_COUNTER_CHAPTER,
	///
	LABEL_COUNTER_SECTION,
	///
	LABEL_COUNTER_SUBSECTION,
	///
	LABEL_COUNTER_SUBSUBSECTION,
	///
	LABEL_COUNTER_PARAGRAPH,
	///
	LABEL_COUNTER_SUBPARAGRAPH,
	///
	LABEL_COUNTER_ENUMI,
	///
	LABEL_COUNTER_ENUMII,
	///
	LABEL_COUNTER_ENUMIII,
	///
	LABEL_COUNTER_ENUMIV,
	///
	LABEL_FIRST_COUNTER = LABEL_COUNTER_CHAPTER
};


/*
 * Fix labels are printed flushright, manual labels flushleft. 
 * MARGIN_MANUAL and MARGIN_FIRST_DYNAMIC are *only* for LABEL_MANUAL,
 * MARGIN_DYNAMIC and MARGIN_STATIC are *not* for LABEL_MANUAL. 
 * This seems a funny restriction, but I think other combinations are
 * not needed, so I will not change it yet. 
 * Correction: MARGIN_FIRST_DYNAMIC also usable with LABEL_STATIC
 */


/*
 * There is a parindent and a parskip. Which one is used depends on the 
 * paragraph_separation-flag of the text-object. 
 * BUT: parindent is only thrown away, if a parskip is defined! So if you
 * want a space between the paragraphs and a parindent at the same time, 
 * you should set parskip to zero and use topsep, parsep and bottomsep.
 * 
 * The standard layout is an exception: its parindent is only set, if the 
 * previous paragraph is standard too. Well, this is LateX and it is good!
 */ 

// The order of the LayoutTags enum is no more important. [asierra300396]
/* tags indexes */
enum _LayoutTags {
	LT_ALIGN, LT_ALIGNPOSSIBLE, LT_MISC,
	LT_BLOCK, LT_MARGIN,
	LT_BOTTOMSEP, LT_CENTER, LT_CENTERED_TOP_ENVIRONMENT, LT_COLUMNS,
	LT_COPYSTYLE, LT_OBSOLETEDBY,
	LT_COMMAND, LT_COUNTER_CHAPTER, LT_COUNTER_ENUMI, LT_COUNTER_ENUMII,
	LT_COUNTER_ENUMIII, LT_COUNTER_ENUMIV, LT_COUNTER_PARAGRAPH,
	LT_COUNTER_SECTION, LT_COUNTER_SUBPARAGRAPH, LT_COUNTER_SUBSECTION,
	LT_COUNTER_SUBSUBSECTION, LT_DEFAULTFONT, LT_DYNAMIC, LT_EMPTY,
	LT_END, LT_ENVIRONMENT, LT_ENVIRONMENT_DEFAULT, LT_FAMILY,
	LT_FANCYHDR, LT_FILL_BOTTOM, LT_FILL_TOP, LT_FIRST_COUNTER,
	LT_FIRST_DYNAMIC, LT_FONT, LT_FREE_SPACING, LT_HEADINGS, LT_INPUT,
	LT_ITEM_ENVIRONMENT, LT_ITEMSEP, LT_KEEPEMPTY,
	LT_LABEL_BOTTOMSEP, LT_LABELFONT, LT_TEXTFONT,
	LT_LABELINDENT, LT_LABELSEP, LT_LABELSTRING, LT_LABELSTRING_APPENDIX, LT_LABELTYPE,
	LT_LATEXNAME, LT_LATEXPARAM, LT_LATEXTYPE, LT_LAYOUT, LT_LEFT,
	LT_LEFTMARGIN,
	LT_LIST_ENVIRONMENT , LT_MANUAL, LT_MAXCOUNTER, 
	LT_NEED_PROTECT, LT_NEWLINE,
	LT_NEXTNOINDENT, LT_NO_LABEL, LT_NOSTYLE,
	LT_PAGESTYLE, LT_PARAGRAPH,
	LT_PARINDENT, LT_PARSEP, LT_PARSKIP, LT_PLAIN, LT_PREAMBLE, LT_RIGHT,
	LT_RIGHT_ADDRESS_BOX, LT_RIGHTMARGIN, LT_SENSITIVE, LT_SIDES,
	LT_SPACING, LT_SPACING_SINGLE, LT_SPACING_ONEHALF,
	LT_SPACING_DOUBLE, LT_OTHER,  LT_CLASSOPTIONS, LT_FONTSIZE,
	LT_STATIC, LT_STYLE, LT_TOP_ENVIRONMENT, LT_TOPSEP, LT_BIBLIO,
        LT_INTITLE, LT_SECNUMDEPTH, LT_TOCDEPTH,
	LT_LAST
};

/// Attributes of a layout/paragraph environment
class LyXTextClass;

///
class LyXLayout {
public:
	///
	LyXLayout();

	///
	~LyXLayout() { }

	///
	LyXLayout & operator=(LyXLayout const &l);

	///
	bool read(LyXLex&, LyXTextClass const&);

	/// Name of the layout/paragraph environment
	string name;

	/** Name of an layout that has replaced this layout. This is
	  used to rename a layout, while keeping backward compatibility 
	*/
	string obsoleted_by;

	/** Default font for this layout/envrionment. The main font for
	  this kind of environment. If an attribute has LyXFont::INHERITED_*,
	  it means that the value is specified by the defaultfont for the
	  entire layout. If we are nested, the font is inherited from the
	  font in the environment one level up until the font is resolved.
	  The values LyXFont::IGNORE_* and LyXFont::TOGGLE are illegal here.
	*/
	LyXFont font;

	/** Default font for labels. Interpretation the same as
	  for font above */
	LyXFont labelfont;

	/** This is a resolved version the default font. The font is resolved
	against the defaultfont of the entire layout. */
	LyXFont resfont;

	/** This is a resolved version the label font. The font is resolved
	against the defaultfont of the entire layout. */
	LyXFont reslabelfont;

	/// Text that dictates how wide the left margin is on the screen
	string leftmargin;

	/// Text that dictates how wide the right margin is on the screen
	string rightmargin;

	/// Text that dictates how much space to leave after a potential label
	string labelsep;

	/// Text that dictates how much space to leave before a potential label
	string labelindent;

	/** Text that dictates the width of the indentation of
	  indented paragraphs */
	string parindent;

	/// Label string. "Abstract", "Reference", "Caption"...
	string labelstring;

	/// Label string inside appendix. "Appendix", ...
	string labelstring_appendix;

	/// LaTeX name for environment
	string latexname;

	/// LaTeX parameter for environment
        string latexparam;   //arrae970411

        /// Macro definitions needed for this layout
	string preamble;

	///
	float parskip;

	///
	float itemsep;

	///
	float topsep;

	///
	float bottomsep;

	///
	float labelbottomsep;

	///
	float parsep;

	///
	Spacing spacing;

	///
	char align; // add approp. signedness

	///
	char alignpossible; // add approp. signedness

	///
	char labeltype; // add approp. signedness

	///
	char margintype; // add approp. signedness

	///
	bool fill_top;

	///
	bool fill_bottom;

	///
	bool newline_allowed;

	///
	bool nextnoindent;

	///
	bool free_spacing;
	
	/** true when the fragile commands in the paragraph need to be
	  \protect'ed. */
	bool needprotect;
	
	/// true when empty paragraphs should be kept.
	bool keepempty;
	
	///
	bool isParagraph() const {
		return latextype == LATEX_PARAGRAPH;
	}
	
	///
	bool isCommand() const { 
	    return latextype == LATEX_COMMAND;
	}
	
	///
	bool isEnvironment() const {
		return (latextype == LATEX_ENVIRONMENT
			|| latextype == LATEX_ITEM_ENVIRONMENT
			|| latextype == LATEX_LIST_ENVIRONMENT);
	}
	
	/// Type of LaTeX object
	LYX_LATEX_TYPES latextype;
	
	/// Does this object belong in the title part of the document?
	bool intitle;
};

#endif
