// -*- C++ -*-
/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 * 	 
 *	    Copyright (C) 1995 Matthias Ettrich
 *          Copyright (C) 1995-1998 The LyX Team
 *
 *======================================================*/
#ifndef LYXSCREEN_H
#define LYXSCREEN_H

#ifdef __GNUG__
#pragma interface
#endif

#include FORMS_H_LOCATION
#include <X11/Xlib.h>

class LyXText;
class BufferView;
struct Row;
typedef unsigned short Dimension;

/** The class LyXScreen is used for the main text body.
    Concretely, the screen is held in a pixmap owner by a BufferView.  
    This class takes care to keep the pixmap up to date and used to 
    optimize drawing on the screen.
    This class also handles the drawing of the cursor.
 */
class LyXScreen {
	friend class BufferView;
	
	///
	LyXScreen(BufferView * own,
		  Window window,
		  Pixmap pix,
		  Dimension width, 
		  Dimension height,
		  Dimension offset_x,
		  Dimension offset_y,
		  LyXText *text_ptr);
	///
	~LyXScreen();

	/** Draws the screen form textposition y. Uses as much of
	  the already printed pixmap as possible */
	void Draw(long y);

	/// Redraws the screen, without using existing pixmap
	void Redraw();
   
	/// Returns a new top so that the cursor is visible
	long TopCursorVisible();

	/** Redraws the screen such that the cursor is visible
	 * Returns true if we moved to get it visible.
	 * Will not draw the cursor */
	bool FitCursor();
	/// If the cursor is on, this will draw it
	void showCursor();
	/// If the cursor is off, this will remove it
	void hideCursor();
	/// This will toggle the state of the cursor and draw as needed
	void CursorToggle();
	/// This will draw a cursor unconditionally at requested point
	void ShowManualCursor(long x, long y, int asc, int desc);
	/// This will remove a cursor unconditionally
	void HideManualCursor(long x, long y, int asc, int desc);
	/** Make sure a cursor at given point is visible.
	  * If the screen is moved to accomplish this, we return true */
	bool FitManualCursor(long, long, int, int);
	/// Redraw selection.  If parameter is true, the selection is removed
	void toggleSelection(bool = true);
	/// Redraw area between toggle_cursor and toggle_end_cursor.
	void ToggleToggle();
	
	/** Updates part of the screen. If text->status is
	  LyXText::NEED_MORE_REFRESH, we update from the
	  point of change and to the end of the screen.
	  If text->status is LyXText::NEED_VERY_LITTLE_REFRESH,
	  we only update the current row. */
	void Update();

	/** Updates part of the screen. Updates till row with cursor,
	  or only current row */
	void SmallUpdate();
	
	///
	//int getWidth() const { return width_; }

	/// first visible pixel-row
	long first;

	///
	bool cursor_visible;

	/// Copies specified area of pixmap to screen
	void expose(int x, int y, int exp_width, int exp_height); 

	/// y1 and y2 are coordinates of the screen
	void DrawFromTo(int y1, int y2);
   
	/// y is a coordinate of the text
	void DrawOneRow(Row* row, long &y_text);

	///
	Pixmap cursor_pixmap;
	///
	int cursor_pixmap_x;
	///
	int cursor_pixmap_y;
	///
	int cursor_pixmap_w;
	///
	int cursor_pixmap_h;
	///
	BufferView * owner;
	///
	Window window_;
	///
	Pixmap pixmap_;
        ///
	Dimension width_;
	///
	Dimension height_;
        ///
	Dimension offset_x_;
	///
	Dimension offset_y_;

	///
	LyXText *text;

	///
	long screen_refresh_y;
	///
	Row *screen_refresh_row;
	///
	friend class InsetFormula;
	///
	GC gc_copy;

};
#endif
