// -*- C++ -*-
/* This file is part of
 * ======================================================
* 
*           LyX, The Document Processor
* 	 
*	    Copyright (C) 1995 Matthias Ettrich
*           Copyright (C) 1995-1998 The LyX Team.
*
*======================================================*/

#ifndef TEXTCLASS_H
#define TEXTCLASS_H

#ifdef __GNUG__
#pragma interface
#endif

#include <vector>

#include "LString.h"
#include "lstrings.h"
#include "layout.h"

///
class LyXTextClass  {
public:
	///
	typedef vector<LyXLayout> LayoutList;
	
	///
	LyXTextClass (string const &fn = string(), 
		      string const &cln = string(), 
		      string const &desc = string());
	///
	~LyXTextClass() { }

	///
	LyXTextClass & operator=(LyXTextClass const &tc) {
		name = tc.name;
		latexname = tc.latexname;
		description = tc.description;
		opt_fontsize = tc.opt_fontsize;
		opt_pagestyle = tc.opt_pagestyle;
		defaultfont = tc.defaultfont;
		columns = tc.columns;
		sides = tc.sides;
		secnumdepth = tc.secnumdepth;
		tocdepth = tc.tocdepth;
		pagestyle = tc.pagestyle;
		preamble = tc.preamble;
		leftmargin = tc.leftmargin;
		rightmargin = tc.rightmargin;
		maxcounter = tc.maxcounter;
		layoutList = tc.layoutList;
		loaded = tc.loaded;
		layoutList = tc.layoutList;
		return *this;
		
	}

	/// Erase a layout.
	bool eraseLayout(string const &name);
	
	/** This is used to sort textclasses in lexicographic order of
	  description. This kind of class/object is called a
	  "function-like object" */
	class compare_desc {
	public:
		///
		bool operator()(LyXTextClass const &tc1, 
				LyXTextClass const &tc2) const
		{
			return compare_no_case(tc1.description,
					       tc2.description)<0;
		}
	};

	/// Reads a textclass structure from file
	int read(string const & filename);

	/// Sees to that the textclass structure has been loaded
	void load();

	///
	string name;
	///
	string latexname;
	///
	string description;

	/// Specific class options
        string opt_fontsize;
	///
        string opt_pagestyle;
	///
	string options;
    
	/** Base font. This one has to be fully instantiated.
	  The paragraph and layout fonts are resolved against
	  this font. Attributes LyXFont::INHERIT, LyXFont::IGNORE, and 
	  LyXFont::TOGGLE are extremely illegal.
	  */
	LyXFont defaultfont;
	
	///
#warning check type
	unsigned int numLayouts() const;
	///
	LyXLayout const * getLayout(unsigned char layout) const;
	///
	LyXLayout const * getLayout(string const & name) const;
	///
#warning check type
	short getNumberOfLayout(string const &name) const;

	///
	string getNameOfLayout(unsigned char layout) const;

	///
	string getCompletePreamble() const;
	///
#warning check type
	short columns;
	///
#warning check type
	short sides;
	///
#warning check type
	short secnumdepth;
	///
#warning check type
	short tocdepth;
	///
	string pagestyle;
	///
	string preamble;
	/// Text that dictates how wide the left margin is on the screen
	string leftmargin;
	/// Text that dictates how wide the right margin is on the screen
	string rightmargin;
        ///
#warning check type
	short maxcounter; // add approp. signedness
private:
	/// Non-const method is private.
	LyXLayout * getLayout(string const &name);
	///
	LayoutList layoutList;
	/// Has this layout file been loaded yet?
	bool loaded;
};

#endif
