// -*- C++ -*-
/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 * 	 
 *	    Copyright (C) 1995 Matthias Ettrich
 *          Copyright (C) 1995-1998 The LyX Team
 *
 * ======================================================*/

#ifndef TEXTCLASSLIST_H
#define TEXTCLASSLIST_H

#ifdef __GNUG__
#pragma interface
#endif

#include "textclass.h"


///
class LyXTextClassList {
public:
	///
	typedef vector<LyXTextClass> TextClassList;
	///
	LyXTextClassList() { }

	///
	~LyXTextClassList() { }

	///
	LyXTextClassList & operator=(LyXTextClassList const &ltc)
	{
		classList = ltc.classList;
		return *this;
	}
	
	/// Gets layout structure from layout number and textclass number
	LyXLayout const *style(unsigned char textclass,
			       unsigned char layout) const;

	/// Gets layout number from textclass number and layout name
	signed char numberOfLayout(unsigned char textclass,
				   string const & name) const;

	///
	unsigned int size() const {
		return classList.size();
	}

	/// Gets a layout name from layout number and textclass number
	string nameOfLayout(unsigned char textclass,
			     unsigned char layout) const;

	/** Gets textclass number from name. Returns -1 if textclass
	  name does not exist */
	signed char numberOfClass(string const & textclass) const;

	///
	string nameOfClass(unsigned char number) const;

	///
	string latexNameOfClass(unsigned char number) const;

	///
	string descOfClass(unsigned char number) const;

	///
	LyXTextClass const * textClass(unsigned char textclass) const;

	/// Reads the style files.
	void readClasses();

	/** Load textclass. Returns false if this fails */
	bool load(unsigned char const number);
private:
	/** Read textclass list. Returns false if this fails */
	bool read();
	
	/** Mutable so that const methods can do a load
	  on the textclasses. */
	mutable TextClassList classList;
};

extern LyXTextClassList textclasslist;
 
#endif
