// -*- C++ -*-
/* This file is part of
 * ======================================================
* 
*           LyX, The Document Processor
*	    Copyright (C) 1995 Matthias Ettrich
*
*           This file is Copyright (C) 1996-1998
*           Lars Gullik Bjnnes
*
*======================================================*/

#ifndef _TOOLBARBACKEND_H_
#define _TOOLBARBACKEND_H_

#ifdef __GNUG__
#pragma interface
#endif

#include <vector> // STL vector
#include "lyxfunc.h"
#include "lyxlex.h"

/** The GUI independant part of the toolbar code.
    This holds enough information to build the visible toolbar.
*/
class ToolbarBackend {
public:
	///
	ToolbarBackend()
	{
		reset();
		init(); // set default toolbar.
		dump();
	}

	///
	~ToolbarBackend() {
	}

	///
	ToolbarBackend & operator=(const ToolbarBackend & tb) {
		toollist = tb.toollist;
		return *this;
	}
	
	void dump();
	///
	int get_toolbar_func(string const & func);
	
        /// The special toolbar actions
	enum TOOLBARITEMS {
		/// none
                TOOL_NONE,
		/// adds space between buttons in the toolbar
		TOOL_SEPARATOR,
		/// a special combox insead of a button
		TOOL_LAYOUTS,
		/// begin a new line of button (not working)
		TOOL_NEWLINE,
		/// a regular lyxfunc
		TOOL_FUNC
	};

	///
	void read(LyXLex&);
	/// sets up the default toolbar
	void init();

	/// add a new button to the toolbar.
    	void add(TOOLBARITEMS bi, int action = LFUN_NOACTION);
	///
	void clean();
private:
	///
	struct toolbarFunc
	{
		///
		TOOLBARITEMS baritem;
		///
		int action;
		///
		toolbarFunc(){
			baritem = TOOL_NONE;
			action = LFUN_NOACTION;
		}
		///
		~toolbarFunc(){
		}
		///
		toolbarFunc & operator=(const toolbarFunc & tf) {
			baritem = tf.baritem;
			action = tf.action;
			return *this;
		}
	};
public:
	typedef vector<toolbarFunc> FuncList;
private:
	///
	friend class Toolbar;
	///
	enum tooltags_ {
		TO_ADD = 1,
		TO_ENDTOOLBAR,
		TO_SEPARATOR,
		TO_LAYOUTS,
		TO_NEWLINE,
		TO_LAST
	};
	
	/// a list containing all the functions in the toolbar
	FuncList toollist;

	/** more...
	 */
	void reset(){
		// G++ vector does not have clear defined
		//toollist.clear();
		toollist.erase(toollist.begin(), toollist.end());
	}
};
#endif
