/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 * 	 
 *	    Copyright (C) 1995 Matthias Ettrich
 *          Copyright (C) 1995-1998 The LyX Team.
 *
 *======================================================*/

#include <config.h>

#ifdef __GNUG__
#pragma implementation "lyxinset.h"
#endif

#include "lyxinset.h"
#include "debug.h"
#include "lyxfont.h"
#include "BufferView.h"
#include "lyxscreen.h"
#include "commandtags.h"
#include "lstrings.h"

// 	$Id: inset.C,v 1.7 1998/10/08 14:25:49 larsbj Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: inset.C,v 1.7 1998/10/08 14:25:49 larsbj Exp $";
#endif /* lint */

/* Insets default methods */

bool Inset::Deletable() const
{
	return true;
}


Inset::EDIT Inset::Editable() const
{
	return Inset::NOT_EDITABLE;
}


void Inset::Validate(LaTeXFeatures &) const
{
	// nothing by default
}


bool Inset::AutoDelete() const
{
	return false;
}


void Inset::Edit(int, int, unsigned int, BufferView *)
{
}


LyXFont Inset::ConvertFont(LyXFont font)
{
	return font;
}

 /* some stuff for inset locking */

void UpdatableInset::InsetButtonPress(int x, int y, int button)
{
	lyxerr[Debug::INFO] << "Inset Button Press x=" << x
			    << ", y=" << y
			    << ", button=" << button << '\n';
}


void UpdatableInset::InsetButtonRelease(int x, int y, int button)
{
	lyxerr[Debug::INFO] << "Inset Button Release x=" << x
			    << ", y=" << y
			    << ", button=" << button << '\n';
}


void UpdatableInset::InsetKeyPress(XKeyEvent *)
{
	lyxerr[Debug::INFO] << "Inset Keypress\n";
}


void UpdatableInset::InsetMotionNotify(int x, int y, int state)
{
	lyxerr[Debug::INFO] << "Inset Motion Notify x=" << x
			    << ", y=" << y
			    << ", state=" << state << '\n';
}


void UpdatableInset::InsetUnlock()
{
	scx = 0;
	lyxerr[Debug::ANY] << "Inset Unlock\n";
}


// An updatable inset is highly editable by definition
Inset::EDIT UpdatableInset::Editable() const
{
	return Inset::HIGHLY_EDITABLE;
}


void UpdatableInset::ToggleInsetCursor()
{
}


void UpdatableInset::scroll(float s)
{
    /*    if (abs(scx) > 1) 
	scx = s;
	else*/ {
	    //	if (!display()) s = 2*s;
	scx = int(s*mx_scx);
	//     if (scx > mx_scx) scx = mx_scx;
	//     if (scx < -mx_scx) scx = -mx_scx;
	if (!display()) scx += 20;
    }
    UpdateLocal(false);
}


void UpdatableInset::Edit(int, int, unsigned int, BufferView *bv)
{
    LyXFont
	font;
    bview = bv;
    scx = 0;

    mx_scx=abs((width(bv->getPainter(), font) - bv->paperWidth())/2);
}


void UpdatableInset::draw(Painter &, LyXFont const &, int baseline, float &x)
{
    if (scx) x += (float)scx;
    top_x = int(x);
    top_baseline = baseline;
}


void UpdatableInset::UpdateLocal(bool /*flag*/)
{
    LyXFont
	font;

    mx_scx=abs((width(bview->getPainter(),font)-bview->paperWidth())/2);
    bview->updateInset(this, true);
}


///  An updatable inset could handle lyx editing commands
UpdatableInset::RESULT UpdatableInset::LocalDispatch(int action, char const *arg) 
{
    if (action==LFUN_SCROLL_INSET)
	{
	    float xx;
	    sscanf(arg, "%f", &xx);	
	    scroll(xx);

	    return DISPATCHED;
	}

    return UNDISPATCHED; 
}


