
#include <config.h>

#ifdef __GNUG__
#pragma implementation
#endif

#include "insetbibkey.h"
#include "LyXView.h"
#include "LString.h"
#include "lstrings.h"
#include "buffer.h"


InsetBibKey::InsetBibKey(string const & key, string const & label):
	InsetCommand("bibitem", key, label)
{
	counter = 1;
	if (key.empty())
		contents = ' ';
}


InsetBibKey::InsetBibKey(InsetBibKey const *b):
	InsetCommand("bibitem", b->contents, b->options)
{
	counter = b->counter;
}


InsetBibKey::~InsetBibKey()
{
	gone();
}


void InsetBibKey::setCounter(int c) 
{ 
	counter = c; 
    
	if (contents.empty())
		contents += counter;
}


// I'm sorry but this is still necessary because \bibitem is used also
// as a LyX 2.x command, and lyxlex is not enough smart to understand
// real LaTeX commands. Yes, that could be fixed, but would be a waste 
// of time cause LyX3 won't use lyxlex anyway.  (ale)
void InsetBibKey::Write(FILE *file)
{
	string s;
	if (!options.empty()) {
		s += '[';
		s += options + ']';
	}
	s += '{';
	s += contents + '}';
	fprintf(file,"\\bibitem %s\n", s.c_str());
}


string InsetBibKey::getScreenLabel() const
{
	if (!options.empty())
		return options;
    
	return tostr(counter);
}


/*NB: the following comment should no longer apply (fingers crossed) ARRae 990103
  The value in "Key:" isn't allways set right after a few bibkey insets have
  been added/removed.  Perhaps the wrong object is deleted/used somewhere upwards?
  (Joacim 1998-03-04)
*/
void InsetBibKey::Edit(int, int, unsigned int, BufferView *bv)
{
	bv->getOwner()->getPopups().showFormBibkey(this);
}

////////////////////////////////////////////////////////////////////////////////

// ale070405 This function maybe shouldn't be here. We'll fix this at 0.13.
// ARRae990105 Maybe these two should be in Buffer but then again they're only helpers
int bibitemMaxWidth(Buffer const * buffer, Painter & pain, const class LyXFont &font)
{
	int w = 0;
	// Does look like a hack? It is! (but will change at 0.13)
	LyXParagraph *par = buffer->paragraph;
    
	while (par) {
		if (par->bibkey) {
			int wx = par->bibkey->width(pain, font);
			if (wx>w) w = wx;
		}
		par = par->next;
	}
	return w;
}

// a bit difficult to get rid of this one.
extern BufferView *current_view;

// ale070405 
string bibitemWidthest()
{
	int w = 0;
	// Does look like a hack? It is! (but will change at 0.13)
	LyXParagraph *par = current_view->currentBuffer()->paragraph;
	InsetBibKey *bkey=0;
	LyXFont font;
      
	while (par) {
		if (par->bibkey) {
			int wx = font.stringWidth(par->bibkey->getOptions());

			if (wx > w) {
				w = wx;
				bkey = par->bibkey;
			}
		}
		par = par->next;
	}
    
	if (bkey)
		return bkey->getOptions();
    
	return string();
}
