/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 * 	 
 *          Copyright (C) 1998 The LyX Team.
 *
 *======================================================*/

#include <config.h>

#ifdef __GNUG__
#pragma implementation
#endif

#include "insetcollapsable.h"
#include "gettext.h"
#include "lyxfont.h"
#include "BufferView.h"
#include "painter.h"
#include "lyx_gui_misc.h"


InsetCollapsable::InsetCollapsable(Buffer *bf): InsetText(bf)
{
    collapsed = true;
    label = "Foot";
    autocolapse = true;
}


Inset* InsetCollapsable::Clone()
{
    Inset *result = new InsetCollapsable(buffer);
    return result;
}


int InsetCollapsable::ascent_collapsed(Painter & pain, LyXFont const &) const
{
    int width = 0, ascent = 0, descent = 0;
    pain.buttonText(0, 0, label.c_str(), labelfont, false, 
		    width, ascent, descent);
    return ascent;
}


int InsetCollapsable::descent_collapsed(Painter & pain, LyXFont const &) const
{
    int width = 0, ascent = 0, descent = 0;
    pain.buttonText(0, 0, label.c_str(), labelfont, false, 
		    width, ascent, descent);
    return descent;
}


int InsetCollapsable::width_collapsed(Painter & pain, LyXFont const &) const
{
    int width, ascent, descent;
    pain.buttonText(TEXT_TO_INSET_OFFSET, 0, label.c_str(), labelfont, false,
		    width, ascent, descent);
    return width + (2*TEXT_TO_INSET_OFFSET);
}


int InsetCollapsable::ascent(Painter & pain, LyXFont const &font) const
{
    if (collapsed) 
	return ascent_collapsed(pain, font);
    else 
	return InsetText::ascent(pain, font) + TEXT_TO_TOP_OFFSET;
}


int InsetCollapsable::descent(Painter & pain, LyXFont const &font) const
{
    if (collapsed) 
	return descent_collapsed(pain, font);
    else 
	return InsetText::descent(pain, font) + TEXT_TO_BOTTOM_OFFSET;
}


int InsetCollapsable::width(Painter & pain, LyXFont const &font) const
{
    if (collapsed) 
	return width_collapsed(pain, font);

    return localInsetWidth;
}


void InsetCollapsable::draw_collapsed(Painter &pain, LyXFont const &,
				      int baseline, float &x)
{
    int width = 0;
    pain.buttonText(int(x) + TEXT_TO_INSET_OFFSET, baseline,
		    label.c_str(), labelfont, true, width);
    x += width + (2 * TEXT_TO_INSET_OFFSET);
}


void InsetCollapsable::draw(Painter &pain, LyXFont const &f, 
			    int baseline, float &x)
{
    if (collapsed) {
	draw_collapsed(pain, f, baseline, x);
	return;
    }
    top_x = int(x);
    top_baseline = baseline;
    draw_collapsed(pain, f, baseline, x);
    button_x = int(x - top_x);
    // I think this is fundamentally wrong, we should not go to the topmost
    // inset (BufferView) to get informatin about the immidiate inset
    // (the inset containg this inset). There should rather be passed an owner
    // parameter.
    
    //localInsetWidth = pain.getOwner()->getMaxWidth(this);
    localInsetWidth = pain.paperWidth(); // works for most, except where this inset is inside a locking inset.
    
    maxWidth = localInsetWidth - button_x;
    x += 2;
    int
	w = maxWidth - 4,
	h = ascent(pain,f) + descent(pain,f);
    
    pain.rectangle(int(x), baseline - ascent(pain, f), w, h,
		   LColor::footnoteframe);

    x += 2;
    top_x = int(x - top_x);
    InsetText::draw(pain, f, baseline, x);
}


void InsetCollapsable::Edit(int x, int y, unsigned int button, BufferView *bv)
{
    bview = bv;
    localInsetWidth = bv->getMaxWidth(this);
    if (collapsed) {
	collapsed = false;
	UpdateLocal();
	InsetText::Edit(0, 0, button, bv);
    } else {
	InsetText::Edit(x-top_x, y, button, bv);
    }
}


void InsetCollapsable::InsetUnlock()
{
    if (autocolapse) {
	collapsed = true;
    }
    InsetText::InsetUnlock();
    UpdateLocal();
}

    
void InsetCollapsable::UpdateLocal(bool flag)
{
    maxWidth = bview->getMaxWidth(this) - 
	width_collapsed(bview->getPainter(), labelfont);
    InsetText::UpdateLocal(flag);
}


void InsetCollapsable::InsetButtonPress(int x, int y, int button)
{
    if ((x < button_x)  &&
	(y < (labelfont.maxDescent()+labelfont.maxAscent()))) {
	collapsed = true;
	UpdateLocal();
        bview->unlockInset(this);
    } else if (x >= button_x)
	InsetText::InsetButtonPress(x-top_x, y, button);
}


void InsetCollapsable::InsetButtonRelease(int x, int y, int button)
{
    if (x >= button_x)
	InsetText::InsetButtonRelease(x-top_x, y, button);
}


void InsetCollapsable::InsetMotionNotify(int x, int y, int button)
{
    if (x >= button_x)
	InsetText::InsetMotionNotify(x-top_x, y, button);
}
