/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 * 	 
 *	    Copyright (C) 1995 Matthias Ettrich
 *          Copyright (C) 1995-1998 The LyX Team.
 *
 *======================================================*/

#include <config.h>

#ifdef __GNUG__
#pragma implementation
#endif

#include "inseterror.h"
#include "gettext.h"
#include "lyx_gui_misc.h" // CancelCloseBoxCB
#include "lyxfont.h"
#include "painter.h"

// 	$Id: inseterror.C,v 1.9 1998/10/12 20:58:57 larsbj Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: inseterror.C,v 1.9 1998/10/12 20:58:57 larsbj Exp $";
#endif /* lint */

/* Error, used for the LaTeX-Error Messages */

InsetError::InsetError()
{
	form = 0;
}


InsetError::InsetError(string const & str)
	: contents(str)
{
	form = 0;
}


InsetError::~InsetError()
{
	if (form) {
		fl_hide_form(form);
		fl_free_form(form);
		form = 0;
	}
}


int InsetError::ascent(Painter & pain, LyXFont const &) const
{
	int width, ascent, descent;
	LyXFont font(LyXFont::ALL_SANE);
	font.decSize();
	pain.buttonText(0, 0, _("Error"), font, false, width, ascent, descent);
	return ascent;
}


int InsetError::descent(Painter & pain, LyXFont const &) const
{
	int width, ascent, descent;
	LyXFont font(LyXFont::ALL_SANE);
	font.decSize();
	pain.buttonText(0, 0, _("Error"), font, false, width, ascent, descent);
	return descent;
}


int InsetError::width(Painter & pain, LyXFont const &) const
{
	int width, ascent, descent;
	LyXFont font(LyXFont::ALL_SANE);
	font.decSize();
	pain.buttonText(0, 0, _("Error"), font, false, width, ascent, descent);
	return width;
}


void InsetError::draw(Painter &pain, LyXFont const &, int baseline, float &x)
{
	// Draw as "Error" as a button
	LyXFont font(LyXFont::ALL_SANE);
	font.decSize();
	font.setColor(LColor::error);
	int width;
	pain.buttonText(int(x), baseline, _("Error"), font, true, width);
	x += width;
}


void InsetError::Write(FILE *)
{
}

void InsetError::Read(LyXLex &)
{
}


int InsetError::Latex(FILE *, signed char /*fragile*/)
{
	return 0;
}


int InsetError::Latex(string &, signed char /*fragile*/)
{
	return 0;
}


bool InsetError::AutoDelete() const
{
	return true;
}


Inset::EDIT InsetError::Editable() const
{
	return Inset::EDITABLE;
}


void InsetError::CloseErrorCB(FL_OBJECT *, long data)
{
	InsetError *inset = (InsetError*) data;
	if (inset->form) {
		fl_hide_form(inset->form);
		fl_free_form(inset->form);
		inset->form = 0;
	}
}

void InsetError::Edit(int, int, unsigned int, BufferView *)
{
	if (!form) {
		FL_OBJECT *obj;
		form = fl_bgn_form(FL_UP_BOX,400,240);
		strobj = fl_add_box(FL_FRAME_BOX,10,10,380,180,"");
		fl_set_object_color(strobj,FL_MCOL,FL_MCOL);
		fl_set_object_gravity(strobj, FL_NorthWest, FL_SouthEast);
		obj = fl_add_button(FL_RETURN_BUTTON,140,200,120,30,_("Close"));
		fl_set_object_callback(obj, CloseErrorCB, (long)this);
		fl_set_object_gravity(obj, FL_South, FL_South);
		fl_set_object_resize(obj, FL_RESIZE_NONE);
		fl_end_form();
		fl_set_form_atclose(form, CancelCloseBoxCB, 0);
	}
	fl_set_object_label(strobj, contents.c_str());
	if (form->visible) {
		fl_raise_form(form);
	} else {
		fl_show_form(form,FL_PLACE_MOUSE | FL_FREE_SIZE, FL_FULLBORDER, 
			     _("LaTeX Error"));
	}
}


Inset* InsetError::Clone()
{
	InsetError *result = new InsetError(contents);
	return result;
}
