// -*- C++ -*-
/* This file is part of*
 * ======================================================
 *
 *           LyX, The Document Processor
 * 	 
 *	    Copyright (C) 1995 Matthias Ettrich
 *
 *======================================================*/

#ifndef _INSET_ERROR_H
#define _INSET_ERROR_H

#ifdef __GNUG__
#pragma interface
#endif

#include FORMS_H_LOCATION
#include "lyxinset.h"
#include "LString.h"
#include "gettext.h"

/** Used for error messages from LaTeX runs.
  
  The edit-operation opens a 
  dialog with the text of the error-message. The inset is displayed as 
  "Error" in a box, and automatically deleted. */
class InsetError: public Inset {
public:
	///
	InsetError(string const & string);
	///
	InsetError();
	///
	~InsetError();

	///
	int ascent(Painter &, LyXFont const &) const;
	///
	int descent(Painter &, LyXFont const &) const;
	///
	int width(Painter &, LyXFont const &) const;
	///
	void draw(Painter &, LyXFont const &, int baseline, float &x);

	///
	void Write(FILE *file);
	///
	void Read(LyXLex &lex);
	///
	int Latex(FILE *file, signed char fragile);
	///
	int Latex(string &file, signed char fragile);
	///
	bool AutoDelete() const;
	/// what appears in the minibuffer when opening
	string EditMessage() const { return _("Opened error"); }
	///
	void Edit(int, int, unsigned int, BufferView *);
	///
	Inset::EDIT Editable() const;
	///
	Inset* Clone();
	///
	Inset::Code LyxCode() const { return Inset::NO_CODE; }
private:
	///
	string contents;
	///
	FL_FORM *form;
	///
	FL_OBJECT *strobj;
	///
	static void CloseErrorCB(FL_OBJECT *, long data);
};
#endif
