/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 *
 *	     Copyright (C) 1995 Matthias Ettrich
 *           Copyright (C) 1995-1998 The LyX Team.
 *
 *======================================================
 */

#include <config.h>

#include "insetfig.h"
#include "debug.h"
#include "lyxfont.h"
#include "LaTeXFeatures.h"
#include "painter.h"

// keep this in case we need it.
const float DEG2PI = 57.295779513;


InsetFig::InsetFig(int /*tmpx*/, int /*tmpy*/, Buffer *o)
	: owner(o)
{
}


InsetFig::~InsetFig()
{
}


int InsetFig::ascent(Painter &, LyXFont const&) const
{
	return 5;
}


int InsetFig::descent(Painter &, LyXFont const&) const
{
	return 5;
}


int InsetFig::width(Painter &, LyXFont const&) const
{
	return 5;
}


void InsetFig::draw(Painter &pain, LyXFont const &, int baseline, float &x)
{
	string justname("Graphics File");
	LyXFont f(LyXFont::ALL_SANE);
	f.setFamily(LyXFont::SANS_FAMILY);
	f.setSize(LyXFont::SIZE_FOOTNOTE);
	int width;
	pain.rectText(int(x), baseline, justname, f,
		      LColor::insetbg, LColor::insetframe,
		      true, width);
	x += width;
}


void InsetFig::Write(FILE *)
{
}


void InsetFig::Read(LyXLex &/*lex*/)
{
}


int InsetFig::Latex(FILE *, signed char /* fragile*/ )
{
	return 0;
}


int InsetFig::Latex(string &, signed char /* fragile*/ )
{
	return 0;
}


void InsetFig::Validate(LaTeXFeatures &features) const
{
	features.graphics = true;
}


Inset::EDIT InsetFig::Editable() const
{
	return Inset::EDITABLE;
}


bool InsetFig::Deletable() const
{
	return false;
}


void InsetFig::Edit(int, int, unsigned int, BufferView *)
{
	lyxerr[Debug::INFO] << "Editing InsetFig.\n";
}


Inset *InsetFig::Clone()
{
	InsetFig *tmp = new InsetFig(100, 100, owner);
	return tmp;
}


Inset::Code InsetFig::LyxCode() const
{
	return Inset::GRAPHICS_CODE;
}
