// -*- C++ -*-

#ifndef INSETFIG_H_
#define INSETFIG_H_

#include "lyxinset.h"
#include "LString.h"
//#include FORMS_H_LOCATION
//#include "form1.h"
#include "figlist.h"

class LaTeXFeatures;
class Buffer;

/* The InsetFig is used for displaying graphics images on screen. It uses
   a filebrowser dialog to set the file, and requests this file from the
   GraphicsCache, the graphichs cache returns a pixmap. Transformations is
   also requested from the GraphicsCache. So InsetFig is a class to view
   the pixmaps, and set the image parameters. */

///
class InsetFig: public Inset {
public:
	///
	InsetFig(int tmpx, int tmpy, Buffer *);
	///
	~InsetFig();
	///
	int ascent(Painter &, LyXFont const &) const;
	///
	int descent(Painter &, LyXFont const &) const;
	///
	int width(Painter &, LyXFont const &) const;
	///
	void draw(Painter &, LyXFont const &, int baseline, float &x);

	///
	void Write(FILE *file);
	///
	void Read(LyXLex &lex);
	///
	int Latex(FILE *file, signed char fragile);
	///
	int Latex(string &file, signed char fragile);
	/// Updates needed features for this inset.
	void Validate(LaTeXFeatures &features) const;

	/// what appears in the minibuffer when opening
	string EditMessage() const { return "Opened Graphics Inset"; }
	///
	void Edit(int, int, unsigned int, BufferView *);
	///
	Inset::EDIT Editable() const;
	///
	bool Deletable() const;
	///
	Inset::Code LyxCode() const;
	///
	Inset* Clone();
private:
	///
	Buffer *owner;
};
#endif
