/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 * 	 
 *          Copyright (C) 1998 The LyX Team.
 *
 *======================================================*/

#include <config.h>

#ifdef __GNUG__
#pragma implementation
#endif

#include "insetfoot.h"
#include "gettext.h"
#include "lyxfont.h"
#include "BufferView.h"
#include "lyxscreen.h"
#include "painter.h"
#include "lyx_gui_misc.h"


InsetFoot::InsetFoot(Buffer *bf): InsetCollapsable(bf)
{
    setLabel("foot");
    LyXFont font(LyXFont::ALL_SANE);
    font.decSize();
    font.decSize();
    font.setColor(LColor::footnote);
    setFont(font);
    setAutoCollapse(false);
}


Inset* InsetFoot::Clone()
{
    InsetFoot *result = new InsetFoot(buffer);
    return result;
}


string InsetFoot::EditMessage() const
{
    return _("Opened Footnote Inset");
}


int InsetFoot::Latex(FILE *file, signed char ch)
{
    string l;
    int i = Latex(l, ch);
    fprintf(file, "%s", l.c_str());
    return i;
}


int InsetFoot::Latex(string &latex, signed char fragile)
{
    int i;
    
    if (fragile) 
	latex += "\\footnotetext{";
    else 
	latex += "\\footnote{";

    i = InsetText::Latex(latex, fragile);
    latex += "}";

    return i;
}

void InsetFoot::Write(FILE *file)
{
    fprintf(file, "\\begin_inset Foot\n");
    WriteParagraphData(file);
    fprintf(file, "\\end_inset\n");
}

bool InsetFoot::InsertInset(Inset *inset)
{
    if ((inset->LyxCode() == Inset::FOOT_CODE) ||
	(inset->LyxCode() == Inset::MARGIN_CODE)) {
	return false;
    }
    return InsetText::InsertInset(inset);
}
