/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 * 	 
 *	    Copyright (C) 1995 Matthias Ettrich
 *          Copyright (C) 1995-1998 The LyX Team.
 *
 *======================================================*/

#include <config.h>
#include <ctype.h>

#ifdef __GNUG__
#pragma implementation
#endif

#include "insetinfo.h"
#include "debug.h"
#include "gettext.h"
#include "painter.h"
#include "LyXView.h"
#include "LString.h"
#include "lstrings.h"
#include "lyxlex.h"

// 	$Id: insetinfo.C,v 1.12 1998/10/22 16:56:12 larsbj Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: insetinfo.C,v 1.12 1998/10/22 16:56:12 larsbj Exp $";
#endif /* lint */

/* Info, used for the Info boxes */

InsetInfo::InsetInfo()
{}


InsetInfo::InsetInfo(string const & str)
	: contents(str)
{}


InsetInfo::~InsetInfo()
{
	gone();
}

int InsetInfo::ascent(Painter & pain, LyXFont const &) const
{
	LyXFont font(LyXFont::ALL_SANE);
	int width, ascent, descent;
	pain.rectText(0, 0, _("Note"), font, LColor::none, LColor::none, false,
		      width, ascent, descent);
	return ascent;
}


int InsetInfo::descent(Painter & pain, LyXFont const &) const
{
	LyXFont font(LyXFont::ALL_SANE);
	int width, ascent, descent;
	pain.rectText(0, 0, _("Note"), font, LColor::none, LColor::none, false,
		      width, ascent, descent);
	return descent;
}


int InsetInfo::width(Painter & pain, LyXFont const &) const
{
	LyXFont font(LyXFont::ALL_SANE);
	int width, ascent, descent;
	pain.rectText(0, 0, _("Note"), font, LColor::none, LColor::none, false,
		      width, ascent, descent);
	return width;
}

void InsetInfo::draw(Painter & pain, LyXFont const &, int baseline, float & x)
{
	// Draw as "Note" in a framed box
	LyXFont font(LyXFont::ALL_SANE);
	font.setColor(LColor::note);
	int width;
	pain.rectText(int(x), baseline, _("Note"), font, LColor::notebg,
		      LColor::noteframe, true, width);
	x += width;
}


string const & InsetInfo::getContents() const
{
	return contents;
}


void InsetInfo::setContents(string const & arg)
{
	contents = arg;
}


void InsetInfo::Write(FILE *file)
{
	fprintf(file, "\\begin_inset Info %s\n\\end_inset\n",
		contents.c_str());
}


void InsetInfo::Read(LyXLex &lex)
{
	string tmp = lex.GetString(); // should be "Info"
	if (tmp != "Info")
		lex.printError("(InsetInfo::Read) Missing token 'Info'.");

	while (lex.IsOK()) {
		if (!lex.EatLine())
			// blank line in the file being read
			// should we skip blank lines?
			continue;

		string const token = strip(lex.GetString());
		lyxerr[Debug::PARSER] << "Note: " << token << '\n';
		
		if (token != "\\end_inset") {
			contents += token + '\n';
		}
		else // token == "\\end_inset"
			break;
	}
	// now remove the last '\n's
	contents = strip(contents, '\n');
}
      

int InsetInfo::Latex(FILE *, signed char /*fragile*/)
{
	return 0;
}


int InsetInfo::Latex(string &, signed char /*fragile*/)
{
	return 0;
}


Inset::EDIT InsetInfo::Editable() const
{
	return Inset::EDITABLE;
}


void InsetInfo::Edit(int, int, unsigned int, BufferView *bv)
{
	bv->getOwner()->getPopups().showFormInfo(this);
}


Inset* InsetInfo::Clone()
{
	InsetInfo *result = new InsetInfo(contents);
	return result;
}


Inset::Code InsetInfo::LyxCode() const
{
	return Inset::IGNORE_CODE;
}
