/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 * 	 
 *	    Copyright (C) 1995 Matthias Ettrich
 *          Copyright (C) 1995-1998 The LyX Team.
 *
 *======================================================*/

#include <config.h>

#ifdef __GNUG__
#pragma implementation
#endif

#include "insetlatexaccent.h"
#include "debug.h"
#include "lyxrc.h"
#include "lyxfont.h"
#include "painter.h"
#include "lstrings.h"

extern LyXRC * lyxrc;

// 	$Id: insetlatexaccent.C,v 1.8 1998/10/22 16:56:13 larsbj Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: insetlatexaccent.C,v 1.8 1998/10/22 16:56:13 larsbj Exp $";
#endif /* lint */

/* LatexAccent. Proper handling of accented characters */
/* This part is done by Ivan Schreter, schreter@ccsun.tuke.sk */
/* Later modified by Lars G. Bjonnes, larsbj@ifi.uio.no */

InsetLatexAccent::InsetLatexAccent()
{
	candisp = false;
}


InsetLatexAccent::InsetLatexAccent(InsetLatexAccent const & other)
    :contents(other.contents),
     candisp(other.candisp),
     modtype(other.modtype),
     remdot(other.remdot),
     plusasc(other.plusasc),
     plusdesc(other.plusdesc),
     ic(other.ic)
{}


InsetLatexAccent::InsetLatexAccent(string const & str)
	: contents(str)
{
	checkContents();
}


InsetLatexAccent::~InsetLatexAccent()
{
}


void InsetLatexAccent::checkContents()
        // check, if we know the modifier and can display it ok on screen
{
        candisp = false;

	if (contents.empty() || contents.length()<2) return;

        if (contents[0] != '\\') return; // demand that first char is a '\\'

	lyxerr[Debug::INFO] << "Decode: " << contents << '\n';

        remdot = false; plusasc = false; plusdesc = false;

        switch (contents[1]) { // second char should be one of these
	case '\'':  // acute
                modtype = ACUTE;    // acute
                plusasc = true;    // at the top of character
                break;
	case '`':   // grave
                modtype = GRAVE;    // grave
                plusasc = true;    // at the top
                break;
	case '=':   // macron
                modtype = MACRON;    // macron
                plusasc = true;    // at the top
                break;
	case '~':   // tilde
                modtype = TILDE;    // tilde
                plusasc = true;    // at the top
                break;
	case 'b':   // underbar
                modtype = UNDERBAR;    // underbar
                plusdesc = true;   // at the bottom
                break;
	case 'c':   // cedilla
                modtype = CEDILLA;    // cedilla
                plusdesc = true;   // at the bottom
                break;
	case 'd':   // underdot
                modtype = UNDERDOT;    // underdot
                plusdesc = true;   // at the bottom
                break;
	case 'r':   // circle
                modtype = CIRCLE;    // circle
                plusasc = true;    // at the top
                break;
	case 't':   // tie
                modtype = TIE;    // tie
                plusasc = true;    // at the top
                break;
	case 'u':   // breve
                modtype = BREVE;    // breve
                plusasc = true;    // at the top
                break;
	case 'v':   // caron
                modtype = CARON;   // caron
                plusasc = true;    // at the top
                break;
	case 'q':   // special caron
                modtype = SPECIAL_CARON;   // special caron
                plusasc = true;    // at the top
                break;
	case 'H':   // hungarian umlaut
                modtype = HUNGARIAN_UMLAUT;   // hungarian umlaut
                plusasc = true;    // at the top
                break;
	case '"':   // umlaut
                modtype = UMLAUT;   // umlaut
                plusasc = true;    // at the top
                break;
	case '.':   // dot
                modtype = DOT;   // dot
                plusasc = true;    // at the top
                break;
	case '^':   // circumflex
                modtype = CIRCUMFLEX;   // circumflex
                plusasc = true;    // at the top
                break;
	case 'k':   // ogonek
		modtype = OGONEK;  // ogonek
		plusdesc = true;
		break;
	case 'i': // dot-less-i
		modtype = DOT_LESS_I;  // dot-less-i
		plusasc = true; // at the top (not really needed)
		remdot = true;
		break;
	case 'j': // dot-less-j
		modtype = DOT_LESS_J; // dot-less-j
		plusasc = true; // at the top (not really needed)
		remdot = true;
		break;
 	case 'l': // lslash
 		modtype = lSLASH;
 		plusasc = true; // at the top (not really needed)
 		break;
 	case 'L': // lslash
 		modtype = LSLASH;
 		plusasc = true; // at the top (not really needed)
 		break;
	default:
 	        printf ("Default\n");
		// unknow accent (or something else)
                return;
        }

	// we demand that third char is a '{' (Lgb)
	if (contents[2] != '{') return;

	// special clause for \i{}, \j{} \l{} and \L{}
	if ((modtype == DOT_LESS_I || modtype == DOT_LESS_J
		|| modtype == lSLASH || modtype == LSLASH)
	    && contents[3] == '}' ) {
		switch (modtype) {
		case DOT_LESS_I: ic = 'i'; break;
		case DOT_LESS_J: ic = 'j'; break;
		case lSLASH:     ic = 'l'; break;
		case LSLASH:     ic = 'L'; break;
		default:
			// if this happens something is really wrong
			lyxerr << "InsetLaTexAccent: weird error.\n";
			break;
		}
		//ic = (modtype == DOT_LESS_J ? 'j' : 'i');
		lyxerr[Debug::INFO] << "Contents: [" << contents << "], ic: "
				    << ic << ", top: " << plusasc
				    << ", bot: " << plusdesc
				    << ", dot: " << remdot
				    << ", mod: " << int(modtype) << '\n';
	// Special case for space
	} else if (contents[3] == '}') {
		ic = ' ';
	} else {
		int i = 3;
		
		// now get the char
		ic = contents[3]; // i will always be 3 here

		// ic should now be a alfa-char or '\\'
		if (ic == '\\') {
			ic = contents[++i]; // will only allow \<foo>{\i} and \<foo>{\j}
			if (ic == 'i' || ic == 'j')
				remdot = true;
			else
				return;
		} else if ( (ic=='i'|| ic=='j') && contents[4]=='}') {
			// Do a rewrite: \<foo>{i} -->	 \<foo>{\i}
#if 0	
			string temp = contents.substr(0,2);
			temp+='\\';
			temp+=char(ic);
			for(unsigned int j = 4; j < contents.length(); ++j)
				temp += contents[j];
			contents = temp;
			++i;
			remdot = true;
#endif
			contents.insert(3,1, '\\');
			++i;
			remdot = true;
		}    

		// demand a '}' at the end
		  if (contents[++i] != '}' && contents[++i]) return;
					   
		// fine, the char is properly decoded now (hopefully)
		  lyxerr[Debug::INFO] << "Contents: [" << contents << "], ic: "
				      << ic 
				      << ", top: " << plusasc
				      << ", bot: " << plusdesc
				      << ", dot: " << remdot
				      << ", mod: " << int(modtype) << '\n';
	}
        candisp = true;
}


int InsetLatexAccent::ascent(Painter &, LyXFont const & font) const
{
	int max;
	if (candisp) {
		if (ic == ' ')
			max = font.ascent('a');
		else
			max = font.ascent(ic);
		if (plusasc) 
			max += (font.maxAscent()+3) / 3;
	} else
		max = font.maxAscent()+4;
	return max;
}


int InsetLatexAccent::descent(Painter &, LyXFont const & font) const
{
	int max;
	if (candisp) {
		if (ic==' ') 
			max = font.descent('a');
                else 
                	max = font.descent(ic);
                if (plusdesc)
                	max += 3;
	} else
		max = font.maxDescent() + 4;
	return max;
}


int InsetLatexAccent::width(Painter & pain, LyXFont const & font) const
{
	if (candisp) {
		string ch;
		ch = ic;
		return pain.width(ch, font);
	} else
                return pain.width(contents, font) + 4;
}


bool InsetLatexAccent::displayISO8859_9(Painter & pain, LyXFont const & font,
					int baseline, 
					float &x)
{
	unsigned char tmpic = ic;
	
	switch (modtype) {
	case CEDILLA:
	{
		if (ic == 'c') tmpic = 0xe7;
		if (ic == 'C') tmpic = 0xc7;
		if (ic == 's') tmpic = 0xfe;
		if (ic == 'S') tmpic = 0xde;
		break;
	}
	case BREVE:
	{	if (ic == 'g') tmpic = 0xf0;
		if (ic == 'G') tmpic = 0xd0;
		break;
	}
	case UMLAUT:
	{
		if (ic == 'o') tmpic = 0xf6;
		if (ic == 'O') tmpic = 0xd6;
		if (ic == 'u') tmpic = 0xfc;
		if (ic == 'U') tmpic = 0xdc;
		break;
	}
	case DOT: 	 if (ic == 'I') tmpic = 0xdd; break;
	case DOT_LESS_I: tmpic = 0xfd; break;
	default:	 return false;
	}
	if (tmpic != ic) {
		pain.text(int(x), baseline, char(tmpic), font);
		x += pain.width(char(tmpic), font);
		return true;
	} else
		return false;
}

void InsetLatexAccent::draw(Painter &pain, LyXFont const & f, int baseline, 
			    float &x)
{
	LyXFont font = f;
	if (lyxrc->font_norm=="iso8859-9")
		if (displayISO8859_9 (pain, f, baseline, x))	
			return;
	
	/* draw it! */ 
	if (candisp) {
		int asc = ascent(pain, font);
		int desc = descent(pain, font);
		int wid = width(pain, font);
		float x2 = x+(float(wid)/2);
		float hg35;
		int hg, y;
		if (plusasc) {
			// mark at the top
			hg = font.maxDescent();
			y = baseline - asc;

			if (font.shape() == LyXFont::ITALIC_SHAPE) x2 += (4*hg)/5; // italic
		} else {
			// at the bottom
			hg = desc;
			y = baseline;
		}

		hg35 = float(hg*3)/5;

		// display with proper accent mark
		// first the letter
		pain.text(int(x), baseline , ic, font);

		if (remdot) {
			int tmpvar = baseline - font.ascent('i');
			float tmpx = 0;
			if (font.shape() == LyXFont::ITALIC_SHAPE) tmpx += (8*hg)/10; // italic
			lyxerr[Debug::ANY] << "Removing dot.\n";
			// remove the dot first
			pain.fillRectangle(int(x + tmpx),
					   tmpvar, wid,
					   font.ascent('i') -
					   font.ascent('x')-1);
		}

		// now the rest - draw within (x,y, x+wid, y+hg)
		switch (modtype) {
		case ACUTE:     // acute
 		{
			pain.line(int(x2), int(y+hg35), int(x2+hg35), y);
			break;
		}
		case GRAVE:     // grave
		{
			pain.line(int(x2), int(y+hg35), int(x2-hg35), y); 
			break;
		}
		case MACRON:     // macron
		{
			pain.line(int(x2-(3*wid/7)), int(y+(hg/2) + hg35),
				  int(x2+(3*wid/7)), int(y+(hg/2) + hg35));
			break;
		}
		case TILDE:     // tilde
		{
			if (hg35 > 2) --hg35;
			x2 += (hg35/2);
			int xc[4], yc[4];
			xc[0] = int(x2 - 2*hg35); yc[0] = int(y + hg);
			xc[1] = int(x2 - hg35);   yc[1] = int(y + hg35);
			xc[2] = int(x2);          yc[2] = int(y + hg);
			xc[3] = int(x2 + hg35);   yc[3] = int(y + hg35);
			pain.lines(xc, yc, 4);
			break;
		}
		case UNDERBAR:     // underbar
		{
			pain.line(int(x2-(3*wid/7)), y+(hg/2),
				  int(x2+(3*wid/7)), y+(hg/2));
			break;
		}
		case CEDILLA:     // cedilla
		{
			int xc[4], yc[4];
			xc[0] = int(x2);          yc[0] = y;
			xc[1] = int(x2);          yc[1] = y + (hg/3);
			xc[2] = int(x2 + (hg/3)); yc[2] = y + (hg/2);
			xc[3] = int(x2 - (hg/4)); yc[3] = y + hg;
			pain.lines(xc, yc, 4);
			break;
		}
		case UNDERDOT:     // underdot
		case DOT:    // dot
		{
			pain.arc(int(x2), y+(hg/2),
				 1, 1, 0, 360*64); 
			break;
		}
		case CIRCLE:     // circle
		{
			pain.arc(int(x2-(hg/2)), y, hg, hg, 0,
				 360*64);
			break;
		}
		case TIE:     // tie
		{
			pain.arc(int(x2), y+(hg/4),
				 hg, hg, 0, 11519);
			break;
		}
		case BREVE:     // breve
		{
			pain.arc(int(x2-(hg/2)), y-(hg/4),
				 hg, hg, 0, -11519);
			break;
		}
		case CARON:    // caron
		{
			int xc[3], yc[3];
			xc[0] = int(x2 - hg35); yc[0] = y;
			xc[1] = int(x2);        yc[1] = int(y+hg35);
			xc[2] = int(x2 + hg35); yc[2] = y;
			pain.lines(xc, yc, 3);
			break;
		}
		case SPECIAL_CARON:    // special caron
		{
			switch (ic) {
			case 'L': wid = 4*wid/5; break;
			case 't': y -= int(hg35/2); break;
			}
			int xc[3], yc[3];
			xc[0] = int(x+wid);          yc[0] = int(y+hg35+hg);
			xc[1] = int(x+wid+(hg35/2)); yc[1] = int(y+ hg+(hg35/2));
			xc[2] = int(x+wid+(hg35/2)); yc[2] = y + hg;
			pain.lines(xc, yc, 3);
			break;
		}
		case HUNGARIAN_UMLAUT:    // hung. umlaut
		{
			int x1c[2], y1c[2], x2c[2], y2c[2];
			x1c[0] = int(x2-(hg/2));      y1c[0] = int(y + hg35);
			x2c[0] = int(x2+hg35-(hg/2)); y2c[0] = y;
			x1c[1] = int(x2+(hg/2));      y1c[1] = int(y + hg35);
			x2c[1] = int(x2+hg35+(hg/2)); y2c[1] = y;

			pain.segments(x1c, y1c, x2c, y2c, 2);
			break;
		}
		case UMLAUT:    // umlaut
		{
			int tmpadd = y;
			tmpadd += (remdot) ? ((19*hg)/10) : ((20*hg)/27); // if (remdot) -> i or j
			int rad = ((hg*4)/8);
			if (rad <= 1) {
				pain.point(int(x2-((4*hg)/7)), tmpadd);
				pain.point(int(x2+((4*hg)/7)), tmpadd);
			} else {
				pain.arc(int(x2-((2*hg)/4)), tmpadd,
					 rad-1, rad-1, 0, 360*64);
				pain.arc(int(x2+((2*hg)/4)), tmpadd,
					 rad-1, rad-1, 0, 360*64);
			}
			break;
		}
		case CIRCUMFLEX:    // circumflex
		{
			int xc[3], yc[3];
			xc[0] = int(x2 - hg35); yc[0] = y + hg;
			xc[1] = int(x2);        yc[1] = int(y + hg35);
			xc[2] = int(x2 + hg35); yc[2] = y + hg;
			pain.lines(xc, yc, 3);
			break;
		}
		case OGONEK:    // ogonek
		{
			// this does probably not look like an ogonek, so
			// it should certainly be refined
			int xc[4], yc[4];
			xc[0] = int(x2);          yc[0] = y;
			xc[1] = int(x2);          yc[1] = y + (hg/3);
			xc[2] = int(x2 - (hg/3)); yc[2] = y + (hg/2);
			xc[3] = int(x2 + (hg/4)); yc[3] = y + hg;
			pain.lines(xc, yc, 4);
			break;
		}
		case lSLASH:
		case LSLASH:
		{
 			int xc[2], yc[2];
 			xc[0] = int(x);                yc[0] = y+3*hg;
 			xc[1] = int(x+float(wid)*.75); yc[1] = y+hg;
 			pain.lines(xc, yc, 2);
 			break;
		}
		case DOT_LESS_I: // dotless-i
		case DOT_LESS_J: // dotless-j
		{
			// nothing to do for these
			break;
		}
		}
	} else {
		font.setColor(LColor::accent);
		int width;
		pain.rectText(int(x), baseline, contents, font, 
			      LColor::accentbg, LColor::accentframe, 
			      true, width);
		x += width;
		return;
	}
	x +=  width(pain, font);
}


void InsetLatexAccent::Write(FILE *file)
{
	fprintf(file, "\\begin_inset LaTeXAccent %s\n\\end_inset\n",
		contents.c_str());
}


void InsetLatexAccent::Read(LyXLex &lex)
{
#ifndef NO_COMPABILITY
	bool old_format = (lex.GetString() == "\\i");
#endif
	lex.EatLine();
	contents = lex.GetString();
	checkContents();

#ifndef NO_COMPABILITY
	if (old_format) return;
#endif

	// step \end_inset
	lex.nextToken();
	if (lex.GetString() != "\\end_inset") {
		lex.printError("Missing \\end_inset at this point. "
			       "Token: `$$Token'");
	}
}


int InsetLatexAccent::Latex(FILE *file, signed char /*fragile*/)
{
	fprintf(file, "%s", contents.c_str());
	return 0;
}


int InsetLatexAccent::Latex(string &file, signed char /*fragile*/)
{
	file += contents;
	return 0;
}


bool InsetLatexAccent::Deletable() const
{
	return true;
}


Inset* InsetLatexAccent::Clone()
{
	InsetLatexAccent *result = new InsetLatexAccent(contents);
	return result;
}


Inset::Code InsetLatexAccent::LyxCode() const
{
	return Inset::ACCENT_CODE;
}


bool InsetLatexAccent::IsEqual(Inset* other)
{
	if (other && other->LyxCode() == Inset::ACCENT_CODE){
		InsetLatexAccent* otheraccent = (InsetLatexAccent*) other;
		return (contents == otheraccent->contents);
	}
	return false;
}
