#include <config.h>

#include <stdlib.h>

#ifdef __GNUG__
#pragma implementation
#endif

#include "insetref.h"
#include "buffer.h"
#include "debug.h"
#include "LyXView.h"
#include "lyxfunc.h"
#include "commandtags.h"
#include "gettext.h"


InsetRef::InsetRef(string const & cmd, Buffer *bf)
	: master(bf)
{
	scanCommand(cmd);
	if (getCmdName() == "ref")
		flag = InsetRef::REF;
	else
		flag = InsetRef::PAGE_REF;
}


InsetRef::InsetRef(InsetCommand const &inscmd, Buffer *bf)
	: master(bf)
{
	setCmdName(inscmd.getCmdName());
	setContents(inscmd.getContents());
	setOptions(inscmd.getOptions());
	if (getCmdName() == "ref")
		flag = InsetRef::REF;
	else
		flag = InsetRef::PAGE_REF;
}


InsetRef::~InsetRef()
{
}


void InsetRef::Edit(int, int, unsigned int, BufferView *bv)
{
        bv->getOwner()
		->getLyXFunc().Dispatch(LFUN_REFGOTO,
					 getContents().c_str());
}


string InsetRef::getScreenLabel() const
{
	string temp;
	if (flag == InsetRef::PAGE_REF)
		temp += _("Page: ");
	else 
		temp += _("Ref: ");
	temp += getContents();
	return temp;
}


int InsetRef::Latex(FILE *file, signed char /*fragile*/)
{
	fprintf(file, "%s", escape(getCommand()).c_str());
	return 0;
}


int InsetRef::Latex(string &file, signed char /*fragile*/)
{
	file += escape(getCommand());
	return 0;
}


// This function escapes 8-bit characters and other problematic characters
// It's exactly the same code as in insetlabel.C.
string InsetRef::escape(string const & lab) const {
	char hexdigit[16] = { '0', '1', '2', '3', '4', '5', '6', '7',
			      '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
	string enc;
	for (unsigned int i = 0; i < lab.length(); ++i) {
		unsigned char c = lab[i];
		if (c >= 128 || c == '=' || c == '%') {
			enc += '=';
			enc += hexdigit[c >> 4];
			enc += hexdigit[c & 15];
		} else {
			enc += (char) c;
		}
	}
	return enc;
}


void InsetRef::gotoLabel(BufferView *bv)
{
	//if (master) {
	bv->gotoLabel(getContents());
	//}
}
