/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 * 	 
 *	    Copyright (C) 1997 Asger Alstrup
 *
 *======================================================*/

#include <config.h>

#ifdef __GNUG__
#pragma implementation
#endif

#include "insetspecialchar.h"
#include "debug.h"
#include "lyxfont.h"
#include "painter.h"
#include "lyxlex.h"

// 	$Id: insetspecialchar.C,v 1.12 1998/10/17 14:46:16 larsbj Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: insetspecialchar.C,v 1.12 1998/10/17 14:46:16 larsbj Exp $";
#endif /* lint */

InsetSpecialChar::InsetSpecialChar()
{
}


InsetSpecialChar::InsetSpecialChar(Kind k)
	: kind(k)
{
}


InsetSpecialChar::~InsetSpecialChar()
{
}


int InsetSpecialChar::ascent(Painter &, LyXFont const& font) const
{
#warning Move this to the painter? What do you think Asger?
	return font.maxAscent();
}


int InsetSpecialChar::descent(Painter &, LyXFont const& font) const
{
#warning Move this to the painter? What do you think Asger?
	return font.maxDescent();
}


int InsetSpecialChar::width(Painter & pain, LyXFont const& font) const
{
	LyXFont f = font;
	int ret = 0;
	switch (kind) {
	case HYPHENATION:
	{
		int w = pain.width("-", f);
		if (w > 5) 
			w -= 2; // to make it look shorter
		ret = w;
		break;
	}
	case END_OF_SENTENCE:
		ret = pain.width(".", f);
		break;
	case LDOTS:
		ret = pain.width(". . .", f);
		break;
	case PROTECTED_SPACE:
		ret = pain.width("~", f);
		break;
	case HFILL:
		ret = pain.width("\\hfill", f);
		break;
	}

	return ret;
}


void InsetSpecialChar::draw(Painter &pain, LyXFont const & f,
			    int baseline, float &x)
{
	LyXFont font = f;
	font.setColor(LColor::special);

	string str;
	switch (kind) {
	case HYPHENATION:
		str = "-";
		break;
	case END_OF_SENTENCE:
		str = ".";
		break;
	case LDOTS:
		str = ". . .";
		break;
	case PROTECTED_SPACE:
		str = "~";
		break;
	case HFILL:
		str = "\\hfill";
		break;
	}
	pain.text(int(x), baseline, str, font);
	x += width(pain, f);
}


// In lyxf3 this will be just LaTeX
void InsetSpecialChar::Write(FILE *file)
{
	string command;
	switch (kind) {
	case HYPHENATION:	command = "\\-";	break;
	case END_OF_SENTENCE:	command = "\\@.";	break;
	case LDOTS:		command = "\\ldots{}";	break;
	case PROTECTED_SPACE:   command = "~";          break;
	case HFILL:             command = "\\hfill{}";    break;
	}
	fprintf(file, "\\begin_inset SpecialChar %s\n\\end_inset\n",
		command.c_str());
}


// This function will not be necessary when lyx3
void InsetSpecialChar::Read(LyXLex &lex)
{    
	lex.nextToken();
	string command = lex.GetString();

	if (command == "\\-")
		kind = HYPHENATION;
	else if (command == "\\@.")
		kind = END_OF_SENTENCE;
	else if (command == "\\ldots{}")
		kind = LDOTS;
	else if (command == "~")
		kind = PROTECTED_SPACE;
	else if (command == "\\hfill{}")
		kind = HFILL;
	else
		lex.printError("InsetSpecialChar: Unknown kind: '$$Token'");
	
	// step \end_inset
	lex.nextToken();
	if (lex.GetString() != "\\end_inset") {
		lex.printError("Missing \\end_inset at this point. "
			       "Token: `$$Token'");
	}
}


int InsetSpecialChar::Latex(FILE *file, signed char /*fragile*/)
{
	string command;
	signed char dummy = 0;
	Latex(command, dummy);
	fprintf(file, "%s", command.c_str());
	return 0;
}


int InsetSpecialChar::Latex(string &file, signed char /*fragile*/)
{
	switch (kind) {
	case HYPHENATION:	file += "\\-";       break;
	case END_OF_SENTENCE:	file += "\\@.";      break;
	case LDOTS:		file += "\\ldots{}"; break;
	case PROTECTED_SPACE:   file += "~";         break;
	case HFILL:             file += "\\hfill{}";   break;
	}
	return 0;
}


Inset* InsetSpecialChar::Clone()
{
	InsetSpecialChar *result = new InsetSpecialChar(kind);
	return result;
}
