/* This file is part of
* ======================================================
* 
*           LyX, The Document Processor
* 	 
*	    Copyright (C) 1995 Matthias Ettrich
*           Copyright (C) 1995-1999 The LyX Team.
*
*======================================================*/

#include <config.h>


#ifdef __GNUG__
#pragma implementation
#endif

#include <algorithm>
#include FSTREAM_HEADER

#include "FileInfo.h"
#include "lastfiles.h"
#include "filetools.h"
#include "debug.h"
#include "lstrings.h"


// 	$Id: lastfiles.C,v 1.5 1998/10/20 13:10:21 larsbj Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: lastfiles.C,v 1.5 1998/10/20 13:10:21 larsbj Exp $";
#endif /* lint */


LastFiles::LastFiles(string const & filename, bool st, unsigned short num)
	: dostat(st), max_files(num)
{
	readFile(filename);
}


LastFiles::~LastFiles() {}


void LastFiles::setNumberOfFiles(unsigned short no)
{
	if (1 <= no && no <= ABSOLUTEMAXLASTFILES)
		max_files = no;
	else {
		lyxerr << "LyX: lastfiles: too many files\n"
			"\tdefault (=" << DEFAULTFILES << ") used.\n";
		max_files = DEFAULTFILES;
	}
}


void LastFiles::readFile(string const & filename)
{
	// we will not complain if we can't find filename nor will
	// we issue a warning. Lgb.
	string tmp;
	FileInfo fileInfo;
	int i = 0;
	ifstream in(filename.c_str());
	while ((in >> tmp) && i < max_files) {
		if (dostat) {
			if (!(fileInfo.newFile(tmp).exist() &&
			      fileInfo.isRegular()))
				continue; // the file does not exist
		}
		++i; // we only increment when we actually add a file
		files.push_back(tmp);
	}
}

class LastFiles::write_display_path {
public:
	write_display_path(ostream & o)
		: ost(o) {}
	void operator()(FileStack::value_type const & vt) {
		ost << MakeDisplayPath(vt).c_str() << '\n';
	}
private:
	ostream & ost;
};

void LastFiles::writeFile(string const & filename) const
{
	ofstream out(filename.c_str());
	if (!out) {
		lyxerr << "LyX: Warning: unable to save LastFile: "
		       << filename << '\n';
		return;
	}

	for_each(files.begin(), files.end(), write_display_path(out));
}


void LastFiles::newFile(string const & file)
{
	FileStack::iterator fi = find(files.begin(), files.end(), file);
	if (fi != files.end()) {
		files.erase(fi);
	} else if (files.size() >= max_files) {
		files.pop_back();
	}
	files.push_front(file);
}
