/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 * 	 
 *	    Copyright (C) 1995 Matthias Ettrich
 *          Copyright (C) 1995-1998 The LyX Team.
 *
 * ======================================================*/

#include <config.h>

#ifdef __GNUG__
#pragma implementation
#endif

#include "layout.h"
#include "textclass.h"
#include "lyxlex.h"
#include "debug.h"

// 	$Id: layout.C,v 1.9 1998/10/22 16:56:14 larsbj Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: layout.C,v 1.9 1998/10/22 16:56:14 larsbj Exp $";
#endif /* lint */


// Constructor for layout
LyXLayout::LyXLayout ()
{
	margintype = MARGIN_STATIC;
	latextype = LATEX_PARAGRAPH;
	intitle = false;
	needprotect = false;
	keepempty = false;
	font = LyXFont(LyXFont::ALL_INHERIT);
	labelfont = LyXFont(LyXFont::ALL_INHERIT);
	resfont = LyXFont(LyXFont::ALL_SANE);
	reslabelfont = LyXFont(LyXFont::ALL_SANE);
	nextnoindent = false;
	parskip = 0.0;
	itemsep = 0;
	topsep = 0.0;
	bottomsep = 0.0;
	labelbottomsep = 0.0;
	parsep = 0;
	align = LYX_ALIGN_BLOCK;
	alignpossible = LYX_ALIGN_BLOCK;
	labeltype = LABEL_NO_LABEL;
	// Should or should not. That is the question.
	// spacing.set(Spacing::OneHalf);
	fill_top = false;
	fill_bottom = false;
	newline_allowed = true;
	free_spacing = false;
}


LyXLayout & LyXLayout::operator=(LyXLayout const &l) {
	name = l.name;
	obsoleted_by = l.obsoleted_by;
	margintype = l.margintype;
	latextype = l.latextype;
	intitle = l.intitle;
	needprotect = l.needprotect;
	keepempty = l.keepempty;
	latexname = l.latexname;
	latexparam = l.latexparam;   //arrae970411
	preamble = l.preamble;
	font = l.font;
	labelfont = l.labelfont;
	resfont = l.resfont;
	reslabelfont = l.reslabelfont;
	nextnoindent = l.nextnoindent;
	leftmargin = l.leftmargin;
	rightmargin = l.rightmargin;
	labelsep = l.labelsep;
	labelindent = l.labelindent;
	parindent = l.parindent;
	parskip = l.parskip;
	itemsep = l.itemsep;
	topsep = l.topsep;
	bottomsep = l.bottomsep;
	labelbottomsep = l.labelbottomsep;
	parsep = l.parsep;
	align = l.align;
	alignpossible = l.alignpossible;
	labeltype = l.labeltype;
	spacing = l.spacing;
	labelstring = l.labelstring;
	labelstring_appendix = l.labelstring_appendix;
	fill_top = l.fill_top;
	fill_bottom = l.fill_bottom;
	newline_allowed = l.newline_allowed;
	free_spacing = l.free_spacing;
	return *this;
}

/* Reads a layout definition from file */
bool LyXLayout::read (LyXLex & lexrc, LyXTextClass const &textclass)
{
	bool error = false;
	bool finished = false;
	
	/* parse style section */
	while (!finished && lexrc.IsOK() && !error) {
		switch(lexrc.lex()) {

		case -2:
			break;

		case -1:		/* parse error */
			lexrc.printError("Unknown tag `$$Token'");
			error = true;
			break;

		case LT_END:		/* end of structure */
			finished = true;
			break;

		case LT_COPYSTYLE:     // initialize with a known style
		        if (lexrc.next()) {
		        	LyXLayout const * layout = textclass.getLayout(lexrc.GetString());
				if (layout) {
					string tmpname = name;
					operator=(*layout);
					name = tmpname;
				} else {
					lexrc.printError("Cannot copy unknown "
                                                 "style `$$Token'");
				}
			}
			break;

		case LT_OBSOLETEDBY:     // replace with a known style
		        if (lexrc.next()) {
		        	LyXLayout const * layout = textclass.getLayout(lexrc.GetString());
				if (layout) {
					string tmpname = name;
					operator=(*layout);
					name = tmpname;
					if (obsoleted_by.empty())
						obsoleted_by = lexrc.GetString();
				} else {
					lexrc.printError("Cannot replace with" 
							 " unknown style "
							 "`$$Token'");
				}
			}
			break;

		case LT_MARGIN:		/* margin style definition */
		       
			switch(lexrc.lex()) {
			case LT_STATIC:
				margintype = MARGIN_STATIC;
				break;
			case LT_MANUAL:
				margintype = MARGIN_MANUAL;
				break;
			case LT_DYNAMIC:
				margintype = MARGIN_DYNAMIC;
				break;
			case LT_FIRST_DYNAMIC:
				margintype = MARGIN_FIRST_DYNAMIC;
				break;
			case LT_RIGHT_ADDRESS_BOX:
				margintype = MARGIN_RIGHT_ADDRESS_BOX;
				break;
			default:
				lexrc.printError("Unknown margin type `$$Token'");
				break;
			}
			break;

		case LT_LATEXTYPE:	/* latex style definition */
		        switch (lexrc.lex()) {
			case LT_PARAGRAPH:
				latextype=LATEX_PARAGRAPH;
				break;
			case LT_COMMAND:
				latextype=LATEX_COMMAND;
				break;
			case LT_ENVIRONMENT:
				latextype=LATEX_ENVIRONMENT;
				break;
			case LT_ITEM_ENVIRONMENT:
				latextype=LATEX_ITEM_ENVIRONMENT;
				break;
			case LT_LIST_ENVIRONMENT:
				latextype=LATEX_LIST_ENVIRONMENT;
				break;
			default:
				lexrc.printError("Unknown latextype `$$Token'");
				break;
			}
			break;

		case LT_INTITLE:
			intitle = lexrc.next() && lexrc.GetInteger();
			break;
			
		case LT_NEED_PROTECT:
			needprotect = lexrc.next() && lexrc.GetInteger();
			break;
			
		case LT_KEEPEMPTY:
			keepempty = lexrc.next() && lexrc.GetInteger();
			break;

		case LT_FONT:
			font.lyxRead(lexrc);
			labelfont=font;
			break;

		case LT_TEXTFONT:
			font.lyxRead(lexrc);
			break;

		case LT_LABELFONT:
			labelfont.lyxRead(lexrc);
			break;

		case LT_NEXTNOINDENT:	/* indent next paragraph ? */
			if (lexrc.next() && lexrc.GetInteger())
				nextnoindent = true;
			else
				nextnoindent = false;
			break;

		case LT_LATEXNAME:	/* latex name */
		        if (lexrc.next())
		                latexname = lexrc.GetString();
			break;
                        
                //arrae970411
		case LT_LATEXPARAM:	/* latex parameter */
			if (lexrc.next())
				latexparam = lexrc.GetString();
			break;

		case LT_PREAMBLE:
			preamble = lexrc.getLongString("EndPreamble");
			break;

		case LT_LABELTYPE:	/* label type */
		        switch (lexrc.lex()) {
			case LT_NO_LABEL:
				labeltype = LABEL_NO_LABEL;
				break;
			case LT_MANUAL:
				labeltype = LABEL_MANUAL;
				break;
			case LT_TOP_ENVIRONMENT:
				labeltype = LABEL_TOP_ENVIRONMENT;
				break;
			case LT_CENTERED_TOP_ENVIRONMENT:
				labeltype = LABEL_CENTERED_TOP_ENVIRONMENT;
				break;
			case LT_STATIC:
				labeltype = LABEL_STATIC;
				break;
			case LT_SENSITIVE:
				labeltype = LABEL_SENSITIVE;
				break;
			case LT_COUNTER_CHAPTER:
				labeltype = LABEL_COUNTER_CHAPTER;
				break;
			case LT_COUNTER_SECTION:
				labeltype = LABEL_COUNTER_SECTION;
				break;
			case LT_COUNTER_SUBSECTION:
				labeltype = LABEL_COUNTER_SUBSECTION;
				break;
			case LT_COUNTER_SUBSUBSECTION:
				labeltype = LABEL_COUNTER_SUBSUBSECTION;
				break;
			case LT_COUNTER_PARAGRAPH:
				labeltype = LABEL_COUNTER_PARAGRAPH;
				break;
			case LT_COUNTER_SUBPARAGRAPH:
				labeltype = LABEL_COUNTER_SUBPARAGRAPH;
				break;
			case LT_COUNTER_ENUMI:
				labeltype = LABEL_COUNTER_ENUMI;
				break;
			case LT_COUNTER_ENUMII:
				labeltype = LABEL_COUNTER_ENUMII;
				break;
			case LT_COUNTER_ENUMIII:
				labeltype = LABEL_COUNTER_ENUMIII;
				break;
			case LT_COUNTER_ENUMIV:
				labeltype = LABEL_COUNTER_ENUMIV;
				break;
			case LT_BIBLIO:
				labeltype = LABEL_BIBLIO;
				break;
			default:
				lexrc.printError("Unknown labeltype `$$Token'");
			}
			break;

		case LT_LEFTMARGIN:	/* left margin type */
		        if (lexrc.next())
				leftmargin = lexrc.GetString();
			break;			

		case LT_RIGHTMARGIN:	/* right margin type */
			if (lexrc.next())
				rightmargin = lexrc.GetString();
			break;

		case LT_LABELINDENT:	/* label indenting flag */
			if (lexrc.next())
				labelindent = lexrc.GetString();
			break;

		case LT_PARINDENT:	/* paragraph indent. flag */
			if (lexrc.next())
				parindent = lexrc.GetString();
			break;

		case LT_PARSKIP:	/* paragraph skip size */
			if (lexrc.next())
				parskip = lexrc.GetFloat();
			break;

		case LT_ITEMSEP:	/* item separation size */
			if (lexrc.next())
				itemsep = lexrc.GetFloat();
			break;

		case LT_TOPSEP:		/* top separation size */
			if (lexrc.next())
				topsep = lexrc.GetFloat();
			break;

		case LT_BOTTOMSEP:	/* bottom separation size */
			if (lexrc.next())
				bottomsep = lexrc.GetFloat();
			break;

		case LT_LABEL_BOTTOMSEP:/* label bottom separation size */
			if (lexrc.next())
				labelbottomsep = lexrc.GetFloat();
			break;

		case LT_LABELSEP:	/* label separator */
			if (lexrc.next()) {
				labelsep = subst(lexrc.GetString(), 'x', ' ');
			}
			break;

		case LT_PARSEP:		/* par. separation size */
			if (lexrc.next())
				parsep = lexrc.GetFloat();
			break;

		case LT_FILL_TOP:	/* fill top flag */
			if (lexrc.next())
				fill_top = lexrc.GetInteger();
			break;

		case LT_FILL_BOTTOM:	/* fill bottom flag */
			if (lexrc.next())
				fill_bottom = lexrc.GetInteger();
			break;

		case LT_NEWLINE:	/* newlines allowed ? */
			if (lexrc.next())
				newline_allowed = lexrc.GetInteger();
			break;

		case LT_ALIGN:		/* paragraph align */
			switch (lexrc.lex()) {
			case LT_BLOCK:
				align = LYX_ALIGN_BLOCK;
				break;
			case LT_LEFT:
				align = LYX_ALIGN_LEFT;
				break;
			case LT_RIGHT:
				align = LYX_ALIGN_RIGHT;
				break;
			case LT_CENTER:
				align = LYX_ALIGN_CENTER;
				break;
			case LT_LAYOUT:
				align = LYX_ALIGN_LAYOUT;
				break;
			default:
				lexrc.printError("Unknown alignment `$$Token'");
			}
			break;

		case LT_ALIGNPOSSIBLE:	/* paragraph allowed align */
		{
			alignpossible = 0;
		      
			int lineno = lexrc.GetLineNo();
			do {
				switch (lexrc.lex()) {
				case LT_BLOCK:
					alignpossible |= LYX_ALIGN_BLOCK;
					break;
				case LT_LEFT:
					alignpossible |= LYX_ALIGN_LEFT;
					break;
				case LT_RIGHT:
					alignpossible |= LYX_ALIGN_RIGHT;
					break;
				case LT_CENTER:
					alignpossible |= LYX_ALIGN_CENTER;
					break;
				case LT_LAYOUT:
					alignpossible |= LYX_ALIGN_LAYOUT;
					break;
				default:
					lexrc.printError("Unknown alignment `$$Token'");
					
				}
			} while (lineno==lexrc.GetLineNo());
		}
		break;

		case LT_LABELSTRING:	/* label string definition */
			if (lexrc.next())
				labelstring = lexrc.GetString();
			break;

		case LT_LABELSTRING_APPENDIX:	/* label string appendix definition */
			if (lexrc.next())
				labelstring_appendix = lexrc.GetString();
			break;

		case LT_FREE_SPACING:	/* Allow for free spacing. */
			if (lexrc.next())
				free_spacing = lexrc.GetInteger();
			break;

		case LT_SPACING: // setspace.sty
			switch(lexrc.lex()) {
			case LT_SPACING_SINGLE:
				spacing.set(Spacing::Single);
				break;
			case LT_SPACING_ONEHALF:
				spacing.set(Spacing::Onehalf);
				break;
			case LT_SPACING_DOUBLE:
				spacing.set(Spacing::Double);
				break;
			case LT_OTHER:
				lexrc.next();
				spacing.set(Spacing::Other, lexrc.GetFloat());
				break;
			default:
				lexrc.printError("Unknown spacing `$$Token'");
			}
			break;
		default:		/* context error */
			lexrc.printError("Tag `$$Token' is not "
					 "allowed in layout");
			error = true;
			break;
		}
	}
	return error;
}
