/* This file is part of
* ======================================================
* 
*           LyX, The Document Processor
* 	 
*	    Copyright (C) 1995 Matthias Ettrich,
*           Copyright (C) 1995-1998 The LyX Team.
*
*======================================================*/

#include <config.h>

#include <ctype.h>

#include "lyx_main.h"
#include FORMS_H_LOCATION
#include "layout_forms.h"
#include "print_form.h"
#include "form1.h"
#include "minibuffer.h"
#include "combox.h"
#include "bufferlist.h"
#include "filetools.h"
#include "path.h"
#include "LyXView.h" // only because of form_main
#include "syscall.h"
#include "FontLoader.h"
#include "lyxrc.h"
#include "insetfig.h"
#include "insetref.h"
#include "libgui.h"
#include "textclasslist.h"
#include "lyx.h"
#include "lyxscreen.h"
#include "lstrings.h"
#include "lyxtext.h"

// 	$Id: lyx_cb.C,v 1.33 1998/10/22 16:56:15 larsbj Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: lyx_cb.C,v 1.33 1998/10/22 16:56:15 larsbj Exp $";
#endif /* lint */

extern MiniBuffer *minibuffer;
extern BufferList bufferlist;
extern FD_form_character *fd_form_character;
extern FD_form_print *fd_form_print;
extern FD_form_figure *fd_form_figure;
extern FD_form_screen *fd_form_screen;
extern FD_form_toc *fd_form_toc;
extern FD_form_ref *fd_form_ref;

extern BufferView *current_view; // called too many times in this file...

extern FontLoader fontloader;

// Whether the work area should get callbacks.
bool input_prohibited = false;

// Some function prototypes

unsigned char getCurrentTextClass(Buffer *c_b)
{
	return c_b->params.textclass;
}


#warning discuss
// How should this actually work? Should it prohibit input in all BufferViews,
// or just in the current one? If "just the current one", then it should be
// placed in BufferView. If "all BufferViews" then LyXGUI (I think) should
// run "ProhibitInput" on all LyXViews which will run prohibitInput on all
// BufferViews. Or is it perhaps just the (input in) BufferViews in the
// current LyxView that should be prohibited (Lgb) (This applies to
// "AllowInput" as well.)

// I think we should have both:
// Locking on buffer level, and locking on application level. 
// I.e, methods Buffer::Lock() and Buffer::Unlock() that
// disallow changes in a buffer.  This will tell all views of
// this buffer to change the cursor and disallow interaction. 
// Also, we would have similar methods for application wide
// locking.  (Asger)
void ProhibitInput()
{
	input_prohibited = true;
	current_view->hideCursor();

#if 0
	/* set the cursor to the watch for all forms and the canvas */ 
	setXwatchCursor(fl_display, fd_form_main->form_main->window);
	if (fd_form_character->form_character->visible)
		setXwatchCursor(fl_display,
				fd_form_character->form_character->window);
#endif
	
	fl_deactivate_all_forms();
}


void AllowInput()
{
	input_prohibited = false;

	// Reset the cursor from the watch for all forms and the canvas.

#if 0   
	XUndefineCursor(fl_display, fd_form_main->form_main->window);
	if (fd_form_character->form_character->visible)
		XUndefineCursor(fl_display,
				fd_form_character->form_character->window);
	if (current_view->getWorkArea()->belowmouse)
		setXtermCursor(fl_display, fd_form_main->form_main->window);
#endif

	fl_activate_all_forms();
}


//
// File menu
//




#warning move where?
// To Buffer:  We make DVI output for a specific buffer.
// It should return a bool, and maybe take an optional path
// of where the DVI output should be placed (for Export). (Asger)
int MakeDVIOutput(Buffer *buffer)
{
	// bad test
	if (!buffer)
		return 1;

	int ret = 0;

	string path = buffer->getFileDir();
	if (lyxrc->use_tempdir || (IsDirWriteable(path) < 1)) {
		path = buffer->getTmpDir();
	}
	if (!buffer->isDviClean()) {
		Path p(path);
		ret = current_view->runLaTeX();
	}
	return ret;
}


/* wait == false means don't wait for termination */
/* wait == true means wait for termination       */
// The bool should be placed last on the argument line. (Lgb)
// Returns false if we fail.
bool RunScript(Buffer *buffer, bool wait,
	       string const & command, string const & orgname = string(),
	       bool need_shell=true)
{
	string path;
	string cmd;
	string name= orgname;
	
	if (MakeDVIOutput(buffer) > 0)
		return false;
	// Get DVI-Filename.
	if (name.empty())
		name = ChangeExtension(buffer->getFileName(),
				       ".dvi", true);

	path = OnlyPath(name);
	if (lyxrc->use_tempdir || (IsDirWriteable(path) < 1)) {
		path = buffer->getTmpDir();
	}
	Path p(path);

	cmd = command + ' ' + SpaceLess(name);
	if (need_shell) {
#ifndef __EMX__
		if (!wait)
			cmd += " &";
#else
		if (!wait) {
                       string sh = getenv ("EMXSHELL");
                       if (sh.empty())
                               sh = getenv ("COMSPEC");
                       if (contains(sh, "cmd.exe"))
                               cmd = "start /min/n " + cmd;
                       else
                               cmd += " &";
               }
#endif
		minibuffer->Set(_("Executing command:"), cmd);
		Systemcalls one(Systemcalls::System, cmd);
	} else {
		minibuffer->Set(_("Executing command:"), cmd);
		Systemcalls one(wait ? Systemcalls::Wait
                                     : Systemcalls::DontWait, cmd);
	}
	return true;
}


#warning lyxfunc?
// No, I think Buffer.  Same reason as for MakeDVIoutput:
// This function creates a PostScript for a specific buffer.

// Returns false if we fail
bool MenuRunDvips(Buffer *buffer, bool wait=false)
{
	// bad test
	if (!buffer)
		return false;

	ProhibitInput();

	// Generate dvi file
        if (MakeDVIOutput(buffer) > 0) {
            	AllowInput();
		return false;
        }
	// Generate postscript file
	string ps = ChangeExtension (buffer->getFileName(),
				      ".ps_tmp", true);

	string paper;
	
	char real_papersize = buffer->params.papersize;
	if (real_papersize == PAPER_DEFAULT)
		real_papersize = lyxrc->default_papersize;

	switch (real_papersize) {
	case PAPER_USLETTER:
		paper = "letter";
		break;
	case PAPER_A3PAPER:
		paper = "a3";
		break;
	case PAPER_A4PAPER:
		paper = "a4";
		break;
	case PAPER_A5PAPER:
		paper = "a5";
		break;
	case PAPER_B5PAPER:
		paper = "b5";
		break;
	case PAPER_EXECUTIVEPAPER:
		paper = "foolscap";
		break;
	case PAPER_LEGALPAPER:
		paper = "legal";
		break;
	default: /* If nothing else fits, keep an empty value... */
		break;
	}

	// Make postscript file.
	string command = "dvips -o " + SpaceLess(ps);
	// dvips won't accept -t letter -t landscape.  In all other
	// cases, include the paper size explicitly.
	if (!paper.empty() 
	    && (real_papersize != PAPER_USLETTER ||
		buffer->params.orientation == ORIENTATION_PORTRAIT))
	  command += " -t " + paper;
	if (buffer->params.orientation == ORIENTATION_LANDSCAPE)
		command += " -t landscape";
	// push directorypath, if necessary 
        string path = OnlyPath(buffer->getFileName());
        if (lyxrc->use_tempdir || (IsDirWriteable(path) < 1)){
		path = buffer->getTmpDir();
        }
	Path p(path);
	bool ret = RunScript(buffer, wait, command);
	AllowInput();
	return ret;
}





#warning encapsulate this
void LayoutsCB(int sel, void *)
{
	string tmp;
	tmp += sel;
	current_view->getOwner()->getLyXFunc().Dispatch(LFUN_LAYOUTNO,
							 tmp.c_str());
}





#warning gui specific
void MenuLayoutCharacter()
{
	if (fd_form_character->form_character->visible) {
		fl_raise_form(fd_form_character->form_character);
	} else {
		fl_show_form(fd_form_character->form_character,
			     FL_PLACE_MOUSE | FL_FREE_SIZE,FL_FULLBORDER,
			     _("Character Style"));
	}
}


/* Callbacks added for LinuxDoc support (Pascal Andr) : 
 *   - marks and cross references TeX macros
 *   - url display macros
 *   - htmlurl macros
 */

// Inserts an url skeleton.
void UrlCB()
{
#warning do it right!
}


// Inserts an html url skeleton.
void HtmlUrlCB()
{
#warning do it right!
}

/* end of LinuxDoc SGML special CB */



/* -------> Returns a string containing the current relative font
 * setting and the current depth. In the
 * future perhaps we could try to implement a callback to the button-bar.
 * That is, `light' the bold button when the font is currently bold, etc.
 */
string currentState(BufferView *b_view)
{
	string state;
	if (b_view->available()) { 
		// I think we should only show changes from the default
		// font. (Asger)
		Buffer * buffer = b_view->currentBuffer();
		LyXFont font = b_view->text->real_current_font;
		LyXFont defaultfont =
			textclasslist.textClass(buffer->params.textclass)->defaultfont;
		font.reduce(defaultfont);
		state = _("Font: ") + font.stateText();

		int depth = b_view->text->GetDepth();
		if (depth>0) 
			state += string(_(", Depth: ")) + tostr(depth);
	}
	return state;
}


#warning gui specific
void FigureCB(FL_OBJECT *, long)
{
	if (fd_form_figure->form_figure->visible) {
		fl_raise_form(fd_form_figure->form_figure);
	} else {
		fl_show_form(fd_form_figure->form_figure,
			     FL_PLACE_MOUSE, FL_FULLBORDER,
			     _("Insert Figure"));
	}
}

/* callbacks for form form_character */
// this should probably be changed to dispatch the freefont LFUN
void CharacterApplyCB(FL_OBJECT *, long)
{
	current_view->userFree();
}


void CharacterCancelCB(FL_OBJECT *ob, long)
{
	fl_hide_form(ob->form);
}


void CharacterOKCB(FL_OBJECT *ob, long data)
{
	CharacterApplyCB(ob,data);
	CharacterCancelCB(ob,data);
}


// Callbacks for form form_print.

void PrintCancelCB(FL_OBJECT *ob, long)
{
	fl_hide_form(ob->form);
}


void PrintApplyCB(FL_OBJECT *, long)
{
	if (!current_view->available())
		return;
        Buffer *buffer = current_view->currentBuffer();
        string path = OnlyPath(buffer->getFileName());

	string pageflag;
	if (fl_get_button(fd_form_print->radio_even_pages))
		pageflag = lyxrc->print_evenpage_flag + ' ';
	else if (fl_get_button(fd_form_print->radio_odd_pages))
		pageflag = lyxrc->print_oddpage_flag + ' ';
   
	string reverseflag;
	if (fl_get_button(fd_form_print->radio_order_reverse))
		reverseflag = lyxrc->print_reverse_flag + ' ';
   
	string orientationflag;
	if (buffer->params.orientation == ORIENTATION_LANDSCAPE)
		orientationflag = lyxrc->print_landscape_flag + ' ';
   
	string ps_file = fl_get_input(fd_form_print->input_file);
	string printer = fl_get_input(fd_form_print->input_printer);

	string printerflag;
	if (lyxrc->print_adapt_output // printer name should be passed to dvips
	    && ! printer.empty()) // a printer name has been given
		printerflag = lyxrc->print_to_printer + printer + ' ';
     
	string command = lyxrc->print_command + ' ' 
		+ printerflag + pageflag + reverseflag 
		+ orientationflag + lyxrc->print_extra_options;
 
	if (fl_get_button(fd_form_print->radio_file))
		command += lyxrc->print_to_file + " \""
			+ MakeAbsPath(ps_file, path)
			+ '\"';
	else if (!lyxrc->print_spool_command.empty())
		command += lyxrc->print_to_file 
			+ " \"" + ps_file + '\"';
	
	// push directorypath, if necessary 
        if (lyxrc->use_tempdir || (IsDirWriteable(path) < 1)){
		path = buffer->getTmpDir();
        }
	Path p(path);
	if (!lyxrc->print_spool_command.empty() && 
	    !fl_get_button(fd_form_print->radio_file)) {
	    	string command2 = lyxrc->print_spool_command + ' ';
		if (!printer.empty())
			command2 += lyxrc->print_spool_printerprefix 
				    + printer;
		// First run dvips and, if succesful, then spool command
		if (RunScript(buffer, true, command)) {
			RunScript(buffer, false, command2, ps_file);
		}
        } else
		RunScript(buffer, false, command);
}


void PrintOKCB(FL_OBJECT *ob, long data)
{
	PrintCancelCB(ob, data);  
	PrintApplyCB(ob,data);
}


// Callbacks for form form_figure.

void FigureApplyCB(FL_OBJECT *, long)
{
	if (!current_view->available())
		return;

	Buffer * buffer = current_view->currentBuffer();
	if(buffer->isReadonly()) // paranoia
		return;
	
	minibuffer->Set(_("Inserting figure..."));
	if (fl_get_button(fd_form_figure->radio_inline)) {
		current_view->insertInset(new InsetFig(100, 20, buffer));
		minibuffer->Set(_("Figure inserted"));
		return;
	}
	
	current_view->hideCursor();
	current_view->update(BufferView::ResetSelection
		     |BufferView::FitCursor);
	current_view->beforeChange();
      
	current_view->text->SetCursorParUndo(); 
	current_view->text->FreezeUndo();

	current_view->text->BreakParagraph();
	current_view->update(BufferView::ResetSelection
			     |BufferView::FitCursor
			     |BufferView::BufferChange);
      
	if (current_view->text->cursor.par->Last()) {
		current_view->text->CursorLeft();
	 
		current_view->text->BreakParagraph();
		current_view->update(BufferView::ResetSelection
				     |BufferView::FitCursor
				     |BufferView::BufferChange);
	}

	// The standard layout should always be numer 0;
	current_view->text->SetLayout(0);
	
	if (current_view->text->cursor.par->footnoteflag ==
	    LyXParagraph::NO_FOOTNOTE) {
		current_view->text->SetParagraph(0, 0,
					   0, 0,
					   VSpace (0.3 * buffer->params.spacing.getValue(),
						   LyXLength::CM),
					   VSpace (0.3 *
						   buffer->params.spacing.getValue(),
						   LyXLength::CM),
					   LYX_ALIGN_CENTER, string(), 0);
	} else
		current_view->text->SetParagraph(0, 0,
					   0, 0,
					   VSpace::NONE, VSpace::NONE,
					   LYX_ALIGN_CENTER, 
					   string(),
					   0);
	
	current_view->update(BufferView::ResetSelection
			     |BufferView::FitCursor
			     |BufferView::BufferChange);
      
	Inset *new_inset = 0;
	
	new_inset = new InsetFig(100, 100, buffer);
	current_view->insertInset(new_inset);
	
	current_view->update(BufferView::ResetSelection
			     |BufferView::FitCursor);
	minibuffer->Set(_("Figure inserted"));
	current_view->text->UnFreezeUndo();
}

   
void FigureCancelCB(FL_OBJECT *ob, long)
{
	fl_hide_form(ob->form);
}


void FigureOKCB(FL_OBJECT *ob, long data)
{
	FigureApplyCB(ob,data);
	FigureCancelCB(ob,data);
}


void ScreenApplyCB(FL_OBJECT *, long)
{
	lyxrc->roman_font_name = fl_get_input(fd_form_screen->input_roman);
	lyxrc->sans_font_name = fl_get_input(fd_form_screen->input_sans);
	lyxrc->typewriter_font_name = fl_get_input(fd_form_screen->input_typewriter);
	lyxrc->font_norm = fl_get_input(fd_form_screen->input_font_norm);
	lyxrc->zoom = atoi(fl_get_input(fd_form_screen->intinput_size));
	fontloader.update();
   
	// All buffers will need resize
	bufferlist.resize();

	minibuffer->Set(_("Screen options set"));
}


void ScreenCancelCB(FL_OBJECT *ob, long)
{
	fl_hide_form(ob->form);
}


void ScreenOKCB(FL_OBJECT *ob, long data)
{
	ScreenCancelCB(ob,data);
	ScreenApplyCB(ob,data);
}


// This function runs "configure" and then rereads lyx.defaults to
// reconfigure the automatic settings.
void Reconfigure()
{
	minibuffer->Set(_("Running configure..."));

	// Run configure in user lyx directory
	Path p(user_lyxdir);
	Systemcalls one(Systemcalls::System, 
                          AddName(system_lyxdir,"configure"));
	minibuffer->Set(_("Reloading configuration..."));
	lyxrc->Read(LibFileSearch(string(), "lyxrc.defaults"));
	WriteAlert(_("The system has been reconfigured."), 
		   _("You need to restart LyX to make use of any"),
		   _("updated document class specifications."));
}


#warning looks like the below functions belong in BufferView
/* these functions are for the spellchecker */
#if 0
string NextWord(float &value)
{
	if (!current_view->available()){
		value = 1;
		return 0;
	}
   
	return current_view->text->SelectNextWord(value);
}
#endif

#warning move to BufferView
void SelectLastWord()
{
	if (!current_view->available())
		return;
   
	current_view->hideCursor();
	current_view->beforeChange(); 
	current_view->text->SelectSelectedWord();
	current_view->toggleSelection(false);
	current_view->update(BufferView::ResetSelection
			     |BufferView::FitCursor);
}


#warning move to BufferView
void EndOfSpellCheck()
{
	if (!current_view->available())
		return;
   
	current_view->hideCursor();
	current_view->beforeChange(); 
	current_view->text->SelectSelectedWord();
	current_view->text->ClearSelection();
	current_view->update(BufferView::ResetSelection
			     |BufferView::FitCursor);
}


#warning make it a BufferView function
void ReplaceWord(string const & replacestring)
{
	//if (!current_view->getScreen())
	//	return;

	current_view->hideCursor();
	current_view->update(BufferView::ResetSelection
			     |BufferView::FitCursor);
   
	// Clear the selection (if there is any).
	current_view->toggleSelection(false);
	current_view->text->
		ReplaceSelectionWithString(replacestring.c_str());
   
	current_view->text->SetSelectionOverString(replacestring.c_str());

	// Go back so that replacement string is also spellchecked
	for (unsigned int i=0; i < replacestring.length() + 1; ++i) {
		current_view->text->CursorLeftIntern();
	}
	current_view->update(BufferView::ResetSelection
			     |BufferView::FitCursor
			     |BufferView::BufferChange);
}
// End of spellchecker stuff



//
// Table of Contents
//

struct TocList {
	char counter[6];
	bool appendix;
	TocList *next;
};


static TocList* toclist = 0;


void TocSelectCB(FL_OBJECT *ob, long)
{
	if (!current_view->available())
		return;
   
	TocList* tmptoclist = toclist;
	int i = fl_get_browser(ob);
	int a = 0;

	for (a=1; a<i && tmptoclist->next; ++a){
		tmptoclist = tmptoclist->next;
	}

	if (!tmptoclist)
		return;
     
	LyXParagraph *par = current_view->currentBuffer()->paragraph;
	while (par && (par->GetCounter(0) != tmptoclist->counter[0] ||
		       par->GetCounter(1) != tmptoclist->counter[1] ||
		       par->GetCounter(2) != tmptoclist->counter[2] ||
		       par->GetCounter(3) != tmptoclist->counter[3] ||
		       par->GetCounter(4) != tmptoclist->counter[4] ||
		       par->GetCounter(5) != tmptoclist->counter[5] ||
		       par->appendix != tmptoclist->appendix)) {
		par = par->LastPhysicalPar()->Next();
	}
   
	if (par) {
		current_view->beforeChange();
		current_view->text->SetCursor(par, 0);
		current_view->text->sel_cursor =
			current_view->text->cursor;
		current_view->update(BufferView::ResetSelection
				     |BufferView::FitCursor);
	}
	else {
		WriteAlert(_("Error"), 
			   _("Couldn't find this label"), 
			   _("in current document."));
	}
	  
}


void TocCancelCB(FL_OBJECT *ob, long)
{
	fl_hide_form(ob->form);
}


void TocUpdateCB(FL_OBJECT *, long)
{
	static LyXParagraph* stapar = 0;
	TocList *tmptoclist = 0;
   
	// Deleted the toclist 
	if (toclist){
		while (toclist){
			tmptoclist = toclist->next;
			delete toclist;
			toclist = tmptoclist;
		}
	}
	toclist = 0;
	tmptoclist = toclist;


	fl_clear_browser(fd_form_toc->browser_toc);
	if (!current_view->available()) {
		fl_add_browser_line(fd_form_toc->browser_toc, _("*** No Document ***"));
		return;
	}
	fl_hide_object(fd_form_toc->browser_toc);
	// Get the table of contents.
	LyXParagraph *par = current_view->currentBuffer()->paragraph;
	char labeltype;
	char* line = new char[200];
	unsigned int i = 0;
	unsigned int pos = 0;
	unsigned char c;
	int topline = 0;
   
	if (stapar == par)
		topline = fl_get_browser_topline(fd_form_toc->browser_toc);
	stapar = par;
   
	while (par) {
		labeltype = textclasslist.style(current_view->currentBuffer()->params.textclass, 
					   par->GetLayout())->labeltype;
      
		if (labeltype >= LABEL_COUNTER_CHAPTER
		    && labeltype <= LABEL_COUNTER_CHAPTER +
		    current_view->currentBuffer()->params.tocdepth) {
			/* insert this into the table of contents */ 
			/* first indent a little bit */ 
			
			for (pos=0; 
			     pos < (labeltype - 
				    textclasslist.textClass(current_view->currentBuffer()->
						       params.textclass)->maxcounter) * 4 + 2;
			     ++pos)
				line[pos] = ' ';
			
			// Then the labelstring.
			i = 0;
			if (!par->labelstring.empty()) {
				while (pos < 199 && i < par->labelstring.length()) {
					line[pos] = par->labelstring[i];
					++i;
					++pos;
				}
			}
	 
			line[pos] = ' ';
			++pos;
			
			// Tow the contents.
			i = 0;
			while (pos < 199 && i < par->last()) {
				c = par->GetChar(i);
				if (isprint((unsigned char) c) || c >= 128) {
					line[pos] = c;
					++pos;
				}
				++i;
			}
			line[pos] = '\0';
			fl_add_browser_line(fd_form_toc->browser_toc, line);
			
			// Make a toclist entry.
			if (!tmptoclist){
				tmptoclist = new TocList;
				toclist = tmptoclist;
			} else {
				tmptoclist->next = new TocList;
				tmptoclist = tmptoclist->next;
			}
			
			tmptoclist->next = 0;
			int a = 0;
			for (a=0; a<6; ++a){
				tmptoclist->counter[a] = par->GetCounter(a);
			}
			tmptoclist->appendix = par->appendix;
		}
		par = par->LastPhysicalPar()->Next();
		
	}
	delete[] line;
	fl_set_browser_topline(fd_form_toc->browser_toc, topline);
	fl_show_object(fd_form_toc->browser_toc);
}


// Callbacks for form form_ref.
void RefSelectCB(FL_OBJECT *, long data)
{
	if (!current_view->available())
		return;

	string s =
		fl_get_browser_line(fd_form_ref->browser_ref,
				    fl_get_browser(fd_form_ref->browser_ref));
	if (s.empty())
		return;

	if (!current_view->currentBuffer()->isLinuxDoc()) {
	        if (data==2) {
	                current_view->getOwner()->getLyXFunc().Dispatch(LFUN_REFGOTO, s.c_str());
		        return;
		}
	    
		string t;
		if (data==0)
			t += "\\ref{";
		else
			t += "\\pageref{";
		t += s;
		t += '}';
		Inset *new_inset =
			new InsetRef(t, current_view->currentBuffer());
		current_view->insertInset(new_inset);
	} else {
#ifdef WITH_WARNINGS
#warning FIXME!
#endif
		// probably changes should be made to InsetRef to handle
		// linuxdoc as well. (Lgb)
	}
}


void RefUpdateCB(FL_OBJECT *, long)
{
	fl_clear_browser(fd_form_ref->browser_ref);
	if (!current_view->available())
		return;

	string refs = current_view->currentBuffer()->getReferenceList('\n');
	fl_addto_browser_chars(fd_form_ref->browser_ref,refs.c_str());

	if (!fl_get_browser_maxline(fd_form_ref->browser_ref)) {
		fl_add_browser_line(fd_form_ref->browser_ref, 
				    _("*** No labels found in document ***"));
		fl_deactivate_object(fd_form_ref->browser_ref);
	} else {
		fl_select_browser_line(fd_form_ref->browser_ref, 1);
		fl_activate_object(fd_form_ref->browser_ref);
	}
}


void RefHideCB(FL_OBJECT *ob, long)
{
	fl_hide_form(ob->form);
}
