/* This file is part of
* ======================================================
* 
*           LyX, The Document Processor
* 	 
*	    Copyright (C) 1995 Matthias Ettrich
*           Copyright (C) 1995-1998 The LyX Team.
*
* ======================================================*/

#include <config.h>
#include <stdlib.h>

#ifdef __GNUG__
#pragma implementation
#endif


#include FORMS_H_LOCATION
#include "libgui.h"
#include "splashscreen.h"
#include "lyx_gui.h"
#include "combox.h"
#include "lyx.h"
#include "form1.h"
#include "layout_forms.h"
#include "print_form.h"
#include "tex-strings.h"
#include "lyx_main.h"
#include "debug.h"
#include "version.h"
#include "LyXView.h"
#include "buffer.h"
#include "bufferlist.h"
#include "lyxserver.h"
#include "lyxrc.h"
#include "gettext.h"
#include "lyx_gui_misc.h"
#include "lyx_cb.h"
#include "lyxlookup.h"

// 	$Id: lyx_gui.C,v 1.18 1998/10/20 13:10:22 larsbj Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: lyx_gui.C,v 1.18 1998/10/20 13:10:22 larsbj Exp $";
#endif /* lint */

FD_form_search *fd_form_search;
FD_form_character *fd_form_character;
FD_form_print *fd_form_print;
FD_form_sendto *fd_form_sendto;
FD_form_figure *fd_form_figure;
FD_form_screen *fd_form_screen;
FD_form_toc *fd_form_toc;
FD_form_ref *fd_form_ref;

extern LyXServer *lyxserver;
extern BufferList bufferlist;


#warning there should be a function to read these. ARRae
// What you could do is to send the LyXGui to the splash
// then we would get rid of the global all together. Lgb
int width = 690;
int height = 510;
int xpos = -1;
int ypos = -1;
bool cursor_follows_scrollbar;


static int LyX_XErrHandler(Display *display, XErrorEvent *xeev)
{
	// emergency save
	if (!bufferlist.empty())
		bufferlist.emergencyWriteAll();

	// Get the reason for the crash.
	char etxt[513];
	XGetErrorText(display, xeev->error_code, etxt, 512);
	lyxerr << etxt << '\n';
	// By doing an abort we get a nice backtrace. (hopefully)
	abort();
	return -1; // to please certain compilers
}


LyXGUI::LyXGUI(LyX *owner, int *argc, char *argv[], bool GUI)
	:_owner(owner), gui(GUI)
{
	if (!gui) return;

	setDefaults();

	GUI_initialize(argc, argv);
	
	// X Error handler install goes here
	XSetErrorHandler(LyX_XErrHandler);
	
	splashscreen = new SplashScreen();

	// Make sure default screen is not larger than monitor
	
	if (width == 690 && height == 510) {
		Screen * scr=(DefaultScreenOfDisplay(fl_get_display()));
		if (HeightOfScreen(scr)-24<height)
			height = HeightOfScreen(scr)-24;
		if (WidthOfScreen(scr)-8<width)
			width = WidthOfScreen(scr)-8;
	}
}


// A destructor is always necessary  (asierra-970604)
LyXGUI::~LyXGUI() {
        // Lyxserver was created in this class so should be destroyed
        // here.  asierra-970604
        if (lyxserver) {
		delete lyxserver;
		lyxserver = 0;
	}

	CloseLyXLookup();
	
	// it'd be better to put this just after/before showing the
	// first LyXView.
	delete splashscreen;
}


void LyXGUI::setDefaults()
{
	GUI_defaults();
}


// This is called after we have parsed lyxrc
void LyXGUI::init()
{
	if (!gui) return;

	create_forms();

	GUI_postlyxrc(*lyxrc);

	// Update parameters.
	lyxViews->redraw();

	// Initialize the views.
	lyxViews->init();

	// in 0.12 the initialisation of the LyXServer must be done here
	// 0.13 it should be moved again...
	lyxserver = new LyXServer(lyxViews->getLyXFunc(), lyxrc->lyxpipes);
}


void LyXGUI::create_forms()
{
	lyxerr[Debug::INFO] << "Initializing LyXView...\n";
	lyxViews = new LyXView(width, height);
	lyxerr[Debug::INFO] << "Initializing LyXView...done\n";

	// From here down should be done by somebody else. (Lgb)

	//
	// Create forms
	//

	// the search form
	fd_form_search = create_form_form_search();
	fd_form_search->vdata = (void*)lyxViews;
	fl_set_form_atclose(fd_form_search->form_search,
			    CancelCloseBoxCB, 0);

	// the character form
	fd_form_character = create_form_form_character();
	fl_set_form_atclose(fd_form_character->form_character,
			    CancelCloseBoxCB, 0);
	fl_addto_choice(fd_form_character->choice_family, 
			_(" No change %l| Roman | Sans Serif | Typewriter %l| Reset "));
	fl_addto_choice(fd_form_character->choice_series, 
			_(" No change %l| Medium | Bold %l| Reset "));
	fl_addto_choice(fd_form_character->choice_shape,
			_(" No change %l| Upright | Italic | Slanted | Small Caps "
			"%l| Reset "));
	fl_addto_choice(fd_form_character->choice_size, 
			_(" No change %l| Tiny | Smallest | Smaller | Small "
			"| Normal | Large | Larger | Largest | Huge | Huger "
			"%l| Increase | Decrease | Reset "));
	fl_addto_choice(fd_form_character->choice_bar, 
			_(" No change %l| Emph | Underbar | Noun | LaTeX mode %l| Reset "));
	fl_addto_choice(fd_form_character->choice_color, 
			_(" No change %l| No color | Black | White | Red | Green "
			"| Blue | Cyan | Magenta | Yellow %l| Reset "));

	// the print form
	fd_form_print = create_form_form_print();
	fl_set_form_atclose(fd_form_print->form_print, CancelCloseBoxCB, 0);
       	fl_set_button(fd_form_print->radio_printer, 1);
	fl_set_button(fd_form_print->radio_file, 0);
	fl_set_button(fd_form_print->radio_order_normal, 1);
	fl_set_button(fd_form_print->radio_order_reverse, 0);
	fl_set_button(fd_form_print->radio_all_pages, 1);
	fl_set_button(fd_form_print->radio_odd_pages, 0);
	fl_set_button(fd_form_print->radio_even_pages, 0);

	// the sendto form
	fd_form_sendto = create_form_form_sendto();
	fl_set_form_atclose(fd_form_sendto->form_sendto,CancelCloseBoxCB,0);

	// the figure form
	fd_form_figure = create_form_form_figure();
	fl_set_form_atclose(fd_form_figure->form_figure,
			    CancelCloseBoxCB, 0);
	fl_set_button(fd_form_figure->radio_postscript, 1);

	// the screen form
	fd_form_screen = create_form_form_screen();
	fl_set_form_atclose(fd_form_screen->form_screen,
			    CancelCloseBoxCB, 0);

	// the toc form
	fd_form_toc = create_form_form_toc();
	fl_set_form_atclose(fd_form_toc->form_toc, CancelCloseBoxCB, 0);

	// the ref form
	fd_form_ref = create_form_form_ref();
	fl_set_form_atclose(fd_form_ref->form_ref, CancelCloseBoxCB, 0);

	// Show the main form
	// Commented out until we have the commandline parsing working
#if 0	
	if (xpos >= 0 && ypos >= 0) {
		lyxViews->setPosition(xpos, ypos);
	}
#endif	
	lyxViews->show();
	// make sure the splashscreen isn't hidden
	splashscreen->raise();
}


void LyXGUI::runTime()
{
	if (!gui) return;

	GUI_runtime();
}


void LyXGUI::regBuf(Buffer *b)
{
	lyxViews->currentView()->setBuffer(b);
}
