// -*- C++ -*-
/* This file is part of
* ======================================================
* 
*           LyX, The Document Processor
*        
*           Copyright (C) 1995 Matthias Ettrich
*           Copyright (C) 1995-1998 The LyX Team.
*
*======================================================*/

#include <config.h>

#include <errno.h>
#include "lyx_gui_misc.h"
#include "BufferView.h"
#include "buffer.h"
#include "form1.h"
#include "gettext.h"
#include "layout_forms.h"
#include "lyx.h"
#include "lyx_cb.h"
#include "lyx_main.h"
#include "math_forms.h"
#include "minibuffer.h"
#include "print_form.h"
#include "sp_form.h"

extern MiniBuffer *minibuffer;

// 	$Id: lyx_gui_misc.C,v 1.11 1998/10/18 03:20:44 arrae Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: lyx_gui_misc.C,v 1.11 1998/10/18 03:20:44 arrae Exp $";
#endif /* lint */


extern FD_form_search * fd_form_search;
extern FD_form_character * fd_form_character;
extern FD_form_print * fd_form_print;
extern FD_form_sendto * fd_form_sendto;
extern FD_form_figure * fd_form_figure;
extern FD_form_toc * fd_form_toc;
extern FD_form_ref * fd_form_ref;
extern FD_form_spell_check * fd_form_spell_check;
extern FD_panel  * fd_panel;
extern FD_delim  * fd_delim;
extern FD_deco   * fd_deco;
extern FD_space  * fd_space;
extern FD_matrix * fd_matrix;

extern void TocUpdateCB();

// Prevents LyX from being killed when the close box is pressed in a popup.
int CancelCloseBoxCB(FL_FORM *, void *)
{
	return FL_CANCEL;
}


// Prevents LyX from being killed when the close box is pressed in a popup.
int IgnoreCloseBoxCB(FL_FORM *, void *)
{
	return FL_IGNORE;
}


// Prevents LyX from crashing when no buffers available
// This is also one of the functions that we _really_ dont want
// we should try to finds way to help us with that.
// The signal/slot mechanism can probably help. 
void CloseAllBufferRelatedPopups()
{
	// don't forget to check that dynamically created forms
	// have been created otherwise hiding one could cause a crash
	// need the visible check otherwise XForms prints a warning
	// if hiding an invisible form
	if (fd_form_search->form_search->visible) {
		fl_hide_form(fd_form_search->form_search);
	}
	if (fd_form_character->form_character->visible) {
		fl_hide_form(fd_form_character->form_character);
	}
	if (fd_form_print->form_print->visible) {
		fl_hide_form(fd_form_print->form_print);
	}
	if (fd_form_figure->form_figure->visible) {
		fl_hide_form(fd_form_figure->form_figure);
	}
	if (fd_form_toc->form_toc->visible) {
		fl_hide_form(fd_form_toc->form_toc);
	}
	if (fd_form_ref->form_ref->visible) {
		fl_hide_form(fd_form_ref->form_ref);
	}
	if (fd_form_sendto->form_sendto->visible) {
		fl_hide_form(fd_form_sendto->form_sendto);
	}
	if (fd_form_spell_check) {
		if (fd_form_spell_check->form_spell_check->visible) {
			fl_trigger_object(fd_form_spell_check->done);
		}
	}
	if (fd_panel) {
		if (fd_panel->panel->visible) {
			fl_hide_form(fd_panel->panel);
		}
	}
	if (fd_delim) {
		if (fd_delim->delim->visible) {
			fl_hide_form(fd_delim->delim);
		}
	}
	if (fd_deco) {
		if (fd_deco->deco->visible) {
			fl_hide_form(fd_deco->deco);
		}
	}
	if (fd_space) {
		if (fd_space->space->visible) {
			fl_hide_form(fd_space->space);
		}
	}
	if (fd_matrix) {
		if (fd_matrix->matrix->visible) {
			fl_hide_form(fd_matrix->matrix);
		}
	}
}

// This is another function we really don't want.
// Again the Signal/Slot mechanism is tailor made for this task.
void updateAllVisibleBufferRelatedPopups()
{
	if (fd_form_toc->browser_toc->visible) {
		TocUpdateCB(0,0);
	}
}

// Extract shortcut from <ident>|<shortcut> string
const char* flyx_shortcut_extract(char const *sc)
{
	// Find '|' in the sc and return the string after that.

	register const char *sd = sc;
	while(sd[0]!=0 && sd[0] != '|') ++sd;

	if (sd[0]=='|') {
		++sd;
		//fprintf(stderr, "%s\n", sd);
		return sd;
	}
	return "";
}

// Extract identifier from <ident>|<shortcut> string
const char* flyx_ident_extract(char const *sc)
{
	register const char *se = sc;
	while(se[0]!=0 && se[0] != '|') ++se;

	if (se[0]==0) return sc;
	
	char * sb = new char[se-sc + 1];
	int index = 0;
	register const char *sd = sc;
	while (sd != se) {
		sb[index] = sd[0];
		++index; ++sd;
	}
	sb[index]=0;
	return sb;
}


//
void WriteAlert(string const & s1, string const & s2, string const & s3)
{
	if (minibuffer) {
		ProhibitInput();
		minibuffer->Set(s1, s2, s3);
	}

	fl_set_resource("flAlert.dismiss.label", _("Dismiss"));
	fl_show_alert(s1.c_str(), s2.c_str(), s3.c_str(), 0);
	if (minibuffer) {
		AllowInput();
	}
}


// Alarms user of something related to files
void WriteFSAlert(string const & s1, string const & s2)
{
	WriteAlert (s1, s2, strerror(errno));
}


bool AskQuestion(string const & s1, string const & s2, string const & s3)
{
	fl_set_resource("flQuestion.yes.label", idex(_("Yes|Yy#y")));
	fl_set_resource("flQuestion.no.label", idex(_("No|Nn#n")));

	return fl_show_question((s1 + "\n" + s2 + "\n" + s3).c_str(), 0);
}

// Returns 1 for yes, 2 for no, 3 for cancel.
int AskConfirmation(string const & s1, string const & s2, string const & s3)
{
	fl_set_choices_shortcut(scex(_("Yes|Yy#y")),
				scex(_("No|Nn#n")),
				scex(_("Cancel|^[")));
        return fl_show_choice(s1.c_str(), s2.c_str(), s3.c_str(), 
			      3, idex(_("Yes|Yy#y")),
			      idex(_("No|Nn#n")),
                              idex(_("Cancel|^[")), 3);
}


// Asks for a text
string askForText(string const & msg, string const & dflt)
{
	string tmp;
	tmp = fl_show_input(msg.c_str(), dflt.c_str());
	return tmp;
}

// Inform the user that the buffer is read-only, and that changes in the
// dialog box that is to appear will be ignored.
void WarnReadonly(BufferView *view)
{
	WriteAlert(_("Any changes will be ignored"),
		   _("The document is read-only:"),
		   view->currentBuffer()->getFileName());
}
