/* This file is part of
* ======================================================
* 
*           LyX, The Document Processor
* 	 
*	    Copyright (C) 1995 Matthias Ettrich
*           Copyright (C) 1995-1998 The LyX Team.
*
*======================================================*/

#include <config.h>

#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#ifdef __GNUG__
#pragma implementation
#endif

#include "definitions.h"
#include "lyxlookup.h"
#include "kbmap.h"
#include "lyxfunc.h"
#include "LyXAction.h"
#include "bufferlist.h"
#include "lyxserver.h"
#include "lyx.h"
#include "intl.h"
#include "lyx_main.h"
#include "lyx_cb.h"
#include "insetlatexaccent.h"
#include "insettoc.h"
#include "insetlof.h"
#include "insetloa.h"
#include "insetlot.h"
#include "insetref.h"
#include "insetparent.h"
#include "insettabular.h"
#include "insetfoot.h"
#include "insetmargin.h"
#include "insetert.h"
#include "formulamacro.h"
#include "toolbar.h"
#include "spellchecker.h" // RVDK_PATCH_5
#include "minibuffer.h"
#include "vspace.h"
#include "insetbibtex.h"
#include "insetcitation.h"
#include "LyXView.h"
#include "Popups.h"
#include "insetindex.h"
#include "insetinclude.h"
#include "filedlg.h"
#include "lyx_gui_misc.h"
#include "filetools.h"
#include "FileInfo.h"
#include "lyxscreen.h"
#include "debug.h"
#include "lyxrc.h"
#include "lyxtext.h"
#include "gettext.h"
#include "trans_mgr.h"
#include "ImportLaTeX.h"
#include "textclasslist.h"
#include "menus.h"

// 	$Id: lyxfunc.C,v 1.41 1998/10/22 16:56:17 larsbj Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: lyxfunc.C,v 1.41 1998/10/22 16:56:17 larsbj Exp $";
#endif /* lint */

extern bool cursor_follows_scrollbar;

extern void InsertAsciiFile(string const &, bool);
//extern void math_insert_symbol(char const*);
//extern bool math_insert_greek(char const); // why "Bool"?
extern BufferList bufferlist;
extern LyXServer *lyxserver;
extern short greek_kb_flag;
extern FD_form_toc *fd_form_toc;
extern bool selection_possible;

extern kb_keymap *toplevel_keymap;

extern bool MenuRunDvips(Buffer *,bool);
extern void MenuPrint(Buffer*);
extern void MenuSendto();
extern void QuitLyX();
extern void MenuFax(Buffer *);
extern void MenuMakeLaTeX(Buffer *);
extern void MenuMakeLinuxDoc(Buffer *);
extern void MenuMakeAscii(Buffer *);
extern LyXAction lyxaction;
// (alkis)
extern tex_accent_struct get_accent(kb_action action);

extern void MenuSearch(BufferView*);
extern bool MenuPreview(Buffer*);
extern bool MenuPreviewPS(Buffer*);
extern void MenuInsertLabel();
extern void MenuInsertRef();
extern void MenuLayoutCharacter();
extern void MenuLayoutParagraph();
extern void bulletForm();

extern Buffer * NewLyxFile(string const &);
extern void LoadLyXFile(string const &);
extern void Reconfigure();

extern int getISOCodeFromLaTeX(char *);


/* === globals =========================================================== */

bool LyXFunc::show_sc = true;


LyXFunc::LyXFunc(LyXViewBase *o)
	:owner(o)
{
	meta_fake_bit = 0;
	lyx_dead_action = LFUN_NOACTION;
	lyx_calling_dead_action = LFUN_NOACTION;
	setupLocalKeymap();
}


LyXFunc::~LyXFunc()
{
}


string LyXFunc::argAsString(char const *const argument)
{
	string tmp(argument);

	if (tmp.empty()) {
		// get the arg from somewhere else, a popup, or ask for
		// it in the minibuffer.
	}
	lyxerr[Debug::INFO] << "argAsString: <" << tmp << ">\n";
	return tmp;
}


int LyXFunc::processKeyEvent(XEvent *ev)
{
	char s_r[10];
	s_r[9] = '\0';
	int num_bytes;
	int action; 
	char *argument = 0;
	XKeyEvent *keyevent = &ev->xkey;
	KeySym keysym_return;

	num_bytes = LyXLookupString(ev, s_r, 10, &keysym_return);

	if (lyxerr.debugging(Debug::KEY)) {
		lyxerr << "KeySym is "
		       << XKeysymToString(keysym_return)
		       << "["
		       << long(keysym_return) << "]"
		       << " and num_bytes is "
		       << num_bytes
		       << " the string returned is \""
		       << (s_r) << "\"\n";
	}
	
	// Do nothing if we have nothing (JMarc)
	if (num_bytes == 0 && keysym_return == NoSymbol) {
		lyxerr[Debug::KEY]
			<< "Empty kbd action (probably composing)\n";
		return 0;
	}
	
	// this function should be used always [asierra060396]
	if (owner->currentView()->available() &&
	    owner->currentView()->the_locking_inset &&
	    keysym_return == XK_Escape) {
		UpdatableInset *inset = owner->currentView()->the_locking_inset;
		if (inset==inset->GetLockingInset()) {
			owner->currentView()->unlockInset(inset);
			owner->currentView()->text->CursorRight();
		}
		else {
		    inset->UnlockInsetInInset(inset->GetLockingInset(), true);
		}
		return 0;
	}

	// Can we be sure that this will work for all X-Windows
	// implementations? (Lgb)
	// This code snippet makes lyx ignore some keys. Perhaps
	// all of them should me explictly mentioned?
	if((keysym_return >= XK_Shift_L && keysym_return <= XK_Hyper_R)
	   || keysym_return == XK_Mode_switch || keysym_return == 0x0)
		return 0;

	// Do a one-deep top-level lookup for
	// cancel and meta-fake keys. RVDK_PATCH_5
	cancel_meta_seq.reset();

	action = cancel_meta_seq.addkey(keysym_return, keyevent->state
					&(ShiftMask|ControlMask
					  |Mod1Mask)); 

	// When not cancel or meta-fake, do the normal lookup. 
	// Note how the meta_fake Mod1 bit is OR-ed in and reset afterwards.
	// Mostly, meta_fake_bit = 0. RVDK_PATCH_5.
	if ( (action != LFUN_CANCEL) && (action != LFUN_META_FAKE) ) {

		// remove Caps Lock and Mod2 as a modifiers
		action = keyseq.addkey(keysym_return,
				       (keyevent->state | meta_fake_bit)
				       &(ShiftMask|ControlMask
					 |Mod1Mask));      
	}
	// Dont remove this unless you know what you are doing.
	meta_fake_bit = 0;
		
	if (action == 0) action = LFUN_PREFIX;

	if (lyxerr.debugging(Debug::KEY)) {
		lyxerr << "Key ["
		       << action << "]["
		       << keyseq.print() << "]["
		       << num_bytes << "]\n";
	}

	// already here we know if it any point in going further
	// why not return already here if action == -1 and
	// num_bytes == 0? (Lgb)

	if(keyseq.length>1 || keyseq.length<-1){
		owner->getMiniBuffer()->Set(keyseq.print());
	}

	if (action==-1) {
		if (keyseq.length<-1) { // unknown key sequence...
			LyXBell();
			owner->getMiniBuffer()->Set(_("Unknown sequence:"),
						    keyseq.print());
			return 0;
		}
	
		char isochar = keyseq.getiso();
		if (!(keyevent->state&ControlMask) &&
		    !(keyevent->state&Mod1Mask) &&
		    (isochar && keysym_return < 0xF000)) {
			argument = &s_r[0]; // shall this be here
			argument[0] = isochar;
			argument[1] = 0;
		}
		if (!argument) {
			lyxerr[Debug::INFO] << "Empty argument!\n";
			// This can`t possibly be of any use
			// so we`ll skip the dispatch.
			return 0;
		}
	} else
		if (action==LFUN_SELFINSERT) {
			argument = &s_r[0];
			argument[1] = 0;
		}
    
        bool tmp_sc = show_sc;
	show_sc = false;
	Dispatch(action, argument);
	show_sc = tmp_sc;
	
	// Need this for deadkeys (alkis)
	//keyseq.length=0;
	// ...but that breaks the minibuffer's display of "M-"...
#ifdef WITH_WARNINGS
#warning How does the deadkeys react to this?
#endif
	return 0;
} 


string LyXFunc::Dispatch(string const &cmd, string const &arg)
{
	return Dispatch(lyxaction.LookupFunc(cmd), arg.c_str());
}


string LyXFunc::Dispatch(int ac,
			  char const *do_not_use_this_arg)
{
#ifdef WITH_WARNINGS
#warning Why not just use an string instead of a char *
	// That would then mean we could eliminate argu and tmparg
	// and do most of the testing (especially the empty()) without
	// extra conversions.  ARRae.
#endif
	char const * argument = 0;
	kb_action action;
        

	FL_OBJECT *ob = 0;  // This will disapear soon
    
        // we have not done anything wrong yet.
        errorstat = false;
	dispatch_buffer.erase();
	
	// if action is a pseudo-action, we need the real action
	if (lyxaction.isPseudoAction(ac)) {
		string tmparg;
		action = lyxaction.retrieveActionArg(ac, tmparg);
		if (!tmparg.empty())
			argument = tmparg.c_str();
	} else {
		action = (kb_action)ac;
		argument = do_not_use_this_arg; // except here
	}
    
	selection_possible = false;
	
	if (owner->currentView()->available()) 
		owner->currentView()->hideCursor();

	if(!owner->currentView()->available()) {
		// This lists the allowed funcs when we have no
		// buffer loaded
		switch(action){
		case LFUN_MENU_OPEN_BY_NAME:
		case LFUN_PREFIX:
		case LFUN_MENUNEW:
		case LFUN_MENUNEWTMPLT:
		case LFUN_MENUOPEN:
		case LFUN_QUIT:
		case LFUN_PUSH_TOOLBAR:
		case LFUN_ADD_TO_TOOLBAR:
		case LFUN_EXEC_COMMAND:
		case LFUN_DROP_LAYOUTS_CHOICE:
		case LFUN_FILE_NEW:
		case LFUN_FILE_OPEN:
		case LFUN_IMPORT:
		case LFUN_RECONFIGURE:
		case LFUN_CANCEL:
		case LFUN_APROPOS:
		case LFUN_META_FAKE:
		case LFUN_HELP_COPYRIGHT:
		case LFUN_HELP_CREDITS:

			break;
		case LFUN_UNKNOWN_ACTION:
			setErrorMessage(N_("Unknown action"));
			goto exit_with_message;
		default:
			setErrorMessage(N_("Command not allowed with"
					"out any document open"));
			goto exit_with_message;  // action not allowed
		}
	}
	
	commandshortcut.erase();
	
	if (lyxrc->display_shortcuts && show_sc) {
		if (action != LFUN_SELFINSERT) {
			// Put name of command and list of shortcuts
			// for it in minibuffer
			string comname = lyxaction.getActionName(action);

			int pseudoaction = action;
			bool argsadded = false;

			string argu = !argument ? "" : argument;
			if (!argu.empty()) {
				// If we have the command with argument, 
				// this is better
				pseudoaction = 
					lyxaction.searchActionArg(action,
							  	  argument);

				if (pseudoaction == -1) {
					pseudoaction = action;
				} else {
					comname += " " + argu;
					argsadded = true;
				}
			}

			string shortcuts = toplevel_keymap->findbinding(pseudoaction);

			if (!shortcuts.empty()) {
				comname += ": " + shortcuts;
			} else if (!argsadded) {
				comname += " " + argu;
			}

			if (!comname.empty()) {
				comname = strip(comname);
				commandshortcut = "(" + comname + ')';
				owner->getMiniBuffer()->Set(commandshortcut);
				// Here we could even add a small pause,
				// to annoy the user and make him learn
				// the shortcuts.
				// No! That will just annoy, not teach
				// anything. The user will read the messages
				// if they are interested. (Asger)
			}
		}
        }
    
        // Now that we know which action, if the buffer is RO let's check 
	// whether the action is legal.  Alejandro 970603
        if (owner->currentView()->available() && 
            owner->currentBuffer()->isReadonly() && 
            lyxaction.isFuncRO(action)) {
		setErrorMessage(N_("Document is read-only"));
		lyxerr[Debug::INFO] << "Error: Document is read-only.\n";
		goto exit_with_message;
	}
	
	// If in math mode pass the control to
	// the math inset [asierra060396]
	if (owner->currentView()->available() &&
	    owner->currentView()->the_locking_inset) {
		if (action > 1 || 
		    (action == LFUN_UNKNOWN_ACTION && keyseq.length >= -1)) {
		        char s[8] = "";
			if (action == LFUN_UNKNOWN_ACTION && !argument) {
				sprintf(s, "%c", keyseq.getiso());
				argument = &s[0];
			}
			// Undo/Redo pre 0.13 is a bit tricky for insets.
		        if (action == LFUN_UNDO) {
				int slx, sly;
				UpdatableInset* inset = owner->currentView()->the_locking_inset;
				inset->GetCursorPos(slx, sly);
				owner->currentView()->unlockInset(inset);
				owner->currentView()->undo();
				inset = (UpdatableInset*)owner->currentView()->text->cursor.par->GetInset(owner->currentView()->text->cursor.pos);
				if (inset) inset->Edit(slx, sly, 0, owner->currentView());
				return string();
			} else if (action==LFUN_REDO) {
				int slx, sly;
				UpdatableInset* inset =	owner->currentView()->the_locking_inset;
				inset->GetCursorPos(slx, sly);
				owner->currentView()->unlockInset(inset);
				owner->currentView()->redo();
				inset = (UpdatableInset*)owner->currentView()->text->cursor.par->GetInset(owner->currentView()->text->cursor.pos);
				if (inset) inset->Edit(slx, sly, 0, owner->currentView());
				return string();
			} else {
			    UpdatableInset::RESULT lresult = 
			      owner->currentView()->the_locking_inset->
			      LocalDispatch(action, argument);
			    
			    if (lresult==UpdatableInset::DISPATCHED) 
				return string();
			    else 
			      if (lresult==UpdatableInset::FINISHED) {
				setMessage(N_("Text mode"));
				if (action==LFUN_RIGHT || action==-1)
					owner->currentView()->text->CursorRight();
				if (action==LFUN_LEFT || action==LFUN_RIGHT)
					return string();
			      }
			}
		}
	}

	switch(action) {
		// --- Misc -------------------------------------------
	case LFUN_WORDFINDFORWARD: 
	case LFUN_WORDFINDBACKWARD: 
	{ 
		LyXText *ltCur;
		static string last_search;
		string searched_string;
		
		string arg = argAsString(argument);
		
		if (!arg.empty()) {
			last_search = arg;
			searched_string = arg;
		} else {
			searched_string = last_search;
		}
		
		ltCur =	 owner->currentView()->text ;	 
		
		if (!searched_string.empty() &&
		    (	 (action == LFUN_WORDFINDBACKWARD) ? 
			 ltCur->SearchBackward( searched_string.c_str() ) :
			 ltCur->SearchForward(	searched_string.c_str() ) 
			    )){

			//	??? What is that ???
			owner->currentView()->update(BufferView::ResetSelection
		     |BufferView::FitCursor);
			
			// ??? Needed ???
			// clear the selection (if there is any) 
			owner->currentView()->toggleSelection();
			owner->currentView()->text->ClearSelection();
			// Move cursor so that successive C-s 's will not stand in place. 
			if( action == LFUN_WORDFINDFORWARD ) 
				owner->currentView()->text->CursorRightOneWord();
			owner->currentView()->text->FinishUndo();
			owner->currentView()->moveCursorUpdate(false);
			
			// ??? Needed ???
			// set the new selection 
			// SetSelectionOverLenChars(owner->currentView()->currentBuffer()->text, iLenSelected);
			owner->currentView()->toggleSelection(false);
		} else 
			LyXBell();	
		
		owner->currentView()->showCursor();
	}
	break;
	
	case LFUN_PREFIX:
	{
		if (owner->currentView()->available()) {
			owner->currentView()->update(BufferView::ResetSelection
						     |BufferView::FitCursor);
		}
		owner->getMiniBuffer()->Set(keyseq.print(true), 
					    string(), string(), 1);
	}
	break;

	// --- Misc -------------------------------------------
	case LFUN_EXEC_COMMAND:
		owner->getMiniBuffer()->ExecCommand(); 
		break;
		
	case LFUN_CANCEL:                   // RVDK_PATCH_5
		keyseq.reset();
		meta_fake_bit = 0;
		if(owner->currentView()->available())
			// cancel any selection
			Dispatch(int(LFUN_MARK_OFF), 0);
		setMessage(N_("Cancel"));
		break;

	case LFUN_META_FAKE:                                 // RVDK_PATCH_5
	{
		meta_fake_bit = Mod1Mask;
		setMessage("M-"+keyseq.print(true)); // RVDK_PATCH_5
	}
	break;  

	case LFUN_READ_ONLY_TOGGLE:
		owner->currentView()->toggleReadOnly();
		// force all the popups to reflect the change.
		owner->getPopups().updateBufferDependent();
		break;
		
	case LFUN_CENTER: // this is center and redraw.
		owner->currentView()->center();
		break;
		
	case LFUN_APPENDIX:
		owner->currentView()->appendix();
		break;


	// -- Window operatrions
	case LFUN_SPLIT_WINDOW_HORIZONTALLY:
		owner->getFrameHandler()->splitHorizontally();
		break;
		
	case LFUN_SPLIT_WINDOW_VERTICALLY:
		owner->getFrameHandler()->splitVertically();
		break;

		
	// --- Menus -----------------------------------------------
	case LFUN_MENUNEW:
		MenuNew(false);
		break;
		
	case LFUN_MENUNEWTMPLT:
		MenuNew(true);
		break;
		
	case LFUN_MENUOPEN:
		MenuOpen();
		break;
		
	case LFUN_CLOSEBUFFER:
		CloseBuffer();
		break;
		
	case LFUN_MENUWRITE:
		bufferlist.menuWrite(owner->currentBuffer());
		break;
		
	case LFUN_MENUWRITEAS:
		bufferlist.menuWriteAs(owner->currentBuffer());
		break;
		
	case LFUN_MENURELOAD:
		reloadBuffer();
		break;
		
	case LFUN_PREVIEW:
		MenuPreview(owner->currentBuffer());
		break;
			
	case LFUN_PREVIEWPS:
		MenuPreviewPS(owner->currentBuffer());
		break;
		
	case LFUN_RUNLATEX:
		owner->currentView()->runLaTeX();
		break;
		
	case LFUN_RUNCHKTEX:
		owner->currentView()->runChktex();
		break;
		
	case LFUN_RUNDVIPS:
		MenuRunDvips(owner->currentBuffer(), false);
		break;
		
	case LFUN_MENUPRINT:
		MenuPrint(owner->currentBuffer());
		break;
		
	case LFUN_FAX:
		MenuFax(owner->currentBuffer());
		break;
			
	case LFUN_EXPORT:
	{
		//needs argument as string
		string extyp=argAsString(argument);
		
		// latex
		if (extyp == "latex") {
			// make sure that this buffer is not linuxdoc
			owner->currentView()->MenuMakeLaTeX();
		}
		// linuxdoc
		else if (extyp == "linuxdoc") {
			// make sure that this buffer is not latex
			MenuMakeLinuxDoc(owner->currentBuffer());
		}
		// dvi
		else if (extyp == "dvi") {
			// Run LaTeX as "Update dvi..." Bernhard.
			// We want the dvi in the current directory. This
			// is achieved by temporaily disabling use of
			// temp directory. As a side-effect, we get
			// *.log and *.aux files also. (Asger)
			bool flag = lyxrc->use_tempdir;
			lyxrc->use_tempdir = false;
			owner->currentView()->runLaTeX();
			lyxrc->use_tempdir = flag;
		}
		// postscript
		else if (extyp == "postscript") {
			// Start Print-dialog. Not as good as dvi... Bernhard.
			MenuPrint(owner->currentBuffer());
			// Since the MenuPrint is a pop-up, we can't use
			// the same trick as above. (Asger)
			// MISSING: Move of ps-file :-(
		}
		// ascii
		else if (extyp == "ascii") {
			MenuMakeAscii(owner->currentBuffer());
		}
		else if (extyp == "custom") {
			MenuSendto();
			break;
		}
		else {
			setErrorMessage(string(N_("Unknown export type: "))
					+ extyp);
		}
	}
	break;

	case LFUN_IMPORT:
	{
		//needs argument as string
		string imtyp=argAsString(argument);
	
		// latex
		if (imtyp == "latex") {
			doImportLaTeX();
		}
		// ascii
		else if (imtyp == "ascii") {
			doImportASCII();
		}
		else {
			setErrorMessage(string(N_("Unknown import type: "))
					+ imtyp);
		}
		break;
	}
		
	case LFUN_QUIT:
		QuitLyX();
		break;
		
	case LFUN_TOCVIEW:
		TocUpdateCB(ob, 0);
		if (fd_form_toc->form_toc->visible) {
			fl_raise_form(fd_form_toc->form_toc);
		} else {
			fl_show_form(fd_form_toc->form_toc,
				     FL_PLACE_MOUSE |
				     FL_FREE_SIZE, FL_FULLBORDER,
				     _("Table of Contents"));
		}
		break;
		
	case LFUN_TOC_INSERT:
		owner->currentView()->insertTOC();
		break;
	
	case LFUN_LOF_INSERT:
		owner->currentView()->insertLOF();
		break;
	
	case LFUN_LOA_INSERT:
		owner->currentView()->insertLOA();
		break;

	case LFUN_LOT_INSERT:
		owner->currentView()->insertLOT();
		break;
		
	case LFUN_TABLE:
                owner->getPopups().showFormTableNew();
		break;
		
	case LFUN_FIGURE:
		FigureCB(ob, 0);
		break;
		
	case LFUN_AUTOSAVE:
		owner->currentView()->autoSave();
		break;
		
	case LFUN_UNDO:
		owner->currentView()->undo();
		break;
		
	case LFUN_REDO:
		owner->currentView()->redo();
		break;
		
	case LFUN_MENUSEARCH:
		MenuSearch(owner->currentView());
		break;
		
	case LFUN_PASTE:
		owner->currentView()->paste();
		break;
		
	case LFUN_PASTESELECTION:
	{
	        bool asPara = false;
		if (string(argument) == "paragraph") asPara = true;
		owner->currentView()->pasteSelection(asPara);
		break;
	}

	case LFUN_CUT:
		owner->currentView()->cut();
		break;
		
	case LFUN_COPY:
		owner->currentView()->copy();
		break;
		
	case LFUN_LAYOUT_COPY:
		owner->currentView()->copyEnvironment();
		break;
		
	case LFUN_LAYOUT_PASTE:
		owner->currentView()->pasteEnvironment();
		break;
		
	case LFUN_GOTOERROR:
		owner->currentView()->gotoError();
		break;
		
	case LFUN_REMOVEERRORS:
		owner->currentView()->removeErrors();
		break;
		
	case LFUN_GOTONOTE:
		owner->currentView()->gotoNote();
		break;
		
	case LFUN_OPENSTUFF:
		owner->currentView()->openStuff();
		break;
		
	case LFUN_HYPHENATION:
		owner->currentView()->hyphenationPoint();
		break;

	case LFUN_PROTECTEDSPACE:
		owner->currentView()->protectedSpace();
		break;
		
	case LFUN_LDOTS:
		owner->currentView()->ldots();
		break;
		
	case LFUN_END_OF_SENTENCE:
		owner->currentView()->endOfSentenceDot();
		break;
		
	case LFUN_HFILL:
		owner->currentView()->hfill();
		break;
		
	case LFUN_DEPTH:
		owner->currentView()->changeDepth(ob, 0);
		break;
		
	case LFUN_DEPTH_MIN:
		owner->currentView()->changeDepth(ob, -1);
		break;
		
	case LFUN_DEPTH_PLUS:
		owner->currentView()->changeDepth(ob, 1);
		break;
		
	case LFUN_FREE:
		owner->currentView()->userFree();
		break;
		
	case LFUN_TEX:
		owner->currentView()->tex();
		break;
		
	case LFUN_MELT:
		owner->currentView()->melt();
		break;
		
	case LFUN_RECONFIGURE:
		Reconfigure();
		break;

	case LFUN_FOOTMELT:
		owner->currentView()->footMelt();
		break;

	case LFUN_MARGINMELT:
		owner->currentView()->marginMelt();
		break;
		
		// --- version control -------------------------------
	case LFUN_VC_REGISTER:
	{
		if (!owner->currentBuffer()->lyxvc.inUse())
			owner->currentBuffer()->lyxvc.registrer();
	}
	break;
		
	case LFUN_VC_CHECKIN:
	{
		if (owner->currentBuffer()->lyxvc.inUse()
		    && !owner->currentBuffer()->isReadonly())
			owner->currentBuffer()->lyxvc.checkIn();
	}
	break;
		
	case LFUN_VC_CHECKOUT:
	{
		if (owner->currentBuffer()->lyxvc.inUse()
		    && owner->currentBuffer()->isReadonly())
			owner->currentBuffer()->lyxvc.checkOut();
	}
	break;
	
	case LFUN_VC_REVERT:
	{
		owner->currentBuffer()->lyxvc.revert();
	}
	break;
		
	case LFUN_VC_UNDO:
	{
		owner->currentBuffer()->lyxvc.undoLast();
	}
	break;
		
	case LFUN_VC_HISTORY:
	{
		owner->currentBuffer()->lyxvc.showLog();
		break;
	}
	
	// --- buffers ----------------------------------------
	case LFUN_PREVBUFFER:
		lyxerr << "Sorry. Not implemented yet.\n";
		break;
			
	case LFUN_FILE_INSERT:
	{
		// needs argument as string
		string fil = argAsString(argument);
		MenuInsertLyXFile(fil);
	}
	break;
	
	case LFUN_FILE_INSERT_ASCII:
	{
	        bool asPara = (string(argument) == "paragraph");
		owner->currentView()->insertAsciiFile(string(), asPara);
	}
	break;
	
	case LFUN_FILE_NEW:
	{
		// servercmd: argument must be <file>:<template>
		Buffer *tmpbuf = NewLyxFile(argument);
		if (tmpbuf)
			owner->currentView()->setBuffer(tmpbuf);
	}
		break;
			
	case LFUN_FILE_OPEN:
		owner->currentView()->setBuffer(
			bufferlist.loadLyXFile(argument));
		break;

	case LFUN_LATEX_LOG:
		owner->getPopups().showLaTeXLog();
		break;
		
	case LFUN_LAYOUTNO:
	{
		lyxerr[Debug::INFO] << "LFUN_LAYOUTNO: (arg) " 
				    << argument << '\n';
		int sel = atoi(argument);
		lyxerr[Debug::INFO] << "LFUN_LAYOUTNO: (sel) "
				    << sel << '\n';
		
		// Should this give a setMessage instead?
		if (sel == 0) 
			return string(); // illegal argument

		--sel; // sel 1..., but layout 0...

		// Pretend we got the name instead.
		Dispatch(int(LFUN_LAYOUT), 
			 textclasslist.nameOfLayout(owner->currentView()->
					       text->parameters->
					       textclass,
					       sel).c_str());
		return string();
	}
		
	case LFUN_LAYOUT:
	{
		lyxerr[Debug::INFO] << "LFUN_LAYOUT: (arg) "
				    << argument << '\n';
		
		// Derive layout number from given argument (string)
		// and current buffer's textclass (number). */    
		int layoutno =
			textclasslist.numberOfLayout(owner->
						currentView()->
						text->parameters->
						textclass,
						argument);

		// see if we found the layout number:
		if (layoutno == -1) {
			setErrorMessage(string(N_("Layout ")) + argument + 
					N_(" not known"));
			break;
		}
			
		if (owner->currentView()->text->cursor.par->GetLayout() != layoutno) {
			owner->currentView()->hideCursor();
			owner->currentView()->update(BufferView::ResetSelection
						     |BufferView::FitCursor);
			owner->currentView()->text->SetLayout(layoutno);
			owner->updateLayoutChoice();
			owner->currentView()->update(BufferView::ResetSelection
						     |BufferView::FitCursor
						     |BufferView::BufferChange);
		}
	}
	break;

	case LFUN_LAYOUT_DOCUMENT:
                owner->getPopups().showFormDocument();
		break;
		
	case LFUN_LAYOUT_PARAGRAPH:
                owner->getPopups().showFormParagraph();
		break;
		
	case LFUN_LAYOUT_CHARACTER:
		MenuLayoutCharacter();
		break;
		
	case LFUN_LAYOUT_TABLE:
		owner->getPopups().showFormTable();
		break;
		
	case LFUN_LAYOUT_PREAMBLE:
		owner->getPopups().showLaTeXPreamble();
		break;

	case LFUN_LAYOUT_SAVE_DEFAULT:
                owner->currentView()->layoutSave();
                break;
                
	case LFUN_DROP_LAYOUTS_CHOICE:
		owner->getToolbar()->combox->Show();
		break;

	case LFUN_EMPH:
		owner->currentView()->emph();
		break;
		
	case LFUN_BOLD:
		owner->currentView()->bold();
		break;
		
	case LFUN_NOUN:
		owner->currentView()->noun();
		break;
		
	case LFUN_CODE:
		owner->currentView()->code();
		break;
		
	case LFUN_SANS:
		owner->currentView()->sans();
		break;
		
	case LFUN_ROMAN:
		owner->currentView()->roman();
		break;
		
	case LFUN_DEFAULT:
		owner->currentView()->styleReset();
		break;
		
	case LFUN_UNDERLINE:
		owner->currentView()->underline();
		break;
		
	case LFUN_FONT_SIZE:
		owner->currentView()->fontSize(argument);
		break;
		
	case LFUN_FONT_STATE:
		setMessage(currentState(owner->currentView()));
		break;
		
	case LFUN_UPCASE_WORD:
		owner->currentView()->upcaseWord();
		break;
		
	case LFUN_LOWCASE_WORD:
		owner->currentView()->lowercaseWord();
		break;
		
	case LFUN_CAPITALIZE_WORD:
		owner->currentView()->capitalizeWord();
		break;
		
	case LFUN_INSERT_LABEL:
		owner->currentView()->insertLabel();
		break;
		
	case LFUN_INSERT_REF:
		MenuInsertRef();
		break;
		
	case LFUN_REFTOGGLE:
	{
		InsetRef *inset =
			(InsetRef*)getInsetByCode(Inset::REF_CODE);
		if (inset) {
			if (inset->getFlag()==InsetRef::REF)
				inset->setFlag(InsetRef::PAGE_REF);
			else
				inset->setFlag(InsetRef::REF);
			owner->currentView()->updateInset(inset, true);
		} else {
			setErrorMessage(N_("No cross-reference to toggle"));
		}
	}
	break;
	
	case LFUN_REFBACK:
		owner->currentView()->restorePosition();
		break;

	case LFUN_REFGOTO:
	{
		string label(argument);
		if (label.empty()) {
			InsetRef *inset =
				(InsetRef*)getInsetByCode(Inset::REF_CODE);
			if (inset)
                                label = inset->getContents();
		}
		
		if (!label.empty()) {
			owner->currentView()->savePosition();
			owner->currentView()->gotoLabel(label.c_str());
		}
	}
	break;
		
	case LFUN_MENU_OPEN_BY_NAME:
		owner->getMenus()->openByName(argument);
		break;
		
	case LFUN_SPELLCHECK:
		if (lyxrc->isp_command != "none")
			ShowSpellChecker(owner->currentView());
		break;
		
		// --- Cursor Movements -----------------------------
	case LFUN_RIGHT:
		owner->currentView()->right(false);
		break;
		
	case LFUN_LEFT:
		owner->currentView()->left(false);
		break;
		
	case LFUN_UP:
		owner->currentView()->up(false);
		break;
		
	case LFUN_DOWN:
		owner->currentView()->down(false);
		break;

	case LFUN_UP_PARAGRAPH:
		owner->currentView()->upParagraph(false);
		break;
		
	case LFUN_DOWN_PARAGRAPH:
		owner->currentView()->downParagraph(false);
		break;
		
	case LFUN_PRIOR:
		owner->currentView()->prior(false);
		break;
		
	case LFUN_NEXT:
		owner->currentView()->next(false);
		break;
		
	case LFUN_HOME:
		owner->currentView()->home(false);
		break;
		
	case LFUN_END:
		owner->currentView()->end(false);
		break;
		
	case LFUN_TAB:
		owner->currentView()->tab();
		break;
		
	case LFUN_WORDRIGHT:
		owner->currentView()->wordRight(false);
		break;
		
	case LFUN_WORDLEFT:
		owner->currentView()->wordLeft(false);
		break;
		
	case LFUN_BEGINNINGBUF:
		owner->currentView()->beginBuff(false);
		break;
		
	case LFUN_ENDBUF:
		owner->currentView()->endBuff(false);
		break;

      
		/* cursor selection ---------------------------- */
	case LFUN_RIGHTSEL:
		owner->currentView()->right(true);
		break;
		
	case LFUN_LEFTSEL:
		owner->currentView()->left(true);
		break;
		
	case LFUN_UPSEL:
		owner->currentView()->up(true);
		break;
		
	case LFUN_DOWNSEL:
		owner->currentView()->down(true);
		break;

	case LFUN_UP_PARAGRAPHSEL:
		owner->currentView()->upParagraph(true);
		break;
		
	case LFUN_DOWN_PARAGRAPHSEL:
		owner->currentView()->downParagraph(true);
		break;
		
	case LFUN_PRIORSEL:
		owner->currentView()->prior(true);
		break;
		
	case LFUN_NEXTSEL:
		owner->currentView()->next(true);
		break;
		
	case LFUN_HOMESEL:
		owner->currentView()->home(true);
		break;
		
	case LFUN_ENDSEL:
		owner->currentView()->end(true);
		break;
		
	case LFUN_WORDRIGHTSEL:
		owner->currentView()->wordRight(true);
		break;
		
	case LFUN_WORDLEFTSEL:
		owner->currentView()->wordLeft(true);
		break;
		
	case LFUN_BEGINNINGBUFSEL:
		owner->currentView()->beginBuff(true);
		break;
		
	case LFUN_ENDBUFSEL:
		owner->currentView()->endBuff(true);
		break;

		// --- text changing commands ------------------------
	case LFUN_BREAKLINE:
		owner->currentView()->insertSpecialChar(LYX_META_NEWLINE);
		break;
		
	case LFUN_SETMARK:
		if (owner->currentView()->setMark())
			setMessage(N_("Mark set"));
		else
			setMessage(N_("Mark removed"));
		break;
		
	case LFUN_DELETE:
		owner->currentView()->regionDelete();
		break;

	case LFUN_DELETE_SKIP:
		owner->currentView()->regionDeleteSkip();
		break;

		// -------> Delete word forward.
	case LFUN_DELETE_WORD_FORWARD:
		owner->currentView()->deleteWordForward();
		break;

		// -------> Delete word backward.
	case LFUN_DELETE_WORD_BACKWARD:
		owner->currentView()->deleteWordBackward();
		break;
		
		// -------> Kill to end of line.
	case LFUN_DELETE_LINE_FORWARD:
		owner->currentView()->deleteLineForward();
		break;
		
		// -------> Set mark off.
	case LFUN_MARK_OFF:
		owner->currentView()->markOff();
		setMessage(N_("Mark off"));
		break;

		// -------> Set mark on.
	case LFUN_MARK_ON:
		owner->currentView()->markOn();
		setMessage(N_("Mark on"));
		break;
		
	case LFUN_BACKSPACE:
		owner->currentView()->backspace();
		break;

	case LFUN_BACKSPACE_SKIP:
		owner->currentView()->backspaceSkip();
		break;

	case LFUN_BREAKPARAGRAPH:
		owner->currentView()->paragraphBreak(false);
		break;

	case LFUN_BREAKPARAGRAPHKEEPLAYOUT:
		owner->currentView()->paragraphBreak(true);
		break;
	
	case LFUN_BREAKPARAGRAPH_SKIP:
		owner->currentView()->paragraphBreakSkip();
		break;
	
	case LFUN_QUOTE:
		owner->currentView()->insertChar('\"');
		break;

	case LFUN_HTMLURL:
	        HtmlUrlCB();
		break;

	case LFUN_URL:
	        UrlCB();
		break;
		
		// --- lyxserver commands ----------------------------

	case LFUN_CHARATCURSOR:
		dispatch_buffer = owner->currentView()->getCurrentChar();
		break;
	
	case LFUN_GETXY:
		dispatch_buffer = 
			tostr(owner->currentView()->text->cursor.x) + ' '
			+ tostr(owner->currentView()->text->cursor.y);
		break;
		
	case LFUN_SETXY:
	{
		int  x;
		long y;
		sscanf(argument, " %d %ld", &x, &y);
		owner->currentView()->text->SetCursorFromCoordinates(x, y);
	}
	break;
	
	case LFUN_GETLAYOUT:
		dispatch_buffer = 
			tostr(owner->currentView()->text->cursor.par->layout);
		break;
			
	case LFUN_GETFONT:
	{
		LyXFont *font = &(owner->currentView()->text->current_font);
                if(font->shape() == LyXFont::ITALIC_SHAPE)
			dispatch_buffer = 'E';
                else if(font->shape() == LyXFont::SMALLCAPS_SHAPE)
			dispatch_buffer = 'N';
                else
			dispatch_buffer = '0';

	}
	break;

	case LFUN_GETLATEX:
	{
		LyXFont *font = &(owner->currentView()->text->current_font);
                if(font->latex() == LyXFont::ON)
			dispatch_buffer = 'L';
                else
			dispatch_buffer = '0';
	}
	break;

	case LFUN_GETNAME:
		setMessage(owner->currentBuffer()->getFileName());
		lyxerr[Debug::INFO] << "FNAME["
				    <<owner->currentBuffer()->getFileName()
				    << "]\n";
		break;
		
	case LFUN_NOTIFY:
	{
		dispatch_buffer = keyseq.print();
		lyxserver->notifyClient(dispatch_buffer);
	}
	break;

	case LFUN_APROPOS:
	case LFUN_GETTIP:
	{
		int qa = lyxaction.LookupFunc((const char*)argument);
		setMessage(lyxaction.helpText((kb_action)qa));
	}
	break;

	// --- accented characters ---------------------------
		
	case LFUN_UMLAUT:
	case LFUN_CIRCUMFLEX:
	case LFUN_GRAVE:
	case LFUN_ACUTE:
	case LFUN_TILDE:
	case LFUN_CEDILLA:
	case LFUN_MACRON:
	case LFUN_DOT:
	case LFUN_UNDERDOT:
	case LFUN_UNDERBAR:
	case LFUN_CARON:
	case LFUN_SPECIAL_CARON:
	case LFUN_BREVE:
	case LFUN_TIE:
	case LFUN_HUNG_UMLAUT:
	case LFUN_CIRCLE:
	case LFUN_OGONEK:
	{
		char c;
		
		if (keyseq.length==-1 && keyseq.getiso()!=0) 
			c=keyseq.getiso();
		else
			c=0;
		
		owner->getIntl().getTrans()->
			deadkey(owner->currentView(), c,
				get_accent(action).accent);
		
		// Need to reset, in case the minibuffer calls these
		// actions
		keyseq.reset();
		keyseq.length=0;
		
		// copied verbatim from do_accent_char
		owner->currentView()->smallUpdate(1);
		owner->currentView()->setUpdateTimer();
		owner->currentView()->text->sel_cursor = 
			owner->currentView()->text->cursor;
	}   
	break;
	
	// --- toolbar ----------------------------------
	case LFUN_PUSH_TOOLBAR:
	{
		int nth = atoi(argument);
		if (lyxerr.debugging(Debug::TOOLBAR)) {
			lyxerr << "LFUN_PUSH_TOOLBAR: argument = `"
			       << argument << "'\n"
			       << "LFUN_PUSH_TOOLBAR: nth = `"
			       << nth << "'\n";
		}
		
		if (nth <= 0) {
			LyXBell();
			setErrorMessage(N_("Push-toolbar needs argument > 0"));
		} else {
			owner->getToolbar()->push(nth);
		}
	}
	break;
	
	case LFUN_ADD_TO_TOOLBAR:
	{
		if (lyxerr.debugging(Debug::TOOLBAR)) {
			lyxerr << "LFUN_ADD_TO_TOOLBAR: argument = `" 
			       << argument << "'\n";
		}
		string tmp(argument);
		if (tmp.empty()) {
			LyXBell();
			setErrorMessage(N_("Usage: toolbar-add-to <LyX command>"));
		} else {
			owner->getToolbar()->add(ToolbarBackend::TOOL_FUNC,
						 lyxaction
						 .LookupFunc(argument));
			owner->getToolbar()->set();
		}
	}
	break;
	
	   
	// ---  Mathed stuff. If we are here, there is no locked inset yet.
	
	// Greek mode     
	case LFUN_GREEK:
	{
		if (!greek_kb_flag) {
			greek_kb_flag = 1;
			setMessage(N_("Math greek mode on"));
		} else
			greek_kb_flag = 0;
	}  
	break;
      
	// Greek keyboard      
	case LFUN_GREEK_TOGGLE:
	{
		greek_kb_flag = (greek_kb_flag) ? 0: 2;
		if (greek_kb_flag) {
			setMessage(N_("Math greek keyboard on"));
		} else {
			setMessage(N_("Math greek keyboard off"));
		}
	}
	break;
	
	case LFUN_MATH_DELIM:     
	case LFUN_INSERT_MATRIX:
#warning make into BufferView::func
	{ 	   
		if (owner->currentView()->available()) { 
		    if (owner->currentView()->
			open_new_inset(new InsetFormula(false))) {
			owner->currentView()->
			    the_locking_inset->LocalDispatch(action, argument);
		    }
		}
	}	   
	break;
	       
	case LFUN_INSERT_MATH:
	{
		owner->currentView()->InsertMathSymbol(argument);
	}
	break;
	
	case LFUN_MATH_DISPLAY:
#warning make into BufferView::func
	{
		if (owner->currentView()->available())
			owner->currentView()->open_new_inset(new InsetFormula(true));
		break;
	}
		    
	case LFUN_MATH_MACRO:
#warning make into BufferView::func
	{
		if (owner->currentView()->available()) {
			string s(argument);
		        if (s.empty())
		            setErrorMessage(N_("Missing argument"));
		        else {
			    string s1 = token(s, ' ', 1);
			    int na = s1.empty() ? 0: atoi(s1.c_str());
			    owner->currentView()->
			      open_new_inset(new InsetFormulaMacro(token(s, ' ', 0), na));
			}
		}
	}
	break;

	case LFUN_MATH_MODE:   // Open or create a math inset
#warning make into BufferView::func
	{
		
	    if (owner->currentView()->available() &&
		owner->currentView()->open_new_inset(new InsetFormula))
		setMessage(N_("Math editor mode"));
	}
	break;
	  
	case LFUN_TABULAR_MODE:   // Open or create a tabular inset
#warning make into BufferView::func
	{
		if (owner->currentView()->available() &&
		    owner->currentView()->currentBuffer())
			owner->getPopups().showFormTableNew();
	}
	break;

#warning make into BufferView::func
	case LFUN_ERT_MODE:   // Open or create a LaTeX inset
	{
	    if (owner->currentView()->available() &&
		owner->currentView()->currentBuffer()) {
		Buffer *buf = owner->currentView()->currentBuffer();
		if (owner->currentView()->open_new_inset(new InsetERT(buf)))
		    setMessage(N_("ERT mode"));
	    }
	}
	break;

	case LFUN_MARGIN_INSET:   // Open or create a margin note
	{
	    if (owner->currentView()->available() &&
		owner->currentView()->currentBuffer()) {
		Buffer *buf = owner->currentView()->currentBuffer();
		if (owner->currentView()->open_new_inset(new InsetMargin(buf)))
		    setMessage(N_("Margin mode"));
	    }
	}
	break;


	case LFUN_FOOT_INSET:   // Open or create a Foot inset
	{
	    if (owner->currentView()->available() &&
		owner->currentView()->currentBuffer()) {
		Buffer *buf = owner->currentView()->currentBuffer();
		if (owner->currentView()->open_new_inset(new InsetFoot(buf)))
		    setMessage(N_("Foot mode"));
	    }
	}
	break;

	  
	case LFUN_MATH_NUMBER:
	case LFUN_MATH_LIMITS:
	{
		setErrorMessage(N_("This is only allowed in math mode!"));
	}
	break;
	
	case LFUN_INSERT_CITATION:
	{   
		InsetCitation *new_inset = new InsetCitation();
		// ale970405
		// The note, if any, must be after the key, delimited
		// by a | so both key and remark can have spaces.
		if (argument) {
			string lsarg(argument);
			if (contains(lsarg, "|")) {
				new_inset->setContents(token(lsarg, '|', 0));
				new_inset->setOptions(token(lsarg, '|', 1));
			} else
				new_inset->setContents(lsarg);
			owner->currentView()->insertInset(new_inset);
		} else {
			owner->currentView()->insertInset(new_inset);
			new_inset->Edit(0, 0, 0, owner->currentView());
		}
	}
	break;
		    
	case LFUN_INSERT_BIBTEX:
	{   
		// ale970405+lasgoutt970425
		// The argument can be up to two tokens separated 
		// by a space. The first one is the bibstyle.
		string lsarg(argument);
		string bibstyle = token(lsarg, ' ', 1);
		if (bibstyle.empty())
			bibstyle = "plain";
		InsetBibtex *new_inset 
			= new InsetBibtex(token(lsarg, ' ', 0),
					  bibstyle,
					  owner->currentBuffer());
		
		owner->currentView()->insertInset(new_inset);
		if (lsarg.empty()) {
			new_inset->Edit(0, 0, 0, owner->currentView());
		}
	}
	break;
		
	// BibTeX data bases
	case LFUN_BIBDB_ADD:
	{
		InsetBibtex *inset =
			(InsetBibtex*)getInsetByCode(Inset::BIBTEX_CODE);
		if (inset) {
			inset->addDatabase(argument);
		}
	}
	break;
		    
	case LFUN_BIBDB_DEL:
	{
		InsetBibtex *inset =
			(InsetBibtex*)getInsetByCode(Inset::BIBTEX_CODE);
		if (inset) {
			inset->delDatabase(argument);
		}
	}
	break;
	
	case LFUN_BIBTEX_STYLE:
	{
		InsetBibtex *inset =
			(InsetBibtex*)getInsetByCode(Inset::BIBTEX_CODE);
		if (inset) {
			inset->setOptions(argument);
		}
	}
	break;
		
	case LFUN_INDEX_INSERT:
	{
		InsetIndex *new_inset = new InsetIndex();
		if (argument) {
			string lsarg(argument);
			new_inset->setContents(lsarg);
			owner->currentView()->insertInset(new_inset);
		} else {
			owner->currentView()->insertInset(new_inset);
			new_inset->Edit(0, 0, 0, owner->currentView());
		}
	}
	break;
		
	case LFUN_INDEX_PRINT:
	{
		Inset *new_inset = new
			InsetPrintIndex(owner->currentBuffer());
		owner->currentView()->insertInset(new_inset,
						    "Standard", true);
	}
	break;

	case LFUN_PARENTINSERT:
	{
		lyxerr[Debug::INFO] << "arg " << argument << '\n';
		Inset *new_inset = new InsetParent(argument, owner->currentBuffer());
		owner->currentView()->insertInset(new_inset,
						    "Standard", true);
	}
	break;

	case LFUN_CHILDINSERT:
	{
		Inset *new_inset = new InsetInclude(argument,owner->currentBuffer());
		owner->currentView()->insertInset(new_inset,
						  "Standard", true);
	}
	break;

	case LFUN_CHILDOPEN:
#warning make into BufferView::func
	{
		string filename = MakeAbsPath(argument, 
					       OnlyPath(owner->currentBuffer()->getFileName()));
		setMessage(N_("Opening child document ") +
			   MakeDisplayPath(filename) + "...");
		owner->currentView()->savePosition();
		owner->currentView()->setBuffer(
			bufferlist.loadLyXFile(filename));
	}
	break;

	case LFUN_INSERT_NOTE:
		owner->currentView()->note();
		break;
		
	case LFUN_INSERTFLOAT: 
	{
		LyXParagraph::footnote_kind kind;
		string arg = argument;
		if (arg == "footnote")
			{ kind = LyXParagraph::FOOTNOTE; }
		else if (arg == "margin")
			{ kind = LyXParagraph::MARGIN; }
		else if (arg == "figure")
			{ kind = LyXParagraph::FIG; }
		else if (arg == "table")
			{ kind = LyXParagraph::TAB; }
		else if (arg == "wide-fig")
			{ kind = LyXParagraph::WIDE_FIG; }
		else if (arg == "wide-tab")
			{ kind = LyXParagraph::WIDE_TAB; }
		else if (arg == "algorithm")
			{ kind = LyXParagraph::ALGORITHM; }
		else {
			setErrorMessage(N_("Unknown kind of footnote"));
			break;
		}
		owner->currentView()->text->InsertFootnoteEnvironment(kind);
		owner->currentView()->update(BufferView::ResetSelection
					     |BufferView::FitCursor
					     |BufferView::BufferChange);
	}
	break;
	
	case LFUN_TOGGLECURSORFOLLOW:
		cursor_follows_scrollbar = !cursor_follows_scrollbar;
		break;

	case LFUN_HELP_COPYRIGHT:
		owner->getPopups().showCopyright();
		break;

	case LFUN_HELP_CREDITS:
		owner->getPopups().showCredits();
		break;

	case LFUN_KMAP_OFF:		// keymap off
		owner->getIntl().KeyMapOn(false);
		break;
		
	case LFUN_KMAP_PRIM:	// primary keymap
		owner->getIntl().KeyMapPrim();
		break;
		
	case LFUN_KMAP_SEC:		// secondary keymap
		owner->getIntl().KeyMapSec();
		break;
		
	case LFUN_KMAP_TOGGLE:	// toggle keymap
		owner->getIntl().ToggleKeyMap();
		break;

	case LFUN_SELFINSERT:
		owner->currentView()->selfInsert(argument);
		break;

	case LFUN_UNKNOWN_ACTION:
	{
		if (owner->currentBuffer()->isReadonly()) {
			LyXBell();
			setErrorMessage(N_("Document is read only"));
			break;
		}
			 
		if (argument) {
			
			/* Automatically delete the currently selected
			 * text and replace it with what is being
			 * typed in now. Depends on lyxrc settings
			 * "auto_region_delete", which defaults to
			 * true (on). */
		
			if ( lyxrc->auto_region_delete ) {
				if (owner->currentView()->text->selection){
					owner->currentView()->text->CutSelection(false);
					owner->currentView()->update(BufferView::ResetSelection
								     |BufferView::FitCursor
								     |BufferView::BufferChange);
				}
			}
			
			owner->currentView()->beforeChange();
			for(char const *p = argument; *p; ++p) {
				if (greek_kb_flag) {
					if (!owner->currentView()->InsertGreek(*p))
						owner->getIntl().getTrans()->TranslateAndInsert(owner->currentView(), *p);
					
				} else
					owner->getIntl().getTrans()->TranslateAndInsert(owner->currentView(), *p);
			}
			owner->currentView()->smallUpdate(1);
			owner->currentView()->setUpdateTimer();

			owner->currentView()->text->sel_cursor =
				owner->currentView()->text->cursor;
			owner->currentView()->moveCursorUpdate(false);
			return string();
		} else {
			// why is an "Unknown action" with empty
			// argument even dispatched in the first
			// place? I`ll probably change that. (Lgb)
			LyXBell();
			setErrorMessage(N_("Unknown action"));
		}
		break;
	default:
		lyxerr << "A truly unknown func!\n";
		break;
	}
	} // end of switch
  exit_with_message:

	string res=getMessage();

	if (res.empty()) {
		if (!commandshortcut.empty()) {
			string newbuf = owner->getMiniBuffer()->GetText();
			if (newbuf != commandshortcut) {
				owner->getMiniBuffer()->Set(newbuf
							    + " " +
							    commandshortcut);
			}
		}
	} else {
		owner->getMiniBuffer()->Set(string(_(res.c_str()))
					    + " " + commandshortcut);
	}

	return getMessage();
}
	    

void LyXFunc::setupLocalKeymap()
{
	keyseq.stdmap = keyseq.curmap = toplevel_keymap;
	cancel_meta_seq.stdmap = cancel_meta_seq.curmap = toplevel_keymap;
}


void LyXFunc::MenuNew(bool fromTemplate)
{
	string fname, initpath = lyxrc->document_path;
	LyXFileDlg fileDlg;

	if (owner->currentView()->available()) {
		string trypath = owner->currentBuffer()->getFileDir();
		// If directory is writeable, use this as default.
		if (IsDirWriteable(trypath) == 1)
			initpath = trypath;
	}

	ProhibitInput();
	fileDlg.SetButton(0, _("Documents"), lyxrc->document_path);
	fileDlg.SetButton(1, _("Templates"), lyxrc->template_path);
	fname = fileDlg.Select(_("Enter Filename for new document"), 
			       initpath, "*.lyx", _("newfile"));
 	AllowInput();
	
	if (fname.empty()) {
		owner->getMiniBuffer()->Set(_("Canceled."));
		lyxerr[Debug::INFO] << "New Document Cancelled.\n";
		return;
	}
        
	// get absolute path of file and make sure the filename ends
	// with .lyx
	string s = MakeAbsPath(fname);
	if (!IsLyXFilename(s))
		s += ".lyx";

	// Check if the document already is open
	if (bufferlist.exists(s)){
		switch(AskConfirmation(_("Document is already open:"), 
				       MakeDisplayPath(s,50),
				       _("Do you want to close that document now?\n"
					 "('No' will just switch to the open version)")))
		{
		case 1: // Yes: close the document
			if (!bufferlist.close(bufferlist.getBuffer(s)))
				// If close is canceled, we cancel here too.
				return;
			break;
		case 2: // No: switch to the open document
			owner->currentView()->setBuffer(bufferlist.getBuffer(s));
			return;
		case 3: // Cancel: Do nothing
			owner->getMiniBuffer()->Set(_("Canceled."));
			return;
		}
	}
        
	// Check whether the file already exists
	FilePtr myfile(s, FilePtr::read);
	if (myfile() &&
	    AskQuestion(_("File already exists:"), 
			MakeDisplayPath(s,50),
			_("Do you want to open the document?"))) {
		// loads document
		owner->getMiniBuffer()->Set(_("Opening document"), 
					    MakeDisplayPath(s), "...");
		owner->currentView()->setBuffer(bufferlist.loadLyXFile(s));
		owner->getMiniBuffer()->Set(_("Document"),
					    MakeDisplayPath(s),
					    _("opened."));
		return;
	}

	// The template stuff
	string templname;
	if (fromTemplate) {
		ProhibitInput();
		fname = fileDlg.Select(_("Choose template"),
				       lyxrc->template_path,
				       "*.lyx");
                templname = fname;
		AllowInput();
	}
  
	// find a free buffer
	lyxerr[Debug::INFO] << "Find a free buffer.\n";
	owner->currentView()->setBuffer(bufferlist.newFile(s,templname));
}


void LyXFunc::MenuOpen()
{
	string initpath = lyxrc->document_path;
	LyXFileDlg fileDlg;
  
	if (owner->currentView()->available()) {
		string trypath = owner->currentBuffer()->getFileDir();
		// If directory is writeable, use this as default.
		if (IsDirWriteable(trypath) == 1)
			initpath = trypath;
	}

	// launches dialog
	ProhibitInput();
	fileDlg.SetButton(0, _("Documents"), lyxrc->document_path);
	fileDlg.SetButton(1, _("Examples"), 
			  AddPath(system_lyxdir, "examples"));
	string filename = fileDlg.Select(_("Select Document to Open"),
					  initpath, "*.lyx");
	AllowInput();
 
	// check selected filename
	if (filename.empty()) {
		owner->getMiniBuffer()->Set(_("Canceled."));
		return;
	}

	// get absolute path of file and make sure the filename ends
	// with .lyx
	filename = MakeAbsPath(filename);
	if (!IsLyXFilename(filename))
		filename += ".lyx";

	// loads document
	owner->getMiniBuffer()->Set(_("Opening document"),
				    MakeDisplayPath(filename), "...");
	Buffer * openbuf = bufferlist.loadLyXFile(filename);
	if (openbuf) {
		owner->currentView()->setBuffer(openbuf);
		owner->getMiniBuffer()->Set(_("Document"),
					    MakeDisplayPath(filename),
					    _("opened."));
	} else {
		owner->getMiniBuffer()->Set(_("Could not open document"),
					    MakeDisplayPath(filename));
	}
}


void LyXFunc::doImportASCII()
{
	string initpath = lyxrc->document_path;
	LyXFileDlg fileDlg;
  
	if (owner->currentView()->available()) {
		string trypath = owner->currentBuffer()->getFileDir();
		// If directory is writeable, use this as default.
		if (IsDirWriteable(trypath) == 1)
			initpath = trypath;
	}

	// launches dialog
	ProhibitInput();
	fileDlg.SetButton(0, _("Documents"), lyxrc->document_path);
	fileDlg.SetButton(1, _("Examples"), 
			  AddPath(system_lyxdir, "examples"));
	string filename = fileDlg.Select(_("Select ASCII file to Import"),
					  initpath, "*.txt");
	AllowInput();
 
	// check selected filename
	if (filename.empty()) {
		owner->getMiniBuffer()->Set(_("Canceled."));
		return;
	}

	// get absolute path of file
	filename = MakeAbsPath(filename);

	string s = ChangeExtension(filename, ".lyx", false);

	// Check if the document already is open
	if (bufferlist.exists(s)){
		switch(AskConfirmation(_("Document is already open:"), 
				       MakeDisplayPath(s,50),
				       _("Do you want to close that document now?\n"
					 "('No' will just switch to the open version)")))
		{
		case 1: // Yes: close the document
			if (!bufferlist.close(bufferlist.getBuffer(s)))
				// If close is canceled, we cancel here too.
				return;
			break;
		case 2: // No: switch to the open document
			owner->currentView()->setBuffer(bufferlist.getBuffer(s));
			return;
		case 3: // Cancel: Do nothing
			owner->getMiniBuffer()->Set(_("Canceled."));
			return;
		}
	}

	// Check if a LyX document by the same root exists in filesystem
	FileInfo f(s, true);
	if (f.exist() && !AskQuestion(_("A document by the name"), 
				      MakeDisplayPath(s),
				      _("already exists. Overwrite?"))) {
		owner->getMiniBuffer()->Set(_("Canceled."));
		return;
	}

	owner->currentView()->setBuffer(bufferlist.newFile(s,string()));
	owner->getMiniBuffer()->Set(_("Importing ASCII file"),
				    MakeDisplayPath(filename), "...");
	// Insert ASCII file
	owner->currentView()->insertAsciiFile(filename,true);
	owner->getMiniBuffer()->Set(_("ASCII file "),
				    MakeDisplayPath(filename),
				    _("imported."));
}


void LyXFunc::doImportLaTeX()
{
	string initpath = lyxrc->document_path;
	LyXFileDlg fileDlg;
  
	if (owner->currentView()->available()) {
		string trypath = owner->currentBuffer()->getFileDir();
		// If directory is writeable, use this as default.
		if (IsDirWriteable(trypath) == 1)
			initpath = trypath;
	}

	// launches dialog
	ProhibitInput();
	fileDlg.SetButton(0, _("Documents"), lyxrc->document_path);
	fileDlg.SetButton(1, _("Examples"), 
			  AddPath(system_lyxdir, "examples"));
	string filename = fileDlg.Select(_("Select LaTeX file to Import"),
					  initpath, "*.tex");
	AllowInput();
 
	// check selected filename
	if (filename.empty()) {
		owner->getMiniBuffer()->Set(_("Canceled."));
		return;
	}

	// get absolute path of file
	filename = MakeAbsPath(filename);

	// Check if the document already is open
	string LyXfilename = ChangeExtension(filename, ".lyx", false);
	if (bufferlist.exists(LyXfilename)){
		switch(AskConfirmation(_("Document is already open:"), 
				       MakeDisplayPath(LyXfilename,50),
				       _("Do you want to close that document now?\n"
					 "('No' will just switch to the open version)")))
		{
		case 1: // Yes: close the document
			if (!bufferlist.close(bufferlist.getBuffer(LyXfilename)))
				// If close is canceled, we cancel here too.
				return;
			break;
		case 2: // No: switch to the open document
			owner->currentView()->setBuffer(
				bufferlist.getBuffer(LyXfilename));
			return;
		case 3: // Cancel: Do nothing
			owner->getMiniBuffer()->Set(_("Canceled."));
			return;
		}
	}

	// Check if a LyX document by the same root exists in filesystem
	FileInfo f(LyXfilename, true);
	if (f.exist() && !AskQuestion(_("A document by the name"), 
				      MakeDisplayPath(LyXfilename),
				      _("already exists. Overwrite?"))) {
		owner->getMiniBuffer()->Set(_("Canceled."));
		return;
	}

	// loads document
	owner->getMiniBuffer()->Set(_("Importing LaTeX file"),
				    MakeDisplayPath(filename), "...");
	ImportLaTeX myImport(filename);
	Buffer * openbuf = myImport.run();
	if (openbuf) {
		owner->currentView()->setBuffer(openbuf);
		owner->getMiniBuffer()->Set(_("LateX file "),
					    MakeDisplayPath(filename),
					    _("imported."));
	} else {
		owner->getMiniBuffer()->Set(_("Could not import LaTeX file"),
					    MakeDisplayPath(filename));
	}
}
 

void LyXFunc::MenuInsertLyXFile(string const & filen)
{
	string filename = filen;

	if (filename.empty()) {
		// Launch a file browser
		string initpath = lyxrc->document_path;
		LyXFileDlg fileDlg;

		if (owner->currentView()->available()) {
			string trypath = owner->currentBuffer()->getFileDir();
			// If directory is writeable, use this as default.
			if (IsDirWriteable(trypath) == 1)
				initpath = trypath;
		}

		// launches dialog
		ProhibitInput();
		fileDlg.SetButton(0, _("Documents"), lyxrc->document_path);
		fileDlg.SetButton(1, _("Examples"), 
				  AddPath(system_lyxdir, "examples"));
		filename = fileDlg.Select(_("Select Document to Insert"),
					  initpath, "*.lyx");
		AllowInput();

		// check selected filename
		if (filename.empty()) {
			owner->getMiniBuffer()->Set(_("Canceled."));
			return;
		}
	} 

	// get absolute path of file and make sure the filename ends
	// with .lyx
	filename = MakeAbsPath(filename);
	if (!IsLyXFilename(filename))
		filename += ".lyx";

	// Inserts document
	owner->getMiniBuffer()->Set(_("Inserting document"),
				    MakeDisplayPath(filename), "...");
	bool res = owner->currentBuffer()->insertLyXFile(filename);
	if (res) {
		owner->getMiniBuffer()->Set(_("Document"),
					    MakeDisplayPath(filename),
					    _("inserted."));
	} else {
		owner->getMiniBuffer()->Set(_("Could not insert document"),
					    MakeDisplayPath(filename));
	}
}


void LyXFunc::reloadBuffer()
#warning make into BufferView::func
{
	string fn = owner->currentBuffer()->getFileName();
	if (bufferlist.close(owner->currentBuffer()))
		owner->currentView()->setBuffer(bufferlist.loadLyXFile(fn));
}


void LyXFunc::CloseBuffer()
#warning make into BufferView::func
{
	if (bufferlist.close(owner->currentBuffer()) && !quitting) {
		if (bufferlist.empty()) {
			// need this otherwise SEGV may occur while trying to
			// set variables that don't exist
			// since there's no current buffer
			CloseAllBufferRelatedPopups();
			// temporary until signal is built.
			owner->getPopups().hideBufferDependent();
		} else {
			owner->currentView()->setBuffer(bufferlist.first());
		}
	}
}


Inset* LyXFunc::getInsetByCode(Inset::Code code)
{
	bool found = false;
	Inset* inset = 0;
	LyXCursor cursor = owner->currentView()->text->cursor;
	int pos = cursor.pos;
	LyXParagraph *par = cursor.par;
	
	while (par && !found) {
		while ((inset = par->ReturnNextInsetPointer(pos))){
			if (inset->LyxCode()==code) {
				found = true;
				break;
			}
			++pos;
		} 
		par = par->next;
	}
	return (found) ? inset: 0;
}


// Each "owner" should have it's own message method. lyxview and
// the minibuffer would use the minibuffer, but lyxserver would
// send an ERROR signal to its client.  Alejandro 970603
// This func is bit problematic when it comes to NLS, to make the
// lyx servers client be language indepenent we must not translate
// strings sent to this func.
void LyXFunc::setErrorMessage(string const &m) 
{
	dispatch_buffer = m;
	errorstat = true;
}


void LyXFunc::setMessage(string const & m)
{
	dispatch_buffer = m;
}
