/* This file is part of
* ======================================================
* 
*           LyX, The Document Processor
* 	 
*	    Copyright (C) 1995 Matthias Ettrich
*           Copyright (C) 1995-1998 The LyX Team. 
*
*======================================================*/

#include <config.h>

#ifdef __GNUG__
#pragma implementation "lyxparagraph.h"
#endif

#include "lyxparagraph.h"
#include "lyxrc.h"
#include "textclasslist.h"
#include "tex-strings.h"
#include "bufferparams.h"
#include "FileInfo.h"
#include "debug.h"
#include "LaTeXFeatures.h"
#include "insetinclude.h"
#include "filetools.h"
#include "texrow.h"
#include "LAssert.h"
#include "BufferView.h"
#include "buffer.h"

// 	$Id: paragraph.C,v 1.28 1998/10/22 16:56:29 larsbj Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: paragraph.C,v 1.28 1998/10/22 16:56:29 larsbj Exp $";
#endif /* lint */

// this should be fixed/removed
extern unsigned char getCurrentTextClass(Buffer*);

int tex_code_break_column;
// this is a bad idea, but how can LyXParagraph find its buffer to get
// parameters? (JMarc)
// I think we need to pass it as a parameter. (Lgb)
extern BufferView *current_view;
extern LyXRC *lyxrc;


// ale970405
extern string bibitemWidthest();

// This is a minibuffer
static char minibuffer_char;
static LyXFont minibuffer_font;
static Inset *minibuffer_inset;


// The counter for the paragraph id's 
unsigned int LyXParagraph::paragraph_id = 0;


LyXParagraph::LyXParagraph()
{
	for (int i=0; i<10; ++i) counter[i] = 0;
	start_of_appendix = false;
	appendix = false;
	enumdepth = 0;
	itemdepth = 0;
	next = 0;
	previous = 0;
	child = 0;
	footnoteflag = LyXParagraph::NO_FOOTNOTE;
	id = paragraph_id++;
        bibkey = 0; // ale970302
	
	Clear();
}


// this konstruktor inserts the new paragraph in a list */ 
LyXParagraph::LyXParagraph(LyXParagraph *par)
{
	for (int i=0; i<10; ++i) counter[i] = 0;
	appendix = false;
	enumdepth = 0;
	itemdepth = 0;
	next = par->next;
	if (next)
		next->previous = this;
	previous = par;
	previous->next = this;
	child = 0;
	footnoteflag = LyXParagraph::NO_FOOTNOTE;
	footnotekind = LyXParagraph::FOOTNOTE;
	id = paragraph_id++;

        bibkey = 0; // ale970302
	
	Clear();
}


// The destructor removes the new paragraph from the list.
LyXParagraph::~LyXParagraph()
{
	if (previous)
		previous->next = next;
	if (next)
		next->previous = previous;

	if (child) delete child;
        // ale970302
	if (bibkey)
	        delete bibkey;
}


/// Used by the spellchecker
bool LyXParagraph::IsLetter(int pos){
	unsigned char c = GetChar(pos);
	if (IsLetterChar(c))
		return true;
	// We want to pass the ' and escape chars to ispell
	string extra = lyxrc->isp_esc_chars + '\'';
	char ch[2];
	ch[0] = c;
	ch[1] = 0;
	return contains(extra, ch);
}


void LyXParagraph::writeFile(FILE *file, BufferParams &params,
			     char footflag, char dth)
{
	LyXFont font1, font2;

	if (footnoteflag != LyXParagraph::NO_FOOTNOTE
	    || !previous
	    || previous->footnoteflag == LyXParagraph::NO_FOOTNOTE){
		
		// The beginning or the end of a footnote environment? 
		if (footflag != footnoteflag) {
			footflag = footnoteflag;
			if (footflag) {
				fprintf(file, "\\begin_float %s\n", 
					string_footnotekinds[footnotekind]);
			} else {
				fprintf(file, "\\end_float\n");
			}
		}

		// The beginning or end of a deeper (i.e. nested) area?
		if (dth != depth) {
			if (depth > dth) {
				while (depth > dth) {
					fprintf(file, "\\begin_deeper\n");
					++dth;
				}
			} else {
				while (depth < dth) {
					fprintf(file, "\\end_deeper\n");
					--dth;
				}
			}
		}

		// First write the layout
		// ensure that there is a space before the layout.
		fprintf(file, "\\begin_paragraph %s\n",
			textclasslist.nameOfLayout(params.textclass,layout).c_str());
		pparams.writeFile(file, params, footflag, dth);
		
		// The labelwidth string used in lists.
		// Should perhaps be changed to be a real vspace.
		// pparams candidate
		if (!labelwidthstring.empty())
			fprintf(file, "\\labelwidthstring %s\n",
				labelwidthstring.c_str());

		// The extra paramters
		// pparams candidate
                if (pextra_type != PEXTRA_NONE) {
                        fprintf(file, "\\pextra_type %d", pextra_type);
                        if (pextra_type == PEXTRA_MINIPAGE) {
				fprintf(file, " \\pextra_alignment %d",
					pextra_alignment);
				if (pextra_hfill)
					fprintf(file, " \\pextra_hfill %d",
						pextra_hfill);
				if (pextra_start_minipage)
					fprintf(file, " \\pextra_start_minipage %d",
						pextra_start_minipage);
                        }
                        if (!pextra_width.empty()) {
				fprintf(file, " \\pextra_width %s",
                                        VSpace(pextra_width).asLyXCommand().c_str());
                        } else if (!pextra_widthp.empty()) {
				fprintf(file, " \\pextra_widthp %s",pextra_widthp.c_str());
                        }
                }
	} else {
   		// Dummy layout. This means that a footnote ended
		fprintf(file, "\\end_float\n");
		footflag = LyXParagraph::NO_FOOTNOTE;
	}
	// bibitem  ale970302
	if (bibkey)
		bibkey->Write(file);

	font1 = LyXFont(LyXFont::ALL_INHERIT);

	int column = 0;
	char c = 0;
	for (int i = 0; i < last(); ++i) {
		// Write font changes
		font2 = GetFontSettings(i);
		if (font2 != font1) {
			font2.lyxWriteChanges(font1, file);
			column = 0;
			font1 = font2;
		}

		c = GetChar(i);
		// A newline before tags
		if (column > 0 &&
		    (c == LYX_META_INSET ||
		     c == LYX_META_NEWLINE ||
		     c == '\\')) {
			fprintf(file, "\n");
			column = 0;
		}
			
		switch (c) {
		case LYX_META_INSET: {
			Inset *inset = GetInset(i);
			if (inset) {
				inset->Write(file);
				column = 0;
			}
		}
			break;
		case LYX_META_NEWLINE: 
			fprintf(file, "\\newline\n");
			column = 0;
			break;
		case '\\': 
			fprintf(file, "\\backslash\n");
			column = 0;
			break;
		default:
			if (column > 65 && c==' ') {
				fprintf(file, "\n");
				column = 0;
			}
			// this check is to amend a bug. LyX sometimes
			// inserts '\0' this could cause problems.
			if (c != '\0')
				fprintf(file, "%c", c);
			else
				lyxerr << "ERROR (LyXParagraph::writeFile):"
					" '\\0' char in structure.\n";
			++column;
			break;
		}
	}

	// A newline if the last c was not a tag.
	if (column > 0 &&
	    (c != LYX_META_INSET &&
	     c != LYX_META_NEWLINE &&
	     c != '\\')) {
		fprintf(file, "\n");
		column = 0;
	}

	fprintf(file, "\\end_paragraph\n");
	
	// now write the next paragraph
	if (next)
		next->writeFile(file, params, footflag, dth);
}


void LyXParagraph::validate(LaTeXFeatures &features)
{
	if (lyxerr.debugging(Debug::LATEX))
		lyxerr << "Validating paragraph: " << (void*)this << '\n';
	
	// this will be useful later
	LyXLayout const *layout =
		textclasslist.style(getCurrentTextClass(current_view->currentBuffer()), 
			       GetLayout());
	
	// check the params.
	if (pparams.line_top || pparams.line_bottom)
		features.lyxline = true;
	
	// then the layouts
	features.layout[GetLayout()] = true;

	// then the fonts
	for (FontTable::const_iterator ci = fonttable.begin();
	    ci != fonttable.end(); ++ci) {
		if (ci->font.noun() == LyXFont::ON) {
			lyxerr[Debug::LATEX] << "font.noun: "
					     << int(ci->font.noun()) << '\n';
			features.noun = true;
			lyxerr[Debug::LATEX] << "Noun enabled. Font: "
					     << ci->font.stateText() << '\n';
		}
		switch (ci->font.color()) {
		case LColor::none: 
		case LColor::inherit:
		case LColor::ignore:
			break;
		default:
			features.color = true;
			lyxerr[Debug::LATEX] << "Color enabled. Font: "
					     << ci->font.stateText() << '\n';
		}
	}
	
	// then the insets
	for (InsetTable::const_iterator ici = insettable.begin();
	     ici != insettable.end(); ++ici) {
		// This test looks a bit weird. Should we ever have
		// insettable items that has a inset == 0?
		if (ici->inset)
			ici->inset->Validate(features);
	}
        if (pextra_type == PEXTRA_INDENT)
                features.LyXParagraphIndent = true;
        if (pextra_type == PEXTRA_FLOATFLT)
                features.floatflt = true;
        if (layout->needprotect 
	    && next && next->footnoteflag != LyXParagraph::NO_FOOTNOTE)
		features.NeedLyXFootnoteCode = true;
        if ((current_view->currentBuffer()->params.paragraph_separation == LYX_PARSEP_INDENT) &&
            (pextra_type == PEXTRA_MINIPAGE))
		features.NeedLyXMinipageIndent = true;
	if (next)
		next->validate(features);
	return;
}


// First few functions needed for cut and paste and paragraph breaking.
void LyXParagraph::CopyIntoMinibuffer(int pos) const
{
	minibuffer_char = GetChar(pos);
	minibuffer_font = GetFontSettings(pos);
	if (minibuffer_char == LYX_META_INSET) {
		if (GetInset(pos)) {
			minibuffer_inset = GetInset(pos)->Clone();
		} else {
			minibuffer_inset = 0;
			minibuffer_char = ' ';
			// This reflects what GetInset() does (ARRae)
		}
	}
}


void LyXParagraph::CutIntoMinibuffer(int pos)
{
	minibuffer_char = GetChar(pos);
	minibuffer_font = GetFontSettings(pos);
	if (minibuffer_char == LYX_META_INSET) {
		if (GetInset(pos)) {
			minibuffer_inset = GetInset(pos);
		} else {
			minibuffer_inset = 0;
			minibuffer_char = ' ';
			// This reflects what GetInset() does (ARRae)
		}

		/* This is a little hack since I want exactly the inset,
		   not just a clone. Otherwise the inset would be deleted
		   when calling Erase(pos) */
		/* And it's only useful inside the if{} above 
		   since the else{} means its not there (ARRae) */

		// Find the entry.
		for (InsetTable::iterator it= insettable.begin();
		     it != insettable.end(); ++it) {
			if (it->pos == pos) {
				it->inset = 0;
				break;
			}
		}
	}
}


void LyXParagraph::InsertFromMinibuffer(int pos)
{
	InsertChar(pos, minibuffer_char);
	SetFont(pos, minibuffer_font);
	if (minibuffer_char == LYX_META_INSET)
		InsertInset(pos, minibuffer_inset);
}

/* end of minibuffer */ 



void LyXParagraph::Clear()
{
	depth = 0;

        pextra_type = PEXTRA_NONE;
        pextra_width.erase();
        pextra_widthp.erase();
        pextra_alignment = MINIPAGE_ALIGN_TOP;
        pextra_hfill = false;
        pextra_start_minipage = false;

        labelstring.erase();
	labelwidthstring.erase();
	layout = 0;
	start_of_appendix = false;
}


void LyXParagraph::Erase(int pos)
{
	/* > because last is the next unused position, and you can 
	 * use it if you want  */
	if (pos > last()) {
		if (next && next->footnoteflag == LyXParagraph::CLOSED_FOOTNOTE) 
			NextAfterFootnote()->Erase(pos - last() - 1);
		else 
			lyxerr << "ERROR (LyXParagraph::Erase):"
				" position does not exist.\n";
		return;
	}
	if (pos < last()) { // last is free for insertation, but should be empty
      
		// If it is an inset, delete the inset entry.
		if (textvec[pos] == LYX_META_INSET) {
			// Find the entry
			for (InsetTable::iterator it = insettable.begin();
			     it != insettable.end(); ++it) {
				if (it->pos == pos) {
					insettable.erase(it);
					break;
				}
			}
		}

		textvec.erase(textvec.begin() + pos);

		// Erase entries in the tables.
		for (FontTable::iterator fit=fonttable.begin();
		    fit != fonttable.end(); ++fit) {
			if (pos >= fit->pos && pos<= fit->pos_end) {
				if (fit->pos == fit->pos_end)
					fonttable.erase(fit);
				break;
			}
		}
	       
		// Update positions in the fonttable.
		for (FontTable::iterator fti = fonttable.begin();
		     fti != fonttable.end(); ++fti) {
			if (fti->pos > pos)
				fti->pos--;
			if (fti->pos_end >= pos)
				fti->pos_end--;
		}
		
		// Update positions in the insettable.
		for (InsetTable::iterator it= insettable.begin();
		     it != insettable.end(); ++it) {
			if (it->pos > pos)
				it->pos--;
		}
      
	} else {
		lyxerr << "ERROR (LyXParagraph::Erase): "
			"can't erase non-existant char.\n";
	}
}


void LyXParagraph::InsertChar(int pos, char c)
{
	Assert(pos >= 0);
	
	/* > because last is the next unused position, and you can 
	 * use it if you want  */
	if (pos > last()) {
		if (next && next->footnoteflag == LyXParagraph::CLOSED_FOOTNOTE) 
			NextAfterFootnote()->InsertChar(pos - last() - 1, c);
		else 
			lyxerr << "ERROR (LyXParagraph::InsertChar): "
				"position does not exist.\n";
		return;
	} else if (pos == last()) {
		textvec.push_back(c);
	} else {
		textvec.insert(textvec.begin() + pos, c);
	}
   
	// update the font table
	for (FontTable::iterator it = fonttable.begin();
	     it != fonttable.end(); ++it) {
		if (it->pos >= pos)
			it->pos++;
		if (it->pos_end >= pos)
			it->pos_end++;
	}
	
	// update the inset table
	for (InsetTable::iterator tit = insettable.begin();
	     tit != insettable.end(); ++tit) {
		if (tit->pos >= pos)
			tit->pos++;
	}
}


void LyXParagraph::InsertInset(int pos, Inset *inset)
{
	/* > because last is the next unused position, and you can 
	 * use it if you want  */
	if (pos > last()) {
		if (next && next->footnoteflag == LyXParagraph::CLOSED_FOOTNOTE) 
			NextAfterFootnote()->InsertInset(pos - last() - 1, inset);
		else 
			lyxerr << "ERROR (LyXParagraph::InsertInset): " 
				"position does not exist: " << pos << '\n';
		return;
	}

	if (textvec[pos] != LYX_META_INSET) {
		lyxerr << "ERROR (LyXParagraph::InsertInset): "
			"there is no LYX_META_INSET\n";
		return;
	}

	if (inset) {
		// Add a new entry in the inset table.
		InsetEntry tmpi;
		tmpi.pos = pos;
		tmpi.inset = inset;
		insettable.insert(insettable.begin(), tmpi);
	}
}


Inset* LyXParagraph::GetInset(int pos) const
{
	if (pos >= last()) {
		if (next && next->footnoteflag == LyXParagraph::CLOSED_FOOTNOTE) 
			return NextAfterFootnote()->GetInset(pos - last() - 1);
		else { 
		        lyxerr << "ERROR (LyXParagraph::GetInset): "
				"position does not exist: "
			       << pos << '\n';
		}
		return 0;
	}

	// find the inset
	for (InsetTable::const_iterator ci = insettable.begin();
	     ci !=insettable.end(); ++ci) {
		if (ci->pos == pos) {
			return ci->inset;
		}
	}
	// not found
	lyxerr << "ERROR (LyXParagraph::GetInset): "
		"Inset does not exist: " << pos << '\n';
	//textvec[pos] = ' '; /// WHY!!! does this set the pos to ' '????
	// Did this commenting out introduce a bug? So far I have not
	// seen any, please enlighten me. (Lgb)
	// My guess is that since the inset does not exist, we might
	// as well replace it with a space to prevent crashes. (Asger)
	return 0;
}


// Gets uninstantiated font setting at position.
// Optimized after profiling. (Asger)
LyXFont LyXParagraph::GetFontSettings(int pos) const
{
	if (pos < last()) {
		for (FontTable::const_iterator ci = fonttable.begin();
		     ci != fonttable.end(); ++ci) {
			if (pos >= ci->pos && pos <= ci->pos_end)
				return ci->font;
		}
	}

	/* > because last is the next unused position, and you can 
	 * use it if you want  */
	else if (pos > last()) {
		if (next && next->footnoteflag == LyXParagraph::CLOSED_FOOTNOTE) 
			return NextAfterFootnote()->GetFontSettings(pos - last() - 1);
		else {
			// Why is it an error to ask for the font of a
			// position that does not exist? Would it be
			// enough for this to be enable on debug?
			// We want strict error checking, but it's ok to only
			// have it when debugging. (Asger)
			lyxerr << "ERROR (LyXParagraph::GetFontSettings): "
				"position does not exist. " << pos << '\n';
		}
	} else if (pos) {
		return GetFontSettings(pos - 1);
	}
	return LyXFont(LyXFont::ALL_INHERIT);
}


// Gets the fully instantiated font at a given position in a paragraph
// This is basically the same function as LyXText::GetFont() in text2.C.
// The difference is that this one is used for generating the LaTeX file,
// and thus cosmetic "improvements" are disallowed: This has to deliver
// the true picture of the buffer. (Asger)
// If position is -1, we get the layout font of the paragraph.
// If position is -2, we get the font of the manual label of the paragraph.
LyXFont LyXParagraph::getFont(int pos)
{
	LyXFont tmpfont;
	LyXLayout const *layout = textclasslist.style(getCurrentTextClass(current_view->currentBuffer()), 
					   GetLayout());
   
	int main_body=0;
	if (layout->labeltype == LABEL_MANUAL)
		main_body = BeginningOfMainBody();

	if (pos >= 0){
		LyXFont layoutfont;
		if (pos < main_body)
			layoutfont = layout->labelfont;
		else
			layoutfont = layout->font;
		tmpfont = GetFontSettings(pos);
		tmpfont.realize(layoutfont);
	} else {
		// process layoutfont for pos == -1 and labelfont for pos < -1
		if (pos == -1)
			tmpfont = layout->font;
		else
			tmpfont = layout->labelfont;
	}

	// check for environment font information
	char par_depth = GetDepth();
	LyXParagraph * par = this;
	while (par && par_depth && !tmpfont.resolved()) {
		par = par->DepthHook(par_depth - 1);
		if (par) {
			tmpfont.realize(textclasslist.
					style(getCurrentTextClass(current_view->currentBuffer()),
					      par->GetLayout())->font);
			par_depth = par->GetDepth();
		}
	}

	tmpfont.realize(textclasslist.textClass(getCurrentTextClass(current_view->currentBuffer()))->defaultfont);
	return tmpfont;
}


/// Returns the height of the highest font in range
LyXFont::FONT_SIZE
LyXParagraph::HighestFontInRange(int startpos, int endpos) const
{
	LyXFont::FONT_SIZE maxsize = LyXFont::SIZE_TINY;

	for (FontTable::const_iterator ci = fonttable.begin();
	     ci != fonttable.end(); ++ci) {
		if (startpos <= ci->pos_end && endpos >= ci->pos) {
			LyXFont::FONT_SIZE size = ci->font.size();
			if (size > maxsize && size <= LyXFont::SIZE_HUGER)
				maxsize = size;
		}
	}
	return maxsize;
}


char LyXParagraph::GetChar(int pos) const
{
	// pos < 0 that is a bug!
	Assert(pos >= 0);
	
	char retval = 'F';
	if (pos < last()) {
		retval = textvec[pos];
	}

	/* > because last is the next unused position, and you can 
	 * use it if you want  */
	else if (pos > last()) {
		if (next && next->footnoteflag != LyXParagraph::NO_FOOTNOTE) 
			retval = NextAfterFootnote()->GetChar(pos - last() - 1);
		else 
			lyxerr << "ERROR (LyXParagraph::GetChar): "
				"position does not exist.\n";
	} else { // pos==last
		// We should have a footnote environment.
		if (next && next->footnoteflag != LyXParagraph::NO_FOOTNOTE) {
			switch (next->footnotekind) {
			case LyXParagraph::FOOTNOTE:
				retval = LYX_META_FOOTNOTE;
				break;
			case LyXParagraph::MARGIN:
				retval = LYX_META_MARGIN;
				break;
			case LyXParagraph::FIG:
			case LyXParagraph::WIDE_FIG:
				retval = LYX_META_FIG;
				break;
			case LyXParagraph::TAB:
			case LyXParagraph::WIDE_TAB:
				retval = LYX_META_TAB;
				break;
			case LyXParagraph::ALGORITHM:
				retval = LYX_META_ALGORITHM;
				break;
			}
		}
	}
	return retval;
}


int LyXParagraph::Last()
{
	if (next && next->footnoteflag == LyXParagraph::CLOSED_FOOTNOTE)
		return last() + NextAfterFootnote()->Last() + 1;   /* the 1 is the symbol
								  * for the footnote */
	else
		return last();
}


LyXParagraph *LyXParagraph::ParFromPos(int pos)
{
	/* > because last is the next unused position, and you can 
	 * use it if you want  */
	if (pos > last()) {
		if (next && next->footnoteflag == LyXParagraph::CLOSED_FOOTNOTE) 
			return NextAfterFootnote()->ParFromPos(pos - last() - 1);
		else 
			lyxerr << "ERROR (LyXParagraph::ParFromPos): "
				"position does not exist.\n";
		return this;
	} else
		return this;
}


int LyXParagraph::PositionInParFromPos(int pos)
{
	/* > because last is the next unused position, and you can 
	 * use it if you want  */
	if (pos > last()) {
		if (next && next->footnoteflag == LyXParagraph::CLOSED_FOOTNOTE) 
			return NextAfterFootnote()->PositionInParFromPos(pos - last() - 1);
		else 
			lyxerr << "ERROR(LyXParagraph::PositionInParFromPos): "
				"position does not exist.\n";
		return pos;
	} else
		return pos;
}


void LyXParagraph::SetFont(int pos, LyXFont const & font)
{
	/* > because last is the next unused position, and you can 
	 * use it if you want  */
	if (pos > last()) {
		if (next && next->footnoteflag == LyXParagraph::CLOSED_FOOTNOTE) {
			NextAfterFootnote()->SetFont(pos - last() - 1, font);
		} else {
			lyxerr << "ERROR (LyXParagraph::SetFont): "
				"position does not exist.\n";
		}
		return;
	}

	LyXFont patternfont(LyXFont::ALL_INHERIT);

// First, reduce font against layout/label font
// Update: The SetCharFont() routine in text2.C already reduces font, so
// we don't need to do that here. (Asger)
// No need to simplify this because it will disappear in a new kernel. (Asger)

	// Next search font table
	bool found = false;
	FontTable::iterator it = fonttable.begin();
	for (; it != fonttable.end(); ++it) {
		if (pos >= it->pos && pos <= it->pos_end) {
			found = true;
			break;
		}
	}
	
	if (!found) {
		// If we did not find a font entry, but if the font at hand
		// is the same as default, we just forget it.
		if (font == patternfont)
			return;

		// Ok, we did not find a font entry. But maybe there is exactly
		// the needed font entry one position left
		FontTable::iterator ti = fonttable.begin();
		for (; ti != fonttable.end(); ++ti) {
			if (pos -1 >= ti->pos && pos -1 <= ti->pos_end) {
				found = true;
				break;
			}
		}
		if (found) {
			// ok there is one. maybe it is exactly the needed font
			if (ti->font == font) {
				// put the position under the font
				ti->pos_end++;
				return;
			}
		}
		/* Add a new entry in the 
		 * fonttable for the position */
		FontEntry tmp;
		tmp.pos = pos;
		tmp.pos_end = pos;
		//tmp.font = patternfont;
		tmp.font = font;
		fonttable.insert(fonttable.begin(), tmp);
	} else 
		/* we found a font entry. maybe we have to split it and create
		 * a new one */ 
		
		if (it->pos != it->pos_end) {  // more than one character
			
			if (pos == it->pos) {
				// maybe we could enlarge the left fonttable
				
				found = false;
				FontTable::iterator ti = fonttable.begin();
				for (; ti != fonttable.end(); ++ti) {
					if (pos -1 >= ti->pos
					    && pos -1 <= ti->pos_end) {
						found = true;
						break;
					}
				}
				// Is there one, and is it exactly
				// the needed font?
				if (found && ti->font == font) {
					/* put the position under the font */ 
					ti->pos_end++;
					it->pos++;
					return;
				}
				
				/* Add a new entry in the 
				 * fonttable for the position */
				FontEntry tmp;
				tmp.pos = pos + 1;
				tmp.pos_end = it->pos_end;
				tmp.font = it->font;
				it->pos_end = pos;
				fonttable.insert(fonttable.begin(), tmp);
			} else if (pos == it->pos_end) {
				/* Add a new entry in the 
				 * fonttable for the position */
				FontEntry tmp;
				tmp.pos = it->pos;
				tmp.pos_end = it->pos_end - 1;
				tmp.font = it->font;
				it->pos = it->pos_end;
				fonttable.insert(fonttable.begin(), tmp);
			} else {
				/* Add a new entry in the 
				 * fonttable for the position */
				FontEntry tmp;
				tmp.pos = it->pos;
				tmp.pos_end = pos - 1;
				tmp.font = it->font;
				fonttable.insert(fonttable.begin(), tmp);
				
				FontEntry tmp2;
				tmp2.pos = pos + 1;
				tmp2.pos_end = it->pos_end;
				tmp2.font = it->font;
				fonttable.insert(fonttable.begin(), tmp2);
				
				it->pos = pos;
				it->pos_end = pos;
			}
	}
}

   
// This function is able to hide closed footnotes.
LyXParagraph *LyXParagraph::Next()
{
	LyXParagraph *tmp;
	if (next && next->footnoteflag == LyXParagraph::CLOSED_FOOTNOTE) {
		tmp = next;
		while (tmp && tmp->footnoteflag == LyXParagraph::CLOSED_FOOTNOTE)
			tmp = tmp->next;
		if (tmp && tmp->footnoteflag != LyXParagraph::CLOSED_FOOTNOTE) 
			return tmp->Next(); /* There can be more than one
					     * footnote in a logical paragraph
					     */
		else
			return next;	// this should never happen!
	} else
		return next;
}


LyXParagraph *LyXParagraph::NextAfterFootnote() const
{
	LyXParagraph *tmp;
	if (next && next->footnoteflag != LyXParagraph::NO_FOOTNOTE) {
		tmp = next;
		while (tmp && tmp->footnoteflag != LyXParagraph::NO_FOOTNOTE)
			tmp = tmp->next;
		if (tmp && tmp->footnoteflag != LyXParagraph::CLOSED_FOOTNOTE) 
			return tmp;  /* there can be more than one footnote
				      * in a logical paragraph */
		else
			return next; // this should never happen!
	} else
		return next;
}


LyXParagraph *LyXParagraph::PreviousBeforeFootnote()
{
	LyXParagraph *tmp;
	if (previous && previous->footnoteflag != LyXParagraph::NO_FOOTNOTE) {
		tmp = next;
		while (tmp && tmp->footnoteflag != LyXParagraph::NO_FOOTNOTE)
			tmp = tmp->previous;
		if (tmp && tmp->footnoteflag != LyXParagraph::CLOSED_FOOTNOTE) 
			return tmp;   /* there can be more than one footnote
				       * in a logical paragraph */
		else
			return previous;  // this should never happen!
	} else
		return previous;
}


LyXParagraph *LyXParagraph::LastPhysicalPar()
{
	LyXParagraph *tmp = this;
	if (footnoteflag != LyXParagraph::NO_FOOTNOTE)
		return tmp;
   
	while (tmp->next
	       && tmp->next->footnoteflag != LyXParagraph::NO_FOOTNOTE)
		tmp = tmp->NextAfterFootnote();
   
	return tmp;
   
}


const LyXParagraph *LyXParagraph::FirstPhysicalPar() const
{
	if (!IsDummy())
		return this;
	LyXParagraph const *tmppar = this;

	while (tmppar
	       && (tmppar->IsDummy()
		   || tmppar->footnoteflag != LyXParagraph::NO_FOOTNOTE))
		tmppar = tmppar->previous;
   
	if (!tmppar)
		return this; // this should never happen!
	else
		return tmppar;
}


LyXParagraph *LyXParagraph::FirstPhysicalPar()
{
	if (!IsDummy())
		return this;
	LyXParagraph *tmppar = this;

	while (tmppar
	       && (tmppar->IsDummy()
		   || tmppar->footnoteflag != LyXParagraph::NO_FOOTNOTE))
		tmppar = tmppar->previous;
   
	if (!tmppar)
		return this; // this should never happen!
	else
		return tmppar;
}


// This function is able to hide closed footnotes.
LyXParagraph *LyXParagraph::Previous()
{
	LyXParagraph *tmp = previous;
	if (!tmp)
		return tmp;
   
	if (tmp->previous
	    && tmp->previous->footnoteflag == LyXParagraph::CLOSED_FOOTNOTE) {
		tmp = tmp->previous;
		while (tmp
		       && tmp->footnoteflag == LyXParagraph::CLOSED_FOOTNOTE)
			tmp = tmp->previous;
		if (tmp && tmp->footnoteflag != LyXParagraph::CLOSED_FOOTNOTE) 
			return tmp->next->Previous();	

		else
			return previous; 
	} else
		return previous;
}

   
void LyXParagraph::BreakParagraph(int pos, int flag)
{
	int i, pos_end, pos_first;

	// create a new paragraph
	LyXParagraph *par = ParFromPos(pos);
	LyXParagraph *firstpar = FirstPhysicalPar();
   
	LyXParagraph *tmp = new LyXParagraph(par);
   
	tmp->footnoteflag = footnoteflag;
	tmp->footnotekind = footnotekind;
   
	/* this is an idea for a more userfriendly layout handling, I will
	 * see what the users say */
   
	// layout stays the same with latex-environments
	if (flag) {
		tmp->SetOnlyLayout(firstpar->layout);
		tmp->SetLabelWidthString(firstpar->labelwidthstring);
	}

	if (Last() > pos || !Last() || flag == 2) {
		tmp->SetOnlyLayout(firstpar->layout);
		tmp->pparams.align = firstpar->pparams.align;
		tmp->SetLabelWidthString(firstpar->labelwidthstring);
      
		tmp->pparams.line_bottom = firstpar->pparams.line_bottom;
		firstpar->pparams.line_bottom = false;
		tmp->pparams.pagebreak_bottom = firstpar->pparams.pagebreak_bottom;
		firstpar->pparams.pagebreak_bottom = false;
		tmp->pparams.added_space_bottom = firstpar->pparams.added_space_bottom;
		firstpar->pparams.added_space_bottom = VSpace::NONE;
      
		tmp->depth = firstpar->depth;
		tmp->pparams.noindent = firstpar->pparams.noindent;
   
		/* copy everything behind the break-position to the
		 * new paragraph */
		pos_first = 0;
		while (ParFromPos(pos_first) != par)
			++pos_first;
   
		pos_end = pos_first + par->last() - 1;

		for (i = pos; i <= pos_end; ++i) {
			par->CutIntoMinibuffer(i - pos_first);
			tmp->InsertFromMinibuffer(i - pos);
		}

		for (i = pos_end; i >= pos; --i)
			par->Erase(i - pos_first);
	}

	// just an idea of me (who?)
	if (!pos) {
		tmp->pparams.line_top = firstpar->pparams.line_top;
		tmp->pparams.pagebreak_top = firstpar->pparams.pagebreak_top;
		tmp->pparams.added_space_top = firstpar->pparams.added_space_top;
		tmp->bibkey = firstpar->bibkey;
		firstpar->bibkey = 0;
		firstpar->Clear();
		// layout stays the same with latex-environments
		if (flag) {
			firstpar->SetOnlyLayout(tmp->layout);
			firstpar->SetLabelWidthString(tmp->labelwidthstring);
			firstpar->depth = tmp->depth;
		}
	}
}


void LyXParagraph::MakeSameLayout(LyXParagraph const & lp)
{
	LyXParagraph const *par = lp.FirstPhysicalPar();
	
	footnoteflag = par->footnoteflag;
	footnotekind = par->footnotekind;

	layout = par->layout;
	pparams = par->pparams;
	SetLabelWidthString(par->labelwidthstring);

        pextra_type = par->pextra_type;
        pextra_width = par->pextra_width;
        pextra_widthp = par->pextra_widthp;
        pextra_alignment = par->pextra_alignment;
        pextra_hfill = par->pextra_hfill;
        pextra_start_minipage = par->pextra_start_minipage;

	depth = par->depth;
}


LyXParagraph *LyXParagraph::FirstSelfrowPar()
{
	LyXParagraph *tmppar;
   
	tmppar = this;
	while (tmppar && (
		(tmppar->IsDummy() &&
		 tmppar->previous->footnoteflag == LyXParagraph::CLOSED_FOOTNOTE)
		|| tmppar->footnoteflag == LyXParagraph::CLOSED_FOOTNOTE))
		tmppar = tmppar->previous;
   
	if (!tmppar)
		return this; // This should never happen!
	else
		return tmppar;
}


#warning verify that this does what it is supposed to.
LyXParagraph & LyXParagraph::operator=(LyXParagraph const & lp)
{
	MakeSameLayout(lp);
	
	// This is becuse of the dummy layout of the paragraphs that
	// follow footnotes.
	layout = lp.layout;

	// ale970302  (moved to operator= by Lgb)
	bibkey = lp.bibkey ? new InsetBibKey(lp.bibkey) : 0;

	// copy everything behind the break-position to the new paragraph
	for (int i = 0; i < last(); ++i) {
		lp.CopyIntoMinibuffer(i);
		InsertFromMinibuffer(i);
	}
	
	return *this;
}


bool LyXParagraph::HasSameLayout(LyXParagraph const & lp)
{
	LyXParagraph const *par = lp.FirstPhysicalPar();

	return (
		par->footnoteflag == footnoteflag &&
		par->footnotekind == footnotekind &&

		par->layout == layout &&
		par->pparams == pparams &&
                par->pextra_type == pextra_type &&
                par->pextra_width == pextra_width && 
                par->pextra_widthp == pextra_widthp && 
                par->pextra_alignment == pextra_alignment && 
                par->pextra_hfill == pextra_hfill && 
                par->pextra_start_minipage == pextra_start_minipage && 
		par->pparams.noindent == pparams.noindent &&
		par->depth == depth);
}


void LyXParagraph::BreakParagraphConservative(int pos)
{
	int i, pos_end, pos_first;
   
	// create a new paragraph
	LyXParagraph *par = ParFromPos(pos);

	LyXParagraph *tmp = new LyXParagraph(par);
   
	tmp->MakeSameLayout(par);
   
	if (Last() > pos) {   
		/* copy everything behind the break-position to the new
		   paragraph */
		pos_first = 0;
		while (ParFromPos(pos_first) != par)
			++pos_first;
   
		pos_end = pos_first + par->last() - 1;

		for (i = pos; i <= pos_end; ++i) {
      
			par->CutIntoMinibuffer(i - pos_first);
			tmp->InsertFromMinibuffer(i - pos);
		}
		for (i = pos_end; i >= pos; --i)
			par->Erase(i - pos_first);
	}
}
   

// be carefull, this does not make any check at all
void LyXParagraph::PasteParagraph()
{
	int i, pos_end, pos_insert;
	LyXParagraph *the_next;
   
	// copy the next paragraph to this one
	the_next = Next();
   
	LyXParagraph *firstpar = FirstPhysicalPar();
   
	// first the DTP-stuff
	firstpar->pparams.line_bottom = the_next->pparams.line_bottom;
	firstpar->pparams.added_space_bottom = the_next->pparams.added_space_bottom;
	firstpar->pparams.pagebreak_bottom = the_next->pparams.pagebreak_bottom;
   
	pos_end = the_next->last() - 1;
	pos_insert = Last();

	// ok, now copy the paragraph
	for (i = 0; i <= pos_end; ++i) {
		the_next->CutIntoMinibuffer(i);
		InsertFromMinibuffer(pos_insert + i);
	}
   
	// delete the next paragraph
	delete the_next;
}


void LyXParagraph::OpenFootnote(int pos)
{
	LyXParagraph *par = ParFromPos(pos);
	par = par->next;
	while (par && par->footnoteflag == LyXParagraph::CLOSED_FOOTNOTE) {
		par->footnoteflag = LyXParagraph::OPEN_FOOTNOTE;
		par = par->next;
	}
}


void LyXParagraph::CloseFootnote(int pos)
{
	LyXParagraph *par = ParFromPos(pos);
	par = par->next;
	while (par && par->footnoteflag == LyXParagraph::OPEN_FOOTNOTE) {
		par->footnoteflag = LyXParagraph::CLOSED_FOOTNOTE;
		par = par->next;
	}
}


int LyXParagraph::GetLayout()
{
	return FirstPhysicalPar()->layout;
}


char LyXParagraph::GetDepth()
{
	return FirstPhysicalPar()->depth;
}


char LyXParagraph::GetAlign()
{
	return FirstPhysicalPar()->pparams.align;
}


string LyXParagraph::GetLabelstring()
{
	return FirstPhysicalPar()->labelstring;
}


unsigned char LyXParagraph::GetCounter(int i)
{
	return FirstPhysicalPar()->counter[i];
}


// The next two functions are for the manual labels.

string LyXParagraph::GetLabelWidthString()
{
	if (!FirstPhysicalPar()->labelwidthstring.empty())
		return FirstPhysicalPar()->labelwidthstring;
	else
		return _("Senseless with this layout!");
}


void LyXParagraph::SetLabelWidthString(string const & s)
{
	LyXParagraph *par = FirstPhysicalPar();

	par->labelwidthstring = s;
}


void LyXParagraph::SetOnlyLayout(char new_layout)
{
	LyXParagraph *par = FirstPhysicalPar();
	LyXParagraph *ppar = 0;
	LyXParagraph *npar = 0;

	par->layout = new_layout;
        if (par->pextra_type == PEXTRA_NONE) {
                if (par->Previous()) {
                        ppar = par->Previous()->FirstPhysicalPar();
                        while(ppar
			      && ppar->Previous()
			      && (ppar->depth > par->depth))
                                ppar = ppar->Previous()->FirstPhysicalPar();
                }
                if (par->Next()) {
                        npar = par->Next()->NextAfterFootnote();
                        while(npar
			      && npar->Next()
			      && (npar->depth > par->depth))
                                npar = npar->Next()->NextAfterFootnote();
                }
                if (ppar && (ppar->pextra_type != PEXTRA_NONE)) {
                        string
                                p1 = ppar->pextra_width,
                                p2 = ppar->pextra_widthp;
                        ppar->SetPExtraType(ppar->pextra_type,
                                            p1.c_str(),p2.c_str());
                }
                if ((par->pextra_type == PEXTRA_NONE) &&
                    npar && (npar->pextra_type != PEXTRA_NONE)) {
                        string
                                p1 = npar->pextra_width,
                                p2 = npar->pextra_widthp;
                        npar->SetPExtraType(npar->pextra_type,
                                            p1.c_str(),p2.c_str());
                }
        }
}


void LyXParagraph::SetLayout(char new_layout)
{
	LyXParagraph
		*par = FirstPhysicalPar(),
		*ppar = 0,
		*npar = 0;

        par->layout = new_layout;
	par->labelwidthstring.erase();
	par->pparams.align = LYX_ALIGN_LAYOUT;
	par->pparams.added_space_top = VSpace::NONE;
	par->pparams.added_space_bottom = VSpace::NONE;
        if (par->pextra_type == PEXTRA_NONE) {
                if (par->Previous()) {
                        ppar = par->Previous()->FirstPhysicalPar();
                        while(ppar
			      && ppar->Previous()
			      && (ppar->depth > par->depth))
                                ppar = ppar->Previous()->FirstPhysicalPar();
                }
                if (par->Next()) {
                        npar = par->Next()->NextAfterFootnote();
                        while(npar
			      && npar->Next()
			      && (npar->depth > par->depth))
                                npar = npar->Next()->NextAfterFootnote();
                }
                if (ppar && (ppar->pextra_type != PEXTRA_NONE)) {
                        string
                                p1 = ppar->pextra_width,
                                p2 = ppar->pextra_widthp;
                        ppar->SetPExtraType(ppar->pextra_type,
                                            p1.c_str(),p2.c_str());
                }
                if ((par->pextra_type == PEXTRA_NONE) &&
                    npar && (npar->pextra_type != PEXTRA_NONE)) {
                        string
                                p1 = npar->pextra_width,
                                p2 = npar->pextra_widthp;
                        npar->SetPExtraType(npar->pextra_type,
                                            p1.c_str(),p2.c_str());
                }
        }
}


/* If the layout of a paragraph contains a manual label, the beginning of the 
* main body is the beginning of the second word. This is what the par-
* function returns. If the layout does not contain a label, the main
* body always starts with position 0. This differentiation is necessary,
* because there cannot be a newline or a blank <= the beginning of the 
* main body in TeX. */ 

int LyXParagraph::BeginningOfMainBody()
{
	if (FirstPhysicalPar() != this)
		return -1;
   
	int i = 0;
   
	if (i < last()
	    && GetChar(i) != LYX_META_NEWLINE) {
		++i;
		char previous_char = 0;
		if (i < last()
		    && (previous_char = GetChar(i)) != LYX_META_NEWLINE) {
			// Yes, this  ^ is supposed to be "=" not "=="
			++i;
			char temp = 0;
			while (i < last()
			       && previous_char != ' '
			       && (temp = GetChar(i)) != LYX_META_NEWLINE) {
				++i;
				previous_char = temp;
			}
		}
	}

	if (i==0 && i == last() &&
	    !(footnoteflag==LyXParagraph::NO_FOOTNOTE
	      && next && next->footnoteflag != LyXParagraph::NO_FOOTNOTE)
		)
		++i;		  /* the cursor should not jump  
				   * to the main body if there
				   * is nothing in! */
	return i;
}


LyXParagraph* LyXParagraph::DepthHook(int deth)
{
	LyXParagraph *newpar = this;
	if (deth < 0)
		return 0;
   
	do {
		newpar = newpar->FirstPhysicalPar()->Previous();
	} while (newpar && newpar->GetDepth() > deth
		 && newpar->footnoteflag == footnoteflag);
   
	if (!newpar) {
		if (Previous() || GetDepth())
			lyxerr << "ERROR (LyXParagraph::DepthHook): "
				"no hook.\n";
		newpar = this;
	}
	return newpar->FirstPhysicalPar();
}


int LyXParagraph::AutoDeleteInsets()
{
	int i = 0;
	for (InsetTable::iterator it = insettable.begin();
	     it != insettable.end(); ++it) {
		if (it->inset) {
			if (it->inset->AutoDelete()) {
				++i;
				Erase(it->pos);
			}
		} else
			lyxerr << "ERROR (LyXParagraph::AutoDeleteInsets): "
				"cannot auto-delete insets\n";
	}
	return i;
}


Inset* LyXParagraph::ReturnNextInsetPointer(int &pos)
{
	InsetTable::const_iterator ic = 0;
	for (InsetTable::const_iterator ci = insettable.begin();
	     ci != insettable.end(); ++ci) {
		if (ci->pos >= pos) {
			if (!ic || ci->pos < ic->pos)
				ic = ci;
		}
	}
	if (ic) {
		pos = ic->pos;
		return ic->inset;
	} else
		return 0;
}


// returns -1 if inset not found
int LyXParagraph::GetPositionOfInset(Inset* inset)
{
	for(InsetTable::const_iterator ci=insettable.begin();
	    ci != insettable.end(); ++ci) {
		if (ci->inset && ci->inset == inset)
			return ci->pos;
	}
	if (footnoteflag == LyXParagraph::NO_FOOTNOTE 
	    && next && next->footnoteflag == LyXParagraph::CLOSED_FOOTNOTE) {
		int further =
			NextAfterFootnote()->GetPositionOfInset(inset);
		if (further != -1)
			return last() + 1 + further;
	}
	return -1;
}


LyXParagraph* LyXParagraph::TeXOnePar(string &file, TexRow &texrow,
				      string &foot, TexRow &foot_texrow,
				      int &foot_count)
{
	lyxerr[Debug::LATEX] << "TeXOnePar...     " << (void*)this << '\n';
	LyXParagraph *par = next;
	LyXLayout const * style = textclasslist.style(getCurrentTextClass(current_view->currentBuffer()), layout);

	bool further_blank_line = false;
	if (IsDummy())
		lyxerr << "ERROR (LyXParagraph::TeXOnePar) is dummy.\n";

	if (tex_code_break_column && style->isCommand()){
		file += '\n';
		texrow.newline();
	}

	if (pparams.pagebreak_top) {
		file += "\\newpage";
		further_blank_line = true;
	}
	if (pparams.added_space_top.kind() != VSpace::NONE) {
		file += pparams.added_space_top.asLatexCommand(current_view->currentBuffer());
		further_blank_line = true;
	}
      
	if (pparams.line_top) {
		file += "\\lyxline{\\" + getFont(0).latexSize() + '}';
		file += "\\vspace{-1\\parskip}";
		further_blank_line = true;
	}

	if (further_blank_line){
		file += '\n';
		texrow.newline();
	}

	switch (style->latextype) {
	case LATEX_COMMAND:
                //arrae970411
		if (style->latexparam.empty()) { 
			file += '\\';
			file += style->latexname;
		} else {
			file += '\\';
			file += style->latexname;
			file += '[';
			file += style->latexparam;
			file += ']';
		}
		break;
	case LATEX_ITEM_ENVIRONMENT:
	        if (bibkey) 
			bibkey->Latex(file, false);
		else
			file += "\\item ";
		break;
	case LATEX_LIST_ENVIRONMENT:
		file += "\\item ";
		break;
	default:
		break;
	}

	bool need_par = SimpleTeXOnePar(file, texrow);
 
	// Spit out footnotes
	while (par && par->footnoteflag != LyXParagraph::NO_FOOTNOTE
	       && par->footnoteflag != footnoteflag) {
		par = par->TeXFootnote(file, texrow,
				       foot, foot_texrow, foot_count);
	    	par->SimpleTeXOnePar(file, texrow);
	    	par = par->next;
	}

	// Make sure that \\par is done with the font of the last
	// character if this has another size as the default.
	// This is necessary because LaTeX (and LyX on the screen)
	// calculates the space between the baselines according
	// to this font. (Matthias)
	LyXFont font = getFont(Last()-1);
	if (need_par) {
		if (style->resfont.size() != font.size()) {
			file += '\\';
			file += font.latexSize();
			file += ' ';
		}
		file += "\\par}";
	} else if (textclasslist.style(getCurrentTextClass(current_view->currentBuffer()),
				  GetLayout())->isCommand()){
		if (style->resfont.size() != font.size()) {
			file += '\\';
			file += font.latexSize();
			file += ' ';
		}
		file += '}';
	} else if (style->resfont.size() != font.size()){
		file += "{\\" + font.latexSize() + " \\par}";
	}
	
	switch (style->latextype) {
	case LATEX_ITEM_ENVIRONMENT:
	case LATEX_LIST_ENVIRONMENT:
                if (par && (depth < par->depth)) {
                        file += '\n';
                        texrow.newline();
                }
		break;
	case LATEX_ENVIRONMENT:
		// if its the last paragraph of the current environment
		// skip it otherwise fall through
		if (par
		    && !par->HasSameLayout(this)) {
			break;
		}
	default:
		if (!(footnoteflag != LyXParagraph::NO_FOOTNOTE
		      && footnotekind != LyXParagraph::FOOTNOTE
		      && footnotekind != LyXParagraph::MARGIN)) {
			// don't insert this if we would be adding it
			// before or after a table in a float.  This 
			// little trick is needed in order to allow
			// use of tables in \subfigures or \subtables.
			file += '\n';
			texrow.newline();
		}
	}
	
	further_blank_line = false;
	if (pparams.line_bottom) {
		file += "\\lyxline{\\" + getFont(Last()-1).latexSize() + '}';
		further_blank_line = true;
	}

	if (pparams.added_space_bottom.kind() != VSpace::NONE) {
		file += pparams.added_space_bottom.asLatexCommand(current_view->currentBuffer());
		further_blank_line = true;
	}
      
	if (pparams.pagebreak_bottom) {
		file += "\\newpage";
		further_blank_line = true;
	}

	if (further_blank_line){
		file += '\n';
		texrow.newline();
	}

	if (!(footnoteflag != LyXParagraph::NO_FOOTNOTE && par &&
              par->footnoteflag == LyXParagraph::NO_FOOTNOTE) // &&
//            (pextra_type != PEXTRA_MINIPAGE ||
/*             (par && !par->pextra_type == PEXTRA_MINIPAGE)) */ ) {
		file += '\n';
		texrow.newline();
	}

	lyxerr[Debug::LATEX] << "TeXOnePar...done " << (void*)par << '\n';
	return par;
}


// This one spits out the text of the paragraph
bool LyXParagraph::SimpleTeXOnePar(string &file, TexRow &texrow)
{
	lyxerr[Debug::LATEX] << "SimpleTeXOnePar...     "
			     << (void*)this << '\n';
	char c;
	int main_body;
	int column;
	
	bool return_value = false;

	LyXLayout const * style =
		textclasslist
		.style(getCurrentTextClass(current_view->currentBuffer()),
		       GetLayout());
	LyXFont basefont;

	// Maybe we have to create a optional argument.
	if (style->labeltype != LABEL_MANUAL)
		main_body = 0;
	else
		main_body = BeginningOfMainBody();

	if (main_body > 0) {
		file += '[';
		basefont = getFont(-2); // Get label font
	} else {
		basefont = getFont(-1); // Get layout font
	}

	column = 0;

	if (main_body >= 0
	    && !last()
	    && !IsDummy()) {
		if (style->isCommand()) {
			file += '{';
			++column;
		} else if (pparams.align != LYX_ALIGN_LAYOUT) {
			file += '{';
			++column;
			return_value = true;
		}
	}
 
	// Which font is currently active?
	LyXFont running_font = basefont;
	// Do we have an open font change?
	bool open_font = false;

	texrow.start(this, 0);
 
	for (int i = 0; i < last(); ++i) {
		++column;
		// First char in paragraph or after label?
		if (i == main_body && !IsDummy()) {
			if (main_body > 0) {
				if (open_font) {
					column += running_font.latexWriteEndChanges(file, basefont);
					open_font = false;
				}
				// Now use the layout font
				basefont = getFont(-1);
				running_font = basefont;
				file += ']';
				++column;
			}
			if (style->isCommand()) {
				file += '{';
				++column;
			} else if (pparams.align != LYX_ALIGN_LAYOUT) {
				file += '{';
				++column;
				return_value = true;
			}

			if (pparams.noindent) {
				file += "\\noindent ";
				column += 10;
			}
			switch (pparams.align) {
			case LYX_ALIGN_LEFT:
				file += "\\raggedright ";
				column+=13;
				break;
			case LYX_ALIGN_RIGHT:
				file += "\\raggedleft ";
				column+=12;
				break;
			case LYX_ALIGN_CENTER:
				file += "\\centering ";
				column+=11;
				break;
			}	 
		}

		c = GetChar(i);

		// Fully instantiated font
		LyXFont font = getFont(i);

		// Spaces at end of font change are simulated to be
		// outside font change, i.e. we write "\textXX{text} "
		// rather than "\textXX{text }". (Asger) 
		if (open_font && c == ' ' && i <= last()-2
		    && !getFont(i+1).equalExceptLatex(running_font) 
		    && !getFont(i+1).equalExceptLatex(font)) {
			font = getFont(i+1);
		}

		// We end font definition before blanks
		if (!font.equalExceptLatex(running_font) && open_font) {
			column += running_font.latexWriteEndChanges(file, basefont);
			running_font = basefont;
			open_font = false;
		}

		// Blanks are printed before start of fontswitch
		if (c == ' '){
			// Do not print the separation of the optional argument
			if (i != main_body - 1) {
				SimpleTeXBlanks(file, texrow, i, column, font, style);
			}
		}

		// Do we need to change font?
		if (!font.equalExceptLatex(running_font)
		    && i != main_body-1) {
			column += font.latexWriteStartChanges(file, basefont);
			running_font = font;
			open_font = true;
		}

		if (c == LYX_META_NEWLINE) {
			// newlines are handled differently here than
			// the default in SimpleTeXSpecialChars().
			if (!style->newline_allowed
			    || font.latex() == LyXFont::ON) {
				file += '\n';
			} else {
				if (open_font) {
					column += running_font.latexWriteEndChanges(file, basefont);
					open_font = false;
				}
				basefont = getFont(-1);
				running_font = basefont;
				file += "\\\\\n";
			}
			texrow.newline();
			texrow.start(this, i+1);
			column = 0;
		} else {
			SimpleTeXSpecialChars(file, texrow,
					      font, running_font, basefont,
					      open_font, style, i, column, c);
		}
	}

	// If we have an open font definition, we have to close it
	if (open_font) {
		running_font.latexWriteEndChanges(file, basefont);
	}

	// needed if there is an optional argument but no contents
	if (main_body > 0 && main_body == last()) {
		file += "]~";
		return_value = false;
	}

	lyxerr[Debug::LATEX] << "SimpleTeXOnePar...done "
			     << (void*)this << '\n';
	return return_value;
}

void LyXParagraph::SimpleTeXBlanks(string &file, TexRow &texrow,
				   int const i, int &column,
				   LyXFont const &font,
				   LyXLayout const * const style)
{
	if (column > tex_code_break_column
	    && i 
	    && GetChar(i-1) != ' '
	    && (i < last()-1)
	    // In LaTeX mode, we don't want to
	    // break lines since some commands
	    // do not like this
	    && ! (font.latex() == LyXFont::ON)
	    // same in FreeSpacing mode
	    && !style->free_spacing
	    // In typewriter mode, we want to avoid 
	    // ! . ? : at the end of a line
	    && !(font.family() == LyXFont::TYPEWRITER_FAMILY
		 && (GetChar(i-1) == '.'
		     || GetChar(i-1) == '?' 
		     || GetChar(i-1) == ':'
		     || GetChar(i-1) == '!'))) {
		     file += '\n';
		     texrow.newline();
		     texrow.start(this, i+1);
		     column = 0;
	} else if (font.latex() == LyXFont::OFF) {
		if (font.family() == LyXFont::TYPEWRITER_FAMILY) {
			file += '~';
		} else {
			file += ' ';
		}
	}
}


void LyXParagraph::SimpleTeXSpecialChars(string &file, TexRow &texrow,
					 LyXFont &font, LyXFont &running_font,
					 LyXFont &basefont,
					 bool &open_font,
					 LyXLayout const * const style,
					 int &i, int &column, char const c)
{
	// Two major modes:  LaTeX or plain
	// Handle here those cases common to both modes
	// and then split to handle the two modes separately.

	switch (c) {
	case LYX_META_INSET:
		if (Inset *inset = GetInset(i)) {
			int len = file.length();
			int tmp = inset->Latex(file, style->isCommand());
			
			if (tmp) {
				column = 0;
			} else {
				column += file.length() - len;
			}
			for (;tmp--;) {
				texrow.newline();
			}
		}
		break;

	case LYX_META_NEWLINE:
		if (open_font) {
			column += running_font.latexWriteEndChanges(file,
								    basefont);
			open_font = false;
		}
		basefont = getFont(-1);
		running_font = basefont;
		break;

	default:
		// And now for the special cases within each mode
		// Are we in LaTeX mode?
		if (font.latex() == LyXFont::ON) {
			// at present we only have one option
			// but I'll leave it as a switch statement
			// so its simpler to extend. (ARRae)
			switch (c) {
			default:
				// make sure that we will not print
				// error generating chars to the tex
				// file. This test would not be needed
				// if it were done in the buffer
				// itself.
				if (c != '\0') {
					file += c;
				}
				break;
			}
		} else {
			// Plain mode (i.e. not LaTeX)
			switch (c) {
			case '\\': 
				file += "\\textbackslash{}";
				column += 15;
				break;
		
			case '': case '': case '': case '':  
			case '': case '': case '': case '':
			case '': case '': case '':
				if (current_view->currentBuffer()->params.inputenc == "latin1") {
					file += "\\ensuremath{";
					file += c;
					file += '}';
					column += 13;
				} else {
					file += c;
				}
				break;

			case '|': case '<': case '>':
				// In T1 encoding, these characters exist
				if (lyxrc->fontenc == "T1") {
					file += c;
					//... but we should avoid ligatures
					if ((c == '>' || c == '<')
					    && i <= last()-2
					    && GetChar(i+1) == c){
						file += "\\textcompwordmark{}";
						column += 19;
					}
					break;
				}
				// Typewriter font also has them
				if (font.family() == LyXFont::TYPEWRITER_FAMILY) {
					file += c;
					break;
				} 
				// Otherwise, we use what LaTeX
				// provides us.
				switch(c) {
				case '<':
					file += "\\textless{}";
					column += 10;
					break;
				case '>':
					file += "\\textgreater{}";
					column += 13;
					break;
				case '|':
					file += "\\textbar{}";
					column += 9;
					break;
				}
				break;

			case '-': // "--" in Typewriter mode -> "-{}-"
				if (i <= last()-2
				    && GetChar(i+1) == '-'
				    && font.family() == LyXFont::TYPEWRITER_FAMILY) {
					file += "-{}";
					column += 2;
				} else {
					file += '-';
				}
				break;

			case '\"': 
				file += "\\char`\\\"{}";
				column += 9;
				break;

			case '':
				if (current_view->currentBuffer()->params.inputenc == "default") {
					file += "\\pounds{}";
					column += 8;
				} else {
					file += c;
				}
				break;

			case '$': case '&':
			case '%': case '#': case '{':
			case '}': case '_':
				file += '\\';
				file += c;
				column += 1;
				break;

			case '^': case '~':
				file += '\\';
				file += c;
				file += "{}";
				column += 3;
				break;

			case '*': case '[': case ']':
				// avoid being mistaken for optional arguments
				file += '{';
				file += c;
				file += '}';
				column += 2;
				break;

			case ' ':
				// Blanks are printed before font switching.

				// Sure? I am not! (try nice-latex)

				// I am sure it's correct. LyX might be smarter
				// in the future, but for now, nothing wrong is
				// written. (Asger)
				break;

			default:
				/* idea for labels --- begin*/
				// check for LyX
				if (c ==  'L'
				    && i <= last()-3
				    && font.family() != LyXFont::TYPEWRITER_FAMILY
				    && GetChar(i+1) == 'y'
				    && GetChar(i+2) == 'X') {
					file += "\\LyX{}";
					i += 2;
					column += 5;
				}
				// check for TeX
				else if (c == 'T'
					 && i <= last()-3
					 && font.family() != LyXFont::TYPEWRITER_FAMILY
					 && GetChar(i+1) == 'e'
					 && GetChar(i+2) == 'X') {
					file += "\\TeX{}";
					i += 2;
					column += 5;
				}
				// check for LaTeX2e
				else if (c == 'L'
					 && i <= last()-7
					 && font.family() != LyXFont::TYPEWRITER_FAMILY
					 && GetChar(i+1) == 'a'
					 && GetChar(i+2) == 'T'
					 && GetChar(i+3) == 'e'
					 && GetChar(i+4) == 'X'
					 && GetChar(i+5) == '2'
					 && GetChar(i+6) == 'e') {
					file += "\\LaTeXe{}";
					i += 6;
					column += 8;
				}
				// check for LaTeX
				else if (c == 'L'
					 && i <= last()-5
					 && font.family() != LyXFont::TYPEWRITER_FAMILY
					 && GetChar(i+1) == 'a'
					 && GetChar(i+2) == 'T'
					 && GetChar(i+3) == 'e'
					 && GetChar(i+4) == 'X') {
					file += "\\LaTeX{}";
					i += 4;
					column += 7;
					/* idea for labels --- end*/ 
				} else if (c != '\0') {
					file += c;
				}
				break;
			}
		}
	}
}


LyXParagraph * LyXParagraph::TeXDeeper(string &file, TexRow &texrow,
				       string &foot, TexRow &foot_texrow,
				       int &foot_count)
{
	lyxerr[Debug::LATEX] << "TeXDeeper...     " << (void*)this << '\n';
	LyXParagraph *par = this;

	while (par && par->depth == depth) {
		if (par->IsDummy())
			lyxerr << "ERROR (LyXParagraph::TeXDeeper)\n";
		if (textclasslist.style(getCurrentTextClass(current_view->currentBuffer()), 
				   par->layout)->isEnvironment()
		    || par->pextra_type != PEXTRA_NONE) 
			{
				par = par->TeXEnvironment(file, texrow,
							  foot, foot_texrow,
							  foot_count);
			} else {
				par = par->TeXOnePar(file, texrow,
						     foot, foot_texrow,
						     foot_count);
			}
	}
	lyxerr[Debug::LATEX] << "TeXDeeper...done " << (void*)par << '\n';
	return par;
}


LyXParagraph* LyXParagraph::TeXEnvironment(string &file, TexRow &texrow,
					   string &foot, TexRow &foot_texrow,
					   int &foot_count)
{
	bool
		eindent_open = false;
        static bool
		minipage_open = false;
        static int
                minipage_open_depth = 0;
	char
		par_sep =
		current_view->currentBuffer()->params.paragraph_separation;
    
	lyxerr[Debug::LATEX] << "TeXEnvironment...     "
			     << (void*)this << '\n';
	if (IsDummy())
		lyxerr << "ERROR (LyXParagraph::TeXEnvironment)\n";

	LyXLayout const * style = textclasslist.style(getCurrentTextClass(current_view->currentBuffer()),layout);
       
	if (pextra_type == PEXTRA_INDENT) {
		if (!pextra_width.empty()) {
			file += "\\begin{LyXParagraphIndent}{"
				+ pextra_width + "}\n";
		} else {
			//float ib = atof(pextra_widthp.c_str())/100;
			// string can't handle floats at present (971109)
			// so I'll do a conversion by hand knowing that
			// the limits are 0.0 to 1.0. ARRae.
			file += "\\begin{LyXParagraphIndent}{";
			switch (pextra_widthp.length()) {
			case 3:
				file += "1.00";
				break;
			case 2:
				file += "0.";
				file += pextra_widthp;
				break;
			case 1:
				file += "0.0";
				file += pextra_widthp;
			}
			file += "\\columnwidth}\n";
		}
		texrow.newline();
		eindent_open = true;
	}
	if ((pextra_type == PEXTRA_MINIPAGE) && !minipage_open) {
		if (pextra_hfill && Previous() &&
		    (Previous()->pextra_type == PEXTRA_MINIPAGE)) {
			file += "\\hfill{}\n";
			texrow.newline();
		}
		if (par_sep == LYX_PARSEP_INDENT) {
			file += "{\\setlength\\parindent{0pt}\n";
			texrow.newline();
		}
		file += "\\begin{minipage}";
		switch(pextra_alignment) {
		case MINIPAGE_ALIGN_TOP:
			file += "[t]";
			break;
		case MINIPAGE_ALIGN_MIDDLE:
			file += "[m]";
			break;
		case MINIPAGE_ALIGN_BOTTOM:
			file += "[b]";
			break;
		}
		if (!pextra_width.empty()) {
			file += '{';
			file += pextra_width + "}\n";
		} else {
			//float ib = atof(par->pextra_width.c_str())/100;
			// string can't handle floats at present
			// so I'll do a conversion by hand knowing that
			// the limits are 0.0 to 1.0. ARRae.
			file += '{';
			switch (pextra_widthp.length()) {
			case 3:
				file += "1.00";
				break;
			case 2:
				file += "0.";
				file += pextra_widthp;
				break;
			case 1:
				file += "0.0";
				file += pextra_widthp;
			}
			file += "\\columnwidth}\n";
		}
		texrow.newline();
		if (par_sep == LYX_PARSEP_INDENT) {
			file += "\\setlength\\parindent{\\LyXMinipageIndent}\n";
			texrow.newline();
		}
		minipage_open = true;
                minipage_open_depth = depth;
	}
	if (style->isEnvironment()){
		if (style->latextype == LATEX_LIST_ENVIRONMENT)
			file += "\\begin{" + style->latexname + "}{"
				+ labelwidthstring + "}\n";
		else if (style->labeltype == LABEL_BIBLIO) {
			// ale970405
			file += "\\begin{" + style->latexname + "}{"
				+ bibitemWidthest() + "}\n";
		}
		//arrae970411
		else if (style->latexparam.empty())
			file += "\\begin{" + style->latexname + "}\n";
		else
			file += "\\begin{" + style->latexname + "}{"
				+ style->latexparam + "}\n";
		texrow.newline();
	}
	LyXParagraph *par = this;
	do {
		par = par->TeXOnePar(file, texrow,
				     foot, foot_texrow, foot_count);
                if (minipage_open && par && !style->isEnvironment() &&
                    (par->pextra_type == PEXTRA_MINIPAGE) &&
                    par->pextra_start_minipage) {
                    file += "\\end{minipage}\n";
                    texrow.newline();
                    if (par_sep == LYX_PARSEP_INDENT) {
                        file += "}\n";
			texrow.newline();
                    }
                    minipage_open = false;
                }
		if (par && par->depth > depth) {
			if (textclasslist.style(getCurrentTextClass(current_view->currentBuffer()),
					   par->layout)->isParagraph()
			    && !suffixIs(file, "\n\n")) {
			    // There should be at least one '\n' already
			    // but we need there to be two for Standard 
			    // paragraphs that are depth-increment'ed to be
			    // output correctly. ARRae
				file += '\n';
				texrow.newline();
			}
			par = par->TeXDeeper(file, texrow,
					     foot, foot_texrow, foot_count);
		}
		if (par && par->layout == layout && par->depth == depth &&
		    (par->pextra_type == PEXTRA_MINIPAGE) && !minipage_open) {
			if (par->pextra_hfill && par->Previous() &&
			    (par->Previous()->pextra_type == PEXTRA_MINIPAGE))
				file += "\\hfill{}\n";
			else
				file += '\n';
			texrow.newline();
			if (par_sep == LYX_PARSEP_INDENT) {
				file += "{\\setlength\\parindent{0pt}\n";
				texrow.newline();
			}
			file += "\\begin{minipage}";
			switch(par->pextra_alignment) {
			case MINIPAGE_ALIGN_TOP:
				file += "[t]";
				break;
			case MINIPAGE_ALIGN_MIDDLE:
				file += "[m]";
				break;
			case MINIPAGE_ALIGN_BOTTOM:
				file += "[b]";
				break;
			}
			if (!par->pextra_width.empty()) {
				file += '{';
				file += par->pextra_width;
				file += "}\n";
			} else {
				//float ib = atof(par->pextra_widthp.c_str())/100;
				// string can't handle floats at present
				// so I'll do a conversion by hand knowing that
				// the limits are 0.0 to 1.0. ARRae.
				file += '{';
				switch (par->pextra_widthp.length()) {
				case 3:
					file += "1.00";
					break;
				case 2:
					file += "0.";
					file += par->pextra_widthp;
					break;
				case 1:
					file += "0.0";
					file += par->pextra_widthp;
				}
				file += "\\columnwidth}\n";
			}
			texrow.newline();
			if (par_sep == LYX_PARSEP_INDENT) {
				file += "\\setlength\\parindent{\\LyXMinipageIndent}\n";
				texrow.newline();
			}
			minipage_open = true;
                        minipage_open_depth = par->depth;
		}
	} while (par
		 && par->layout == layout
		 && par->depth == depth
		 && par->pextra_type == pextra_type);

	if (style->isEnvironment()) {
		file += "\\end{" + style->latexname + '}';
	}
        if (minipage_open && (minipage_open_depth == depth) &&
            (!par || par->pextra_start_minipage ||
             par->pextra_type != PEXTRA_MINIPAGE)) {
                file += "\\end{minipage}\n";
		texrow.newline();
                if (par_sep == LYX_PARSEP_INDENT) {
                        file += "}\n";
			texrow.newline();
                }
                if (par && par->pextra_type != PEXTRA_MINIPAGE) {
                        file += "\\medskip\n\n";
			texrow.newline();
			texrow.newline();
                }
                minipage_open = false;
        }
	if (eindent_open) {
		file += "\\end{LyXParagraphIndent}\n";
		texrow.newline();
	}
        if (!(par && (par->pextra_type==PEXTRA_MINIPAGE) 
	      && par->pextra_hfill)) {
                file += '\n';
		texrow.newline();
	}
	lyxerr[Debug::LATEX] << "TeXEnvironment...done " << (void*)par << '\n';
	return par;  // ale970302
}


LyXParagraph * LyXParagraph::TeXFootnote(string &file, TexRow &texrow,
					 string &foot, TexRow &foot_texrow,
					 int &foot_count)
{
	lyxerr[Debug::LATEX] << "TeXFootnote...     "
			     << (void*)this << '\n';
	if (footnoteflag == LyXParagraph::NO_FOOTNOTE)
		lyxerr << "ERROR (LyXParagraph::TeXFootnote): "
			"No footnote!\n";

	LyXParagraph *par = this;
	LyXLayout const * style = textclasslist.style(getCurrentTextClass(current_view->currentBuffer()), 
					   previous->GetLayout());
	
	if (style->needprotect && footnotekind != LyXParagraph::FOOTNOTE){
		lyxerr << "ERROR (LyXParagraph::TeXFootnote): "
			"Float other than footnote in command"
			" with moving argument is illegal\n";
	}

	if (footnotekind != LyXParagraph::FOOTNOTE
	    && footnotekind != LyXParagraph::MARGIN
	    && file.length()
	    && !suffixIs(file, '\n')) {
		// we need to ensure that real floats like tables and figures
		// have their \begin{} on a new line otherwise we can get
		// incorrect results when using the endfloat.sty package
		// especially if two floats follow one another.  ARRae 981022
		// NOTE: if the file is length 0 it must have just been
		//       written out so we assume it ended with a '\n'
		file += '\n';
		texrow.newline();
	}
	
	BufferParams *params = &current_view->currentBuffer()->params;
	bool footer_in_body = true;
	switch (footnotekind) {
	case LyXParagraph::FOOTNOTE:
		if (style->intitle) {
			file += "\\thanks{\n";
			footer_in_body = false;
		} else {
			if (foot_count == -1) {
				// we're at depth 0 so we can use:
				file += "\\footnote{\n";
				footer_in_body = false;
			} else {
				file += "\\footnotemark{}\n";
				if (foot_count) {
					// we only need this when there are
					// multiple footnotes
					foot += "\\stepcounter{footnote}";
				}
				foot += "\\footnotetext{\n";
				foot_texrow.start(this,0);
				foot_texrow.newline();
				++foot_count;
			}
		}
		break;
	case LyXParagraph::MARGIN:
		file += "\\marginpar{\n";
		break;
	case LyXParagraph::FIG:
		if (pextra_type == PEXTRA_FLOATFLT
		    && (!pextra_width.empty()
			|| !pextra_widthp.empty())) {
			char bufr[80];
			if (!pextra_width.empty())
				sprintf(bufr, "\\begin{floatingfigure}{%s}\n",
					pextra_width.c_str());
			else
				sprintf(bufr, "\\begin{floatingfigure}{%f\\textwidth}\n",
					atoi(pextra_widthp.c_str())/100.0);
			file += bufr;
		} else {
			file += "\\begin{figure}";
			if (!params->float_placement.empty()) { 
				file += '[';
				file += params->float_placement;
				file += "]\n";
			} else {
				file += '\n';
			}
		}
		break;
	case LyXParagraph::TAB:
		file += "\\begin{table}";
		if (!params->float_placement.empty()) { 
			file += '[';
			file += params->float_placement;
			file += "]\n";
		} else {
			file += '\n';
		}
		break;
	case LyXParagraph::WIDE_FIG:
		file += "\\begin{figure*}";
		if (!params->float_placement.empty()) { 
			file += '[';
			file += params->float_placement;
			file += "]\n";
		} else {
			file += '\n';
		}
		break;
	case LyXParagraph::WIDE_TAB:
		file += "\\begin{table*}";
		if (!params->float_placement.empty()) { 
			file += '[';
			file += params->float_placement;
			file += "]\n";
		} else {
			file += '\n';
		}
		break;
	case LyXParagraph::ALGORITHM:
		file += "\\begin{algorithm}\n";
		break;
	}
	texrow.newline();
   
	if (footnotekind != LyXParagraph::FOOTNOTE
	    || !footer_in_body) {
		// Process text for all floats except footnotes in body
		do {
			LyXLayout const *style = textclasslist.style(getCurrentTextClass(current_view->currentBuffer()),
							  par->layout);
			if (par->IsDummy())
				lyxerr << "ERROR(LyXParagraph::TeXFootnote)\n";
			if (style->isEnvironment()
			    || par->pextra_type == PEXTRA_MINIPAGE) { /* && !minipage_open ?? */
				// Allows the use of minipages within float environments.
				// Shouldn't be circular because we don't support
				// footnotes inside floats (yet). ARRae
				par = par->TeXEnvironment(file, texrow,
							  foot, foot_texrow,
							  foot_count);
			} else {
				par = par->TeXOnePar(file, texrow,
						     foot, foot_texrow,
						     foot_count);
			}
			
			if (par && !par->IsDummy() && par->depth > depth) {
				par = par->TeXDeeper(file, texrow,
						     foot, foot_texrow,
						     foot_count);
			}
		} while (par && par->footnoteflag != LyXParagraph::NO_FOOTNOTE);
	} else {
		// process footnotes > depth 0 or in environments separately
		// NOTE: Currently don't support footnotes within footnotes
		//       even though that is possible using the \footnotemark
		string dummy;
		TexRow dummy_texrow;
		int dummy_count = 0;
		do {
			LyXLayout const *style = textclasslist.style(getCurrentTextClass(current_view->currentBuffer()),
							  par->layout);
			if (par->IsDummy())
				lyxerr << "ERROR(LyXParagraph::TeXFootnote)\n";
			if (style->isEnvironment()
			    || par->pextra_type == PEXTRA_MINIPAGE) { /* && !minipage_open ?? */
				// Allows the use of minipages within float environments.
				// Shouldn't be circular because we don't support
				// footnotes inside floats (yet). ARRae
				par = par->TeXEnvironment(foot, foot_texrow,
							  dummy, dummy_texrow,
							  dummy_count);
			} else {
				par = par->TeXOnePar(foot, foot_texrow,
						     dummy, dummy_texrow,
						     dummy_count);
			}

			if (par && !par->IsDummy() && par->depth > depth) {
				par = par->TeXDeeper(foot, foot_texrow,
						     dummy, dummy_texrow,
						     dummy_count);
			}
		} while (par && par->footnoteflag != LyXParagraph::NO_FOOTNOTE);
		if (dummy_count) {
			lyxerr << "ERROR (LyXParagraph::TeXFootnote): "
				"Footnote in a Footnote -- not supported\n";
		}
	}

	switch (footnotekind) {
	case LyXParagraph::FOOTNOTE:
		if (footer_in_body) {
			// This helps tell which of the multiple
			// footnotetexts an error was in.
			foot += "}%\n";
			foot_texrow.newline();
		} else {
			file += '}';
		}
		break;
	case LyXParagraph::MARGIN:
		file += '}';
		break;
	case LyXParagraph::FIG:
		if (pextra_type == PEXTRA_FLOATFLT
		    && (!pextra_width.empty()
			|| !pextra_widthp.empty()))
			file += "\\end{floatingfigure}";
		else
			file += "\\end{figure}";
		break;
	case LyXParagraph::TAB:
		file += "\\end{table}";
		break;
	case LyXParagraph::WIDE_FIG:
		file += "\\end{figure*}";
		break;
	case LyXParagraph::WIDE_TAB:
		file += "\\end{table*}";
		break;
	case LyXParagraph::ALGORITHM:
		file += "\\end{algorithm}";
		break;
	}

	if (footnotekind != LyXParagraph::FOOTNOTE
	    && footnotekind != LyXParagraph::MARGIN) {
		// we need to ensure that real floats like tables and figures
		// have their \end{} on a line of their own otherwise we can
		// get incorrect results when using the endfloat.sty package.
		file += "\n";
		texrow.newline();
	}

	lyxerr[Debug::LATEX] << "TeXFootnote...done "
			     << (void*)par->next << '\n';
	return par;
}


void LyXParagraph::SetPExtraType(int type, const char *width,
				 const char *widthp)
{
	pextra_type = type;
	pextra_width = width;
	pextra_widthp = widthp;
	
	if (textclasslist.style(getCurrentTextClass(current_view->currentBuffer()), 
			   layout)->isEnvironment()) {
		LyXParagraph
			*par = this,
			*ppar = par;
		
		while(par
		      && par->layout == layout
		      && par->depth == depth) {
			ppar = par;
			par = par->Previous();
			if (par)
				par = par->FirstPhysicalPar();
			while (par && par->depth > depth) {
				par = par->Previous();
				if (par)
					par = par->FirstPhysicalPar();
			}
		}
		par = ppar;
		while(par
		      && (par->layout == layout) && (par->depth == depth)) {
			par->pextra_type = type;
			par->pextra_width = width;
			par->pextra_widthp = widthp;
			par = par->NextAfterFootnote();
			if (par && (par->depth > depth))
				par->SetPExtraType(type,width,widthp);
			while (par && ((par->depth > depth) || par->IsDummy()))
				par = par->NextAfterFootnote();
		}
	}
}


void LyXParagraph::UnsetPExtraType()
{
	if (pextra_type == PEXTRA_NONE)
		return;
    
	pextra_type = PEXTRA_NONE;
	pextra_width.erase();
	pextra_widthp.erase();

	if (textclasslist.style(getCurrentTextClass(current_view->currentBuffer()), 
			   layout)->isEnvironment()) {
		LyXParagraph
			*par = this,
			*ppar = par;
		
		while(par
		      && par->layout == layout && (par->depth == depth)) {
			ppar = par;
			par = par->Previous();
			if (par)
				par = par->FirstPhysicalPar();
			while (par && par->depth > depth) {
				par = par->Previous();
				if (par)
					par = par->FirstPhysicalPar();
			}
		}
		par = ppar;
		while(par
		      && par->layout == layout
		      && (par->depth == depth)) {
			par->pextra_type = PEXTRA_NONE;
			par->pextra_width.erase();
			par->pextra_widthp.erase();
			par = par->NextAfterFootnote();
			if (par && (par->depth > depth))
				par->UnsetPExtraType();
			while (par && ((par->depth > depth) || par->IsDummy()))
				par = par->NextAfterFootnote();
		}
	}
}


LyXCursor LyXParagraph::getLabelCursor(const string &label)
{
	Inset* inset;
	int pos = -1;
	LyXCursor tmpcur;
	while ((inset = ReturnNextInsetPointer(pos))) {
		for (int i = 0; i < inset->GetNumberOfLabels(); ++i) {
			if (label == inset->getLabel(i)) {
				tmpcur.set(this, pos);
				return tmpcur;
			}
		}
		++pos;
	}
	if (next)
		return next->getLabelCursor(label);
	else
		return tmpcur;
}


void LyXParagraph::getReferenceList(string &list, char delim)
{
	Inset *inset;
	int pos = -1;
	while ((inset = ReturnNextInsetPointer(pos))){     
		for (int i = 0; i < inset->GetNumberOfLabels(); ++i) {
			if (!list.empty())
				list += delim;
			list += inset->getLabel(i);
		}
		++pos;
	}
	if (next)
		next->getReferenceList(list, delim);
	return; 
}


void LyXParagraph::getIncludeonlyList(string &list, char delim)
{
	Inset *inset;
	int pos = -1;
	while ((inset = ReturnNextInsetPointer(pos))){
		if (inset->LyxCode()==Inset::INCLUDE_CODE) {
			InsetInclude *insetinc = (InsetInclude*)inset;
			if (insetinc->isInclude() 
			    && insetinc->isNoLoad()) {
				if (!list.empty())
					list += delim;
				list +=ChangeExtension(insetinc->getContents(),
						       string(), true);
			}
		}
		++pos;
	}
	if (next)
		next->getIncludeonlyList(list, delim);
	return;
}
