/* This file is part of
* ======================================================
* 
*           LyX, The Document Processor
*
*	    Copyright (C) 1995 Matthias Ettrich
*           Copyright (C) 1995-1998 The LyX Team.
*
*           This file is Copyright 1996-1998
*           Lars Gullik Bjnnes
*
*======================================================*/

//  Added pseudo-action handling, asierra 180296

#include <config.h>

#ifdef __GNUG__
#pragma implementation
#endif

#include "toolbarbackend.h"
#include "debug.h"
#include "LyXAction.h"
#include "lstrings.h"

// 	$Id: toolbarbackend.C,v 1.6 1998/10/21 19:05:46 jug Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: toolbarbackend.C,v 1.6 1998/10/21 19:05:46 jug Exp $";
#endif /* lint */

extern LyXAction lyxaction;


void ToolbarBackend::init()
{
	add(TOOL_LAYOUTS);
	add(TOOL_FUNC, LFUN_MENUOPEN);
	//add(TOOL_FUNC, LFUN_CLOSEBUFFER);
	add(TOOL_FUNC, LFUN_MENUWRITE);
	add(TOOL_FUNC, LFUN_MENUPRINT);
	add(TOOL_SEPARATOR);

	add(TOOL_FUNC, LFUN_CUT);
	add(TOOL_FUNC, LFUN_COPY);
	add(TOOL_FUNC, LFUN_PASTE);
	add(TOOL_SEPARATOR);
	
	add(TOOL_FUNC, LFUN_EMPH);
	add(TOOL_FUNC, LFUN_NOUN);
	add(TOOL_FUNC, LFUN_FREE);
	add(TOOL_SEPARATOR);
	
	add(TOOL_FUNC, LFUN_FOOTMELT);
	add(TOOL_FUNC, LFUN_MARGINMELT);
	add(TOOL_FUNC, LFUN_DEPTH);
	add(TOOL_SEPARATOR);

	add(TOOL_FUNC, LFUN_TEX);
        add(TOOL_FUNC, LFUN_MATH_MODE);
	add(TOOL_SEPARATOR);

	add(TOOL_FUNC, LFUN_FIGURE);
	add(TOOL_FUNC, LFUN_TABULAR_MODE);
	//add(TOOL_FUNC, LFUN_MELT);
}


void ToolbarBackend::add(TOOLBARITEMS bi, int action)
{
	toolbarFunc newItem;
	newItem.baritem = bi;
	newItem.action = action;
	toollist.push_back(newItem);
}


void ToolbarBackend::dump()
{
	if (!lyxerr.debugging(Debug::TOOLBAR))
		return;

	lyxerr << "Dumping toolbar:\n";
	string outstr;
	FuncList::const_iterator p = toollist.begin();
	for (; p != toollist.end(); ++p) {
		outstr = "\tToolbarFunc:";
		switch(p->baritem) {
		case TOOL_NONE:
			outstr += " TOOL_NONE";
			break;
		case TOOL_SEPARATOR:
			outstr += " TOOL_SEPARATOR";
			break;
		case TOOL_LAYOUTS:
			outstr += " TOOL_LAYOUTS";
			break;
		case TOOL_NEWLINE:
			outstr += " TOOL_NEWLINE";
			break;
		case TOOL_FUNC:
			outstr += string(" lyxaction ")
				+ tostr(p->action);
			break;
		}
		lyxerr << outstr << '\n';
	}
}


void ToolbarBackend::read(LyXLex &lex)
{
	//consistency check
	if (lex.GetString() != "\\begin_toolbar")
		lyxerr << "Toolbar::read: ERROR wrong token:`"
		       << lex.GetString() << "'\n";
	
	struct keyword_item toolTags[TO_LAST-1] = {
		{ "\\add", TO_ADD },
		{ "\\end_toolbar", TO_ENDTOOLBAR },
		{ "\\layouts", TO_LAYOUTS },
		{ "\\newline", TO_NEWLINE },
		{ "\\separator", TO_SEPARATOR }
	};
	
	string func;
	bool quit = false;
	
	lex.pushTable(toolTags, TO_LAST - 1);
	
	if (lyxerr.debugging(Debug::PARSER))
		lex.printTable();
	
	// First, ensure that the toolbar is empty
	reset();

	while (lex.IsOK() && !quit) {
		
		lyxerr[Debug::TOOLBAR] << "Toolbar::read: current lex text: `"
				       << lex.GetString() << "'\n";
		
		switch(lex.lex()) {
		case TO_ADD:
			if (lex.EatLine()) {
				func = lex.GetString();
				lyxerr[Debug::TOOLBAR] 
					<< "Toolbar::read TO_ADD func: `"
					<< func << "'\n";
				add(TOOL_FUNC, lyxaction.LookupFunc(func));
			}
			break;
			
		case TO_SEPARATOR:
			add(TOOL_SEPARATOR);
			break;
			
		case TO_LAYOUTS:
			add(TOOL_LAYOUTS);
			break;
			
		case TO_NEWLINE:
			add(TOOL_NEWLINE);
			break;
			
		case TO_ENDTOOLBAR:
			quit = true;
			break;
		default:
			lex.printError("Toolbar::read: "
				       "Unknown toolbar tag: `$$Token'");
			break;
		}
	}
	lex.popTable();
	dump();
}
