/* This file is part of
* ======================================================
* 
*           LyX, The Document Processor
* 	 
*	    Copyright (C) 1995 Matthias Ettrich
*           Copyright (C) 1995-1998 The LyX Team
*
*======================================================*/

#include <config.h>

#ifdef __GNUG__
#pragma implementation "lyxscreen.h"
#endif

#include "lyxscreen.h"
#include "lyxtext.h"
#include "BufferView.h"
#include "lstrings.h"

// 	$Id: screen.C,v 1.9 1998/10/20 13:10:28 larsbj Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: screen.C,v 1.9 1998/10/20 13:10:28 larsbj Exp $";
#endif /* lint */

static
GC createGC() {
	//if (copy_gc) return copy_gc;
	XGCValues val;
	val.foreground = BlackPixel(fl_display, 
				    DefaultScreen(fl_display));
	//} else {
	//	val.foreground = WhitePixel(fl_display,
	//				    DefaultScreen(fl_display));
	//}
	
	val.function=GXcopy;
	val.graphics_exposures = false;
	val.line_style = LineSolid;
	val.line_width = 0;
	return XCreateGC(fl_display, RootWindow(fl_display, 0), 
			 GCForeground | GCFunction | GCGraphicsExposures
			 | GCLineWidth | GCLineStyle , &val);
}


// Constructor
LyXScreen::LyXScreen(BufferView * own,
		     Window window,
		     Pixmap pix,
		     Dimension width, 
		     Dimension height,
		     Dimension offset_x,
		     Dimension offset_y,
		     LyXText *text_ptr)
	: owner(own), window_(window), pixmap_(pix), 
	  width_(width), height_(height),
	  offset_x_(offset_x), offset_y_(offset_y),
	  text(text_ptr)
{
	first = 0;
   
	// The cursor isn't visible yet
	cursor_visible = false;
	screen_refresh_y = -1;

	// We don't have any save of stuff behind the cursor
	cursor_pixmap = 0;
	cursor_pixmap_x = 0;
	cursor_pixmap_y = 0;
	cursor_pixmap_w = 0;
	cursor_pixmap_h = 0;
	
	// We need this GC
	gc_copy = createGC();
}


// Destructor
LyXScreen::~LyXScreen()
{
	if (cursor_pixmap != 0) {
		XFreePixmap(fl_display, cursor_pixmap);
	}
}


void LyXScreen::Redraw()
{
	DrawFromTo(0, height_);
	screen_refresh_y = -1;
	expose(0, 0, width_, height_);

	// Make sure the cursor is drawn if it should be visible
	if (cursor_visible) {
		cursor_visible = false;
		showCursor();
	}
}


void LyXScreen::expose(int x, int y, int exp_width, int exp_height)
{
	lyxerr << "Screen drawable: " << pixmap_ << '\n';
	XCopyArea(fl_display,
		  pixmap_,
		  window_,
		  gc_copy,
		  x, y,
		  exp_width, exp_height,
		  x+offset_x_, y+offset_y_);
}


void LyXScreen::DrawFromTo(int y1, int y2)
{
	long y_text = first + y1;
	long y = 0;
   
	// Get the first needed row
	Row * row = text->GetRowNearY(y_text);
	// y_text is updated to be the real beginning of the row

	y = y_text - first;

	while (row != 0 && y < y2) {
		text->GetVisibleRow(y, row, y + first);
		y += row->height;
		row = row->next;
	}

	// Maybe we have to clear the screen at the bottom
	if (y < y2) {
		owner->getPainter().fillRectangle(0, y, width_, y2 - y, 
						  LColor::bottomarea);
	}
}


void LyXScreen::DrawOneRow(Row *row, long &y_text)
{
	long y = y_text - first;
      
	if (y + row->height > 0 && y - row->height <= height_) {
		// ok, something is visible
		text->GetVisibleRow(y, row, y + first);
	}
	y_text+=row->height;
}


/* Draws the screen, starting with text position y. Uses as much 
 * of already drawn pixels as possible */
void LyXScreen::Draw(long y)
{
	if (cursor_visible) hideCursor();

	if (y<0) y = 0;
	long old_first = first;
	first = y;

	// Is any optimiziation possible?
	if ((y - old_first) < height_ 
	    && (old_first - y) < height_) {
		if (first < old_first) {
			// Draw new stuff
			DrawFromTo(0, old_first - first);
			XCopyArea (fl_display,
				   window_,
				   window_,
				   gc_copy,
				   offset_x_, offset_y_, 
				   width_ , height_ - old_first + first,
				   offset_x_, offset_y_ + old_first - first);
			// Expose the rest
			expose(0, 0, width_, old_first - first);
		} else  {
			// Draw new stuff
			DrawFromTo(height_ + old_first - first, height_);
			XCopyArea (fl_display,
				   window_,
				   window_,
				   gc_copy,
				   offset_x_, offset_y_ + first - old_first, 
				   width_ , height_ + old_first - first, 
				   offset_x_, offset_y_);
			// Expose the rest
			expose(0, height_ + old_first - first, 
			       width_, first - old_first);
		}
	} else {
		// Make a dumb new-draw
		DrawFromTo(0, height_);
		expose(0, 0, width_, height_);
	}
}


void LyXScreen::showCursor()
{
	long x = 0;
	long y1 = 0;
	long y2 = 0;
   
	if (cursor_visible) return;
   
	x = text->cursor.x;
	
	y1 = text->cursor.y - text->real_current_font.maxAscent() - first;
	if (y1 < 0) y1 = 0;
	
	y2 = text->cursor.y + text->real_current_font.maxDescent() - first;
	if (y2 > height_) y2 = height_;

	// Secure against very strange situations
	if (y2 < y1) y2 = y1;

	if (cursor_pixmap){
		XFreePixmap(fl_display, cursor_pixmap);
		cursor_pixmap = 0;
	}
	
	if (y2 > 0 && y1 < height_) {
		cursor_pixmap_w = 1;
		cursor_pixmap_h = y2 - y1 + 1;
		cursor_pixmap_x = x;
		cursor_pixmap_y = y1;
		cursor_pixmap =
			XCreatePixmap(fl_display,
				      RootWindow(fl_display, 0),
				      cursor_pixmap_w,
				      cursor_pixmap_h,
				      fl_get_visual_depth());
		XCopyArea(fl_display,
			  window_,
			  cursor_pixmap,
			  gc_copy,
			  offset_x_ + cursor_pixmap_x, 
			  offset_y_ + cursor_pixmap_y,
			  cursor_pixmap_w, cursor_pixmap_h,
			  0, 0);
		XDrawLine(fl_display,
			  window_, gc_copy,
			  x + offset_x_,
			  y1 + offset_y_,
			  x + offset_x_,
			  y2 + offset_y_); 
		cursor_visible = true;
	}
}


// Returns true if we moved the screen
bool LyXScreen::FitManualCursor(long /*x*/, long y, int asc, int desc)
{
	long  newtop = first;
  
	if (y + desc  - first >= height_)
		newtop = y - 3*height_ / 4;   /* the scroll region must be so big!! */
	else if (y - asc < first && first > 0) {
		newtop = y - height_ / 4;
	}
	if (newtop < 0)
		newtop = 0;
  
	if (newtop != first){
		Draw(newtop);
		first = newtop;
		return true;
	}
	return false;
}


void  LyXScreen::HideManualCursor(long /*x*/, long /*y*/,
				  int /*asc*/, int /*desc*/)
{
	hideCursor();
}


void LyXScreen::ShowManualCursor(long x, long y, int asc, int desc)
{
	long y1 = 0;
	long y2 = 0;
	
	y1 = y - first - asc;
	if (y1 < 0)
		y1 =0;
	y2 = y -first + desc;
	if (y2 > height_)
		y2 = height_;
	
	if (cursor_pixmap){
		XFreePixmap(fl_display, cursor_pixmap);
		cursor_pixmap = 0;
	}
	
	if (y2 > 0 && y1 < height_) {
		cursor_pixmap_w = 1;
		cursor_pixmap_h = y2 - y1 + 1;
		cursor_pixmap_x = x,
		cursor_pixmap_y = y1;
		cursor_pixmap =
			XCreatePixmap (fl_display,
				       RootWindow(fl_display, 0),
				       cursor_pixmap_w,
				       cursor_pixmap_h,
				       fl_get_visual_depth());
		
		XCopyArea (fl_display,
			   window_,
			   cursor_pixmap,
			   gc_copy,
			   offset_x_ + cursor_pixmap_x,
			   offset_y_ + cursor_pixmap_y,
			   cursor_pixmap_w,
			   cursor_pixmap_h,
			   0, 0);
		XDrawLine(fl_display,
			  window_, gc_copy,
			  x+offset_x_,
			  y1+offset_y_,
			  x+offset_x_,
			  y2+offset_y_);			
	}
	cursor_visible = true;
}


void LyXScreen::hideCursor()
{
	if (!cursor_visible) return;
	
	if (cursor_pixmap){
		XCopyArea (fl_display, 
			   cursor_pixmap,
			   window_,
			   gc_copy,
			   0, 0, 
			   cursor_pixmap_w, cursor_pixmap_h,
			   cursor_pixmap_x + offset_x_,
			   cursor_pixmap_y + offset_y_);
	}
	cursor_visible = false;
}


void LyXScreen::CursorToggle()
{
	if (cursor_visible)
		hideCursor();
	else
		showCursor();
}


// Returns a new top so that the cursor is visible
long LyXScreen::TopCursorVisible()
{
	long  newtop = first;

	if (text->cursor.y
	    - text->cursor.row->baseline
	    + text->cursor.row->height
	    - first >= height_) {
		if (text->cursor.row->height < height_
		    && text->cursor.row->height > height_/4)
			newtop = text->cursor.y
				+ text->cursor.row->height
				- text->cursor.row->baseline - height_;
		else
			newtop = text->cursor.y
				- 3*height_ / 4;   /* the scroll region must be so big!! */
	} else if (text->cursor.y - text->cursor.row->baseline < first 
		   && first > 0) {
		if (text->cursor.row->height < height_
		    && text->cursor.row->height > height_/4)
			newtop = text->cursor.y - text->cursor.row->baseline;
		else {
			newtop = text->cursor.y - height_ / 4;
			if (newtop > first)
				newtop = first;
		}
	}
	if (newtop < 0)
		newtop = 0;
	
	return newtop;
}


// Scrolls the screen so that the cursor is visible, if necessary.
// Returns true if we move, otherwise false.
bool LyXScreen::FitCursor()
{
	// Is a change necessary?
	long newtop = TopCursorVisible();
	bool result = (newtop != first);
	if (result)
		Draw(newtop);
	return result;
}

   
void LyXScreen::Update()
{
	long y = 0;

	if (text->status == LyXText::NEED_MORE_REFRESH
	    || screen_refresh_y > -1 ) {
		if (screen_refresh_y > -1
		    && screen_refresh_y < text->refresh_y)
			y = screen_refresh_y;
		else
			y = text->refresh_y;
		
		if (y < first) y = first;
		
		DrawFromTo(y - first, height_);
		text->refresh_y = 0;
		text->status = LyXText::UNCHANGED;
		screen_refresh_y = -1;
		expose(0, y-first, width_, height_ - (y - first));
	} else if (text->status == LyXText::NEED_VERY_LITTLE_REFRESH) {
		// ok, we will update the current cursor row
		y = text->refresh_y;
		DrawOneRow(text->refresh_row, y);
		text->status = LyXText::UNCHANGED;
		expose(0, text->refresh_y-first,
		     width_, text->refresh_row->height);
	}
}


void LyXScreen::SmallUpdate()
{
	Row *row = 0;
	long y = 0;
	long y2 = 0;
	
	if (text->status == LyXText::NEED_MORE_REFRESH){
		// ok, I will update till the current cursor row
		row = text->refresh_row;
		y = text->refresh_y;
		y2 = y;
      
		if (y > text->cursor.y) {
			Update();
			return;
		}
	 
		while (row && row != text->cursor.row && y < first + height_) {
			DrawOneRow(row, y);
			row = row->next;
		}
      
		DrawOneRow(row, y);
		screen_refresh_y = y;
		screen_refresh_row = row->next;
		text->status = LyXText::UNCHANGED;
		// Is the right region exposed?
		expose(0, y2-first, width_, y-y2);
	} else if (text->status == LyXText::NEED_VERY_LITTLE_REFRESH) {
		// ok, I will update the current cursor row
		row = text->refresh_row;
		y = text->refresh_y;
		DrawOneRow(row, y);
		text->status = LyXText::UNCHANGED;
		expose(0, text->refresh_y - first,
		       width_, row->height);
	}
}


void LyXScreen::toggleSelection(bool kill_selection)
{
	long top = 0;
	long bottom = 0;
   
	// only if there is a selection
	if (!text->selection)
		return;
	
	top = text->sel_start_cursor.y
		- text->sel_start_cursor.row->baseline;
	bottom = text->sel_end_cursor.y
		- text->sel_end_cursor.row->baseline 
		+ text->sel_end_cursor.row->height;
	
	if (top - first < 0)
		top = first;
	if (bottom - first < 0)
		bottom = first;
	
	if (bottom - first > height_)
		bottom = first + height_;
	if (top - first > height_)
		top = first + height_;
	
	if (kill_selection)
		text->selection = 0;
	DrawFromTo(top - first, bottom - first);
	expose(0, top - first, width_, bottom - first - (top - first));
}
  
   
void LyXScreen::ToggleToggle()
{
	long top = 0;
	long bottom = 0;
	
	if (text->toggle_cursor.par == text->toggle_end_cursor.par
	    && text->toggle_cursor.pos == text->toggle_end_cursor.pos)
		return;

	top = text->toggle_cursor.y
		- text->toggle_cursor.row->baseline;
	bottom = text->toggle_end_cursor.y
		- text->toggle_end_cursor.row->baseline 
		+ text->toggle_end_cursor.row->height;
	
	if (top - first < 0)
		top = first;
	if (bottom - first < 0)
		bottom = first;
	
	if (bottom - first > height_)
		bottom = first + height_;
	if (top - first > height_)
		top = first + height_;
	
	DrawFromTo(top - first, bottom - first);
	expose(0, top - first, width_, bottom - first - (top - first));
}
