/* This file is part of
 * ======================================================
 * 
 *           LyX, The Document Processor
 * 	 
 *	    Copyright (C) 1995 Matthias Ettrich
 *          Copyright (C) 1995-1998 The LyX Team.
 *
 *======================================================*/

#include <config.h>

#ifdef __GNUG__
#pragma implementation
#endif

#include "textclass.h"
#include "filetools.h"
#include "debug.h"
#include "lyxlex.h"

// 	$Id: textclass.C,v 1.7 1998/10/22 16:56:20 larsbj Exp $

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: textclass.C,v 1.7 1998/10/22 16:56:20 larsbj Exp $";
#endif /* lint */


// This table is sorted alphabetically [asierra 30March96]
static keyword_item layoutTags[LT_LAST-1] = {
	{ "align",			LT_ALIGN },
	{ "alignpossible",		LT_ALIGNPOSSIBLE },
	{ "bibliography",		LT_BIBLIO },
	{ "block",			LT_BLOCK },
	{ "bottomsep",			LT_BOTTOMSEP },
	{ "center",			LT_CENTER },
	{ "centered_top_environment",	LT_CENTERED_TOP_ENVIRONMENT },
        { "classoptions",               LT_CLASSOPTIONS },
	{ "columns",			LT_COLUMNS },
	{ "command",			LT_COMMAND },
	{ "copystyle",                  LT_COPYSTYLE },
	{ "counter_chapter",		LT_COUNTER_CHAPTER },
	{ "counter_enumi",		LT_COUNTER_ENUMI },
	{ "counter_enumii",		LT_COUNTER_ENUMII },
	{ "counter_enumiii",		LT_COUNTER_ENUMIII },
	{ "counter_enumiv",		LT_COUNTER_ENUMIV },
	{ "counter_paragraph",		LT_COUNTER_PARAGRAPH },
	{ "counter_section",		LT_COUNTER_SECTION },
	{ "counter_subparagraph",	LT_COUNTER_SUBPARAGRAPH },
	{ "counter_subsection",		LT_COUNTER_SUBSECTION },
	{ "counter_subsubsection",	LT_COUNTER_SUBSUBSECTION },
	{ "defaultfont", 		LT_DEFAULTFONT },
	{ "double",                     LT_SPACING_DOUBLE },
	{ "dynamic",			LT_DYNAMIC },
	{ "empty",			LT_EMPTY },
	{ "end",			LT_END },
	{ "environment",		LT_ENVIRONMENT },
	{ "environment_default",	LT_ENVIRONMENT_DEFAULT },
	{ "fancyhdr",			LT_FANCYHDR },
	{ "fill_bottom",		LT_FILL_BOTTOM },
	{ "fill_top",			LT_FILL_TOP },
	{ "first_counter",              LT_FIRST_COUNTER },
	{ "first_dynamic",              LT_FIRST_DYNAMIC },
	{ "font",                  	LT_FONT },
	{ "fontsize",                  	LT_FONTSIZE },
	{ "freespacing",	   	LT_FREE_SPACING },
	{ "headings",              	LT_HEADINGS },
	{ "input", 			LT_INPUT },
	{ "intitle",                    LT_INTITLE },
	{ "item_environment",     	LT_ITEM_ENVIRONMENT },
	{ "itemsep",               	LT_ITEMSEP },
	{ "keepempty",                  LT_KEEPEMPTY },
	{ "labelbottomsep",             LT_LABEL_BOTTOMSEP },
	{ "labelfont",             	LT_LABELFONT },
	{ "labelindent",           	LT_LABELINDENT },
	{ "labelsep",              	LT_LABELSEP },
	{ "labelstring",           	LT_LABELSTRING },
	{ "labelstringappendix",	LT_LABELSTRING_APPENDIX },
	{ "labeltype",             	LT_LABELTYPE },
	{ "latexname",             	LT_LATEXNAME },
	{ "latexparam",			LT_LATEXPARAM },    //arrae970411
	{ "latextype",             	LT_LATEXTYPE },
	{ "layout",                	LT_LAYOUT },
	{ "left",                  	LT_LEFT },
	{ "leftmargin",            	LT_LEFTMARGIN },
	{ "list_environment",     	LT_LIST_ENVIRONMENT },
	{ "manual",                	LT_MANUAL },
	{ "margin",                	LT_MARGIN },
	{ "maxcounter",            	LT_MAXCOUNTER },
	{ "needprotect",                LT_NEED_PROTECT },
	{ "newline",			LT_NEWLINE },
	{ "nextnoindent",		LT_NEXTNOINDENT },
	{ "no_label",			LT_NO_LABEL },
	{ "nostyle",                    LT_NOSTYLE },
	{ "obsoletedby",                LT_OBSOLETEDBY },
	{ "onehalf",                    LT_SPACING_ONEHALF },
	{ "other",                      LT_OTHER },
	{ "pagestyle",			LT_PAGESTYLE },
	{ "paragraph",			LT_PARAGRAPH },
	{ "parindent",			LT_PARINDENT },
	{ "parsep",			LT_PARSEP },
	{ "parskip",			LT_PARSKIP },
	{ "plain",			LT_PLAIN },
	{ "preamble",                   LT_PREAMBLE },
	{ "right",			LT_RIGHT },
	{ "right_address_box",		LT_RIGHT_ADDRESS_BOX },
	{ "rightmargin",		LT_RIGHTMARGIN },
	{ "secnumdepth",                LT_SECNUMDEPTH },
	{ "sensitive",			LT_SENSITIVE },
	{ "sides",			LT_SIDES },
	{ "single",                     LT_SPACING_SINGLE },
	{ "spacing",                    LT_SPACING },
	{ "static",			LT_STATIC },
	{ "style",			LT_STYLE },
	{ "textfont",                   LT_TEXTFONT },
	{ "tocdepth",                   LT_TOCDEPTH },
	{ "top_environment",		LT_TOP_ENVIRONMENT },
	{ "topsep",			LT_TOPSEP }
};


LyXTextClass::LyXTextClass(string const &fn, string const &cln,
			   string const &desc)
	: name(fn), latexname(cln), description(desc)
{
	columns = 1;
	sides = 1;
	secnumdepth = 3;
	tocdepth = 3;
	pagestyle = "default";
	maxcounter = LABEL_COUNTER_CHAPTER;
	defaultfont = LyXFont(LyXFont::ALL_SANE);
	opt_fontsize = "10|11|12";
	opt_pagestyle = "empty|plain|headings|fancy";
	loaded = false;
}


bool LyXTextClass::eraseLayout(string const &name)
{
	LayoutList::iterator lit = layoutList.begin();
	for(; lit != layoutList.end(); ++lit) {
		if (lit->name == name) {
			layoutList.erase(lit);
			return true;
		}
	}
	return false;
}

/* Reads a textclass structure from file */
int LyXTextClass::read(string const &filename)
{
	LyXLex lexrc(layoutTags, sizeof(layoutTags)/sizeof(keyword_item));
	lexrc.setFile(filename);
	if (!lexrc.IsOK()) return -2;
	
	bool error = false;
	/* parsing */
	while (lexrc.IsOK() && !error) {
		switch(lexrc.lex()) {
		case -2:
			break;

		case -1:                                 
			lexrc.printError("Unknown tag `$$Token'");
			error = true;
			break;
			
		case LT_INPUT: // Include file
		        if (lexrc.next()) {
		        	string tmp = LibFileSearch("layouts",
							    lexrc.GetString(), 
							    "layout");
				
				if (read(tmp)) {
					lexrc.printError("Error reading input"
							 "file: "+tmp);
					error = true;
				}
			}
			break;

		case LT_STYLE:
			if (lexrc.next()) {
				string name =
					subst(lexrc.GetString(), '_', ' ');
				bool is_new = false;

				LyXLayout *tmpl = getLayout(name);
				if (!tmpl) {
					is_new = true;
					tmpl = new LyXLayout;
					tmpl->name = name;
				}

				lyxerr[Debug::TCLASS] << "  Reading style "
						      << tmpl->name << '\n';

				if (!tmpl->read(lexrc, *this)) {
					// Resolve fonts
					tmpl->resfont = tmpl->font;
					tmpl->resfont.realize(defaultfont);
					tmpl->reslabelfont = tmpl->labelfont;
					tmpl->reslabelfont.realize(defaultfont);
					if (is_new) {
						layoutList.push_back(*tmpl);
						delete tmpl;
					}
				} else {
				        lexrc.printError(
					               "Error parsing style `"
						       +tmpl->name+'\'');
					error = true;
				}
			} else {
				lexrc.printError("No name given for style: `$$Token'.");
				error = true;
			}
			break;

		case LT_NOSTYLE:
			if (lexrc.next()) {
				string style =
					subst(lexrc.GetString(), '_', ' ');
				if (!eraseLayout(style))
					lexrc.printError("Cannot delete style `$$Token'");
			}
			break;

		case LT_COLUMNS:
			if (lexrc.next())
				columns = lexrc.GetInteger();
			break;
			
		case LT_SIDES:
			if (lexrc.next())
				sides = lexrc.GetInteger();
			break;
			
		case LT_PAGESTYLE:
		        lexrc.next();
			pagestyle = strip(lexrc.GetString());
			break;
			
		case LT_DEFAULTFONT:
			defaultfont.lyxRead(lexrc);
			if (!defaultfont.resolved()) {
				lexrc.printError("Warning: defaultfont should "
						 "be fully instantiated!");
				defaultfont.realize(LyXFont::ALL_SANE);
			}
			break;

		case LT_MAXCOUNTER:
			switch (lexrc.lex()) {
			case LT_COUNTER_CHAPTER:
				maxcounter = LABEL_COUNTER_CHAPTER;
				break;
			case LT_COUNTER_SECTION:
				maxcounter = LABEL_COUNTER_SECTION;
				break;
			case LT_COUNTER_SUBSECTION:
				maxcounter = LABEL_COUNTER_SUBSECTION;
				break;
			case LT_COUNTER_SUBSUBSECTION:
				maxcounter = LABEL_COUNTER_SUBSUBSECTION;
				break;
			case LT_COUNTER_PARAGRAPH:
				maxcounter = LABEL_COUNTER_PARAGRAPH;
				break;
			case LT_COUNTER_SUBPARAGRAPH:
				maxcounter = LABEL_COUNTER_SUBPARAGRAPH;
				break;
			case LT_COUNTER_ENUMI:
				maxcounter = LABEL_COUNTER_ENUMI;
				break;
			case LT_COUNTER_ENUMII:
				maxcounter = LABEL_COUNTER_ENUMII;
				break;
			case LT_COUNTER_ENUMIII:
				maxcounter = LABEL_COUNTER_ENUMIII;
				break;
			case LT_COUNTER_ENUMIV:
				maxcounter = LABEL_COUNTER_ENUMIV;
				break;
			}
			break;

		case LT_SECNUMDEPTH:
			lexrc.next();
			secnumdepth = lexrc.GetInteger();
			break;

		case LT_TOCDEPTH:
			lexrc.next();
			tocdepth = lexrc.GetInteger();
			break;

	 // First step to support options 
	        case LT_CLASSOPTIONS:
	        {
	                bool getout = true;
	                while (getout && lexrc.IsOK()) { 
				switch (lexrc.lex()) {
				case LT_FONTSIZE:
					lexrc.next();
					opt_fontsize =
						strip(lexrc.GetString());
					break;
				case LT_PAGESTYLE:
					lexrc.next();
					opt_pagestyle =
						strip(lexrc.GetString());
					break;
				case LT_OTHER:
					lexrc.next();
					options = lexrc.GetString();
					break;
				case LT_END: getout = false; break;
				default:
					lexrc.printError("Out of context tag `$$Token'");
					break;
				}
			}
		        break;
		}

		case LT_PREAMBLE:
			preamble = lexrc.getLongString("EndPreamble");
			break;

		case LT_LEFTMARGIN:	/* left margin type */
		        if (lexrc.next())
				leftmargin = lexrc.GetString();
			break;			

		case LT_RIGHTMARGIN:	/* right margin type */
			if (lexrc.next())
				rightmargin = lexrc.GetString();
			break;

		default:
			lexrc.printError("Out of context tag `$$Token'");
			break;
		}
	}	

	if (error) {
#warning careful...
		// perhaps we should empty layoutList if we have error
		// that is what the old code did
		lyxerr << "We had errors while reading textclass.\n"
		       << "What shall we do about it.\n";
	}
	lyxerr[Debug::TCLASS] << "Finished reading input file "
			      << MakeDisplayPath(filename) << '\n';
		
	return error;
}


unsigned int LyXTextClass::numLayouts() const
{
	return layoutList.size();
}


LyXLayout const * LyXTextClass::getLayout(unsigned char layout) const
{
	if (layout >= layoutList.size()) {
		return 0;
	}
	return &layoutList[layout];
}


// this is private
LyXLayout* LyXTextClass::getLayout(string const &name)
{
	LayoutList::iterator lit = layoutList.begin();
	for(; lit != layoutList.end(); ++lit) {
		if (lit->name == name) {
			return lit;
		}
	}
	return 0;
}


LyXLayout const* LyXTextClass::getLayout(string const &name) const
{
	LayoutList::const_iterator lit = layoutList.begin();
	for(; lit != layoutList.end(); ++lit) {
		if (lit->name == name) {
			return lit;
		}
	}
	return 0;
}


short LyXTextClass::getNumberOfLayout(string const &name) const
{
	LayoutList::const_iterator tit = layoutList.begin();

	for (; tit != layoutList.end(); ++tit) {
		if (tit->name == name)
			return tit - layoutList.begin();
	}
	return -1;
}


string LyXTextClass::getNameOfLayout(unsigned char layout) const
{
	if (layout >= layoutList.size()) {
		return "@@end@@";
	}
	return layoutList[layout].name;
}


string LyXTextClass::getCompletePreamble() const
{
	LayoutList::const_iterator lit = layoutList.begin();
	string preamble;
	for(; lit != layoutList.end(); ++lit) {
		if (!lit->preamble.empty())
			preamble += lit->preamble;
	}
	return preamble;
}

// Load textclass info if not loaded yet
void LyXTextClass::load()
{
	if (loaded)
		return;

	// Read style-file
	string real_file = LibFileSearch("layouts", name, "layout");

	if (read(real_file)) {
		lyxerr << "Error reading `"
		       << MakeDisplayPath(real_file) << "'\n";
		lyxerr << "(Check `" << name << "')\n";
		lyxerr << "Check your installation and "
			"try Options/Reconfigure...\n";
	}
	loaded = true;
}
